/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



package actions;

import java.awt.event.ActionEvent;

import objets.CElementBase;
import objets.CListeObjets;
import objets.CMathGrafDoc;
import objets.CPrototype;
import types.NatureCalcul;
import types.NatureObjet;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import dialogs.ConstructionDlg;

@SuppressWarnings("serial")
public class ActionConstructionCreer extends MtgAction {
	public ActionConstructionCreer(MtgFrame frameInit) {
		super(frameInit, null, getString("menuConstructionCreer"), null, null, null, false);
	}

	@Override
	public int idMenu() {
		return 33153;
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		frame.activeOutilCapture();
		MtgInternalFrame cadre = frame.cadreActif();
		if (cadre == null) {
			frame.dialogueAvertissement(getString("messageOutilPasDisponible"));
			return;
		}
		if ((cadre.listeObjetsFinauxG.size() == 0) && (cadre.listeObjetsFinauxNG.size() == 0)) {
			frame.dialogueAvertissement(getString("messageAucunObjetFinal"));
			return;
		}
		// liste1 sera forme de tous les objets sources, objets non graphiques en premier
		CListeObjets liste1 = new CListeObjets();
		liste1.ajouteElementsDe(cadre.listeObjetsSourcesNG);
		liste1.ajouteElementsDe(cadre.listeObjetsSourcesG);
		// liste2 sera form de tous les objets finaux
		CListeObjets liste2 = new CListeObjets();
		liste2.ajouteElementsDe(cadre.listeObjetsFinauxNG);
		liste2.ajouteElementsDe(cadre.listeObjetsFinauxG);
		// On rordonne la liste de tous les objets finaux
		CListeObjets listePr = cadre.listePr;
		liste2.reordonne(listePr);
		if (!CPrototype.verifieObjetsSourcesTousUtilises(liste1, liste2)) {
			frame.dialogueAvertissement(getString("ch151"));
			frame.activeOutilCapture();
			return;
		}
		CListeObjets listeInterm = CPrototype.listeIntermediaires(liste1, liste2, listePr);
		// Comme on permet de dsigner comme obet final un point libre qui figurait dans les objets sources,
		// Il faut les retirer de la liste des objets sources
		for (int i = 0; i < liste1.size(); i++) {
			CElementBase elb = liste1.get(i);
			if ((elb.getNature() == NatureObjet.NPointBase) && (liste2.contains(elb))) {
				liste1.remove(elb);
				i--; // Car l'lment a t retir de la liste
				cadre.listeObjetsSourcesG.remove(elb);
			}
		}
		// Modification version 3.1.5 : Comme on peut maintenant choisir comme objet final une variable
		//  condition qu'elle ait d'abord t chosie dans les objets sources, il faut les retirer
		// de la liste des objets sources
		boolean aumoinsUneVariableRetiree = false;
		for (int i = 0; i < liste1.size(); i++) {
			CElementBase elb = liste1.get(i);
			if ((elb.getNatureCalcul() == NatureCalcul.NVariable) && (liste2.contains(elb))) {
				liste1.remove(elb);
				i--; // Car l'lment a t retir de la liste
				cadre.listeObjetsSourcesNG.remove(elb);
				aumoinsUneVariableRetiree = true;
			}
		}
		if (aumoinsUneVariableRetiree) frame.dialogueAvertissement(getString("ch171"));
		int natureSources;
		if (cadre.listeObjetsSourcesNG.size() == 0) natureSources = CPrototype.ConstructionEltsGraphiquesSeuls;
		else {
			if (cadre.listeObjetsSourcesG.size() == 0) natureSources = CPrototype.ConstructionEltsCalculsSeuls;
			else natureSources = CPrototype.ConstructionEltsMixtes;
		}

		CListeObjets liste = new CListeObjets();
		// Attention :il ne faut pas rordonner les lments sources car ceux
		// de type calcul doivent tre dsigns en premier
		liste.ajouteElementsDe(listeInterm);
		liste.ajouteElementsDe(liste2);
		liste.reordonne(listePr); // Important
		// On ajoute en tte de liste les lments de liste1 (sources)
		for (int i = liste1.size() - 1; i >= 0; i--) {
			CElementBase elb = liste1.get(i);
			liste.add(0, elb);
		}
		CPrototype proto = new CPrototype("", natureSources, liste1.size(), listePr, liste, liste2);
		ConstructionDlg dlg = new ConstructionDlg(frame, proto, false);
		dlg.setLocationRelativeTo(frame);
		dlg.setVisible(true);
		if (dlg.retourOK()) {
			CMathGrafDoc doc = cadre.document;
			doc.tablePrototypes.add(proto);
			cadre.videListesPourConstructions();
		}
		frame.actionEnCours = this;
	}
}
