/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Vector;
import javax.swing.JPanel;
import objets.CSimilitude;
import objets.CValeurAngle;
import objets.CValeurAssocieeAVariable;
import mathgraph32.MtgFrame;

@SuppressWarnings("serial")
public class AngleEtRapportSimilitudeDlg extends MtgDlg {
  private boolean modification;
  private EditeurAngle editeurAngle;
  private EditeurRapportHom editeurRapport;
  private MtgButton btnOK, btnCancel;
  private CSimilitude similitude;
  private CValeurAssocieeAVariable valRapport;
  private CValeurAngle valAngle;
  private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsRapport;
  private PaneBoutonsListeValeursEtFonctionsCombo paneListeValeurEtFonctionsAngle;

  /** Creates a new instance of AngleRotDlg */
  public AngleEtRapportSimilitudeDlg(MtgFrame frameInit, CSimilitude similitudeInit, boolean modificationInit) {
    super(getString("similitudeDlg1"), true, frameInit);
    modification = modificationInit;
    setResizable(false);
    similitude = similitudeInit;
    // Comme cette bote contient deux champs, au cas o l'utilisatuer change d'avis en quittant le dialogue
    // aprs avoir soumis sa rponse avec un champ faux et l'autre correct, il faut utiliser des copies
    // des deux champs
    valRapport = new CValeurAssocieeAVariable(similitude.rapport);
    valAngle = new CValeurAngle(similitude.angle);    
    MtgLabel labelAngle = new MtgLabel(frame, getString("angle"));
    editeurAngle = new EditeurAngle(frame, valAngle, similitude, frame.cadreActif().listePr.uniteAngle);
    MtgLabel labelRapport = new MtgLabel(frame, getString("rapportHomothetieDlg2"));
    editeurRapport = new EditeurRapportHom(frame, valRapport, similitude);
    if (modification) {
    	editeurAngle.setSelectedItem(similitude.angle.chaineInfo());
    	editeurRapport.setText(similitude.rapport.chaineInfo());
    }
    else editeurAngle.setSelectedIndex(-1);
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed(evt);
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed(evt);
      }
    });
  	int indmax;
  	if (modification) indmax = frame.cadreActif().listePr.indexOf(similitude) - 1;
  	else indmax = frame.cadreActif().listePr.size() - 1;
    paneListeValeurEtFonctionsRapport = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurRapport, true, indmax, false);
    paneListeValeurEtFonctionsAngle = new PaneBoutonsListeValeursEtFonctionsCombo(frame, editeurAngle, true, indmax, false);

    btnOK.setPreferredSize(btnCancel.getMaximumSize());    
    JPanel pane = new JPanel();
    pane.setLayout(new GridBagLayout());
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    
    setConstraints(constraints, 0, 0, 1, 1, 30, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(labelAngle, constraints);
    
    setConstraints(constraints, 1, 0, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    constraints.insets.bottom = 0;
    pane.add(editeurAngle, constraints);
    constraints.insets.bottom = 5;
    
    setConstraints(constraints, 1, 1, 1, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneListeValeurEtFonctionsAngle, constraints);

    setConstraints(constraints, 0, 2, 1, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(labelRapport, constraints);
    
    setConstraints(constraints, 1, 2, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    constraints.insets.bottom = 0;
    pane.add(editeurRapport, constraints);
    constraints.insets.bottom = 5;
    
    setConstraints(constraints, 1, 3, 1, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneListeValeurEtFonctionsRapport, constraints);

    setConstraints(constraints, 2, 0, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.EAST);
    pane.add(btnOK, constraints);

    setConstraints(constraints, 2, 1, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(btnCancel, constraints);
        
    setContentPane(pane);
    getRootPane().setDefaultButton(btnOK);
    
    // On change l'ordre par dfaut des composants
    Vector<Component> order = new Vector<Component>(4);
    order.add(editeurAngle.getEditor().getEditorComponent());
    order.add(editeurRapport);
    order.add(btnOK);
    order.add(btnCancel);
    mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
    setFocusTraversalPolicy(mtgFocusTraversalPolicy);
    
    pack();
  }  

  private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {
    if (editeurAngle.verifieValidite()) {
    	if (editeurRapport.verifieValidite()) {
        // On abandonne les objets CValeurAssocieeAVariable de similitude pour les remplacer par les locaaux
        similitude.angle = valAngle;
        similitude.rapport = valRapport;    		
	      setRetourOK(true);
	      dispose();
    	}
    	else editeurRapport.requestFocus();
    }
    else editeurAngle.requestFocus();
  }
  
  private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {                                          
    dispose();
  }       
}
