/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import objets.CCalculBase;
import objets.CCalculComplexe;
import objets.CListeObjets;
import types.Complexe;
import types.ErreurCalculException;
import types.Int;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

@SuppressWarnings("serial")
public class CalculComplexeDlg extends MtgDlg {

	private CCalculComplexe calculAModifier;
	private boolean modification;
	private MtgButton btnCancel;
  private MtgButton btnOK;
  private MtgLabel label1;
  private MtgLabel label2;
  private MtgTextField editeurFormule;
  private MtgTextField editeurNom;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctions;
  
  public CalculComplexeDlg(MtgFrame frameInit, CCalculComplexe calculAModifierInit, boolean modificationInit) {
    super (getString("creerCalculDlg0"), true, frameInit);
    calculAModifier = calculAModifierInit;
    modification = modificationInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);    
    if (modification) editeurFormule.requestFocus(); else editeurNom.requestFocus();
  }
  private void initComponents() {
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed(evt);
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed(evt);
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());    
    label1 = new MtgLabel(frame, getString("creerCalculDlg1"), JLabel.RIGHT);
    label1.setRequestFocusEnabled(false);
    label2 = new MtgLabel(frame, getString("creerCalculDlg2"), JLabel.RIGHT);
    label2.setRequestFocusEnabled(false);
    editeurFormule = new MtgTextField(frame, 30);
    editeurFormule.setAutoscrolls(true);
  	int indmax;
  	if (modification) indmax = frame.cadreActif().listePr.indexOf(calculAModifier) - 1;
  	else indmax = frame.cadreActif().listePr.size() - 1;
    paneListeValeurEtFonctions = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurFormule, false, indmax, false);
    editeurNom = new MtgTextField(frame, 8);
    if (modification) {
    	// String[] ch = new String[1];
    	// ch[0] = "";    	
      editeurFormule.setText(calculAModifier.calcul.chaineCalculSansPar(null));
      editeurNom.setText(calculAModifier.nom());
      editeurNom.setEditable(false);
    }
    JPanel panePrincipal = new JPanel();
    JPanel paneLeft = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new FlowLayout());
    paneLeft.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 30, 40, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(label1, constraints);
    
    setConstraints(constraints, 1, 0, 2, 1, 70, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    constraints.insets.bottom = 0;
    paneLeft.add(editeurNom, constraints);
    
    setConstraints(constraints, 0, 1, 1, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.EAST);
    paneLeft.add(label2, constraints);
   
    setConstraints(constraints, 1, 1, 2, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(editeurFormule, constraints);
    
    constraints.insets.bottom = 5;
    setConstraints(constraints, 1, 2, 1, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(paneListeValeurEtFonctions, constraints);
    
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    panePrincipal.add(paneLeft);
    panePrincipal.add(paneRight);
    setContentPane(panePrincipal);
    pack();
  }
  private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {                                      
    String chNom = editeurNom.getText();
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets listePr = cadre.listePr;
    if (!modification) {
	    // On vrifie d'abord la validit du nom de la fonction
	    if (!listePr.validationNomCalcul(frame, chNom)) {
	      editeurNom.requestFocus();
	      return;
	    }
    }
    //Le nom est correct. On vrifie maintenant le calcul
    String chCalcul = editeurFormule.getText();
    Int indErr = new Int();
    // On n'autorise d'utiliser que les objets crs par l'utilisateur
    int indDernier;
    if (modification) indDernier = listePr.indexOf(calculAModifier);
    else indDernier = listePr.size()-1;    
    if (CCalculBase.verifieSyntaxeComplexe(listePr, chCalcul, indErr, indDernier, null)) {
      CCalculBase calc = CCalculBase.creeCalculBaseComplexe(chCalcul ,listePr, 0,
              chCalcul.length()-1, null);
      try {
        Complexe v = new Complexe();
        calc.listeProprietaire.initialiseNombreIterations();
				calc.resultatComplexe(false, v);
      }
      catch (ErreurCalculException e) {
      	if (e.depassementCapacite) {
      		dialogueAvertissement(getString("ch179"));
        	editeurFormule.requestFocus();
        	return;
      	}
        if (!frame.dialogueConfirmation_YES_NO(getString("ch4"))) {
        	editeurFormule.requestFocus();
        	return;
        }
      }
      calculAModifier.calcul = calc;

      // Modification version 4.5.2 : Quand la chane de calcul sera reconstruite elle pourra tre diffrente de celle ici entre
      // On la change tout de suite
      calculAModifier.chaineCalcul = calc.chaineCalculSansPar(null);
      
      if (!modification) calculAModifier.nomCalcul = chNom;
      if (modification) cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil48"));
      setRetourOK(true);
      dispose();
    }
    else {
    	dialogueAvertissement(getString("ch5"));
      editeurFormule.setCaretPosition(indErr.getValue());
      editeurFormule.requestFocus();
      return;     
    }
  }                                     

  private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {                                          
    dispose();
  }  
}
