/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToggleButton;

import mathgraph32.MtgFrame;
import types.FonteNom;
import types.NatureCarSpe;

@SuppressWarnings("serial")
public class CaracteresSpeciauxLatexDlg extends JDialog {

	private MtgFrame frame;
	private JToggleButton btnGrec = null;
	private JToggleButton btnMath = null;
	private JToggleButton btnFleches = null;
	private PaneInsertionCaractere paneInsertionCaractere; // Le panneau qui correspond au choix fait
  private JScrollPane scrollPane;
  private int hauteurFenetreProprietaire;

	public CaracteresSpeciauxLatexDlg(JDialog owner, String title, MtgFrame frameInit, int NatureCarPermis) {
		super(owner, title, false); // le dialogue n'est pas modal
		setFocusableWindowState(false); // Pour que le champ d'dition associ ne perde pas le focus
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE); // Pour qu'on ne puisse pas fermer la fentre
		hauteurFenetreProprietaire = owner.getHeight();
		frame = frameInit;
		ButtonGroup btnGroup = new ButtonGroup();
		if ((NatureCarPermis & NatureCarSpe.Grec) != 0) {
			btnGrec = new JToggleButton(MtgFrame.getString("ch81"));
			btnGrec.addActionListener(new ActionListener() {
	      public void actionPerformed(ActionEvent evt) {
	        btnGrecActionPerformed();
	      }			
			});
		}
		if ((NatureCarPermis & NatureCarSpe.Math) != 0) {
			btnMath = new JToggleButton(MtgFrame.getString("ch82"));
			btnMath.addActionListener(new ActionListener() {
	      public void actionPerformed(ActionEvent evt) {
	        btnMathActionPerformed();
	      }			
			});
		}
		if ((NatureCarPermis & NatureCarSpe.Fleches) != 0) {		
			btnFleches = new JToggleButton(MtgFrame.getString("ch83"));
			btnFleches.addActionListener(new ActionListener() {
	      public void actionPerformed(ActionEvent evt) {
	        btnFlechesActionPerformed();
	      }			
			});
		}
		getContentPane().setLayout(new BorderLayout());
		JPanel paneHigh = new JPanel();
		paneHigh.setLayout(new GridBagLayout());
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.top = 2;
    constraints.insets.bottom = 2;
    constraints.insets.left = 2;
    constraints.insets.right = 2;
		
		int nbBoutons = 0;
		if (btnMath != null) {
			nbBoutons ++;
			btnMath.setFont(FonteNom.getFonteNom(MtgFrame.sizeFontButton, FonteNom.Normal,
					(double)frame.pref_addSystemFont/MtgFrame.sizeFontButton + MtgFrame.coefMultForDlgBoxIcons));
	    MtgDlg.setConstraints(constraints, 0, 0, 1, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
			btnGroup.add(btnMath);
			paneHigh.add(btnMath, constraints);
		}
		if (btnGrec != null) {
			nbBoutons ++;
			btnGrec.setFont(FonteNom.getFonteNom(MtgFrame.sizeFontButton, FonteNom.Normal,
					(double)frame.pref_addSystemFont/MtgFrame.sizeFontButton + MtgFrame.coefMultForDlgBoxIcons));
	    MtgDlg.setConstraints(constraints, nbBoutons - 1, 0, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
			btnGroup.add(btnGrec);
			paneHigh.add(btnGrec, constraints);
		}
		if (btnFleches != null) {
			nbBoutons ++;
			btnFleches.setFont(FonteNom.getFonteNom(MtgFrame.sizeFontButton, FonteNom.Normal,
					(double)frame.pref_addSystemFont/MtgFrame.sizeFontButton + MtgFrame.coefMultForDlgBoxIcons));
			btnGroup.add(btnFleches);
			if (nbBoutons != 3) MtgDlg.setConstraints(constraints, nbBoutons - 1, 0, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
			else MtgDlg.setConstraints(constraints, 0, 1, 2, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER);
			paneHigh.add(btnFleches, constraints);
		}
		if (btnMath != null) {
			btnMath.setSelected(true);
			paneInsertionCaractere = frame.paneLatexInsertionCaractereMath;
		}
		else {
			if (btnGrec != null){
				btnGrec.setSelected(true);
				paneInsertionCaractere = frame.paneLatexInsertionCaractereGrec;
			}
			else if(btnFleches != null) {
				btnFleches.setSelected(true);
				paneInsertionCaractere = frame.paneLatexInsertionCaractereFleche;
			}
		}
		getContentPane().add(paneHigh, "North");
    scrollPane = new JScrollPane
			(paneInsertionCaractere, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		getContentPane().add(scrollPane, "Center");
		pack();
    setPreferredSize(new Dimension(getPreferredSize().width, hauteurFenetreProprietaire/2));
    pack();
	}
	
	public void AssocieA(JComponent champAssocie) {
		frame.paneLatexInsertionCaractereGrec.associeA(champAssocie);
		frame.paneLatexInsertionCaractereMath.associeA(champAssocie);
		frame.paneLatexInsertionCaractereFleche.associeA(champAssocie);
	}
	
	public void btnGrecActionPerformed() {
		if (paneInsertionCaractere != frame.paneLatexInsertionCaractereGrec) {
			getContentPane().remove(scrollPane);
			paneInsertionCaractere = frame.paneLatexInsertionCaractereGrec;
	    scrollPane = new JScrollPane
			(paneInsertionCaractere, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
			getContentPane().add(scrollPane, "Center");
			pack();
		}
	}

	public void btnMathActionPerformed() {
		if (paneInsertionCaractere != frame.paneLatexInsertionCaractereMath) {
			getContentPane().remove(scrollPane);
			paneInsertionCaractere = frame.paneLatexInsertionCaractereMath;
	    scrollPane = new JScrollPane
			(paneInsertionCaractere, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
			getContentPane().add(scrollPane, "Center");
			pack();
		}
	}
	
	public void btnFlechesActionPerformed() {
		if (paneInsertionCaractere != frame.paneLatexInsertionCaractereFleche) {
			getContentPane().remove(scrollPane);
			paneInsertionCaractere = frame.paneLatexInsertionCaractereFleche;
	    scrollPane = new JScrollPane
			(paneInsertionCaractere, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
			getContentPane().add(scrollPane, "Center");
			pack();
		}
	}
	
	public void positionneParRapportDialoguePere() {
		Container cont = getParent();
		Point p1 = cont.getLocation();
		Dimension s = cont.getSize();
		Dimension d = getSize();
		if (((MtgDlg) getParent()).estCentreSurFenetre()) setLocation(p1.x + s.width , p1.y + s.height - d.height);	
		else setLocation(p1.x + s.width - d.width/2 , p1.y + s.height - d.height);		
	}
	
}
