/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import objets.CPrototype;
import mathgraph32.MtgFrame;

/**
 * Dialogue servant  choisir un prototype d'un document pour l'implmenter
 * @author Yves
 *
 */
@SuppressWarnings("serial")
public class ChoixPrototypeFigureDlg extends MtgDlg {
	String chButtonOK; // La chane pour le bouton de validation
	private MtgLabel label1, label2;
	private MtgButton btnOK, btnCancel;
	private ListePrototypes listeProtos;
	private JScrollPane scrollPaneProto;
	private MtgTextArea info;
	private JScrollPane scrollPaneInfo;
	public CPrototype prototypeChoisi;
	
  public ChoixPrototypeFigureDlg(MtgFrame frameInit, String chButtonOKInit, String titreInit) {
  	super(titreInit, true, frameInit);
  	chButtonOK = chButtonOKInit;
  	initComponents();
    getRootPane().setDefaultButton(btnOK);      	
  }
  
  private void initComponents() {
    label1 = new MtgLabel(frame, getString("choixPrototypeFigureDlg2"));
    label2 = new MtgLabel(frame, getString("informationSurConstruction"));
    btnOK = new MtgButton(frame, chButtonOK);
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    listeProtos = new ListePrototypes(frame, frame.cadreActif().document);
    listeProtos.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
      	listeProtosSelectionChanged();
      }
    });
    listeProtos.addMouseListener(new MouseListener() {

			public void mouseClicked(MouseEvent evt) {
				listeProtosMouseClicked(evt);
			}

			public void mouseEntered(MouseEvent evt) {
			}

			public void mouseExited(MouseEvent evt) {
			}

			public void mousePressed(MouseEvent evt) {
			}

			public void mouseReleased(MouseEvent evt) {
			}
    });
    // Modification version 5.0 pour tenri compte des crans haute dfinition
    double k = MtgFrame.coefMultForDlgBoxIcons;
    scrollPaneProto = new JScrollPane(listeProtos, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    scrollPaneProto.setPreferredSize(new Dimension((int)(k*450), (int)(k*150)));  // Ajout version 3.5
		info = new MtgTextArea(frame);
		info.setEditable(false);
		info.setColumns(40);
		info.setLineWrap(false);
		info.setRows(10);
    scrollPaneInfo = new JScrollPane(info, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    scrollPaneInfo.setPreferredSize(new Dimension((int)(k*450), (int)(k*240)));    

    
    JPanel panePrincipal = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;

    setConstraints(constraints, 0, 0, 1, 1, 80, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label1, constraints);
    setConstraints(constraints, 0, 1, 1, 1, 0, 40, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneProto, constraints);
    setConstraints(constraints, 0, 2, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label2, constraints);
    setConstraints(constraints, 0, 3, 1, 1, 0, 40, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneInfo, constraints);
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTH);
    panePrincipal.add(paneRight, constraints);
    
    setContentPane(panePrincipal);
    pack(); 
    listeProtosSelectionChanged();
  }

  private void btnOKActionPerformed() {
  	prototypeChoisi = listeProtos.prototypeChoisi();
  	setRetourOK(true);
  	dispose();
  }
  
  private void btnCancelActionPerformed() {
  	dispose();
  }
  
  private void listeProtosSelectionChanged() {
  	int i = listeProtos.getSelectedIndex();
  	if (i != -1) {
  		info.setText(listeProtos.prototypeChoisi().commentaire);
  		info.setCaretPosition(0);
  		pack();
  	}  	
	
  }
  
  public void listeProtosMouseClicked(MouseEvent evt) {
  	if ((evt.getClickCount() == 2) && (listeProtos.getSelectedIndex() != -1)) btnOKActionPerformed();
  }
  
}
