/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.URI;
import java.net.URL;
import java.text.NumberFormat;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import mathgraph32.Global;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import entreesSorties.MtgFileManager;
import glob.Kernel;

@SuppressWarnings("serial")
// Classe ajoute version 4.7.7
public class Html5ExportDlg extends MtgDlg {
	private MtgCheckBox cbLaunch; // Sera true si on veut lancer le navigateur
																// avec la figure gnre aprs l'exportation
	// Version 4.4.1 : Deux applets proposes
	private MtgLabel label1, label2, label3, label4, label5, label6;
	private MtgButton btnExporter, btnCancel;
	private MtgTextField titleEditor, authorNameEditor;
	private MtgFormattedTextField widthEditor, heightEditor;
	private MtgTextPane textEditor1, textEditor2;
	private JPanel paneHaut, paneBas, paneGeneral;

	class MyFilter extends javax.swing.filechooser.FileFilter {
		public boolean accept(File file) {
			if (file.isDirectory())
				return true;
			String filename = file.getName().toLowerCase();
			return filename.endsWith(Global.GLOBAL_COMMA_HTM) || filename.endsWith(Global.GLOBAL_COMMA_HTML);
		}

		public String getDescription() {
			return Global.GLOBAL_COMMA_STAR_HTM + Global.GLOBAL_COMMA_STAR_HTM;
		}
	}

	public Html5ExportDlg(MtgFrame frameInit) {
		super(getString("dlgExportHtml1"), true, frameInit);
		paneHaut = new JPanel();
		paneBas = new JPanel();
		paneGeneral = new JPanel();
		label1 = new MtgLabel(frame, getString("titrePage"));
		label2 = new MtgLabel(frame, getString("auteur"));
		label3 = new MtgLabel(frame, getString("texteHtm1"));
		label4 = new MtgLabel(frame, getString("texteHtm2"));
		label5 = new MtgLabel(frame, getString("largeur"));
		label6 = new MtgLabel(frame, getString("hauteur"));

		btnExporter = new MtgButton(frame, getString("btnExporter"));
		btnExporter.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnExporterActionPerformed();
			}
		});
		btnCancel = new MtgButton(frame, getString("btnCancel"));
		btnCancel.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnCancelActionPerformed();
			}
		});
		titleEditor = new MtgTextField(frame, 20);
		authorNameEditor = new MtgTextField(frame, 20);
		authorNameEditor.setText(System.getProperty("user.name"));
		textEditor1 = new MtgTextPane(frame);
		textEditor1.setContentType("text/html");
		JScrollPane scrollPaneText1 = new JScrollPane(textEditor1, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		// Modification version 5.0 pour tenri compte des crans haute dfinition
		double k = MtgFrame.coefMultForDlgBoxIcons;
		scrollPaneText1.setPreferredSize(new Dimension((int) (k * 500), (int) (k * 120)));

		textEditor2 = new MtgTextPane(frame);
		textEditor2.setContentType("text/html");
		JScrollPane scrollPaneText2 = new JScrollPane(textEditor2, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPaneText2.setPreferredSize(new Dimension((int) (k * 500), (int) (k * 120)));
		NumberFormat format;
		format = NumberFormat.getInstance();
		format.setParseIntegerOnly(true);
		format.setGroupingUsed(false);
		widthEditor = new MtgFormattedTextField(frame, format);
		widthEditor.setColumns(5);
		MtgInternalFrame cadre = frame.cadreActif();
		widthEditor.setText(Integer.toString(cadre.dimf.longueurAbscisses));
		heightEditor = new MtgFormattedTextField(frame, format);
		heightEditor.setColumns(5);
		heightEditor.setText(Integer.toString(cadre.dimf.longueurOrdonnees));

		cbLaunch = new MtgCheckBox(frame, getString("dlgExport10"), true);

		JPanel panePrincipal = new JPanel();
		paneBas = new JPanel();
		panePrincipal.setLayout(new FlowLayout());
		paneGeneral = new JPanel();
		paneGeneral.setLayout(new GridBagLayout());

		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 5;
		constraints.insets.top = 5;
		constraints.insets.left = 5;
		constraints.insets.right = 5;
		paneHaut.setLayout(new GridBagLayout());
		paneBas.setLayout(new GridBagLayout());
		JPanel paneBas2 = new JPanel();
		paneBas2.setLayout(new BoxLayout(paneBas2, BoxLayout.X_AXIS));
		paneBas2.add(btnExporter);
		paneBas2.add(Box.createRigidArea(new Dimension(10, 0)));
		paneBas2.add(btnCancel);
		setConstraints(constraints, 0, 0, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER);
		paneBas.add(paneBas2, constraints);
		setConstraints(constraints, 0, 1, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneBas.add(cbLaunch, constraints);

		setConstraints(constraints, 0, 0, 1, 1, 20, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneHaut.add(label1, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 80, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneHaut.add(titleEditor, constraints);
		setConstraints(constraints, 0, 1, 1, 1, 20, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneHaut.add(label2, constraints);
		setConstraints(constraints, 1, 1, 1, 1, 80, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneHaut.add(authorNameEditor, constraints);
		JPanel paneDimensions = new JPanel();
		paneDimensions.add(label5);
		paneDimensions.add(widthEditor);
		paneDimensions.add(label6);
		paneDimensions.add(heightEditor);
		setConstraints(constraints, 0, 2, 2, 1, 0, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneHaut.add(paneDimensions, constraints);

		setConstraints(constraints, 0, 0, 1, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneGeneral.add(label3, constraints);
		setConstraints(constraints, 0, 1, 1, 1, 0, 10, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		paneGeneral.add(scrollPaneText1, constraints);
		setConstraints(constraints, 0, 2, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneGeneral.add(label4, constraints);
		setConstraints(constraints, 0, 3, 1, 1, 0, 10, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		paneGeneral.add(scrollPaneText2, constraints);

		panePrincipal.setLayout(new BoxLayout(panePrincipal, BoxLayout.Y_AXIS));
		panePrincipal.add(paneHaut);
		panePrincipal.add(paneGeneral);
		panePrincipal.add(paneBas);
		setContentPane(panePrincipal);
		getRootPane().setDefaultButton(btnExporter);
		pack();
	}

	private void btnCancelActionPerformed() {
		dispose();
	}

	private void btnExporterActionPerformed() {
		int width, height;
		try {
			width = Integer.parseInt(widthEditor.getText());
		} catch (NumberFormatException e) {
			dialogueAvertissement(getString("ch71"));
			widthEditor.requestFocus();
			return;
		}
		try {
			height = Integer.parseInt(heightEditor.getText());
		} catch (NumberFormatException e) {
			dialogueAvertissement(getString("ch71"));
			heightEditor.requestFocus();
			return;
		}
		URL adr = MtgFrame.class.getResource("export/export.html");
		try {
			InputStream is = adr.openStream();
			InputStreamReader ipsr = new InputStreamReader(is);
			BufferedReader br = new BufferedReader(ipsr);
			StringBuffer sb = new StringBuffer(10000); // Gros car peut contenir des
																									// images
			String s;
			while ((s = br.readLine()) != null) {
				sb.append(s + "\r\n");
			}
			br.close();
			String texte = sb.toString();
			// Ajout version 4.9.2
			// Modifi version 5.2.0 en utilisant des replace et il faut tenir compte du fait qu'il y a aussi une rfrence  la feuille css
			texte = texte.replace("*version", MtgFrame.numeroVersion);
			// Fin ajout
			texte = texte.replace("*width=", "width=" + width);
			texte = texte.replace("*height=", "height=" + height);
			

			String nomAuteur = Kernel.convertStringToHTML(authorNameEditor.getText());
			texte = texte.replace("*auteur", getString("auteur") + " " + nomAuteur);
			String title = Kernel.convertStringToHTML(titleEditor.getText());
			// On met en place le titre de la page
			texte = texte.replace("<title>", title + ((title.length() !=0) ? " - "  : "")+ getString("feuilleTravail"));
			texte = texte.replace("*title", title);
			String texte1 = extraitCodeHTML(textEditor1.getText());
			texte = texte.replace("*texte1",  texte1);
			String texte2 = extraitCodeHTML(textEditor2.getText());
			texte = texte.replace("*texte2", texte2);
			texte = texte.replace("*creeAvec", getString("creeAvec"));
			// Le code Base64 dcrivant la figure
			/*
			ind = texte.indexOf("*figureData");
			texte = texte.substring(0, ind) + frame.cadreActif().getXML() + texte.substring(ind + 11);
			*/
			texte = texte.replace("*figureData", frame.cadreActif().getXML());
			// On enregistre le fichier html modifi
			JFileChooser fc = new JFileChooser(frame.pref_constDirectory);
			boolean hres = MtgFrame.screenIsHighResolution;
			fc.setPreferredSize(new Dimension(hres ? 1200 : 800, hres ? 800 : 600));
			fc.setFileFilter(new MyFilter());
			String name = frame.cadreActif().nomDocument();
			int indpoint = name.lastIndexOf(".");
			if (indpoint != -1)
				name = name.substring(0, name.lastIndexOf("."));
			File fileInit = new File(frame.repertoireFichiers + File.separator + name + Global.GLOBAL_COMMA_HTML);
			fc.setSelectedFile(fileInit);
			if (fc.showSaveDialog(frame) == JFileChooser.APPROVE_OPTION) {
				File file = fc.getSelectedFile();
				if (file != null) {
					String ch = file.getAbsolutePath();
					if (!ch.toLowerCase().endsWith(Global.GLOBAL_COMMA_HTML)) {
						ch = ch + Global.GLOBAL_COMMA_HTML;
						file = new File(ch);
					}
					if (!MtgFileManager.accepteEnregistrement(frame, ch))
						return;
					FileOutputStream fos;
					try {
						fos = new FileOutputStream(file);
						OutputStreamWriter opsr = new OutputStreamWriter(fos);
						BufferedWriter bfw = new BufferedWriter(opsr);
						bfw.write(texte);
						bfw.close();
						// Si la case est reste coche, on lance le navigateur par dfaut
						// avec le fichier html cr
						if (cbLaunch.isSelected()) {
							try {
								String path = file.getAbsolutePath();
								path = path.replaceAll(" ", "%20");
								// Pas d'appel  replaceAll  cause des caractres antislash
								StringBuffer buf = new StringBuffer(path.length());
								for (int i = 0; i < path.length(); i++) {
									char car = path.charAt(i);
									if (car == '\\')
										car = '/';
									buf.append(car);
								}
								path = buf.toString();
								Desktop.getDesktop().browse(new URI("file:///" + path));
							} catch (Exception e2) {
							}
						}
					} catch (IOException e1) {
						frame.dialogueAvertissement(getString("ch45"));
						return;
					}

				}
			}
			dispose();

		} catch (IOException e) {
		}
	}

	private final static String extraitCodeHTML(String ch) {
		String ch1 = ch.toLowerCase();
		int ind1 = ch1.indexOf("<body");
		int ind2 = ch1.indexOf("</body>");
		String ch2 = ch.substring(ch1.indexOf(">", ind1) + 1, ind2);
		ind1 = ch2.indexOf("<p>");
		ind2 = ch2.lastIndexOf("</p>");
		// Modification version 5.2.0.2 : On donne une taille de 2 dans le code html
		// return ch2.substring(ind1 + 1, ind2);
		ch2 = ch2.substring(ind1 + 1, ind2);
		int ind = ch2.toLowerCase().indexOf("size=");
		if (ind != -1) {
			int ind3 = ch2.substring(ind+6).indexOf("\"");
			ch2 = ch2.substring(0, ind) + "size=\"4\"" + ch2.substring(ind + 7 + ind3);
		}
		return ch2;
	}

}