/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.text.NumberFormat;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;

import objets.CLieuDeBase;
import types.InfoLieu;

import mathgraph32.MtgFrame;

@SuppressWarnings("serial")
public class LieuDlg extends MtgDlg {
	private CLieuDeBase lieuAModifier;
  private MtgButton btnCancel;
  private MtgButton btnOK;
  private MtgLabel label1;
  private MtgLabel label2;
  private MtgFormattedTextField editeurNombrePoints;
  private MtgCheckBox checkLieuFerme;
  private MtgCheckBox checkGestionAuto;

  public LieuDlg(MtgFrame frameInit, CLieuDeBase lieuAModifierInit) {
    super (getString("lieuDialog1"), true, frameInit);
    lieuAModifier = lieuAModifierInit;
    initComponents();
    editeurNombrePoints.setSelectionStart(0);
    editeurNombrePoints.setSelectionEnd(editeurNombrePoints.getText().length());
    editeurNombrePoints.requestFocus();
    getRootPane().setDefaultButton(btnOK);
  }

  private void initComponents() {
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed(evt);
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed(evt);
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());  
    label1 = new MtgLabel(frame, getString("lieuDialog2"), JLabel.RIGHT);
    label1.setRequestFocusEnabled(false);
    label2 = new MtgLabel(frame, getString("lieuDialog3"), JLabel.RIGHT);
    label2.setRequestFocusEnabled(false);
    NumberFormat format;
    format = NumberFormat.getInstance();
    format.setParseIntegerOnly(true);
    format.setGroupingUsed(false);
    editeurNombrePoints = new MtgFormattedTextField(frame, format);
    editeurNombrePoints.setColumns(4);
    checkLieuFerme = new MtgCheckBox(frame, getString("lieuDialog4"));
    checkGestionAuto = new MtgCheckBox(frame, getString("GestionAutoDiscontinuites"));
    
  	InfoLieu info = lieuAModifier.infoLieu;
  	editeurNombrePoints.setText(Integer.toString(info.nombreDePoints));
  	editeurNombrePoints.setSelectionStart(0);
  	editeurNombrePoints.setSelectionEnd(editeurNombrePoints.getText().length());
  	checkLieuFerme.setSelected(info.fermeOuNon);
  	checkGestionAuto.setSelected(info.gestionDiscontinuite);
    	
    JPanel panePrincipal = new JPanel();
    JPanel paneLeft = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new FlowLayout());
    paneLeft.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 0;
    constraints.insets.top = 0;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 50, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(label1, constraints);	
    setConstraints(constraints, 1, 0, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(editeurNombrePoints, constraints);
    constraints.insets.bottom = 5;
    setConstraints(constraints, 0, 1, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(label2, constraints);
    setConstraints(constraints, 0, 2, 2, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(checkLieuFerme, constraints);
    setConstraints(constraints, 0, 3, 2, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(checkGestionAuto, constraints);

    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    panePrincipal.add(paneLeft);
    panePrincipal.add(paneRight);
    setContentPane(panePrincipal);
    pack();
  }
  
  private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {
  	int nombrePoints;
  	try {
  		nombrePoints = Integer.parseInt(editeurNombrePoints.getText());
  	}
		catch (NumberFormatException e) {
			dialogueAvertissement(getString("ch71"));
			editeurNombrePoints.requestFocus();
			return;
		}
 	
    if ((nombrePoints < 4 ) || nombrePoints > 5000) {
    	dialogueAvertissement(getString("ch71"));
    	editeurNombrePoints.requestFocus();
    	return;
    }
    int anciennombrePoints = lieuAModifier.infoLieu.nombreDePoints;
    lieuAModifier.infoLieu.nombreDePoints = nombrePoints;
    lieuAModifier.infoLieu.fermeOuNon = checkLieuFerme.isSelected();
    lieuAModifier.infoLieu.gestionDiscontinuite = checkGestionAuto.isSelected();
    // Si on a chang le nombre de points du lieu il faut
    // revoir toutes les surfaces ayant ce lieu pour bord et recrer leurs tabelaux de coorodnnes
  	if (anciennombrePoints != nombrePoints) {
  		lieuAModifier.changeCaracteristiques();
  		frame.cadreActif().listePr.metAJourSurfacesAyantPourBord(lieuAModifier);
  	}
    setRetourOK(true);
    dispose();
  }
  
  private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {                                          
    dispose();
  }       
}