/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.util.ListIterator;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.ListSelectionModel;

import mathgraph32.MtgFrame;
import objets.CCalcul;
import objets.CElementBase;
import objets.CListeObjets;
import objets.CValeurDynamique;
import types.FonteNom;

/**
 * Cr pour version 4.5.2
 * Liste prsentant les calculs dfinis par une formule et les fonctions utlisateur
 * Utilis dans le dialogue de cration  d'un CEditeurFormule
 * Modifi version 4.6 pour tre utilis dans le dialogue de cration d'un test d'quivalence entre deux calculs.
 * @author Yves
 *
 */
@SuppressWarnings("serial")
public class ListeCalculsOuFonctionsParFormule extends JList<String> {
  private CListeObjets liste; // La liste contenant les valeurs  afficher
  private int indiceMaxi; // l'indice maxi permis pour un objet  insrer dans la liste
  private boolean intermPermis; // True si les objets intermdiaires sont autoriss dans la liste
  public Vector<CValeurDynamique> pointeurs;
  private DefaultListModel<String> listModel;

  /**
   * 
   * @param listeInit
   * @param indiceMaxiInit
   * @param intermPermisInit
   * @param natureInit : La nature des objets  afficher initialement
   */
  public ListeCalculsOuFonctionsParFormule(MtgFrame frame, CListeObjets listeInit, int indiceMaxiInit, boolean intermPermisInit, long natureInit) {
    super();
		setFont(FonteNom.getFonteEdition(MtgFrame.sizeFontList, FonteNom.Normal,
				(double)frame.pref_addSystemFont/MtgFrame.sizeFontList + MtgFrame.coefMultForSystemFonts));		
    pointeurs = new Vector<CValeurDynamique>();
    listModel = new DefaultListModel<String>();
    setModel(listModel);
    setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    liste = listeInit;
    indiceMaxi = indiceMaxiInit;
    intermPermis = intermPermisInit;
    int i = 0;
    // On ajoute les noms des objets de type valeurs  la liste
    ListIterator<CElementBase> li = liste.listIterator();
    while (i <= indiceMaxi)
    {
      CElementBase elb = li.next();
      // if ((!elb.estElementIntermediaire() || intermPermis) && elb.estDeNatureCalcul(NatureCalcul.NCalculOuFonctionParFormule)) {
      if ((!elb.estElementIntermediaire() || intermPermis) && elb.estDeNatureCalcul(natureInit)) {
      	CValeurDynamique vald = (CValeurDynamique) elb;
      	String ch = vald.nom();
      	if (!elb.existe) ch = ch + " " + MtgFrame.getString("chinfo0");
        listModel.addElement(ch);
        pointeurs.add((CValeurDynamique) elb);
      }
      i++;
    }
    setVisibleRowCount(7); // Diminu version 4.7.5.2
    setSelectedIndex(0);
  }
  
  public void setSel(CCalcul calc) {
  	int ind = pointeurs.indexOf(calc);
  	setSelectedIndex(ind);
  	ensureIndexIsVisible(ind);
  }
  
  public CCalcul calculSelectionne() {
  	int ind = getSelectedIndex();
  	if (ind == -1) return null;
  	else return (CCalcul) pointeurs.get(ind);
  }
  
  public final void metAJour(long nature, CValeurDynamique exclu) {
  	listModel.removeAllElements();
  	pointeurs.removeAllElements();
    int i = 0;
    // On ajoute les noms des objets de type valeurs  la liste
    ListIterator<CElementBase> li = liste.listIterator();
    while (i <= indiceMaxi)
    {
      CElementBase elb = li.next();
      // if ((!elb.estElementIntermediaire() || intermPermis) && elb.estDeNatureCalcul(NatureCalcul.NCalculOuFonctionParFormule)) {
      if (((elb != exclu) && !elb.estElementIntermediaire() || intermPermis) && elb.estDeNatureCalcul(nature)) {
      	CValeurDynamique vald = (CValeurDynamique) elb;
      	String ch = vald.nom();
      	if (!elb.existe) ch = ch + " " + MtgFrame.getString("chinfo0");
        listModel.addElement(ch);
        pointeurs.add((CValeurDynamique) elb);
      }
      i++;
    }
  }
  
  public void select(CCalcul calc) {
  	for (int i = 0; i < pointeurs.size(); i++) {
  		CValeurDynamique val = pointeurs.get(i);
  		if (val == calc) {
  			setSelectedIndex(i);
  			break;
  		}
  	}
  }
}
