/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.util.Vector;

import javax.swing.JList;
import javax.swing.ListSelectionModel;

import mathgraph32.MtgFrame;
import objets.CElementBase;
import objets.CMathGrafDoc;
import objets.CPrototype;
import types.FonteNom;
import types.NatureObjet;

/**
 * 
 * Liste prsentant les prototypes pouvant tre utiliss pour une utilisation itrative ou rcursive
 * @author Yves
 *
 */
@SuppressWarnings("serial")
public class ListePrototypesPourRecurOuIter extends JList<String> {
	private CMathGrafDoc doc;
  Vector<CPrototype> pointers = new Vector<CPrototype>();
	
	public ListePrototypesPourRecurOuIter(MtgFrame frame, CMathGrafDoc docInit) {
		super();
		setFont(FonteNom.getFonteEdition(MtgFrame.sizeFontList, FonteNom.Normal,
				(double)frame.pref_addSystemFont/MtgFrame.sizeFontList + MtgFrame.coefMultForSystemFonts));		
		doc = docInit;
    setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    Vector<String> values = new Vector<String> ();
    for (int i = 0; i < doc.tablePrototypes.size(); i++) {
    	CPrototype proto = doc.tablePrototypes.get(i);
    	// On regarde si le protype n'implante que des objets autoriss
    	boolean res = true;
    	for (int j = proto.nbObjSources; j < proto.size(); j++) {
    		CElementBase elb = proto.get(j);
    		res = res && (elb.estDeNature(NatureObjet.NObjetPourConstructionIterOuRecur) ||
    				(elb.getNature() == NatureObjet.NAucunObjet));
    		if (!res) break;
    	}
    	if (res) {
    		pointers.add(proto);
    		values.add(proto.nom);
    	}
    }
    setListData(values);
    setVisibleRowCount(6);
    setSelectedIndex(0);
	}

	public CPrototype prototypeChoisi() {
		return pointers.get(getSelectedIndex());
	}
	
  public void setSel(CPrototype proto) {
  	int ind = pointers.indexOf(proto);
  	setSelectedIndex(ind);
  }

}
