/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.util.Vector;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.ListSelectionModel;

import mathgraph32.MtgFrame;
import objets.CElementBase;
import objets.CListeObjets;
import types.FonteNom;

@SuppressWarnings("serial")
public class ListeTousObjets extends JList<String>{
  private CListeObjets liste; // La liste contenant les valeurs  afficher
  public boolean objetsIntermediaires; // Si true, les objets intermdiaires sont aussi dans la liste.
  public Vector<CElementBase> pointeurs;
  private DefaultListModel<String> listModel;

  public ListeTousObjets(MtgFrame frame, CListeObjets listeInit) {
    super();
		setFont(FonteNom.getFonteEdition(MtgFrame.sizeFontList, FonteNom.Normal,
				(double)frame.pref_addSystemFont/MtgFrame.sizeFontList + MtgFrame.coefMultForSystemFonts));		
    objetsIntermediaires = false;
    pointeurs = new Vector<CElementBase>();
    listModel = new DefaultListModel<String>();
    setModel(listModel);
    setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    liste = listeInit;
    Vector<String> values = new Vector<String> ();
    // On ajoute les noms des objets de type valeurs  la liste
    for  (int i = 0; i < liste.size(); i++) {
      CElementBase elb = liste.get(i); 
      if ((!elb.estElementIntermediaire() || objetsIntermediaires) && elb.estVisibledansHistorique()) {
      	String chinf = elb.info();
      	if (elb.estElementIntermediaire()) chinf = "*" + chinf;
      	values.add(chinf);
      	pointeurs.add(elb);
      }
    }
    setListData(values);
    setVisibleRowCount(15);
    setSelectedIndex(0);
  }
  
  public void metAJourListe() {
  	pointeurs.removeAllElements();
    Vector<String> values = new Vector<String> ();
    // On ajoute les noms des objets de type valeurs  la liste
    for  (int i = 0; i < liste.size(); i++) {
      CElementBase elb = liste.get(i); 
      if ((!elb.estElementIntermediaire() || objetsIntermediaires) && elb.estVisibledansHistorique()) {
      	String chinf = elb.info();
      	if (elb.estElementIntermediaire()) chinf = "*" + chinf;
      	values.add(chinf);
      	pointeurs.add(elb);
      }
    }  
    setListData(values);
    setVisibleRowCount(15);
    setSelectedIndex(0);
  }
  
  public final CElementBase selectedObject() {
  	int ind = getSelectedIndex();
  	if (ind == -1) return null;
  	else return pointeurs.get(ind);
  }
  
  public final void selectObject(CElementBase elb) {
  	for (int i = 0; i < pointeurs.size(); i++) {
  		if (pointeurs.get(i) == elb) {
  			setSelectedIndex(i);
  			ensureIndexIsVisible(i);
  			break;
  		}
  	}
  }

}
