/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
// Ajout version 4.8

package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionListener;
import java.text.NumberFormat;
import java.util.Vector;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JFormattedTextField;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import outils.Outil;
import mathgraph32.MtgColorButton;
import mathgraph32.MtgFrame;
import objets.CListeObjets;
import objets.CMacroConstructionRecursive;
import objets.CPrototype;
import types.FonteNom;
import types.NatureCarSpe;

@SuppressWarnings("serial")
public class MacroConstructionRecursiveDlg extends MtgDlg {
	private CMacroConstructionRecursive macro;
	private boolean modification;
	private MtgLabel label1, label2, label3, label4, label5, label5bis, label6, label9;
	private TextFieldAvecCarSpeciaux	editeurIntitule;
	private MtgTextArea editeurCommentaire;
	private ListePrototypesPourRecurOuIter listePrototypes;
	private JScrollPane scrollPaneListePrototypes;
	private PaneTaillePolice paneTaillePolice;
	private PaneModeAffichagePourMacro paneModeAffichage;
	private MtgButton btnOK, btnCancel;
	private MtgRadioButton[] br = new MtgRadioButton[13]; // Les six boutons radios associs aux profondeurs de rcursion disponibles
  private javax.swing.JFormattedTextField editeurPasRec, editeurNbImp, editeurNbSources;
	private MtgCheckBox checkImposerCouleur, checkConstructionNiveauMax, checkPolygoneRempli;
	private MtgColorButton colorButtonImp;
  private MtgRadioButton radioSansLigne, radioAvecPolygone, radioAvecLigneBrisee;
  // private MtgListeStyleTrait listeStyleTrait; // Modifi version 5.0
  private PaneStyleTrait paneStyleTrait;
	private MtgColorButton colorButtonLigne;
	private int[] indconst = new int[13]; // L'indice de la construction associe  chaque niveau de rcursion
  private DefaultListModel<String> listModel;
	private JList<String> listeProf; // La liste contenant les niveaus de profondeur
	private int profondMax;

  public MacroConstructionRecursiveDlg(MtgFrame frameInit, CMacroConstructionRecursive macroInit, boolean modificationInit) {
    super (getString("macroConstructionRecursiveDlg1"), true, frameInit);
    macro = macroInit;
    modification = modificationInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);    
  }

  private void initComponents() {
		label1 = new MtgLabel(frame, getString("macroApparitionDlg2"));
		label2 = new MtgLabel(frame, getString("macroApparitionDlg3"));
  	label3 = new MtgLabel(frame, getString("macroConstructionIterativeDlg2"));  	
  	label4 = new MtgLabel(frame, getString("recursionConstructionDlg2"));  	
  	label5 = new MtgLabel(frame, getString("recursionConstructionDlg3"));
  	label5bis = new MtgLabel(frame, getString("recursionConstructionDlg4"));
  	label6 = new MtgLabel(frame, getString("iterationConstructionDlg4"));
  	label9 = new MtgLabel(frame, getString("recursionConstructionDlg8"));
  	

  	
		editeurIntitule = new TextFieldAvecCarSpeciaux(this, frame, NatureCarSpe.Tout);
		editeurIntitule.setColumns(30);
		editeurCommentaire = new MtgTextArea(frame);
		editeurCommentaire.setColumns(20);
		editeurCommentaire.setLineWrap(false);
		editeurCommentaire.setRows(3);
    JScrollPane scrollPaneCommentaire = new JScrollPane(editeurCommentaire,
    		JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    scrollPaneCommentaire.setPreferredSize(new Dimension(300, 60));
    
	  btnOK = new MtgButton(frame, getString("btnOK"));
	  btnOK.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnOKActionPerformed();
	    }
	  });    
	  btnCancel = new MtgButton(frame, getString("btnCancel"));
	  btnCancel.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnCancelActionPerformed();
	    }
	  });
	  btnOK.setMaximumSize(btnCancel.getPreferredSize());	
	  paneTaillePolice = new PaneTaillePolice(macro.indiceTaillePolice);
	  paneModeAffichage = new PaneModeAffichagePourMacro(frame, macro);
		listeProf = new JList<String>(); // La liste contenant les profondeurs de rcursion
		listeProf.setFont(FonteNom.getFonteNom(MtgFrame.sizeFontList, FonteNom.Normal,
				(double)frame.pref_addSystemFont/MtgFrame.sizeFontList + MtgFrame.coefMultForDlgBoxIcons));		
    listModel = new DefaultListModel<String>();
    listeProf.setModel(listModel);
		listeProf.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		listeProf.setLayoutOrientation(JList.VERTICAL);
		listeProf.setVisibleRowCount(4);
		listeProf.addListSelectionListener(new ListSelectionListener() {
			
			public void valueChanged(ListSelectionEvent e) {
				listeProfSelectionChanged();
			}
		});
		JScrollPane scrollpaneList = new JScrollPane(listeProf, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

		// Cration de 7 boutons radios pour la pprofondeur de rcursion maximale
		ButtonGroup group1 = new ButtonGroup();
		for (int i = 0; i < 13; i++) {
			br[i] = new MtgRadioButton(frame, Integer.toString(i));
			group1.add(br[i]);
		  br[i].addActionListener(new ActionListener() {
	      public void actionPerformed(java.awt.event.ActionEvent evt) {
	      	radioProfActionPerformed();
	      }
	 		});
		}
	  for (int i = 0; i <= 12; i++) indconst[i]=0; // par dfaut le protoype associ  chaque niveau est le premier prototype de la liste
	  listePrototypes = new ListePrototypesPourRecurOuIter(frame, frame.cadreActif().document);
	  listePrototypes.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				listePrototypesListSelectionChanged();
			}
		});
    listePrototypes.setVisibleRowCount(3); // Surcharge
  	prepareListeProf();
	  if (modification) {
	  	listeProf.setSelectedIndex(0);
	  	CListeObjets liste = frame.cadreActif().listePr;
	  	profondMax = macro.profondMax;
	  	for (int i = 0; i <= profondMax; i++) indconst[i] = liste.documentProprietaire.tablePrototypes.indexOf(macro.tableProto.get(i));
	  	// Modifi version 5.0.2
	  	// int ind = frame.cadreActif().document.tablePrototypes.indexOf(macro.tableProto.get(0));
	  	int ind = indconst[0];
	  	listePrototypes.setSelectedIndex(ind);
	  	listePrototypes.ensureIndexIsVisible(ind);
	  }
	  else {
	  	profondMax = 3; // Par dfaut prodfonduer de rcursion de 3}
	  	br[3].setSelected(true);
	  }
  	prepareListeProf(); // Deuxime appel ncessaire
  	// Ajout version 5.0.2
		listeProf.setSelectedIndex(0);
		listeProf.ensureIndexIsVisible(0);
		//

	  scrollPaneListePrototypes = new JScrollPane(listePrototypes,
    		JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	  checkConstructionNiveauMax = new MtgCheckBox(frame, getString("recursionConstructionDlg9"));
	  checkConstructionNiveauMax.setSelected(false);
		checkImposerCouleur = new MtgCheckBox(frame, getString("macroConstructionIterativeDlg3"));  	
		checkImposerCouleur.setSelected(modification ? macro.imposerCouleur : false);
		checkImposerCouleur.addActionListener(new ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
      	checkImposerCouleurActionPerformed();
      }
 		});
		colorButtonImp = new MtgColorButton(frame);
		colorButtonImp.couleur = modification ?(macro.imposerCouleur ? macro.couleurImp : frame.couleurActive()) : frame.couleurActive();
		if (!checkImposerCouleur.isSelected()) {
			// label4.setVisible(false);
			colorButtonImp.setVisible(false);
		}
    NumberFormat format;
    format = NumberFormat.getInstance();
    format.setParseIntegerOnly(true);
    format.setGroupingUsed(false);
    editeurNbImp = new JFormattedTextField(format);
    editeurNbImp.setEditable(!modification); // Ajout version 5.1.1 car on pourrait choisir une autre macro d'une autre type
    editeurNbImp.setColumns(2);
    editeurPasRec = new JFormattedTextField(format);
    editeurPasRec.setEditable(!modification); // Ajout version 5.1.1 car on pourrait choisir une autre macro d'une autre type
    editeurPasRec.setColumns(2);
    editeurNbSources = new JFormattedTextField(format);
    editeurNbSources.setEditable(!modification); // Ajout version 5.1.1 car on pourrait choisir une autre macro d'une autre type
    editeurNbSources.setColumns(2);
    editeurNbSources.setText("0");
	  radioSansLigne = new MtgRadioButton(frame, getString("recursionConstructionDlg5"));
	  radioSansLigne.addActionListener(new ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
      	radioSansLigneActionPerformed();
      }
 		});
	  radioAvecPolygone = new MtgRadioButton(frame, getString("recursionConstructionDlg6"));
	  radioAvecPolygone.addActionListener(new ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
      	radioSansLigneActionPerformed();
      }
 		});
	  radioAvecLigneBrisee = new MtgRadioButton(frame, getString("recursionConstructionDlg7"));
	  radioAvecLigneBrisee.addActionListener(new ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
      	radioSansLigneActionPerformed();
      }
 		});
	  radioSansLigne.setSelected(true);
	  ButtonGroup group = new ButtonGroup();
	  group.add(radioSansLigne);
	  group.add(radioAvecPolygone);
	  group.add(radioAvecLigneBrisee);
		checkPolygoneRempli = new MtgCheckBox(frame, getString("recursionConstructionDlg10"));
		checkPolygoneRempli.setSelected(modification ? macro.polygoneRempli : false);
		colorButtonLigne= new MtgColorButton(frame);
		colorButtonLigne.couleur = modification ? (macro.choixLigne == 0 ? frame.couleurActive() : macro.couleurLigne) : frame.couleurActive();
    // listeStyleTrait = new MtgListeStyleTrait(frame); Modifi version 5.0
		paneStyleTrait = new PaneStyleTrait(frame, this);
   //  scrollPaneStyleTrait = new JScrollPane(listeStyleTrait, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	  if (modification) {
    	editeurIntitule.setText(macro.intitule);
    	editeurCommentaire.setText(macro.commentaireMacro);
			editeurCommentaire.setCaretPosition(0);
			editeurNbSources.setText(Integer.toString(macro.nbSourcesPourIter));
			checkConstructionNiveauMax.setSelected(macro.creationNiveauMax);
			editeurPasRec.setText(Integer.toString(macro.pasRec));
			editeurNbImp.setText(Integer.toString(macro.nbImp));
			br[macro.profondMax].setSelected(true);
			checkImposerCouleur.setSelected(macro.imposerCouleur);
			// listeStyleTrait.setSelectedIndex(macro.styleLigne.style);
			// listeStyleTrait.ensureIndexIsVisible(macro.styleLigne.style);
			if (macro.choixLigne != 0) paneStyleTrait.setSelection(macro.styleLigne);
	  	switch (macro.choixLigne) {
			case 0: 
				radioSansLigne.setSelected(true);			
				break;
			case 1:
				radioAvecPolygone.setSelected(true);
				checkPolygoneRempli.setSelected(macro.polygoneRempli);
				break;
			case 2:
				radioAvecLigneBrisee.setSelected(true);
			default:
				break;
			}
	  }
	  else radioSansLigne.setSelected(true);

    JPanel panePrincipal = (JPanel) getContentPane();
    JPanel paneRight = new JPanel();
    JPanel paneBas = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    JPanel paneChoixProf = new JPanel();
    paneChoixProf.setLayout(new BoxLayout(paneChoixProf, BoxLayout.X_AXIS));
    paneChoixProf.add(label5bis);
    for (int i = 0; i < 13; i++) paneChoixProf.add(br[i]);
    JPanel paneChoixNiv = new JPanel();
    paneChoixNiv.setLayout(new BoxLayout(paneChoixNiv, BoxLayout.X_AXIS));
    paneChoixNiv.add(label9);
    paneChoixNiv.add(scrollpaneList);
    paneChoixNiv.add(label3);
    paneChoixNiv.add(scrollPaneListePrototypes);
    

		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 0;
		constraints.insets.top = 2;
		constraints.insets.left = 5;
		constraints.insets.right = 5;
		
    JPanel paneLigne = new JPanel();
    paneLigne.setLayout(new GridBagLayout());
	  setConstraints(constraints, 0, 0, 2, 1, 40, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  paneLigne.add(radioSansLigne, constraints);
	  setConstraints(constraints, 0, 1, 1, 1, 40, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  paneLigne.add(radioAvecPolygone, constraints);
	  setConstraints(constraints, 1, 1, 1, 1, 20, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  paneLigne.add(checkPolygoneRempli, constraints);
	  setConstraints(constraints, 0, 2, 2, 1, 40, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  paneLigne.add(radioAvecLigneBrisee, constraints);
	  setConstraints(constraints, 2, 0, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  paneLigne.add(colorButtonLigne, constraints);
	  setConstraints(constraints, 2, 1, 1, 2, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  // paneLigne.add(scrollPaneStyleTrait, constraints);
	  paneLigne.add(paneStyleTrait, constraints);

		JPanel paneIntitule = new JPanel();
		paneIntitule.setLayout(new BoxLayout(paneIntitule , BoxLayout.X_AXIS));
		paneIntitule.add(label1);
		paneIntitule.add(editeurIntitule);
		setConstraints(constraints, 0, 0, 2, 1, 80, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneIntitule, constraints);
		setConstraints(constraints, 0, 1, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label2, constraints);
		setConstraints(constraints, 0, 2, 2, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneCommentaire, constraints);
		setConstraints(constraints, 0, 3, 2, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(paneChoixProf, constraints);
		
		setConstraints(constraints, 0, 4, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneChoixNiv, constraints);
    setConstraints(constraints, 0, 5, 1, 1, 50, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label4, constraints);	
    setConstraints(constraints, 1, 5, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(editeurNbImp, constraints);
    setConstraints(constraints, 0, 6, 1, 1, 50, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label5, constraints);	
    setConstraints(constraints, 1, 6, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(editeurPasRec, constraints);
    setConstraints(constraints, 0, 7, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label6, constraints);	
    setConstraints(constraints, 1, 7, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(editeurNbSources, constraints);
    setConstraints(constraints, 0, 8, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(checkConstructionNiveauMax, constraints);
    setConstraints(constraints, 0, 9, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(checkImposerCouleur, constraints);
    setConstraints(constraints, 1, 9, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(colorButtonImp, constraints);
    /*
	  setConstraints(constraints, 0, 10, 2, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  panePrincipal.add(radioSansLigne, constraints);
	  setConstraints(constraints, 0, 11, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  panePrincipal.add(radioAvecPolygone, constraints);
	  setConstraints(constraints, 1, 11, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  panePrincipal.add(checkPolygoneRempli, constraints);
	  setConstraints(constraints, 0, 12, 2, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
	  panePrincipal.add(radioAvecLigneBrisee, constraints);
    setConstraints(constraints, 0, 13, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label7, constraints);
    setConstraints(constraints, 1, 13, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(colorButtonLigne, constraints);
    setConstraints(constraints, 0, 14, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label8, constraints);
    setConstraints(constraints, 1, 14, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneStyleTrait, constraints);
    */
    setConstraints(constraints, 0, 10, 3, 1, 50, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(paneLigne, constraints);
 
		
		paneBas.add(paneTaillePolice);
		paneBas.add(paneModeAffichage);
		setConstraints(constraints, 0, 11, 2, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneBas, constraints);
 		setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(paneRight, constraints);

		if (radioSansLigne.isSelected()) {
			colorButtonLigne.setVisible(false);
			// scrollPaneStyleTrait.setVisible(false); Modifi version 5.0
			paneStyleTrait.setVisible(false);
			checkPolygoneRempli.setVisible(false);
		}
		else if (!radioAvecPolygone.isSelected()) checkPolygoneRempli.setVisible(false);
		pack();	
  }
  
  private void radioProfActionPerformed() {
  	for (int i = 0; i < 13; i++) {
  		if (br[i].isSelected()) {
  			profondMax = i;
  			break;
  		}
  	}
  	prepareListeProf();
  	// Ajout version 5.0.2
		listeProf.setSelectedIndex(0);
		listeProf.ensureIndexIsVisible(0);
		//
  }
  
  private void prepareListeProf() {
  	listModel.removeAllElements();
		for (int i = 0; i <= profondMax; i++) listModel.addElement(Integer.toString(i));
		/* Modifi version 5.0.2 parce que la macro associ au niveau 0 tait toujours la  premire
		listeProf.setSelectedIndex(0);
		listeProf.ensureIndexIsVisible(0);
		listePrototypes.setSelectedIndex(0);
		listePrototypes.ensureIndexIsVisible(0);
		*/
  }

  private void listeProfSelectionChanged() {
  	int ind = listeProf.getSelectedIndex();
  	if (ind != -1) {
	  	listePrototypes.setSelectedIndex(indconst[ind]);
	  	listePrototypes.ensureIndexIsVisible(indconst[ind]);
	  	// pack(); Supprim version 5.0.2
  	}
  }
  
  private void listePrototypesListSelectionChanged() {
  	int ind1 = listeProf.getSelectedIndex();
  	int ind2 = listePrototypes.getSelectedIndex();
  	indconst[ind1] = ind2;
  }
  private void checkImposerCouleurActionPerformed() {
  	colorButtonImp.setVisible(checkImposerCouleur.isSelected());
  	pack();
  }
  
  private void radioSansLigneActionPerformed() {
  	colorButtonLigne.setVisible(!radioSansLigne.isSelected());
  	// scrollPaneStyleTrait.setVisible(!radioSansLigne.isSelected()); Modifi version 5.0
  	paneStyleTrait.setVisible(!radioSansLigne.isSelected());
  	checkPolygoneRempli.setVisible(radioAvecPolygone.isSelected());
  	pack();
  }


  private void btnOKActionPerformed() {
  	int pasRec, nbImp, nbSources;
  	String chintitule = editeurIntitule.getText();
  	if (chintitule.length() == 0) {
  		dialogueAvertissement(getString("ch115"));
  		editeurIntitule.requestFocus();
  		return;
  	}
  	CListeObjets liste = frame.cadreActif().listePr;
  	if (!liste.validationIntituleMacro(chintitule, macro)) {
  		dialogueAvertissement(getString("ch116"));
  		editeurIntitule.requestFocus();
  		return;
  	}
  	try {
  		pasRec = Integer.parseInt(editeurPasRec.getText());
  	}
		catch (NumberFormatException e) {
			dialogueAvertissement(getString("ch71"));
			editeurPasRec.requestFocus();
			return;
		}
 	
    if (pasRec <= 0 ) {
    	dialogueAvertissement(getString("ch71"));
    	editeurPasRec.requestFocus();
    	return;
    }
  	try {
  		nbImp = Integer.parseInt(editeurNbImp.getText());
  	}
		catch (NumberFormatException e) {
			dialogueAvertissement(getString("ch71"));
			editeurNbImp.requestFocus();
			return;
		}
    if (nbImp <= 0 ) {
    	dialogueAvertissement(getString("ch71"));
    	editeurNbImp.requestFocus();
    	return;
    }
  	try {
  		nbSources = Integer.parseInt(editeurNbSources.getText());
  	}
		catch (NumberFormatException e) {
			dialogueAvertissement(getString("ch71"));
			editeurNbSources.requestFocus();
			return;
		}
 	
    if (nbSources < 0) {
    	dialogueAvertissement(getString("ch71"));
    	editeurNbSources.requestFocus();
    	return;
    }
    for (int i = 0; i <= profondMax; i++) {
	    CPrototype proto = liste.documentProprietaire.tablePrototypes.get(indconst[i]);
	    String chErr = proto.implementationPossible(liste);
	    if (chErr.length() != 0) {
	    	frame.dialogueAvertissement(getString("ch142") + "\n" + chErr);
	    	listeProf.setSelectedIndex(i);
	    	return;
	    }
	    if ((liste.pointeurLongueurUnite == null) && proto.utiliseLongueurUnite()) {
     		frame.dialogueAvertissement(getString("ch138"));
     		listeProf.setSelectedIndex(i);
	    	return;
	    }
			if (!proto.estRecursible(nbSources, nbImp, pasRec)) {
				frame.dialogueAvertissement(getString("ch199"));
				listeProf.setSelectedIndex(i);
				return;
			}
    }
    // On fait une estimation du nombre d'objets  crer avec le prototype de niveau de rcursion 0
    CPrototype proto0 = liste.documentProprietaire.tablePrototypes.get(indconst[0]);
    double n;
    if (checkConstructionNiveauMax.isSelected()) {
    	double p = proto0.nombreObjetsJusqueFinal(proto0.nbObjSources-nbSources); // Nombre objets crs sauf dernire tape
    	n = (proto0.size()-proto0.nbObjSources-nbSources)*Math.exp((double)(profondMax)*Math.log(nbImp))
    			+ p*(Math.exp((double)(profondMax)*Math.log(nbImp))-1)/(nbImp-1);
    }
    else n = (double)(proto0.size()-proto0.nbObjSources-nbSources)*(Math.exp((double)(profondMax+1)*Math.log(nbImp))-1)/(nbImp-1);
		if (n > (double) Outil.nombreMaxObjets) {
			frame.dialogueAvertissement(getString("ch198"));
			return;
		}

  	macro.intitule = chintitule;
  	macro.commentaireMacro = editeurCommentaire.getText();
  	macro.indiceTaillePolice = paneTaillePolice.indiceTaille();
  	macro.effacementFond = paneModeAffichage.effacementFond();
  	macro.couleurFondAffichage = paneModeAffichage.couleurFond();
		macro.alignementHorizontal = paneModeAffichage.alignementHorizontal();
		macro.alignementVertical = paneModeAffichage.alignementVertical();
		
		frame.pref_IndiceTaillePoliceAffichages = macro.indiceTaillePolice;
		frame.cadreActif().couleurFondAffichage = macro.couleurFondAffichage;
    macro.profondMax = profondMax;
    macro.tableProto = new Vector<CPrototype>();
    for (int i = 0; i <= profondMax; i++) macro.tableProto.add(liste.documentProprietaire.tablePrototypes.get(indconst[i]));
    macro.pasRec = pasRec;
    macro.nbImp = nbImp;
    macro.nbSourcesPourIter = nbSources;
    macro.creationNiveauMax = checkConstructionNiveauMax.isSelected();
    macro.imposerCouleur = checkImposerCouleur.isSelected();
    macro.couleurImp = colorButtonImp.couleur;
    macro.choixLigne = radioSansLigne.isSelected() ? Outil.sansLigne :
    	(radioAvecPolygone.isSelected() ? Outil.avecPolygone : Outil.avecLigneBrisee);
    macro.polygoneRempli = (macro.choixLigne != Outil.avecPolygone) ? false
    		: radioAvecPolygone.isSelected() ? (checkPolygoneRempli.isSelected() ? true : false) : false;
    // macro.styleLigne = listeStyleTrait.styleTrait();
    macro.styleLigne = paneStyleTrait.getStyle();
    macro.couleurLigne = colorButtonLigne.couleur;
    setRetourOK(true);
    dispose();
  }
  
  private void btnCancelActionPerformed() {                                          
    dispose();
  }       

}