/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import objets.CBarycentre;
import objets.CCommentaire;
import objets.CElementBase;
import objets.CListeObjets;
import objets.CValeurAssocieeAVariable;
import types.NatureObjet;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

@SuppressWarnings("serial")
public class ModificationBarycentreDlg extends MtgDlg {
	private CBarycentre barycentre;
	private ListeModifBarycentre listeModifBarycentre;
	private MtgButton btnFermer;
	private boolean modifie = false; // Sera true si on a modifi un ou plusieurs coefficients
  public ModificationBarycentreDlg(MtgFrame frameInit, CBarycentre barycentreInit) {
  	super(getString("ModificationBarycentreDlg1") + " " + barycentreInit.nom(), true, frameInit);
  	barycentre = barycentreInit;
  	listeModifBarycentre = new ListeModifBarycentre(frame, barycentre);
  	listeModifBarycentre.setSelectedIndex(0);
    JScrollPane scrollPaneListe = new JScrollPane(listeModifBarycentre, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
    		JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
  	JButton btnModifier = new MtgButton(frame, getString("btnModifier"));  
  	btnModifier.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent e) {
		    btnModifierActionPerformed();
			}
		});
  	listeModifBarycentre.addMouseListener(new MouseListener() {
			
			public void mouseReleased(MouseEvent e) {
				// TODO Auto-generated method stub
				
			}
			
			public void mousePressed(MouseEvent e) {
				// TODO Auto-generated method stub
				
			}
			
			public void mouseExited(MouseEvent e) {
				// TODO Auto-generated method stub
				
			}
			
			public void mouseEntered(MouseEvent e) {
				// TODO Auto-generated method stub
				
			}
			 
			public void mouseClicked(MouseEvent e) {
		    if (e.getClickCount() == 2) btnModifierActionPerformed();
				
			}
		});
		btnFermer = new MtgButton(frame, getString("btnFermer"));
		btnFermer.addActionListener(new java.awt.event.ActionListener() {
		  public void actionPerformed(java.awt.event.ActionEvent evt) {
		    btnFermerActionPerformed();
		  }
		});

		JPanel panePrincipal = new JPanel();
		panePrincipal.setLayout(new GridBagLayout());
		JPanel paneRight = new JPanel();
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 0;
		constraints.insets.top = 2;
		constraints.insets.left = 5;
		constraints.insets.right = 5;
		setConstraints(constraints, 0, 0, 1, 1, 70, 80, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneListe, constraints);
		setConstraints(constraints, 0, 1, 1, 1, 50, 80, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(Box.createRigidArea(new Dimension(0,5)), constraints);
		setConstraints(constraints, 0, 2, 1, 1, 25, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(btnModifier, constraints);
		paneRight.add(btnFermer);
		setConstraints(constraints, 1, 0, 1, 1, 30, 100, GridBagConstraints.NONE, GridBagConstraints.NORTH);
		panePrincipal.add(paneRight, constraints);
    addWindowListener(new java.awt.event.WindowAdapter() {
      public void windowClosing(java.awt.event.WindowEvent evt) {
        fermeture();
      } 
    });
    getRootPane().setDefaultButton(btnFermer);
    setContentPane(panePrincipal);
		pack();

  }

  private final void btnModifierActionPerformed() {
  	int ind = listeModifBarycentre.getSelectedIndex();
  	CValeurAssocieeAVariable coef = barycentre.col.get(ind).poids;
  	CoefficientBarycentreDlg dlg = new CoefficientBarycentreDlg(frame, barycentre, coef, true);
  	dlg.setLocationRelativeTo(this);
  	dlg.setVisible(true);  	
    if (dlg.retourOK()) {
    	modifie = true;
    	listeModifBarycentre.metAJour();
    	CListeObjets liste = frame.listeClignotante;
    	// Si un coefficient a t chang on doit modifier le commentaire clignotant
    	for (int j = 0; j < liste.size(); j++) {
    		CElementBase elb =  liste.get(j);
    		if (elb.getNature() == NatureObjet.NCommentaire) {
    			CCommentaire com = (CCommentaire) elb;
    			if (com.pointLie == barycentre.col.get(ind).pointAssocie) {
    				// com.efface(cadre.gbuffer, true, doc.couleurFond, doc.transparencePourSurfaces);
    				com.chaineCommentaire = coef.chaineInfo();
    				com.positionne(false, frame.cadreActif().dimf);
    				break;
    			}
    		}
    	}
    	MtgInternalFrame cadre = frame.cadreActif();
    	// Ligne suivante modifi version 5.1.3 car appel depuis ProtocoleDlf et  ce niveau les points non nomms ont t renomms
		 //  cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil66"));
		  cadre.redessineAvecCalcul();
    }
  	

  }
  
  private final void btnFermerActionPerformed() {
  	fermeture();
  }
  
  private void fermeture() {
  	setRetourOK(modifie);
  	dispose();
  }

}