/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.FlowLayout;
import java.awt.Insets;
import java.net.URL;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTextField;
import objets.CListeObjets;
import objets.CVariableBornee;
import types.FonteNom;
import mathgraph32.MtgFrame;
import mathgraph32.MtgImageIcon;
import mathgraph32.MtgInternalFrame;



/**
 * Classe servant  afficher dans un dialogue non modal en bas et  droite de la figure les variables
 * Le dialogue contiendra une liste dont les lments seront des paneVariablePourDlg
 * @author Yves
 *
 */
@SuppressWarnings("serial")
public class PaneVariablePourDlg extends JPanel {
	private MtgInternalFrame cadre;
	protected CVariableBornee variableAssociee;
	private JTextField affValeur;
	private JButton btnPlus;
	private JButton btnMoins;
	private JButton btnEgal;
	private static final Insets insets = new Insets(0,0,0,0);
	private static final Insets insets1 = new Insets(0,1,0,1);
	
	public PaneVariablePourDlg(MtgInternalFrame cadreInit, CVariableBornee variableAssocieeInit) {
		super();
		cadre = cadreInit;
		variableAssociee = variableAssocieeInit;
		// Version 5.0.2 : On utilise plus un MtgTextField car c'est un peu trop gros
		// affValeur = new MtgTextField(cadre.frame, 12);
		affValeur = new JTextField(12);
		affValeur.setFont(FonteNom.getFonteEdition(11, FonteNom.Normal,
				(double)cadre.frame.pref_addSystemFont/MtgFrame.sizeFontList + MtgFrame.coefMultForSystemFonts));	
		affValeur.setEditable(false);
		affValeur.setMargin(insets1);
		// Modifi pour tenir compte des crans haute rsolution
		double k = MtgFrame.coefMultForDlgBoxIcons;
		double k2 = k;
		if (k>1.5) k = 1.5;
		if (k2>2) k2 = 2;
    URL adr = MtgFrame.class.getResource("images/btnMoins.gif");
    ImageIcon im = new MtgImageIcon(k2, adr);
    btnMoins = new JButton(im);
    btnMoins.setMargin(insets);
    btnMoins.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnMoinsActionPerformed();
      }
    });
    adr = MtgFrame.class.getResource("images/btnPlus.gif");
		// Modifi pour tenir compte des crans haute rsolution
    im = new MtgImageIcon(k2, adr);
    btnPlus = new JButton(im);
    btnPlus.setMargin(insets);
    btnPlus.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnPlusActionPerformed();
      }
    });
    adr = MtgFrame.class.getResource("images/btnEgal.gif");
		// Modifi pour tenir compte des crans haute rsolution
    im = new MtgImageIcon(k2, adr);
    btnEgal = new JButton(im);
    btnEgal.setMargin(insets);
    btnEgal.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnEgalActionPerformed();
      }
    });
    setLayout(new FlowLayout());
    add(affValeur);
    add(btnMoins);
    add(btnPlus);
    add(btnEgal);
    metAJourValeur();
	}
	
	public boolean estAssocieA(CVariableBornee var) {
		return variableAssociee == var;
	}
	
	public void metAJourValeur() {
		affValeur.setText(variableAssociee.nom() + " = " + glob.Kernel.chaineNombre(variableAssociee.valeurActuelle, 8)); // 8 dcimales
	}
	
	private void btnMoinsActionPerformed() {
		CListeObjets liste = cadre.listePr;
		// A revoir
		// Modif version 2.0. Ligne suivante inutile
		// liste.effaceDependants(cadre.gbuffer, cadre.dimf, variableAssociee);
		variableAssociee.decremente();
		liste.positionneDependantsDe(cadre.infRand, cadre.dimf, variableAssociee);
		cadre.repaintFigureSansCalcul();
		metAJourValeur();
		cadre.document.setDirty(true);			
	}

	private void btnPlusActionPerformed() {
		CListeObjets liste = cadre.listePr;
		// A revoir
		// Modif version 2.0. Ligne suivante inutile
		// liste.effaceDependants(cadre.gbuffer, cadre.dimf, variableAssociee);
		variableAssociee.incremente();
		liste.positionneDependantsDe(cadre.infRand, cadre.dimf, variableAssociee);
		cadre.repaintFigureSansCalcul();
		metAJourValeur();
		cadre.document.setDirty(true);			
	}

	private void btnEgalActionPerformed() {
		MtgFrame frame = cadre.frame;
		VariableDlg  dlg = new VariableDlg(frame, variableAssociee, true);
		// Amlioration version 4.7.1
		dlg.editeurValeurActuelle.setSelectionStart(0);
		dlg.editeurValeurActuelle.setSelectionEnd(dlg.editeurValeurActuelle.getText().length());
		dlg.editeurValeurActuelle.requestFocus();
		// Fin amlioration 4.7.1
		dlg.setLocationRelativeTo(frame); // Modifi version 4.7.5.2
		dlg.setVisible(true);
		metAJourValeur();
		frame.mtgDesktopPane.getDesktopManager().activateFrame(cadre);		
		frame.requestFocus();
	}
}
