/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CAppelFonction extends CCalculBase {
	
  Complexe z1loc = new Complexe(); //Ajout version 4.9.7 pour pouvoir utiliser une fonction relle dans un calcul complexe

	CCalculAncetre fonctionAssociee;// Pointera uniquement sur une
																		// fonction ou une suite
	CCalculBase operande;

  /** Creates new CAppelFonction */
  public CAppelFonction() {
  }
  // Constructeur
  public CAppelFonction(CListeObjets liste, CCalculAncetre fonctionInit, CCalculBase operandeInit)
  {
    listeProprietaire = liste;
    fonctionAssociee = fonctionInit;
    operande = operandeInit;
  }
  
  @Override
  protected long nature() {
  	return natAppelFonction;
  }
  
  public CCalculBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    CCalculBase cloneOperande = operande.getClone(listeSource, listeCible); 
    //  revoir car la fonction a dj t cre
    int ind1 = listeSource.indexOf(fonctionAssociee);    
    CCalculBase ptcalc = new CAppelFonction(listeCible, (CCalculAncetre)(listeCible.get(ind1, CCalculAncetre.class)),
      cloneOperande);
    return ptcalc;
  }
  
  @Override
  public boolean existe(boolean infoRandom) {
    return fonctionAssociee.existe && operande.existe(infoRandom);
  }
  
  @Override
  public double resultat(boolean infoRandom) throws ErreurCalculException {
    return fonctionAssociee.rendValeurFonction(infoRandom, operande.resultat(infoRandom));
  }
  
  @Override
  public double resultatFonction(boolean infoRandom, double valeurParametre) throws ErreurCalculException {
    return fonctionAssociee.rendValeurFonction
      (infoRandom, operande.resultatFonction(infoRandom, valeurParametre));
  }

  // Ajout version 3.0
  @Override
  public double resultatFonction(boolean infoRandom, double[] valeurParametre) throws ErreurCalculException {
    return fonctionAssociee.rendValeurFonction
      (infoRandom, operande.resultatFonction(infoRandom, valeurParametre));
  }
  
  //Ajout version 4.9.7 pour pouvoir utiliser une fonction relle dans un calcul complexe
  @Override
  void  resultatFonctionComplexe(boolean infoRandom, Complexe valeurParametre, Complexe zRes)
    throws ErreurCalculException 
  {
    operande.resultatFonctionComplexe(infoRandom, valeurParametre, z1loc);
    fonctionAssociee.rendValeurFonctionComplexe(infoRandom, z1loc, zRes);
  }
  
  //Ajout version 4.9.7 pour pouvoir utiliser une fonction relle dans un calcul complexe
  @Override
  void  resultatFonctionComplexe(boolean infoRandom,Complexe[] valeurParametre, Complexe zRes)
    throws ErreurCalculException 
  {
    operande.resultatFonctionComplexe(infoRandom, valeurParametre, z1loc);
    fonctionAssociee.rendValeurFonctionComplexe(infoRandom, z1loc, zRes);
  }


  // Ajout version 4.8
  @Override
  public void initialisePourDependance() {
  	super.initialisePourDependance();
  	operande.initialisePourDependance();
  }
  
  // Modifi version 4.8
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
  	elementTestePourDependDe = p;
    return memoriseDependance(fonctionAssociee.dependDe(p) || operande.dependDe(p));
  }

  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePouRec(super.dependDePourReclassement(p) || fonctionAssociee.dependDePourReclassement(p) || operande.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return fonctionAssociee.dependDePourBoucle(p) || operande.dependDePourBoucle(p);
  }
  
  @Override
  public boolean dependDeVariable(int ind) {
    return operande.dependDeVariable(ind);
  }
  
  @Override
  public boolean deriveePossible(int indiceVariable) {
  	boolean b = operande.deriveePossible(indiceVariable);
  	if (fonctionAssociee.getClass() == CSuiteRec.class) return b;
  	else {
  		CFonc f = (CFonc) fonctionAssociee;
  		return b || f.deriveePossible(indiceVariable);
  	}
  }
  
  
  public CCalculBase getCopie() {
    return new CAppelFonction(listeProprietaire,fonctionAssociee, operande.getCopie());
  }
  
  @Override
	public String chaineCalcul(String[] varFor) {
  	String cha = operande.chaineCalculSansPar(varFor);
  	// ElimineParenthesesDebut(cha);
  	return fonctionAssociee.nom() + "(" + cha + ")";
	}
  
  @Override
	public String chaineLatex(String[] varFor) {
  	String cha = operande.chaineLatexSansPar(varFor);
  	// ElimineParenthesesDebut(cha);
  	return fonctionAssociee.nom() + parouvLatex + cha + parferLatex;
	}
  
    
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(fonctionAssociee) || fonctionAssociee.estDefiniParObjetDans(listeOb))
		&& 
		operande.estDefiniParObjetDans(listeOb);
	}	 

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    fonctionAssociee = (CCalculAncetre) (list.get(ind1,CCalculAncetre.class));
    operande = (CCalculBase) (inps.readCObject(list));
  }
  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    fonctionAssociee = (CCalculAncetre) (list.get(ind1,CCalculAncetre.class));
    operande = (CCalculBase) (inps.readObject(list));
  }
  
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(fonctionAssociee);
    oups.writeInt(ind1);
    oups.writeObject(operande);
  }
  
  @Override
	public boolean estConstant() {
		return fonctionAssociee.estConstant() && operande.estConstant();
	}
  
  // Ajout version 4.6
  @Override
  public CCalculBase calculAvecValeursRemplacees() {
  	// Modifi version 4.9.6
  	// return new CAppelFonction(listeProprietaire, fonctionAssociee, operande.calculAvecValeursRemplacees());
  	// Modifi pour que si, par exemple on utilise droit(f(x)), ce soit bien la formule de f(x) avec valeurs remplaces qui soit utilise
  	// Modifi version 4.9.8.2
  	// CCalculAncetre fon = (fonctionAssociee.getClass() == CFonc.class) ? new CFonc(listeProprietaire, null, false, "","",
  	//		((CFonc)fonctionAssociee).calcul.calculAvecValeursRemplacees(),((CFonc)fonctionAssociee).chaineVariable) : fonctionAssociee;
  	CCalculAncetre fon = (fonctionAssociee.getClass() == CFonc.class) ? new CFonc(listeProprietaire, null, false,
  			fonctionAssociee.nomCalcul,"",
  			((CFonc)fonctionAssociee).calcul.calculAvecValeursRemplacees(),((CFonc)fonctionAssociee).chaineVariable) : fonctionAssociee;  	

  	return new CAppelFonction(listeProprietaire, fon, operande.calculAvecValeursRemplacees());
  }

  @Override
	public CCalculBase membreGauche() {
  	return fonctionAssociee.membreGauche();
	}
	
	// Ajout version 4.6
	public CCalculBase membreDroit() {
  	return fonctionAssociee.membreDroit();
	}
  
}
