/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;

import types.*;

import dialogs.CalculReelDlg;
import dialogs.MtgDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

import mathgraph32.MtgFrame;


public class CCalcul extends CCalculAncetre {
  public String chaineCalcul;
  public CCalculBase calcul;
  public double resultat;
  
  /** Creates new CCalcul */
  public CCalcul() {
  }
  
  /* Constructeur pour outil de cration */
  public CCalcul(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit) {
    super(liste, impProtoInit, estElementFinalInit);
  }
  
  // Constructeur pour clonage
  public CCalcul(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    String nomCalculInit, String chaineCalculInit, CCalculBase calculInit) {
    super(liste, impProtoInit, estElementFinalInit, nomCalculInit);
    chaineCalcul = new String(chaineCalculInit);
    calcul = calculInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    CCalculBase calculClone = calcul.getClone(listeSource, listeCible);
    int ind1 = listeSource.indexOf(impProto);
    return new CCalcul(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, nomCalcul, chaineCalcul, calculClone);
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NCalculReel;
  }
  
  /***
   * Fonction reconstruisant la chaine reprsentant le calcul (par exemple lorsqu'un autre calcul a t renomm
   * A redfinir pour les fonctions
   */
  public void reconstruitChaineCalcul() {
  	// Erreur rectifie version 3.5.1
  	// String[] ch = new String[1];
  	// ch[0] = "";
  	// chaineCalcul = calcul.chaineCalculSansPar(ch);
  	chaineCalcul = calcul.chaineCalculSansPar(null);
  }
  
  @Override
  public String infoPourHistorique() {
    String ch = nom() + " : " + getString("ch17")+ "\n" + nom() + " = " + chaineCalcul;
    if (existe) ch = ch + "\n" + getString("chinfo97") + " " + glob.Kernel.chaineNombre(resultat, 16);
    else ch = ch + "\n" + getString("ch18");
    return ch;
  }
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
		return nom() + " = " + chaineCalcul;
	}

  // Modfi version 5.0. Plus de paramtre
  @Override
  public double rendValeur() {
    return resultat;
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = calcul.existe(infoRandom);
    if (!existe) return;
    // A revoir en java pour la gestion des erreurs de calcul
    // AnnuleErreurDeCalcul();	// Pour dtecter une ventuelle erreur de
    //                         // calcul lors de l'appel
    try {
      listeProprietaire.initialiseNombreIterations();
      resultat = calcul.resultat(infoRandom);
    } catch (ErreurCalculException e) {
      existe = false;
      resultat = 0;
    }
    if (Double.isNaN(resultat) || Double.isInfinite(resultat)) {
      existe = false;
    }
    // }
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || calcul.dependDe(p));
  }
  
  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || calcul.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || calcul.dependDePourBoucle(p);
  }
  
  // Rajout pour la version 1.8 : nom() qui rend le nom du calcul.
  // Ne sera redfini que pour les calculs et les variables
  public String nom() {
    return nomCalcul;
  }

  @Override
  public boolean modifiableParMenu() {
  	// Modifi version 3.5
  	// return true;
  	return !estElementFinal;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new CalculReelDlg(frame, this, true);
  }
  
   // Inutile de redfinir informationModification() pour les objets de type calcul
  
  /**
   * cration de l'arbre binaire reprsentant la chane de calcul
   */
  public void creeCalcul() {
    calcul = CCalculBase.creeCalculBase(chaineCalcul, listeProprietaire, 0,
            chaineCalcul.length()-1, null);  
  }
  
  /**
   * Rajout pour la version 2.5 pour les prototypes
   * Sert  savoir si un objet est dfini uniquement par des objets figurant dans une liste
   * Attention : cette fonction est spciale pour cet objet
   * En effet, un objet calcul descendant de CCAlculBase ne faisant rfrence
   * qu' des constantes renvoie true dans EstDefiniParObjetDans
   * Il faut donc qu'en plus le calcul dpende des objets sources
   */
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
		boolean res = false;
		java.util.ListIterator<CElementBase> li = listeOb.listIterator();
		while (li.hasNext()) {
			CElementBase ptel = li.next();
			res = res || dependDe(ptel);
		}
		return res && calcul.estDefiniParObjetDans(listeOb);
	}  
  
  /**
   * Ajout pour la version 3.0 : Renvoie true seulement si le calcul associ
   * est constant et ne fait pas appel  la fonction Rand()
   * @return
   */
  public boolean estConstant() {
  	// Ligne suivante ajoute pour la version 3.2.8
  	// Quand une construction a dans ses objets intermdiaires ou finaux une drive et que par exemple
  	// elle utilise une droite dont le coefficient directeur est un calcul utilisant cette drive, au
  	// chargement de la construction, la drive n'a pas encore son calcul effectu et on
  	// obtient une NullPointerException
  	if (calcul == null) return false;
  	return calcul.estConstant();
  }
 
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps,list);
    chaineCalcul = inps.readCString();
    calcul = (CCalculBase) (inps.readCObject(list));
  }

  // Version 1.9.5 : Suite  l'introduction de CDerivee
  // il a fallu dfinir cette produre car CDerivee n'enregistre
  // pas le calcul dans le flux.
  public void chargeDepuisFluxSansCalcul(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps,list);
  }

  /* Lecture depuis un flux java */  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    chaineCalcul = inps.readUTF();
    calcul = (CCalculBase) (inps.readObject(list));
  }
  /**
   * Fonction renvoyant true si le calcul est une fonction utilisateur dont un des paramtres a pour nom ch
   * A redfinir pour les fonctions relles ou complexes de une ou plusieurs variables
   * @param ch
   * @return
   */
  public boolean aPourParametre(String ch) {
  	return false;
  	
  }
  /* Enregistrement dans un flux java */
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeUTF(chaineCalcul);
    oups.writeObject(calcul);
  }
  /* Lecture depuis un flux java pour lecture de drive */  
  public void readSansCalcul(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
  }  
  /* Enregistrement dans un flux java */
  public void writeSansCalcul(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
  }
  
  // Ajout version 3.2.8
  public boolean deriveePossible(int indiceVariable) {
  	return calcul.deriveePossible(indiceVariable);
  }

	// Ajout version 4.6
	/**
	 * Si l'objet associ possde une formule qui est un test renvoie le membre de gauche
	 * sinon renvoie null;
	 * @return
	 */
	public CCalculBase membreGauche() {
		return calcul.membreGauche();
	}
	
	// Ajout version 4.6
	/**
	 * Si l'objet associ possde une formule qui est un test renvoie le membre de gauche
	 * sinon renvoie null;
	 * @return
	 */
	public CCalculBase membreDroit() {
		return calcul.membreDroit();
	}
    
  
}
