/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;


import types.Complexe;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.Kernel;

public class CConstante extends CCalculBase {
  // La valeur de la constante
	double valeur;

  /** Creates new CConstante */
  public CConstante() {
  }
  
  /** Constructeur avec une valeur initiale */
  public CConstante(CListeObjets liste, double valeurInit)
  {
    super(liste);
    valeur = valeurInit;
  }
  
  @Override
  protected long nature() {
  	return natConstante;
  }
  
  public CCalculBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    return new CConstante(listeCible, valeur);
  }
    
  public double resultat(boolean infoRandom) {
    return valeur;
  }

  // Ajout pour la version 2.0 calculs sur les complexes.
  // Le rsultat est renvoy dans z
  public void resultatComplexe(boolean infoRandom, Complexe zRes) {
    /// Sera redfinie pour tous les descendants pouvant ne pas exister
    zRes.x = valeur;// Par dfaut on suppose le calcul rel
    zRes.y = 0;
  }
  
  @Override
	public String chaineCalcul(String[] varFor) {
		return Kernel.chaineNombre(valeur, 16);	
	}
  
  @Override
  public void chargeDepuisFlux ( MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.chargeDepuisFlux(inps, list);
    valeur = inps.readCDouble();
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    valeur = inps.readDouble();
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeDouble(valeur);
  }   
  // Ajout pour la version 1.9.5
  // Procdure rcursive servant  savoir si un calcul dpend de la variable formelle
  public boolean  dependDeVariable(int ind) {
    return false;
  }
  public CCalculBase getCopie() {
	return new CConstante(listeProprietaire, valeur);
  }
  
  @Override
	public boolean estConstant() {
		return true;
	}
	/**
	 * Rajout version 4.7.9 : Renvoie true si la calcul est une constante nulle et 0 sinon
	 */
  @Override
	public boolean estConstante0() {
		return this.valeur == 0;
	}
   
}
