/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/**
 * Ajout version 4.8. Utile pour les constructions itratives et rcursives
 */

package objets;

import java.awt.Color;
import java.io.IOException;

import mathgraph32.MtgFrame;
import types.DimensionFenetre;
import types.NatureObjet;
import types.StyleTrait;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CDemiDroiteClone extends CDemiDroite {
	private CDemiDroite droite; // La droite clone

  /** Constructeur */
  public CDemiDroiteClone() {
  }
  
  public CDemiDroiteClone(CListeObjets liste, CImplementationProto impProtoInit,boolean estElementFinalInit,
      Color couleurInit, boolean masqueInit, StyleTrait styleInit, CDemiDroite droiteInit) {
  	super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
      droite = droiteInit;
  }
  
  @Override  
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(droite);
    int ind2 = listeSource.indexOf(impProto);
    CElementBase ptelb = new CDemiDroiteClone(listeCible,(CImplementationProto)(listeCible.get(ind2,CImplementationProto.class)),
    	estElementFinal, couleur, masque, style, (CDemiDroite)(listeCible.get(ind1, CDemiDroite.class)));
    return ptelb;
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || droite.dependDe(p));
  }

  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || droite.dependDePourReclassement(p));
  }

  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || droite.dependDePourBoucle(p));
  }

	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return listeOb.contains(droite) || droite.estDefiniParObjetDans(listeOb);
	}    

  @Override  
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	// Il faut mmoriser si le'objet est masqu ou non car setClone va le rinitialiser  la valeur du point copi
  	boolean b = masque;
  	setClone(droite);
  	masque = b;
  }
  
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
		return nom() + " : " + getString("natDemiDroiteClone") +  " " + getString("ch31") + " " + droite.nom();
	}
  
  @Override
  public long getNature() {
    return NatureObjet.NDemiDroiteClone;
  }

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(droite);
  }    

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    droite = (CDemiDroite) (list.get(ind1,CDemiDroite.class));
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(droite);
    oups.writeInt(ind1);
  }

}
