/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.IOException;

import dialogs.DeriveePartielleDlg;
import dialogs.MtgDlg;
import mathgraph32.MtgFrame;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import types.DimensionFenetre;
import types.NatureCalcul;
import types.Operateur;

public class CDeriveePartielle extends CFoncNVar {
	public CFoncNVar fonctionAssociee;
	public int indiceVariableDerivation;

	/** Constructeur */
	public CDeriveePartielle() {
		super();
		chaineCalcul = "";
		calcul = null;
		chaineVariable = null;
	}

  public CDeriveePartielle(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      String nomInit, CFoncNVar fonctionAssocieeInit, int indiceVariableDerivationInit) {
  	super(liste, impProtoInit, estElementFinalInit, nomInit, "", null, fonctionAssocieeInit.chaineVariable);
  	fonctionAssociee = fonctionAssocieeInit;
  	indiceVariableDerivation = indiceVariableDerivationInit;
  	creeCalcul();
  }

  /**
   * Constructeur pour clonage
   */
  public CDeriveePartielle(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      String nomInit, CCalculBase calculInit, CFoncNVar fonctionAssocieeInit, int indiceVariableDerivationInit) {
  	super(liste, impProtoInit, estElementFinalInit, nomInit, "", calculInit, fonctionAssocieeInit.nbVar);
  	fonctionAssociee = fonctionAssocieeInit;
  	indiceVariableDerivation = indiceVariableDerivationInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(impProto);
    // Modifi version 4.2
    // CCalculBase calculClone = calcul.getClone(listeSource, listeCible);
    int ind2 = listeSource.indexOf(fonctionAssociee);
    // CElementBase ptelb = new CDerivee(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
    //    estElementFinal, nomCalcul, calculClone, (CFonc)(listeCible.get(ind2, CFonc.class)));
    CDeriveePartielle ptelb = new CDeriveePartielle(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, nomCalcul, null, (CFoncNVar)(listeCible.get(ind2, CFoncNVar.class)), indiceVariableDerivation);
    if (listeCible.getClass() != CPrototype.class) ptelb.creeCalcul();
    return ptelb;    
  }
  
  @Override
	public String[] chaineVariable() {
		return fonctionAssociee.chaineVariable();
	}

  @Override
  public int nombreVariables() {
  	// TODO Auto-generated method stub
  	return fonctionAssociee.nombreVariables();
  }
  
  // Ajout version 4.6. Pas de chaine de calcul pour une drive
  @Override
  public void reconstruitChaineCalcul() {
  	chaineCalcul = "";	
  }
  
  @Override
  public void metAJour() {
		if (! fonctionAssociee.deriveePossible(indiceVariableDerivation)) 
			calcul = new COperation(listeProprietaire, new CConstante(listeProprietaire, 1),
					new CConstante(listeProprietaire, 0), Operateur.Divi);
		else creeCalcul();  	
  }

  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NDeriveePartielle;
  }

  @Override
  public String infoPourHistorique() {
  	return nom() + " : " + getString("ch86") + " " + getString("ch190") + "\n" + fonctionAssociee.nom() + " " + getString("ch191") + " " + fonctionAssociee.chaineVariable[indiceVariableDerivation];
  }
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	return nom() + " : " + getString("ch86") + " " + getString("ch190") + " " + fonctionAssociee.nom() + " " + getString("ch191") + " " + fonctionAssociee.chaineVariable[indiceVariableDerivation];
	}

  @Override
  public String infoFormule() {
  	return nom() + " : " + getString("ch190") + " " + fonctionAssociee.nom() + 
  				"\n" + getString("ch191") + " " + fonctionAssociee.chaineVariable[indiceVariableDerivation];
  }

  @Override
  public boolean modifiableParMenu() {
  	// Modifi version 3.5
  	// return true;
  	return !estElementFinal;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	 return new DeriveePartielleDlg(frame, this, true);
  }

   @Override
   public boolean dependDe(CElementBase p) {
   	if (elementTestePourDependDe == p) return dependDeElementTeste;
     return memoriseDependance(super.dependDe(p) || fonctionAssociee.dependDe(p));
   }

   // Modifi version 4.8
   @Override
   public boolean dependDePourReclassement(CElementBase p) {
   	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || fonctionAssociee.dependDePourReclassement(p));
   }
   
   @Override
   public boolean dependDePourBoucle(CElementBase p) {
     return (p == this) || fonctionAssociee.dependDePourBoucle(p);
   }
   
   @Override
  public boolean confonduAvec(CElementBase p) {
     CDeriveePartielle pt;
     if (p.getClass() == this.getClass()) {
       pt = (CDeriveePartielle) p;
       return (fonctionAssociee == pt.fonctionAssociee)
 			&& (indiceVariableDerivation == pt.indiceVariableDerivation);
     }
     else return false;
  }

   @Override
   public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
     existe = fonctionAssociee.existe;
   }

  public void creeCalcul()
  {
    calcul = CCalculBase.creeDerivee(fonctionAssociee.calcul, indiceVariableDerivation);
  }

  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return listeOb.contains(fonctionAssociee) || fonctionAssociee.estDefiniParObjetDans(listeOb);
	}      
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.readSansCalcul(inps, list);
    int ind1 = inps.readInt();
    fonctionAssociee = (CFoncNVar) (list.get(ind1,CFoncNVar.class));
    nbVar = fonctionAssociee.nbVar;
    indiceVariableDerivation = inps.readInt();
    if (list.getClass() != CPrototype.class) {
			if (! fonctionAssociee.deriveePossible(indiceVariableDerivation)) 
				calcul = new COperation(listeProprietaire, new CConstante(listeProprietaire, 1),
						new CConstante(listeProprietaire, 0), Operateur.Divi);
			else creeCalcul();
		}
  }

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.writeSansCalcul(oups, list);
    int ind1 = list.indexOf(fonctionAssociee);
    oups.writeInt(ind1);
    oups.writeInt(indiceVariableDerivation);
  }    

}
