/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.geom.*;
import java.io.*;


import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

import mathgraph32.MtgFrame;

public class CIntCercleCercle extends CBipoint {
  CCercle C1;
  CCercle C2;
  private Point2D.Double pointInt1 = new Point2D.Double();
  private Point2D.Double pointInt2 = new Point2D.Double();
  
  /** Creates new CIntCercleCercle */
  public CIntCercleCercle() {
    super(); // Ncessaire pour crer les deux points CPOintDansBipoint maintenus dans l'objet
  }
  
  // Constructeur pour clonage
  public CIntCercleCercle(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    CCercle C1Init, CCercle C2Init) {
    super(liste, impProtoInit, estElementFinalInit); // Ncessaire pour crer les deux points CPOintDansBipoint maintenus dans l'objet
    C1 = C1Init;
    C2 = C2Init;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(C1);
    int ind2 = listeSource.indexOf(C2);
    int ind3 = listeSource.indexOf(impProto);
    CElementBase ptelb = new CIntCercleCercle(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, (CCercle)(listeCible.get(ind1, CCercle.class)), (CCercle)(listeCible.get(ind2, CCercle.class)));
    return ptelb;
  }
  
	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
		return getString("chinfo193") + " " + C1.nom() + " " + getString("chinfo194") + " " + C2.nom()  ;
	}

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(C1);
    liste.add(C2);
  } 
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    byte position;
    existe = C1.existe && C2.existe;
    if (!existe) {
      point1.existe = false;
      point2.existe = false;
      return;
    }
    position = Kernel.intersectionCercleCercle(C1.centre_x,C1.centre_y,C1.rayon,
      C2.centre_x,C2.centre_y,C2.rayon,pointInt1,pointInt2);
    double x1 = pointInt1.x;
    double y1 = pointInt1.y;
    double x2 = pointInt2.x;
    double y2 = pointInt2.y;
    
    switch (position) {
      case PositionCercleCercle.Vide:
      case PositionCercleCercle.Confondus:
        point1.existe = false;
        point2.existe = false;
        break;
      case PositionCercleCercle.Secants:
        if (C1.surArc(x1,y1) && C2.surArc(x1,y1)) {
          point1.existe = true;
          point1.placeEn(x1,y1);
          point1.positionne(infoRandom, dimfen); // Ajout versio 4.6.4
        } else point1.existe = false;
        if (C1.surArc(x2,y2) && C2.surArc(x2,y2)) {
          point2.existe = true;
          point2.placeEn(x2,y2);
          point2.positionne(infoRandom, dimfen); // Ajout versio 4.6.4
        } else point2.existe = false;
        break;
      case PositionCercleCercle.Tangents:
        if (C1.surArc(x1,y1) && C2.surArc(x1,y1)) {
          point1.existe = true;
          point1.placeEn(x1,y1);
          point1.positionne(infoRandom, dimfen); // Ajout versio 4.6.4
        } else point1.existe = false;
        point2.existe = false;
        break;
      default :
    }
    super.positionne(infoRandom, dimfen);
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || C1.dependDe(p) || C2.dependDe(p));
  }
  
  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || C1.dependDePourReclassement(p) || C2.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || C1.dependDePourBoucle(p) || C2.dependDePourBoucle(p));
  }
  
  
  @Override
  CElementLigne elementAssocie(byte indice) {
    if (indice == 0) return C1;
    else return C2;
  }
  
  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
   **/
  @Override
  public boolean confonduAvec(CElementBase p) {
    CIntCercleCercle pt;
    if (p.getClass() == this.getClass()) {
      pt = (CIntCercleCercle) p;
      return (((C1 == pt.C1) && (C2 == pt.C2))
      ||
        ((C1 == pt.C2) && (C2 == pt.C1)));
    } else return false;
  }

  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(C1) || C1.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(C2) || C2.estDefiniParObjetDans(listeOb));
	}  	  
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    C1 = (CCercle) (list.get(ind1,CCercle.class));
    C2 = (CCercle) (list.get(ind2,CCercle.class));
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    C1 = (CCercle) (list.get(ind1,CCercle.class));
    C2 = (CCercle) (list.get(ind2,CCercle.class));
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(C1);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(C2);
    oups.writeInt(ind2);
  }     
  // Fonction dfinie pour la version 2.1 pour amliorer l'outil d'intersection
  // Renvoie true si le bipoint est dfini par le cercle Cercle
  // A redfinir pour les descendants
  @Override
  boolean estDefiniPar(CCercle cercle)
  {
    return (cercle == C1) || (cercle == C2);
  }
}
