/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


package objets;

import java.awt.Color;
import java.awt.Graphics2D;
import java.util.StringTokenizer;
import javax.swing.JLabel;
import mathgraph32.MtgFrame;
import org.scilab.forge.jlatexmath.TeXConstants;
import org.scilab.forge.jlatexmath.TeXFormula;
import org.scilab.forge.jlatexmath.TeXIcon;
import dialogs.LatexDlg;
import dialogs.MtgDlg;
import glob.Kernel;
import types.Complexe;
import types.DimensionFenetre;
import types.Int;
import types.NatureCalcul;
import types.NatureObjet;
import types.Pinceau;
import types.StyleEncadrement;
import types.StyleTrait;
import types.TikzColor;

public class CLatex extends CCommentaire {
	// Ajout version 4.9.9 pour gnrer un nom
	public static int indice = 0;
	//
	private TeXIcon icon = null;
	public TeXFormula formula;
	private JLabel jl = new JLabel(); // Sert  afficher l'image de l'icne
	// Correspondance entre les indices de taille et la taille pour l'affichage latex
	// Modifi version 4.7.5.2
	// public static final float tailleLatex[] = {11,12,13,15,18,20,23,25,27};
	// A revoir version 4.7.7 : deux 63
	// Modifi version 5.0 : Devient une fonction
	public static final float tailleLatex(int taille) {
		return taille + 1;
	}
	// public static final float tailleLatex[] = {11,12,13,15,18,20,23,25,27,29,31,33,35,37,39,41,43,45,47,49,51,53,55,57,59,61,63,63};
	private boolean codeLatexOK;
	// Variable static formuleEnCreation pour que creeFormule ne soit pas appel en mm e temps par plusieurs Thread
	// car creeFormule fait appel  setExternalFactory
	// Rajout version 3.1.0. Pass pblic version 4.7.2 car utilis par CEditeurFormule
	// public static boolean formuleEnCreation = false; // Supprim version 4.7.4
	private String chaineLatex; // Rajout version 4.7.4. Contiendra la chaine LaTeX une fois les codes spciaux MathGraph32 traits
	public boolean dejaPositionne; // Rajout version 4.7.4. Mis  true quand la formule a dj t cre.
	public final String errorVal ="Val Error";
	public final String errorFrac ="Frac Error";
	public final String errorIf ="If Error";
	public final String errorFor ="For Error";
	public final String errorCalc ="Calc Error";
	public final String errorCalcSimp ="CalcSimp Error";
	public final String errorForSimp ="ForSimp Error";
  
  /** Creates new CLatex */
  public CLatex() {
  	// mtgConverter = new MtgConverter(this); // Supprim version 4.7.4
  	dejaPositionne = false;
  }

  // Ajout version 4.9.8.1 utilis dans LatexDlg
  public void init() {
  	icon = null;
  }
	/**
	 * Constructeur
	 */
  public CLatex(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit, boolean masqueInit,
      CPointAncetre pointLieInit, int indiceTaillePoliceInit, int encadrementInit,
      boolean effacementFondInit, Color couleurFondInit, String chaineCommentaireInit, int alignementHorizontalInit, int alignementVerticalInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
          dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit, encadrementInit, effacementFondInit,
          couleurFondInit, chaineCommentaireInit, alignementHorizontalInit, alignementVerticalInit);
    // mtgConverter = new MtgConverter(this); // Supprim version 4.7.4
    dejaPositionne = false;
  }
  
  // Rajout version 4.4.5 pour qu'on puisse faire des lieux d'objets de CLatex dynamiques
  @Override
  public void setClone(CElementBase ptel) {
  	super.setClone(ptel);
  	codeLatexOK = ((CLatex) ptel).codeLatexOK;
  	formula = ((CLatex) ptel).formula; // Rajout version 4.7.4
  	if (codeLatexOK) icon = ((CLatex) ptel).icon;
  }
  
  @Override
  public void setCloneTikz(CElementBase ptel) {
  	setClone(ptel);
  	chaineLatex = ((CLatex) ptel).chaineLatex; // Ajout version 4.8.1 pour exportation Tikz
  }
  
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    // Corrig lors de la version 2.0
    int ind1 = listeSource.indexOf(pointLie);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CLatex(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, x_nom, y_nom, dec_x, dec_y,
      masque, (CPointAncetre)listeCible.get(ind1, CPointAncetre.class), indiceTaillePolice, encadrement,
      effacementFond, couleurFondAffichage, chaineCommentaire, alignementHorizontal, alignementVertical);
    ((CCommentaire) ptelb).listeValeursDynamiquesUtilisees = 
      new CListeObjets(listeCible.uniteAngle, listeCible.pointeurLongueurUnite);
    if (listeCible.getClass() != CPrototype.class) {
    	((CLatex) ptelb).determineDependances();
    	// ((CLatex) ptelb).creeFormule(); // Supprim version 4.7.4
    }
    return ptelb;
  }  
	
  @Override
  public String infoPourHistorique() {
    String ch = nom() + " : " + getString("chinfo167");
  	if (pointLie != null) ch += " " + getString("chinfo200") + " " + pointLie.nom();
  	if (effacementFond) ch += "\n" + getString("paneAffichage1");
    return ch;
  }
	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
		String ch =  nom() + " : " + getString("chinfo167");
  	if (pointLie != null) ch += " " + getString("chinfo200") + " " + pointLie.nom();
  	return ch;
	}
  // Ajout version 4.9.9
  public String genereNom() {
  	indice++;
  	return getString("rlat") + indice;
  }
  
  /**
   * Cas particulier : dans le cas o le code latex est erron, renvoie une chaine standard  afficher  l'cran
   * Dans ce cas c'est super.affiche qui est appel
   */
  @Override
  public String rendChaineAffichage() {
  	return chaineCommentaire;
  }
   
  /**
   * Fonction crant la formule
   * A cause de l'applet de setExternalConverterFactory, ne doit pas tre appele
   * simultanment par plusieurs thread
   * Si formuleEnCreation est  true c'est qu'un autre Thread est en train d'utiliser CreeFormule
   * et on attend qu'il ait fini
   */
  public synchronized void creeFormule() {
  	/* Supprim version 4.7.4
  	if (formuleEnCreation) {
  		while (formuleEnCreation) Thread.yield();
  	}
  	formuleEnCreation = true;
  	DynamicAtom.setExternalConverterFactory(this);
  	/* Version 4.7.4 chaineCommentaire remplac par chaineLatex
  	try {
  		if (chaineCommentaire.startsWith("$") && chaineCommentaire.endsWith("$"))
  			formula =  new TeXFormula(chaineCommentaire.substring(1, chaineCommentaire.length() - 1));
  			else formula =  new TeXFormula(chaineCommentaire);
  		formuleEnCreation = false;
  	}
  	catch (Exception e) {
			formula = null;
			formuleEnCreation = false;
		}
		*/
  	try {
  		if (chaineLatex.startsWith("$") && chaineLatex.endsWith("$"))
  			formula =  new TeXFormula(chaineLatex.substring(1, chaineLatex.length() - 1));
  			else formula =  new TeXFormula(chaineLatex);
  	}
  	catch (Exception e) {
			formula = null;
		}
  }
  
  // Fonction appele depuis Glob.Kernel pour les affichages de chanes
  // pass synchronized version 4.9.8
  public static synchronized TeXFormula creeFormule(String ch) {
  	TeXFormula formula1;
  	/* Supprim version 4.7.4
  	if (formuleEnCreation) {
  		while (formuleEnCreation) Thread.yield();
  	}
  	formuleEnCreation = true;
  	DynamicAtom.setExternalConverterFactory(null);
  	*/
  	try {
  		formula1 =  new TeXFormula(ch);
  		// formuleEnCreation = false; // Suprim version 4.7.4
  	}
  	catch (Exception e) {
			formula1 = null;
  		// formuleEnCreation = false; // Suprim version 4.7.4
		}
  	return formula1;
  }
  /**
   * Doit tre appel lors de la srialization (chargement), lors de la cration d'un
   * commentaire ou de sa modification.
   * Est similaire  Positionne() et doit le rester
   */
  public void determineDependances()
  {
    StringBuffer buffer = new StringBuffer(chaineCommentaire);
    int ind;
    while((ind = buffer.indexOf(" ", 0)) != - 1) {
    	buffer.deleteCharAt(ind);
    }
    String chaineAAnalyser = buffer.toString();
    

    listeValeursDynamiquesUtilisees.retireTout();
    // Quand on recherche si c'est bien le nom d'une valeur valide
    // Il ne faut pas que la valeur ait t dfinie aprs l'affichage de valeur
    int indicecommentaire = listeProprietaire.indexOf(this);
    if (indicecommentaire == -1) indicecommentaire = listeProprietaire.size() - 1;          
    // On examine la chaine  la recherche de #Val(
    int len = chaineAAnalyser.length();
    int jdeb;
    int j = 0;
    // if (chaineAAnalyser.indexOf("\\Val{",j) == -1) return;
    while (((jdeb = chaineAAnalyser.indexOf("\\Val{",j)) != -1) && j < len) {
      // On recherche la parenthse fermante correspondante
      int indparf = chaineAAnalyser.indexOf('}',jdeb + 5);
      if (indparf == -1) break;
      else
      {
        // On cre une chaine form de ce qu'il y a entre \Val( et la parenthse fermante
        String ch = chaineAAnalyser.substring(jdeb+5, indparf);
        // On retire les ventuels espaces de cete chaine
        int k;
        while((k = ch.indexOf(' ', 0)) != - 1)
        {
          ch = ch.substring(0,k) + ch.substring(k+1);
        }
        // On spare cette chaine avec les virgules
        StringTokenizer st = new StringTokenizer(ch, ",",false);
        if (! st.hasMoreTokens())
        {
          j = indparf + 1;
        }
        else
        {
          String nomvaleur = st.nextToken();
          // On recherche si c'est bien le nom d'une valeur valide
          // Il ne faut pas que la valeur ait t dfinie aprs l'affichage de valeur
          // Trs important : quand determineDependances est appel
          // par getClone() de CCommentaire, le commentaire n'a pas encor t rajout
          //  la liste clone et ainsi indiceCommentaire renvoie -1.
          // dans ce cas, indiceCommentaire doit valoir le nombre d'lments actuels
          // de la liste clone qui le possde
          // Modifi version 4.7.2
          // CValeurDynamique pValeur = pointeurValeurOuFonction(nomvaleur,indicecommentaire);
          CValeurDynamique pValeur = pointeur(NatureCalcul.NTouteValeurPourCommentaireDynamiqueSaufFonction, nomvaleur,indicecommentaire);
          if (pValeur == null)
          {
            j = indparf + 1;
          }
          else
          {
            listeValeursDynamiquesUtilisees.add(pValeur);
            // Si la valeur n'existe pas, on ne l'inclut pas dans la chane  afficher
            j = indparf + 1;
          }
        }
      }		
    }
    // Ajout version 4.5.2
    j=0;
    while (((jdeb = chaineAAnalyser.indexOf("\\For{",j)) != -1) && j < len) {
      // On recherche la parenthse fermante correspondante
      int indparf = chaineAAnalyser.indexOf('}',jdeb + 5);
      if (indparf == -1) break;
      else
      {
        // On cre une chaine form de ce qu'il y a entre \Val( et la parenthse fermante
        String ch = chaineAAnalyser.substring(jdeb+5, indparf);
        // On retire les ventuels espaces de cete chaine
        int k;
        while((k = ch.indexOf(' ', 0)) != - 1)
        {
          ch = ch.substring(0,k) + ch.substring(k+1);
        }
        // On spare cette chaine avec les virgules
        StringTokenizer st = new StringTokenizer(ch, ",",false);
        if (! st.hasMoreTokens())
        {
          j = indparf + 1;
        }
        else
        {
          String nomvaleur = st.nextToken();
          // On recherche si c'est bien le nom d'une valeur valide
          // Il ne faut pas que la valeur ait t dfinie aprs l'affichage de valeur
          // Trs important : quand determineDependances est appel
          // par getClone() de CCommentaire, le commentaire n'a pas encor t rajout
          //  la liste clone et ainsi indiceCommentaire renvoie -1.
          // dans ce cas, indiceCommentaire doit valoir le nombre d'lments actuels
          // de la liste clone qui le possde
          CValeurDynamique pValeur = pointeur(NatureCalcul.NTouteValeurPourCommentaireDynamique, nomvaleur,indicecommentaire);
          if (pValeur == null)
          {
            j = indparf + 1;
          }
          else
          {
            listeValeursDynamiquesUtilisees.add(pValeur);
            // Si la valeur n'existe pas, on ne l'inclut pas dans la chane  afficher
            j = indparf + 1;
          }
        }
      }		
    }
    // Ajout version 4.6
    j=0;
    while (((jdeb = chaineAAnalyser.indexOf("\\Calc{",j)) != -1) && j < len) {
      // On recherche la parenthse fermante correspondante
      int indparf = chaineAAnalyser.indexOf('}',jdeb + 6);
      if (indparf == -1) break;
      else
      {
        // On cre une chaine form de ce qu'il y a entre \Val( et la parenthse fermante
        String ch = chaineAAnalyser.substring(jdeb+6, indparf);
        // On retire les ventuels espaces de cete chaine
        int k;
        while((k = ch.indexOf(' ', 0)) != - 1)
        {
          ch = ch.substring(0,k) + ch.substring(k+1);
        }
        // On spare cette chaine avec les virgules
        StringTokenizer st = new StringTokenizer(ch, ",",false);
        if (! st.hasMoreTokens())
        {
          j = indparf + 1;
        }
        else
        {
          String nomvaleur = st.nextToken();
          // On recherche si c'est bien le nom d'une valeur valide
          // Il ne faut pas que la valeur ait t dfinie aprs l'affichage de valeur
          // Trs important : quand determineDependances est appel
          // par getClone() de CCommentaire, le commentaire n'a pas encor t rajout
          //  la liste clone et ainsi indiceCommentaire renvoie -1.
          // dans ce cas, indiceCommentaire doit valoir le nombre d'lments actuels
          // de la liste clone qui le possde
          CValeurDynamique pValeur = pointeur(NatureCalcul.NTouteValeurPourCommentaireDynamique, nomvaleur,indicecommentaire);
          if (pValeur == null)
          {
            j = indparf + 1;
          }
          else
          {
            listeValeursDynamiquesUtilisees.add(pValeur);
            // Si la valeur n'existe pas, on ne l'inclut pas dans la chane  afficher
            j = indparf + 1;
          }
        }
      }		
    }

    // Ajout version 4.6
    j=0;
    while (((jdeb = chaineAAnalyser.indexOf("\\ForSimp{",j)) != -1) && j < len) {
      // On recherche la parenthse fermante correspondante
      int indparf = chaineAAnalyser.indexOf('}',jdeb + 9);
      if (indparf == -1) break;
      else
      {
        // On cre une chaine form de ce qu'il y a entre \Val( et la parenthse fermante
        String ch = chaineAAnalyser.substring(jdeb+9, indparf);
        // On retire les ventuels espaces de cete chaine
        int k;
        while((k = ch.indexOf(' ', 0)) != - 1)
        {
          ch = ch.substring(0,k) + ch.substring(k+1);
        }
        // On spare cette chaine avec les virgules
        StringTokenizer st = new StringTokenizer(ch, ",",false);
        if (! st.hasMoreTokens())
        {
          j = indparf + 1;
        }
        else
        {
          String nomvaleur = st.nextToken();
          // On recherche si c'est bien le nom d'une valeur valide
          // Il ne faut pas que la valeur ait t dfinie aprs l'affichage de valeur
          // Trs important : quand determineDependances est appel
          // par getClone() de CCommentaire, le commentaire n'a pas encor t rajout
          //  la liste clone et ainsi indiceCommentaire renvoie -1.
          // dans ce cas, indiceCommentaire doit valoir le nombre d'lments actuels
          // de la liste clone qui le possde
          CValeurDynamique pValeur = pointeur(NatureCalcul.NTouteValeurPourCommentaireDynamique, nomvaleur,indicecommentaire);
          if (pValeur == null)
          {
            j = indparf + 1;
          }
          else
          {
            listeValeursDynamiquesUtilisees.add(pValeur);
            // Si la valeur n'existe pas, on ne l'inclut pas dans la chane  afficher
            j = indparf + 1;
          }
        }
      }		
    }
    // Ajout version 4.6
    j=0;
    while (((jdeb = chaineAAnalyser.indexOf("\\CalcSimp{",j)) != -1) && j < len) {
      // On recherche la parenthse fermante correspondante
      int indparf = chaineAAnalyser.indexOf('}',jdeb + 10);
      if (indparf == -1) break;
      else
      {
        // On cre une chaine form de ce qu'il y a entre \Val( et la parenthse fermante
        String ch = chaineAAnalyser.substring(jdeb+10, indparf);
        // On retire les ventuels espaces de cete chaine
        int k;
        while((k = ch.indexOf(' ', 0)) != - 1)
        {
          ch = ch.substring(0,k) + ch.substring(k+1);
        }
        // On spare cette chaine avec les virgules
        StringTokenizer st = new StringTokenizer(ch, ",",false);
        if (! st.hasMoreTokens())
        {
          j = indparf + 1;
        }
        else
        {
          String nomvaleur = st.nextToken();
          // On recherche si c'est bien le nom d'une valeur valide
          // Il ne faut pas que la valeur ait t dfinie aprs l'affichage de valeur
          // Trs important : quand determineDependances est appel
          // par getClone() de CCommentaire, le commentaire n'a pas encor t rajout
          //  la liste clone et ainsi indiceCommentaire renvoie -1.
          // dans ce cas, indiceCommentaire doit valoir le nombre d'lments actuels
          // de la liste clone qui le possde
          CValeurDynamique pValeur = pointeur(NatureCalcul.NTouteValeurPourCommentaireDynamique, nomvaleur,indicecommentaire);
          if (pValeur == null)
          {
            j = indparf + 1;
          }
          else
          {
            listeValeursDynamiquesUtilisees.add(pValeur);
            // Si la valeur n'existe pas, on ne l'inclut pas dans la chane  afficher
            j = indparf + 1;
          }
        }
      }		
    }

    //
    j = 0;
    while (((jdeb = chaineAAnalyser.indexOf("\\FracRed{",j)) != -1) && j < len) {
      // On recherche la parenthse fermante correspondante
      int indparf = chaineAAnalyser.indexOf('}',jdeb + 9);
      if (indparf == -1) break;
      else
      {
        // On cre une chaine form de ce qu'il y a entre \FracRed{ et la parenthse fermante
        String ch = chaineAAnalyser.substring(jdeb+9, indparf);
        // On retire les ventuels espaces de cete chaine
        int k;
        while((k = ch.indexOf(' ', 0)) != - 1) {
          ch = ch.substring(0,k) + ch.substring(k+1);
        }
        // On spare cette chaine avec les virgules
        StringTokenizer st = new StringTokenizer(ch, ",",false);
        if (! st.hasMoreTokens()) {
          j = indparf + 1;
        }
        else
        {
          String nomvaleur = st.nextToken();
          // Trs important : quand determineDependances est appel
          // par getClone() de CCommentaire, le commentaire n'a pas encor t rajout
          //  la liste clone et ainsi indiceCommentaire renvoie -1.
          // dans ce cas, indiceCommentaire doit valoir le nombre d'lments actuels
          // de la liste clone qui le possde
          CValeurDynamique pValeur = pointeur(NatureCalcul.NTouteValeurReellePourCommentaireDynamique, nomvaleur,indicecommentaire);
          if (pValeur == null)
          {
            j = indparf + 1;
          }
          else
          {
            if (! st.hasMoreTokens()) {
              j = indparf + 1;
            }
            else {
              String nomvaleur2 = st.nextToken();
              // On recherche si c'est bien le nom d'une valeur valide
              // Il ne faut pas que la valeur ait t dfinie aprs l'affichage de valeur
              // Trs important : quand determineDependances est appel
              // par getClone() de CCommentaire, le commentaire n'a pas encor t rajout
              //  la liste clone et ainsi indiceCommentaire renvoie -1.
              // dans ce cas, indiceCommentaire doit valoir le nombre d'lments actuels
              // de la liste clone qui le possde
              CValeurDynamique pValeur2 = pointeur(NatureCalcul.NTouteValeurReellePourCommentaireDynamique, nomvaleur2,indicecommentaire);
              if (pValeur != null) {
                listeValeursDynamiquesUtilisees.add(pValeur);
                listeValeursDynamiquesUtilisees.add(pValeur2);
              }
              j = indparf + 1;
            }
          }
        }
      }		
    }
    j = 0;
    while (((jdeb = chaineAAnalyser.indexOf("\\If{",j)) != -1) && j < len) {
      // On recherche la parenthse fermante correspondante
      int indparf = chaineAAnalyser.indexOf('}',jdeb + 4);
      if (indparf == -1) break;
      else
      {
        // On cre une chaine forme de ce qu'il y a entre \if{ et la parenthse fermante
        String ch = chaineAAnalyser.substring(jdeb+4, indparf);
        // On retire les ventuels espaces de cette chaine
        int k;
        while((k = ch.indexOf(' ', 0)) != - 1)
        {
          ch = ch.substring(0,k) + ch.substring(k+1);
        }
        CValeurDynamique pValeur = pointeur(NatureCalcul.NTouteValeurPourCommentaireDynamiqueSaufFonction, ch,indicecommentaire);
        if (pValeur == null) j = indparf + 1;
        else {
        	listeValeursDynamiquesUtilisees.add(pValeur);
        	j = indparf + 1;
        }
      }
    }
  }
  
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	// super.positionne(infoRandom, dimfen);
  	// Modifi version 4.9.8
  	boolean formuleRecalculee = false;
  	// Ajout version 4.7.4
  	if (!dejaPositionne || listeValeursDynamiquesUtilisees.size() != 0) {
  		traiteCodesLatex();
  		creeFormule();
  		formuleRecalculee = true;
  		dejaPositionne = true; // Rajout version 4.7.7
  	}
  	if (formula == null) {
  		codeLatexOK = false;
  		super.positionnePourCLatex(infoRandom, dimfen);
  	}
  	else {
    	super.positionnePourCLatex(infoRandom, dimfen);  		
	  	try {
	  		// formula =  new TeXFormula(chaineAffichage);
	      // boolean basseDefinition = listeProprietaire.basseDefinition; Modifi version 4.6.4
	  		// Modification version 3.4.4 : A cause du fait qu'une formule LaTeX peut maintenant contenir des formules
	      // utilisateur comme \If pouvant en appeler d'autres, il faut appeler setExternalFactory
	      // avant d'appeler createTeXIcon
	  		// Modifi version 4.7.4
	  		/*
	      if (listeValeursDynamiquesUtilisees.size() > 0){
		    	if (formuleEnCreation) {
		    		while (formuleEnCreation) Thread.yield();
		    	}
		    	formuleEnCreation = true;
		    	DynamicAtom.setExternalConverterFactory(this);
		    	// Modification version 4.6.4
		  		if (basseDefinition) icon = formula.createTeXIcon(TeXConstants.STYLE_DISPLAY, tailleLatex[indiceTaillePolice]);
		  		else icon = formula.createTeXIcon(TeXConstants.STYLE_DISPLAY, tailleLatex[indiceTaillePolice]*10);
		  		//
		    	icon = formula.createTeXIcon(TeXConstants.STYLE_DISPLAY, (float)listeProprietaire.coefMult*tailleLatex[indiceTaillePolice]);
		  		formuleEnCreation = false;
	      }
	      else {
		    	// Modification version 4.6.4
		  		if (basseDefinition) icon = formula.createTeXIcon(TeXConstants.STYLE_DISPLAY, tailleLatex[indiceTaillePolice]);
		  		else icon = formula.createTeXIcon(TeXConstants.STYLE_DISPLAY, tailleLatex[indiceTaillePolice]*10);
		  		//
	      	icon = formula.createTeXIcon(TeXConstants.STYLE_DISPLAY, (float)listeProprietaire.coefMult*tailleLatex[indiceTaillePolice]);
	      }
	      */
	  		// Ligne suivante modifi version 5.0
      	if ((icon == null) || formuleRecalculee) icon = formula.createTeXIcon(TeXConstants.STYLE_DISPLAY,
      			(float)listeProprietaire.coefMult*tailleLatex(indiceTaillePolice));
		  	codeLatexOK = true;
	  	}
	  	catch (Exception e) {
	  		// formula = null;
	  		codeLatexOK = false;
	  		// formuleEnCreation = false; // Supprim version 4.7.4
	  		super.positionne(infoRandom, dimfen);
	  	}
  	}
  }
  
  // Modifi version 4.9.9. Renvoie maintenant un boolen qui vaut true seulement s'il y a bien eu remplacement
  // renvoye n'est pas la mme que celle envoye 
  @Override
  public boolean remplaceNomValeurDynamique(String ancienNom, String nouveauNom) {
  	String chaineInitiale = chaineCommentaire;
  	// On supprime d'abord tous les caractres espace suivant un { ou une virgule ou prcdant une accolade
    StringBuffer buffer = new StringBuffer(chaineCommentaire);
    int ind;
    while((ind = buffer.indexOf("\\Val{ ", 0)) != - 1) {
    	buffer.deleteCharAt(ind+5);
    }
    // Ajout version 4.5.2
    while((ind = buffer.indexOf("\\For{ ", 0)) != - 1) {
    	buffer.deleteCharAt(ind+5);
    }
    // Ajout version 4.6
    while((ind = buffer.indexOf("\\ForSimp{ ", 0)) != - 1) {
    	buffer.deleteCharAt(ind+9);
    }
    // Ajout version 4.6
    while((ind = buffer.indexOf("\\Calc{ ", 0)) != - 1) {
    	buffer.deleteCharAt(ind+6);
    }
    // Ajout version 4.6
    while((ind = buffer.indexOf("\\CalcSimp{ ", 0)) != - 1) {
    	buffer.deleteCharAt(ind+10);
    }
    //
    while((ind = buffer.indexOf("\\FracRed{ ", 0)) != - 1) {
    	// Bug corrig version 4.7.0.4
    	// buffer.deleteCharAt(ind+8);
    	buffer.deleteCharAt(ind+9);
    }
    
    chaineCommentaire = buffer.toString();
  	String ch1 = "\\Val{" + ancienNom + "}";
  	String ch2 = "\\Val{" + nouveauNom + "}";
  	chaineCommentaire = chaineCommentaire.replace(ch1, ch2);
  	ch1 = "\\Val{" + ancienNom + " ";
  	ch2 = "\\Val{" + nouveauNom + " ";
  	chaineCommentaire = chaineCommentaire.replace(ch1, ch2);
  	ch1 = "\\Val{" + ancienNom + ",";
  	ch2 = "\\Val{" + nouveauNom + ",";
  	chaineCommentaire = chaineCommentaire.replace(ch1, ch2);
    
  	ch1 = "\\For{" + ancienNom + "}";
  	ch2 = "\\For{" + nouveauNom + "}";
  	chaineCommentaire = chaineCommentaire.replace(ch1, ch2);
  	ch1 = "\\For{" + ancienNom + " ";
  	ch2 = "\\For{" + nouveauNom + " ";
  	chaineCommentaire = chaineCommentaire.replace(ch1, ch2);

  	ch1 = "\\ForSimp{" + ancienNom + "}";
  	ch2 = "\\ForSimp{" + nouveauNom + "}";
  	chaineCommentaire = chaineCommentaire.replace(ch1, ch2);
  	ch1 = "\\ForSimp{" + ancienNom + " ";
  	ch2 = "\\ForSimp{" + nouveauNom + " ";
  	chaineCommentaire = chaineCommentaire.replace(ch1, ch2);

  	ch1 = "\\Calc{" + ancienNom + "}";
  	ch2 = "\\Calc{" + nouveauNom + "}";
  	chaineCommentaire = chaineCommentaire.replace(ch1, ch2);
  	ch1 = "\\Calc{" + ancienNom + " ";
  	ch2 = "\\Calc{" + nouveauNom + " ";
  	chaineCommentaire = chaineCommentaire.replace(ch1, ch2);

  	ch1 = "\\CalcSimp{" + ancienNom + "}";
  	ch2 = "\\CalcSimp{" + nouveauNom + "}";
  	chaineCommentaire = chaineCommentaire.replace(ch1, ch2);
  	ch1 = "\\CalcSimp{" + ancienNom + " ";
  	ch2 = "\\CalcSimp{" + nouveauNom + " ";
  	chaineCommentaire = chaineCommentaire.replace(ch1, ch2);
  	
  	ch1 = "\\FracRed{" + ancienNom + ",";
  	ch2 = "\\FracRed{" + nouveauNom + ",";
  	chaineCommentaire = chaineCommentaire.replace(ch1, ch2);
  	ch1 = "\\FracRed{" + ancienNom + " ";
  	ch2 = "\\FracRed{" + nouveauNom + " ";
  	chaineCommentaire = chaineCommentaire.replace(ch1, ch2);

  	ch1 = "\\If{" + ancienNom + "}";
  	ch2 = "\\If{" + nouveauNom + "}";
  	chaineCommentaire = chaineCommentaire.replace(ch1, ch2);
  	ch1 = "\\If{" + ancienNom + " ";
  	ch2 = "\\If{" + nouveauNom + " ";
  	chaineCommentaire = chaineCommentaire.replace(ch1, ch2);
  	// On remplace les ventuels deuximes paramtres qui sont aprs une virgule
  	remplaceNomValeurDynamiqueApresVirgule("\\FracRed{", ancienNom, nouveauNom);
  	return (!chaineInitiale.equals(chaineCommentaire));
  }
  
  /**
   * Fonction recherchant dans les codes spciaux utilisant une virgue ancienNom et le replaant par nouveauNom
   * @param code : contient  \FracRed qui est pour le moment le seul code avec virgules dont le deuxime argument
   * peut tre le nom d'un calcul ou une variable
   * @param ancienNom
   * @param nouveauNom
   */
  public void remplaceNomValeurDynamiqueApresVirgule(String code, String ancienNom, String nouveauNom) {
  	int ind = 0;
  	while((ind = chaineCommentaire.indexOf(code, ind)) != -1) {
  		// On recherche la premire accolade fermante
  		int indacf = chaineCommentaire.indexOf('}', ind + code.length());
  		if (indacf != -1) {
  			String left = chaineCommentaire.substring(0, ind + code.length() - 1);
  			String right = chaineCommentaire.substring(indacf + 1);
  			String mid = chaineCommentaire.substring(ind + code.length() - 1, indacf + 1);
  	  	String ch1 = "," + ancienNom + " ";
  	  	String ch2 = "," + nouveauNom + " "; 
  	  	mid = mid.replace(ch1, ch2);
  	  	ch1 = "," + ancienNom + ",";
  	  	ch2 = "," + nouveauNom + ","; 
  	  	mid = mid.replace(ch1, ch2);
  	  	ch1 = " " + ancienNom + ",";
  	  	ch2 = " " + nouveauNom + ",";
  	  	mid = mid.replace(ch1, ch2);
  	  	ch1 = " " + ancienNom + " ";
  	  	ch2 = " " + nouveauNom + " ";
  	  	mid = mid.replace(ch1, ch2);
  	  	ch1 = " " + ancienNom + "}";
  	  	ch2 = " " + nouveauNom + "}";
  	  	mid = mid.replace(ch1, ch2);
  	  	ch1 = "," + ancienNom + "}";
  	  	ch2 = "," + nouveauNom + "}";
  	  	mid = mid.replace(ch1, ch2);
  			chaineCommentaire = left + mid + right;
  		}
  		ind = ind + code.length() - 1;
  	}
  }
  
  public void affiche(Graphics2D pDC, boolean masquage, Color couleurFond, float transparence) {
  	if (!masque || (masque && !masquage)) {
	  	if (!codeLatexOK  || (formula == null)) {
	  		super.affiche(pDC, masquage, couleurFond, transparence);
	  		return;
	  	}
      rectangleAffichage.x = (int) (x_nom + dec_x);
      rectangleAffichage.y = (int) (y_nom + dec_y);
      rectangleAffichage.width = icon.getIconWidth();
      rectangleAffichage.height = icon.getIconHeight();
      switch (alignementHorizontal) {
      case alignementHorizontalCentre :
      	rectangleAffichage.x -= rectangleAffichage.width/2;
      	break;
      case alignementHorizontalDroit :
      	rectangleAffichage.x -= rectangleAffichage.width;
	    }
	    switch (alignementVertical) {
	    	case alignementVerticalCentre :
	    		rectangleAffichage.y -= rectangleAffichage.height/2;
	    		break;
	    	case alignementVerticalBas :
	    		rectangleAffichage.y -= rectangleAffichage.height;
	    }
      trace(pDC, couleurFond, transparence);
    }
  }
  
  
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
  	if (!codeLatexOK) {
  		super.trace(pDC, couleurFond, transparence);
  		return;
  	}
    boolean basseDefinition = listeProprietaire.basseDefinition;
    // if ((chaineAffichage.length() == 0)) return;
    pDC.setStroke(Pinceau.CreePinceau(StyleTrait.traitFin, null)); // Pour le rectangle
    // d'encadrement ventuel et les flches de vecteur ventuelles
    double coef = listeProprietaire.coefMult;
    if (basseDefinition) {
	    xEncadrement = rectangleAffichage.x - 3;
	    yEncadrement = rectangleAffichage.y - 2;
	    widthEncadrement = rectangleAffichage.width + 5;
	    heightEncadrement = rectangleAffichage.height + 2;
    } else {
      // Les dimensions suivantes sont diffrentes dans l'applet car l'espace de coordonnes est 100 fois plus grand
      // et il y a des problmes d'arrondi
      xEncadrement = rectangleAffichage.x - (int)(coef*3);
      yEncadrement = rectangleAffichage.y - (int)(coef*2);
      widthEncadrement = rectangleAffichage.width +(int)(coef*5);
      heightEncadrement = rectangleAffichage.height + (int)(coef*2);    	
    }
    
 
    if (effacementFond) {
      pDC.setColor(couleurFondAffichage);
      // Les bords bas et droits ne sont pas compris semble-t-il par fillRect
      // Diffrence par rapport  la version C++ V1.8
      if (encadrement != 0)
        pDC.fillRect(xEncadrement, yEncadrement, widthEncadrement, heightEncadrement);
      else
        pDC.fillRect(rectangleAffichage.x, rectangleAffichage.y,
          rectangleAffichage.width, rectangleAffichage.height);
      
    }
    pDC.setColor(couleur);
   	// jl.setForeground(couleur);
    icon.setForeground(couleur);
   	icon.paintIcon(jl, pDC, rectangleAffichage.x, rectangleAffichage.y); 
    
    if (encadrement != 0) {
      pDC.drawRect(xEncadrement, yEncadrement, widthEncadrement, heightEncadrement);
    }
    
    if (encadrement == StyleEncadrement.Effet3D) {
    	if (basseDefinition) {
	      int x1 = xEncadrement + widthEncadrement + 1;
	      int y1 = yEncadrement + heightEncadrement + 1;
	      pDC.drawLine(xEncadrement + 1, y1, x1, y1);
	      pDC.drawLine(x1, yEncadrement + 1, x1, y1);
    	}
    	else {
    		int x = xEncadrement + widthEncadrement;
    		int y = yEncadrement + heightEncadrement;
        int x1 = x + (int)coef;
        int y1 = y + (int)coef;
        for (int j = 1; j <= (int)coef; j++) {
	        pDC.drawLine(xEncadrement + j, y + j, x1, y + j);
	        pDC.drawLine(x + j, yEncadrement + j, x + j, y1);
        }
    	}
    }
  }  

  // A revoir
  @Override
  public long getNature() {
    return NatureObjet.NLatex;
  }
  
  // A revoir
  @Override
  public boolean modifiableParMenu() {
  	return true;
  }
  
  // A revoir
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new LatexDlg(frame, this, true, this);
  }
   
   @Override
   public String chaineDesignation() {
     return getString("designationLatex");
   }
   
   // Ajout version 4.6
   @Override
  public void metAJour() {
  	determineDependances();
  }
   
   /* Abandonn version 4.7.4
   @Override
	// Il faut redfinir read car il faut crer la formule une fois la chane charge
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.readForCLatex(inps, list);
   	// mtgConverter = new MtgConverter(this); // Supprim version 4.7.4
    if (list.getClass() != CPrototype.class) creeFormule();
  }
  */ 


  /* Supprim version 4.7.4
	public ExternalConverter getExternalConverter() {
		return mtgConverter;
	}
	*/
	
	/**
	 * Fonction renvoyant un tableau de chanes de caractres correspondant aux arguments
	 * d'une fonction LaTeX de mathGraph32.
	 * @param nbArg
	 * @param ch : contient la chane dans laquelle se fait la recherche
	 * @param debut : contient l'indice du dbut de la recherche du premier argument
	 * @param fin : contiendra en retour l'indice dans ch de la dernire accolade fermante
	 * @return : si tout s'est bien pass, un tableau de String contenant les arguments, sinon null
	 */
	private final String[] parametres(int nbArg, String ch, int debut, Int fin) {
		int ind2 = -1;
		int len = ch.length();
		String[] chret = new String[nbArg];
		int i = debut;
		for (int j = 0; j < nbArg; j++ ) {
			if (i >= len) return null;
			// On saute les espaces et les retours chariot
			while (((ch.charAt(i) == ' ') || (ch.charAt(i) == '\n')) && (i < len - 1)) i++;
			if (ch.charAt(i) != '{') {
				return null;
			}
			int codePar = 1;
			ind2 = i + 1;
			while ((ind2 < len)) {
				char car = ch.charAt(ind2);
				if (car == '{') codePar++;
				else if (car == '}') codePar--;
				if (codePar == 0) break; else ind2++;
			}
			if (codePar != 0) return null;
			// On retire les ventuels espaces de fin
			int k = ind2 - 1;
			while (ch.charAt(k) == ' ') k--;
			chret[j] = ch.substring(i+1, k+1);
			i = ind2 + 1;
		}
		fin.setValue(ind2);
		return chret;
	}
	
	public final void traiteCodesLatex() {
		Int fin = new Int(0);
		// Les codes \If peuvent tre imbriqus les uns dans les autres. Il faut les rechecher tant qu'il y en a et les traiter depuis le dbut
		chaineLatex = new String(chaineCommentaire);
		int ideb = 0;
		while ((ideb = chaineLatex.indexOf("\\If", 0)) != -1) {
			String param[] = parametres(3, chaineLatex, ideb + 3, fin);
			if (param == null) {
				chaineLatex = chaineLatex.substring(0, ideb) + errorIf;
				break;
				
			}
			else {
				String chremp = traiteCodeIf(param);
				int len = chaineLatex.length();
				if (fin.getValue() == len - 1) chaineLatex = chaineLatex.substring(0, ideb) + chremp;
				else chaineLatex = chaineLatex.substring(0, ideb) + chremp + chaineLatex.substring(fin.getValue() + 1, len);
			}
		}
		// Traitement des codes \Val
		while ((ideb = chaineLatex.indexOf("\\Val", 0)) != -1) {
			String[] param = parametres(1, chaineLatex, ideb + 4, fin);
			if (param == null) {
				chaineLatex = chaineLatex.substring(0, ideb) + errorVal;
				break;
			}
			else {
				String chremp = "";
				try {
					chremp = traiteCodeVal(param[0]);
					int len = chaineLatex.length();
					if (fin.getValue() == len - 1) chaineLatex = chaineLatex.substring(0, ideb) + chremp;
					else chaineLatex = chaineLatex.substring(0, ideb) + chremp + chaineLatex.substring(fin.getValue() + 1, len);
				}
				catch (Exception e) {
					chaineLatex = chaineLatex.substring(0, ideb) + errorVal;
				}					
			}
		}
		// Traitement des codes \FracRed
		while ((ideb = chaineLatex.indexOf("\\FracRed", 0)) != -1) {
			String[] param = parametres(1, chaineLatex, ideb + 8, fin);
			if (param == null) {
				chaineLatex = chaineLatex.substring(0, ideb) + errorFrac;
				break;
			}
			String chremp = traiteCodeFracRed(param[0]);
			int len = chaineLatex.length();
			if (fin.getValue() == len - 1) chaineLatex = chaineLatex.substring(0, ideb) + chremp;
			else chaineLatex = chaineLatex.substring(0, ideb) + chremp + chaineLatex.substring(fin.getValue() + 1, len);
		}
		// Traitement des codes \ForSimp. A faire imprativement avant les \For
		while ((ideb = chaineLatex.indexOf("\\ForSimp", 0)) != -1) {
			String[] param = parametres(1, chaineLatex, ideb + 8, fin);
			if (param == null) {
				chaineLatex = chaineLatex.substring(0, ideb) + errorForSimp;
				break;
			}
			String chremp = traiteCodeForSimp(param[0]);
			int len = chaineLatex.length();
			if (fin.getValue() == len - 1) chaineLatex = chaineLatex.substring(0, ideb) + chremp;
			else chaineLatex = chaineLatex.substring(0, ideb) + chremp + chaineLatex.substring(fin.getValue() + 1, len);
		}
		// Traitement des codes \For
		while ((ideb = chaineLatex.indexOf("\\For", 0)) != -1) {
			String[] param = parametres(1, chaineLatex, ideb + 4, fin);
			if (param == null) {
				chaineLatex = chaineLatex.substring(0, ideb) + errorFor;
				break;
			}
			String chremp = traiteCodeFor(param[0]);
			int len = chaineLatex.length();
			if (fin.getValue() == len - 1) chaineLatex = chaineLatex.substring(0, ideb) + chremp;
			else chaineLatex = chaineLatex.substring(0, ideb) + chremp + chaineLatex.substring(fin.getValue() + 1, len);
			ideb = fin.getValue() + 1;
		}
		// Traitement des codes \CalcSimp. A faire imprativement avant les \Calc
		while ((ideb = chaineLatex.indexOf("\\CalcSimp", 0)) != -1) {
			String[] param = parametres(1, chaineLatex, ideb + 9, fin);
			if (param == null) {
				chaineLatex = chaineLatex.substring(0, ideb) + errorCalcSimp;
				break;
			}
			String chremp = traiteCodeCalcSimp(param[0]);
			int len = chaineLatex.length();
			if (fin.getValue() == len - 1) chaineLatex = chaineLatex.substring(0, ideb) + chremp;
			else chaineLatex = chaineLatex.substring(0, ideb) + chremp + chaineLatex.substring(fin.getValue() + 1, len);
		}
		// Traitement des codes \Calc
		while ((ideb = chaineLatex.indexOf("\\Calc", 0)) != -1) {
			String[] param = parametres(1, chaineLatex, ideb + 5, fin);
			if (param == null) {
				chaineLatex = chaineLatex.substring(0, ideb) + errorCalc;
				break;
			}
			String chremp = traiteCodeCalc(param[0]);
			int len = chaineLatex.length();
			if (fin.getValue() == len - 1) chaineLatex = chaineLatex.substring(0, ideb) + chremp;
			else chaineLatex = chaineLatex.substring(0, ideb) + chremp + chaineLatex.substring(fin.getValue() + 1, len);
		}
	}
	private final String traiteCodeVal(String param) {
	  // On spare cette chaine avec les virgules
		String chnombre = "";
	  StringTokenizer st = new StringTokenizer(param, ",", false);
	  if (st.hasMoreTokens()) {
	    String nomvaleur = st.nextToken();
	    // On recherche si c'est bien le nom d'une valeur valide
	    // Il ne faut pas que la valeur ait t dfinie aprs l'affichage de valeur
	    // Si c'est le nom d'une valeur utilise, elle est comprise dans la
	    // liste listeValeursDynamiquesUtilisees
	    int indfin = listeValeursDynamiquesUtilisees.size() - 1;
	    // Modifi version 4.7.2
	    // CValeurDynamique pValeur = latex.listeValeursDynamiquesUtilisees.pointeurValeurOuFonction(nomvaleur,indfin);
	    CValeurDynamique pValeur = listeValeursDynamiquesUtilisees.pointeurParNatureCalcul
	    		(NatureCalcul.NTouteValeurPourCommentaireDynamiqueSaufFonction, nomvaleur,indfin);
	    if (pValeur != null) {
	      // Si la valeur n'existe pas, on ne l'inclut pas dans la chane  afficher
	      if (pValeur.existe) {
	        if (st.hasMoreTokens()) {
	          String chnbdec = st.nextToken(); 
	          if (chnbdec.equals("+")) {
	            chnombre = pValeur.rendChaineValeurPourCommentaire(2);
	          	// if (chnombre.startsWith("-")) chnombre = "\\,-\\," + chnombre.substring(1);  // Modifi version 4.7.4
	            if (chnombre.startsWith("-")) chnombre = " -" + chnombre.substring(1);
	          	// else chnombre = "\\,+\\," + chnombre;
	            else chnombre = " +" + chnombre;
	          }
	          else {
	          	// On retire les espaces de dbut
	          	int k =0;
	          	while (chnbdec.charAt(k) == ' ') k++;
	            Integer nbdec = new Integer(chnbdec.substring(k));
	            int nbdecimales = nbdec.intValue();
	            // chnombre = pValeur.rendChaineValeurPourCommentaire(nbdecimales, false) + "\\,"; // Modifi version 4.7.4
	            chnombre = pValeur.rendChaineValeurPourCommentaire(nbdecimales) + " ";
	            // Si le troisime paramtre :
	            // commence par + : on impose un signe + devant l'affichage si le nombre est positif ou nul
	            // finit par un zro : on impose un signe + devant l'affichage si le nombre est positif ou nul
	            // et on n'affiche rien si le nombre est nul
	            // finit par 1 : on n'affiche pas le nombre si le nombre est gal  1 et on affiche un signe - sil est gal  -1
	            if (st.hasMoreTokens()) {
	              String chplus = st.nextToken();
	              // On retire les ventuels espaces de dbut
		          	k =0;
		          	while (chplus.charAt(k) == ' ') k++;
	              chplus = chplus.substring(k);
	              boolean signePlusImpose = chplus.startsWith("+");
	              if (signePlusImpose) {
	              	/* Modifi version 4.7.4
	              	if (chnombre.startsWith("-")) chnombre = "\\,-\\," + chnombre.substring(1);
	              	else chnombre = "\\,+\\," + chnombre;
	              	*/
	              	if (chnombre.startsWith("-")) chnombre = " -" + chnombre.substring(1);
	              	else chnombre = " +" + chnombre;
	              }
	            	if (chplus.endsWith("0")) {
	            		double nombre = pValeur.rendValeur();
	            		// if (nombre == 0) return "\\,"; // Modifi version 4.7.4
	            		if (nombre == 0) return " ";
	            	}
	            	else {
	            		double nombre = pValeur.rendValeur();
	              	if (chplus.endsWith("1")) {
	              		/* Modifi version 4.7.4
	              		if (nombre == 1) return signePlusImpose ? "\\,+\\," : "\\,";
	              		else if (nombre == -1) return "\\,-\\,";
	              		*/
	              		if (nombre == 1) return signePlusImpose ? " + " : " ";
	              		else if (nombre == -1) return " - ";
	              	}
	            	}
	            }
	          }
	        }
	        else
	        {
	          // Si le nombre de dcimales n'est pas prcis, on en met deux par dfaut
	          return pValeur.rendChaineValeurPourCommentaire(2);
	        }
	      }
	    }
	  }
	  else return errorVal;
	  return chnombre;
	}
	
	private final String traiteCodeFracRed(String param) {
    // On spare cette chaine avec les virgules
		String res = " ";
    StringTokenizer st = new StringTokenizer(param, ",", false);
    if (st.hasMoreTokens()) {
      String nomvaleur1 = st.nextToken();
      // On recherche si c'est bien le nom d'une valeur valide
      // Il ne faut pas que la valeur ait t dfinie aprs l'affichage de valeur
      // Si c'est le nom d'une valeur utilise, elle est comprise dans la
      // liste listeValeursDynamiquesUtilisees
      int indfin = listeValeursDynamiquesUtilisees.size() - 1;
      CValeurDynamique pValeurNum = listeValeursDynamiquesUtilisees.pointeurParNatureCalcul
      		(NatureCalcul.NTouteValeurReellePourCommentaireDynamique, nomvaleur1,indfin);
      if (pValeurNum == null) return errorFrac;
      if (pValeurNum.existe) {
        double num = pValeurNum.rendValeur();
        if (st.hasMoreTokens()) {
		      String nomvaleur2 = st.nextToken();
		      CValeurDynamique pValeurDen = listeValeursDynamiquesUtilisees.pointeurParNatureCalcul
		      		(NatureCalcul.NTouteValeurReellePourCommentaireDynamique, nomvaleur2,indfin);
		      if (pValeurDen == null) return errorFrac;
		      if (pValeurDen.existe) {
	          double den = pValeurDen.rendValeur();
	          if (den == 0) return errorFrac;
	          double numabs = Math.abs(num);
	          double denabs = Math.abs(den);
	          if ((numabs != Math.floor(numabs)) || (numabs > Integer.MAX_VALUE)
	          		|| (denabs != Math.floor(denabs)) || (denabs > Integer.MAX_VALUE) || (denabs == 0)) return errorFrac;
	          else {
  	          boolean signePlusImpose= false;
  	          String chsig = "";
  	          if (st.hasMoreTokens()) {	          	
		            chsig = st.nextToken();
		            if (chsig.startsWith("+")) signePlusImpose = true;
  	          }
  	          if ((num == 0) && chsig.endsWith("0")) return " ";
	          	double g = Kernel.pgcd(numabs, denabs);
	          	double num1 = numabs/g;
	          	double den1 = denabs/g;
	          	boolean positif = (num*den >= 0);
	          	/* Modifi version 4.8.0 pour viter test d'galit en virgule flottante
	          	if (den1 == 1) {
	          		if ((num1 == 1) && chsig.endsWith("1")) {
	          			if (positif) return  signePlusImpose ? " + " : " ";
	          			else return " - ";
	          		}
	          		res = Kernel.chaineNombre(num1, 0);
	          	}
	          	*/
	          	if (Kernel.zero13(den1 - 1)) {
	          		if (Kernel.zero13(num1 - 1) && chsig.endsWith("1")) {
	          			if (positif) return  signePlusImpose ? " + " : " ";
	          			else return " - ";
	          		}
	          		res = Kernel.chaineNombre(num1, 0);
	          	}

	          	else res = " \\frac{" + Kernel.chaineNombre(num1, 0) + "}{" + Kernel.chaineNombre(den1, 0) + "} ";
	          	if (positif && signePlusImpose) res = " +" + res;
	          	
	          	else if (!positif) res = " -" + res;
	          	return res;
	          }
		      }
        }
      }
     }
    return res;
	}
	
	private final String traiteCodeIf(String[] param) {
    int indfin = listeValeursDynamiquesUtilisees.size() - 1;
    // Modifi version 4.7.2
    // CValeurDynamique pValeurNum = latex.listeValeursDynamiquesUtilisees.pointeurValeurOuFonction(nomvaleur,indfin);
    CValeurDynamique pValeurNum = listeValeursDynamiquesUtilisees.pointeurParNatureCalcul
    		(NatureCalcul.NTouteValeurPourCommentaireDynamiqueSaufFonction, param[0], indfin);
    if (pValeurNum == null) return errorIf;
    if (pValeurNum.existe) {
    	/*
    	 * Ajout version 4.7.2
    	 */
    	if (pValeurNum.getNatureCalcul() == NatureCalcul.NCalculComplexe) {
    		Complexe z =new Complexe();
    		((CCalculComplexe) pValeurNum).rendValeurComplexe(z);
    		if ((z.x == 1) && (z.y == 0)) return param[1]; else return param[2];
    	}
    	//
    	else {
    		if (pValeurNum.estDeNatureCalcul(NatureCalcul.NTouteValeurReellePourCommentaireDynamique)) {
	        double num = pValeurNum.rendValeur();
	        if (num == 1) return param[1]; else return param[2];
    		}
    		else return param[2];
    	}
    }
    // Modifi version 4.7.9. Si la valeur n'existe pas on rennvoie le deuxime paramtre comme si le test tait faux
    // else return errorIf;
    else return param[2];
	}
	
	private final String traiteCodeFor(String param) {
    int indfin = listeValeursDynamiquesUtilisees.size() - 1;
    CCalcul calcul = (CCalcul) listeValeursDynamiquesUtilisees.pointeurParNatureCalcul
    		(NatureCalcul.NTouteValeurPourCommentaireDynamique, param.toString(),indfin);
    if (calcul != null) {
    	String[] varFor = null;
  		if (calcul.estDeNatureCalcul(NatureCalcul.NFonction | NatureCalcul.NFonctionComplexe)) {
  			varFor = new String[1];
  			varFor[0] = ((CFonc) calcul).chaineVariable;
  		}
  		else {
    		if (calcul.estDeNatureCalcul(NatureCalcul.NFonction2Var | NatureCalcul.NFonction3Var
    				| NatureCalcul.NFonctionComplexe2Var | NatureCalcul.NFonctionComplexe3Var)) {
    			CFoncNVar fnv = (CFoncNVar) calcul;
    			int nbv = fnv.nbVar;
    			varFor = new String[nbv];
    			for (int k = 0; k < nbv; k++) varFor[k] = fnv.chaineVariable[k];
    		}            			
  		}
  		return calcul.calcul.chaineLatexSansPar(varFor);          		
  	}
  	else return errorFor;

	}
	
	private final String traiteCodeCalc(String param) {
    int indfin = listeValeursDynamiquesUtilisees.size() - 1;
    CCalcul calcul = (CCalcul) listeValeursDynamiquesUtilisees.pointeurParNatureCalcul
    		(NatureCalcul.NTouteValeurPourCommentaireDynamique, param, indfin);
    if (calcul != null) {
    	String[] varFor = null;
  		if (calcul.estDeNatureCalcul(NatureCalcul.NFonction | NatureCalcul.NFonctionComplexe)) {
  			varFor = new String[1];
  			varFor[0] = ((CFonc) calcul).chaineVariable;
  		}
  		else {
    		if (calcul.estDeNatureCalcul(NatureCalcul.NFonction2Var | NatureCalcul.NFonction3Var
    				| NatureCalcul.NFonctionComplexe2Var | NatureCalcul.NFonctionComplexe3Var)) {
    			CFoncNVar fnv = (CFoncNVar) calcul;
    			int nbv = fnv.nbVar;
    			varFor = new String[nbv];
    			for (int k = 0; k < nbv; k++) varFor[k] = fnv.chaineVariable[k];
    		}            			
  		}
  		String chret = "\\text{" + calcul.calcul.chaineCalculSansPar(varFor) + "}";
  		// Il faut remplacer les caractres ^ par \^{}
  		StringBuffer stb = new StringBuffer(chret.length()+256);
  		stb.append(chret);
  		for (int i = 0; i < stb.length(); i++) {
  			if (stb.charAt(i) == '^') {
  				stb.replace(i, i+1, "\\;\\^{}\\;");
  				i = i + 8;
  			}
  		}
  		return stb.toString();
  	}
  	else return errorCalc;
	}
	
	private final String traiteCodeCalcSimp(String param) {
    int indfin = listeValeursDynamiquesUtilisees.size() - 1;
    CCalcul calcul = (CCalcul) listeValeursDynamiquesUtilisees.pointeurParNatureCalcul
    		(NatureCalcul.NTouteValeurPourCommentaireDynamique, param.toString(),indfin);
    if (calcul != null) {
    	String[] varFor = null;
  		if (calcul.estDeNatureCalcul(NatureCalcul.NFonction | NatureCalcul.NFonctionComplexe)) {
  			varFor = new String[1];
  			varFor[0] = ((CFonc) calcul).chaineVariable;
  		}
  		else {
    		if (calcul.estDeNatureCalcul(NatureCalcul.NFonction2Var | NatureCalcul.NFonction3Var
    				| NatureCalcul.NFonctionComplexe2Var | NatureCalcul.NFonctionComplexe3Var)) {
    			CFoncNVar fnv = (CFoncNVar) calcul;
    			int nbv = fnv.nbVar;
    			varFor = new String[nbv];
    			for (int k = 0; k < nbv; k++) varFor[k] = fnv.chaineVariable[k];
    		}            			
  		}
  		String chret = "\\text{" + calcul.calcul.calculAvecValeursRemplacees().
  				calculNormalise(true, false, false).chaineCalculSansPar(varFor) + "}";          		
  		// Il faut remplacer les caractres ^ par \^{}
  		StringBuffer stb = new StringBuffer(chret.length()+256);
  		stb.append(chret);
  		for (int i = 0; i < stb.length(); i++) {
  			if (stb.charAt(i) == '^') {
  				stb.replace(i, i+1, "\\;\\^{}\\;");
  				i = i + 8;
  			}
  		}
  		return stb.toString();
  	}
  	else return errorCalcSimp;
	}
	
	private final String traiteCodeForSimp(String param) {
    int indfin = listeValeursDynamiquesUtilisees.size() - 1;
    CCalcul calcul = (CCalcul) listeValeursDynamiquesUtilisees.pointeurParNatureCalcul
    		(NatureCalcul.NTouteValeurPourCommentaireDynamique, param.toString(),indfin);
    if (calcul != null) {
    	String[] varFor = null;
  		if (calcul.estDeNatureCalcul(NatureCalcul.NFonction | NatureCalcul.NFonctionComplexe)) {
  			varFor = new String[1];
  			varFor[0] = ((CFonc) calcul).chaineVariable;
  		}
  		else {
    		if (calcul.estDeNatureCalcul(NatureCalcul.NFonction2Var | NatureCalcul.NFonction3Var
    				| NatureCalcul.NFonctionComplexe2Var | NatureCalcul.NFonctionComplexe3Var)) {
    			CFoncNVar fnv = (CFoncNVar) calcul;
    			int nbv = fnv.nbVar;
    			varFor = new String[nbv];
    			for (int k = 0; k < nbv; k++) varFor[k] = fnv.chaineVariable[k];
    		}            			
  		}
  		return calcul.calcul.calculAvecValeursRemplacees().calculNormalise(true, false, false).chaineLatexSansPar(varFor);          		
  	}
  	else return errorForSimp;
	}
	
  /**
   * Ajout version 4.8.1
   */
  @Override
  public String tikz(DimensionFenetre dimf, boolean nomaff, double coefmult, boolean bu) {
  	// String st = FonteNom.traiteFractionsTickz(chaineLatex);
  	String style = "";
  	switch (alignementVertical) {
  		case alignementVerticalBas :
  			style = "above ";
  			break;
  		case alignementVerticalHaut :
  			style = "below ";
  	}
  	switch (alignementHorizontal) {
  		case alignementHorizontalGauche :
  			style += "right";
  			break;
  		case alignementHorizontalDroit :
  			style += "left,";
  	}
  	if (!style.isEmpty()) style += ",";
  	style += tikzCouleur() + ",";
  	if (encadrement != StyleEncadrement.Sans) style +=  "draw="+ tikzCouleur() + ",";
  	if (effacementFond) style += "fill=" + TikzColor.colorString(couleurFondAffichage) +",";
  	double dinsepx = listeProprietaire.tikzLongueur(2, dimf, bu)*coefmult;
  	double dinsepy = listeProprietaire.tikzLongueur(4, dimf, bu)*coefmult;
  	String insepx = Kernel.chaineNombre(dinsepx,2);
  	String insepy = Kernel.chaineNombre(dinsepy,2);
  	style += "inner xsep ="  + insepx + "cm,inner ysep =" + insepy + "cm";
    //int k = FonteNom.taille(indiceTailleNom, 1);
    //double d = listeProprietaire.tickzLongueur(k, dimf, bu);
    //double x1 = listeProprietaire.tickzXcoord(x_nom+dec_x+8, dimf, bu) - d;
  	double x1 = listeProprietaire.tikzXcoord(x_nom+dec_x-2, dimf, bu) - dinsepx;// - d;
    // double y = (pointLie == null) ? y_nom+dec_y-6 : y_nom+dec_y-2; // Ajustements ...
    double y1 = listeProprietaire.tikzYcoord(y_nom+dec_y-2, dimf, bu);// - d*0.8;
    // if ((effacementFond) || (encadrement != StyleEncadrement.Sans)) ch += tickzRect(dimf);
    return  "\\node at " + "(" + Kernel.chaineNombre(x1, 3) +", " + Kernel.chaineNombre(y1, 3) +
    		") [align=left," + style + ",font= \\sf " + tikzFont(dimf, indiceTaillePolice, coefmult, bu) + "] {$" + chaineLatex + "$};";
  }
	
}
