/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;

import java.util.*;


import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.Kernel;

import mathgraph32.MtgFrame;

/**
 *
 * @author Yves
 */
public class CMesureAire extends CCalculAncetre{
	private CPolygone polygoneAssocie;
  private double aire;
  
  /** Creates a new instance of CMesureAire */
  public CMesureAire() {
  }
  
  // Constructeur pour clonage
  public CMesureAire(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      CPolygone polygoneAssocieInit, String nomCalculInit) {
    // Inutile d'appeler lengthconstructeur anctre vide
    super(liste, impProtoInit, estElementFinalInit,  nomCalculInit);
    polygoneAssocie = polygoneAssocieInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(polygoneAssocie);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb =  new CMesureAire(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, (CPolygone)(listeCible.get(ind1, CPolygone.class)), nomCalcul);
    return ptelb;
  }
  
  @Override
  public void setClone(CElementBase ptel) {
    aire = ((CMesureAire) ptel).aire;
  }

  /**
   * Fonction renvoyant la chane  afficher dans la bote de dialogue d'historique
   * pour dcrire l'objet.
   * @return
   */
  @Override
  public String infoPourHistorique() {
    String ch =  nom() + " : " + getString("chinfo71") + " " + getString("chinfo72") + " " + polygoneAssocie.nomSommetsPolygone();
    if (existe) ch = ch + "\n" + getString("chinfo97") + " " + Kernel.chaineNombre(aire, 16);
    else ch = ch + "\n" + getString("ch18");
    return ch;    
  }  
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
    return nom() + " : " + getString("chinfo71") + " " + getString("chinfo72") + " " + polygoneAssocie.nomSommetsPolygone();
	}
  
  /**
   * Fonction ajoutant  list les lments qui ont gnr l'objet.
   * @param liste
   */
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(polygoneAssocie);
  }  
  
  @Override
  public boolean nomIndispensable(CElementGraphique el) {
  	for (int i =0; i < polygoneAssocie.collectionPoints.size(); i++) {
  		CNoeudPointeurSurPoint nd = (CNoeudPointeurSurPoint) polygoneAssocie.collectionPoints.get(i);
  		if (nd.pointeurSurPoint == el) return true;
  	}
  	return false;
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NMesureAirePolygone;
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || polygoneAssocie.dependDe(p)
    || listeProprietaire.pointeurLongueurUnite.dependDe(p));
  }
  
  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec( super.dependDePourReclassement(p) || polygoneAssocie.dependDePourReclassement(p)
    || listeProprietaire.pointeurLongueurUnite.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || polygoneAssocie.dependDePourBoucle(p)
    || listeProprietaire.pointeurLongueurUnite.dependDePourBoucle(p);
  }

  @Override
	public
  double rendValeur() {
    return aire;
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    CLongueur pLongUnite = listeProprietaire.pointeurLongueurUnite;
		existe = polygoneAssocie.existe && pLongUnite.existe;
    if (!existe) return;
    double longueurUnite = pLongUnite.rendLongueur();
    // Si la longueur unit est nulle, pLongUnite n'existe pas
    aire = 0;
    Vector<CNoeudPointeurSurPoint> array = polygoneAssocie.collectionPoints;
    for (int i = 0; i < array.size() - 1; i++ ) {
      CPointAncetre ptp1 = (CPointAncetre)((CNoeudPointeurSurPoint) array.get((i))).pointeurSurPoint;
      CPointAncetre ptp2 = (CPointAncetre)((CNoeudPointeurSurPoint) array.get((i+1))).pointeurSurPoint;
      double x1 = ptp1.x;
      double y1 = ptp1.y;
      double x2 = ptp2.x;
      double y2 = ptp2.y;
      aire += (x1*y2 - x2*y1)/2;
    }
    aire = Math.abs(aire)/(longueurUnite*longueurUnite);
  }
  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
  **/
  public boolean confonduAvec(CElementBase p) {
    CMesureAire pt;
    if (p.getClass() == this.getClass()) {
      pt = (CMesureAire) p;
      return polygoneAssocie == pt.polygoneAssocie;
    }
    else return false;
  }    
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return listeOb.contains(polygoneAssocie) || polygoneAssocie.estDefiniParObjetDans(listeOb);
	}    
 
  @Override
  public boolean utiliseLongueurUnite() {
  	return true;
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    polygoneAssocie = (CPolygone) (list.get(ind1,CPolygone.class));
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    polygoneAssocie = (CPolygone) (list.get(ind1,CPolygone.class));
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(polygoneAssocie);
    oups.writeInt(ind1);
  }  
}

