/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;


// Contrairement  la version C++, il est inutile de dfinir getNature pour chaque objet
// car cela ne sert que pour les objets dsignales, les macros
// La nature NPoint qui est la nature du point construit sera la nature des points dfinie
// dans CPointAncetre. Il ne sera redfini que pour les points mobiles (pojnts libres et points lis).
import java.awt.*;
import java.io.*;
import java.awt.geom.*;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

// Modification pour la version 2.5 : la classe n'est plus abstract car il
// faut pouvoir l'instancier dans CListeObjets.get(int indice, Class cla)
// Modifi version 5.0 : Deux styles supplmentaires.
/**
 * Classe anctre de tous les points
 */
public class CPointAncetre extends CElementGraphique {
	// Ajout version 4.9.9 pour gnrer un nom
	public static int indice = 0;
	//
  public double x, y;
  public int motif;
  public boolean horsEcran;
  public boolean dansFenetre; // Rajout version 4.6.4 // Pass public version 4.9.9
  // Modification pour la version 4.4. Le numero de version de document passe  8 et chaque point peut tre marqu ou non pour la trace
  public boolean marquePourTrace;

  /** Creates new CPointAncetre */
  protected CPointAncetre() {
  }
  
  protected CPointAncetre(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
    boolean masqueInit, String nomInit, int indiceTailleNomInit, int motifInit, boolean marquePourTraceInit) {
      super (liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
        dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit);
      motif = motifInit;
      marquePourTrace = marquePourTraceInit;
  }

	/**
	 * Ajout version 5.2 (numro de version 16). Renverra true si l'objet possde un nom qui doit tre enregistr dans le flux
	 * @return
	 */
  @Override
	public boolean hasName() {
		return true;
	}

  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    x = ((CPointAncetre)ptel).x;
    y = ((CPointAncetre)ptel).y;
    horsEcran = ((CPointAncetre)ptel).horsEcran;
    dansFenetre = ((CPointAncetre)ptel).dansFenetre; // Ajout version 4.6.4
    marquePourTrace = ((CPointAncetre)ptel).marquePourTrace;
  }

  public boolean testToile() {
    return Kernel.testToile(x, y);
  }
  
  /**
   * Fonction testant si le point est dans la fentre de dimensions dimf
   * @param dimf
   * @return
   */
  @Override
  public boolean dansFenetre(DimensionFenetre dimf) {
  	return testFenetre(dimf, x, y);
  }
  
  // GetNature ne sera redfini que pour les points libres et points lis
  // Grosse diffrence par rapport au C++
  @Override
  public long getNature() {
    return NatureObjet.NPoint;
  }
  /**
   * Fonction testant si le point est dans la fentre de dimensions dimf
   * @param dimf
   * @return
   */
  boolean testFenetre(DimensionFenetre dimfen, double x, double y)
  {
     return dimfen.dansFenetre(x, y);
  }

  @Override
  public void affiche(Graphics2D pDC, boolean masquage, Color couleurFond, float transparence) {
  	// Modifi version 4.6.4
    // if (!(masquage && masque) && !horsEcran) trace(pDC, couleurFond, transparence);
  	if (!(masquage && masque) && dansFenetre) trace(pDC, couleurFond, transparence);
  }
  
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence)
  {
    int xint,yint,k0,k2,k3,k4,k5,k6,k8;

    if (horsEcran) return;
    xint = (int) Math.round(x);
    yint = (int) Math.round(y);
    // Modification version 5.0 pour augmenter la taille des points pour les fentres de haute dfinition
    double coefMult = (listeProprietaire != null) ? listeProprietaire.coefMult : 1;
    double coef = listeProprietaire.documentProprietaire.coefForPointsAndMarks;
    boolean hd = coef > 1.2;
    //
    // Modification version 5.0
    Stroke stroke = Pinceau.CreePinceau( new StyleTrait(StyleTrait.styleTraitContinu, 1), listeProprietaire);
    // Affichage d'une toile si le point est masqu
    if (masque) {
      pDC.setStroke(stroke);
      pDC.setColor(couleur);
      k3 = hd ? (int)(coefMult*5) : (int)(coefMult*3);
      pDC.drawLine(xint-k3, yint, xint+k3, yint);
      pDC.drawLine(xint-k3, yint-k3, xint+k3, yint+k3);        
      pDC.drawLine(xint-k3, yint+k3, xint+k3, yint-k3);
      pDC.drawLine(xint, yint-k3, xint, yint+k3);
    }
    else {
	    switch (motif)
	    {
		    case MotifPoint.Carre:
		    	k2 = (int)(coefMult*2);
		    	k4 = (int)(coefMult*4);
		      pDC.setStroke(stroke);
		      pDC.setColor(couleur);
		      pDC.fillRect(xint-k2,yint-k2,k4,k4);
		      break;
		    // Modification version 5.0.1 : Pour le rond moyen on ne tient pas compte de la taille de la figure
		    case MotifPoint.Rond:
		    		k3 = (int)(coefMult*3);
		    		k6 = (int)(coefMult*6);
		        pDC.setStroke(stroke);
		        pDC.setColor(couleur);
		        pDC.drawOval(xint-k3,yint-k3,k6,k6);
		        pDC.fillOval(xint-k3,yint-k3,k6,k6);
		        break;
	      case MotifPoint.Croix:
		      	k3 = hd ? (int)(coefMult*5) : (int)(coefMult*3);
	          pDC.setStroke(stroke);
	          pDC.setColor(couleur);
	          pDC.drawLine(xint-k3, yint, xint+k3, yint);
	          pDC.drawLine(xint, yint-k3, xint, yint+k3);
	         break;
	      case MotifPoint.Mult:
		      	k3 = hd ? (int)(coefMult*5) : (int)(coefMult*3);
	          pDC.setStroke(stroke);
	          pDC.setColor(couleur);
	          pDC.drawLine(xint-k3, yint-k3, xint+k3, yint+k3);
	          pDC.drawLine(xint-k3, yint+k3, xint+k3, yint-k3);
	         break;
	      // Pour le petit rond on n'e tient pas compte de la taille de la figure
	      case MotifPoint.PetitRond:
	    		k2 = (int)(coefMult*2);
	    		k4 = (int)(coefMult*4);
          pDC.setStroke(stroke);
          pDC.setColor(couleur);
          pDC.fillOval(xint-k2,yint-k2,k4,k4);
          pDC.drawOval(xint-k2,yint-k2,k4,k4);
          break;
	      case MotifPoint.Losange:
	      		k3 = hd ? (int)(coefMult*5) : (int)(coefMult*3);
	          int[] abscisses = new int[4];
	          int[] ordonnees = new int[4];
	          pDC.setStroke(stroke);
	          pDC.setColor(couleur);
	          abscisses[0] = xint;
	          // 1 pixel de plus par rapport au C++ car seul l'intrieur du
	          //est rempli
	          ordonnees[0] = yint - k3;
	          abscisses[1] = xint - k3;
	          ordonnees[1] = yint;
	          abscisses[2] = xint;
	          ordonnees[2] = yint + k3;
	          abscisses[3] = xint + k3;
	          ordonnees[3] = yint;
	          pDC.fillPolygon(abscisses, ordonnees, 4);
	          break;
	      	case MotifPoint.Pixel:
	          pDC.setStroke(stroke);
	          pDC.setColor(couleur);
		      	/* Modifi version 5.0.1
		      	k0 = (int)(k);
		      	if (k0 == 1) {
		          pDC.drawLine(xint-1,yint,xint+1,yint);
		          pDC.drawLine(xint,yint-1,xint,yint+1);		      		
		      	}
		      	else {
			      	k2 = k0*2;
			        pDC.drawOval(xint-k0,yint-k0,k2,k2);
			        pDC.fillOval(xint-k0,yint-k0,k2,k2);
		      	}
		      	*/
	          /* Modifi version 5.1
		      		k0 = hd ? (int)(coefMult*2) : (int)(coefMult);
		          pDC.drawLine(xint-k0,yint,xint+k0,yint);
		          pDC.drawLine(xint,yint-k0,xint,yint+k0);		      		
		        break;
		        */         
	      		k0 = hd ? (int)(coefMult*2) : (int)(coefMult);
	      		k2 = 2*k0;
	          pDC.drawOval(xint-k0,yint-k0,k2,k2);
	          pDC.fillOval(xint-k0,yint-k0,k2,k2);
	          break;
	          // Ajouts version 5.0. On tient compte de la taille de la figure pour qu'ils soient plus gros sur les crans THD
		      case MotifPoint.GrandRond:
		      	k4 = hd ? (int)(coefMult*6) : (int)(coefMult*4);
		      	k8 = hd ? (int)(coefMult*12) : (int)(coefMult*8);
	          pDC.setStroke(stroke);
	          pDC.setColor(couleur);
	          pDC.drawOval(xint-k4,yint-k4,k8,k8);
	          pDC.fillOval(xint-k4,yint-k4,k8,k8);
	          break;
		      case MotifPoint.GrandMult:
			      k5 = hd ? (int)(coefMult*7) : (int)(coefMult*5);
	          pDC.setStroke(stroke);
	          pDC.setColor(couleur);
	          pDC.drawLine(xint-k5, yint-k5, xint+k5, yint+k5);
	          pDC.drawLine(xint-k5, yint+k5, xint+k5, yint-k5);
	          break;
	        // Ajout version 5.1
		      case MotifPoint.Quadrillage:
	      		k0 = hd ? (int)(coefMult*2) : (int)(coefMult);
	          pDC.drawLine(xint-k0,yint,xint+k0,yint);
	          pDC.drawLine(xint,yint-k0,xint,yint+k0);
	          break;
	    } // switch
    }
  }
  
  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
    if (!(existe) || (masquage && masque))
    return -1;
    else
    {
      VecteurDirecteur u = new VecteurDirecteur(x,y,xp,yp);
      return u.norme();
    }
  }

  @Override
  public void afficheNom(Graphics2D pDC) {
    if (existe && !horsEcran) {
      super.afficheNom(pDC);
    }
  }

  public void positionneNom()
  {
    placeNom(x,y);
  }
  
  @Override  
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    horsEcran = !(testToile());
    dansFenetre = testFenetre(dimfen, x, y); // Ajout version 4.6.4
    positionneNom();
  }
  public void donneMotif(int mot)
  {
    motif = mot;
  }
  public void placeEn(double xn, double yn) {
    existe = true;
    x = xn;
    y = yn;
    placeNom(xn, yn);
    // Diffrence par rapport au C++ : on ne peut pas appeler positionne de CPointAncetre
    horsEcran = !(testToile());
    positionneNom();
  }

  // Java : la procdure suivante a t rajoute en java
  public void placeEn(CPointAncetre point) {
    existe = point.existe;
    x = point.x;
    y = point.y;
    placeNom(x, y);
    // Diffrence par rapport au C++ : on ne peut pas appeler positionne de CPointAncetre
    horsEcran = !(testToile());
    positionneNom();
  }

  // La procdure suivante test si lengthdplacement du point correspondant  la position
  // (xtest, ytest)  de la souris est possible et si oui renvoyant dans pointr les
  // nouvelles coordonnes et dans pointAbs.x l'abscisse pour un point li

  public boolean testDeplacement(DimensionFenetre dimfen, double xtest, double ytest,
        Point2D.Double pointr, Doub abscr)
  {
    pointr.x = xtest;
    pointr.y = ytest;
    return testFenetre(dimfen, xtest, ytest);
  }
  
  /**
   * Fonction renvoyant true si le point est sur la droite (ou descendant de droite)
   * @param droite
   * @return
   */
  public boolean surDroite(CDroiteAncetre droite) {
    if (!(existe)) return false;
    VecteurDirecteur vec = new VecteurDirecteur();
    vec.x = x - droite.point_x;
    vec.y = y - droite.point_y;
    return (Kernel.colineaires(droite.vecteur,vec) && droite.appartientA(x,y));
  }
    
  public boolean egal(CPointAncetre pt) {
    return ((this.x == pt.x) && (this.y == pt.y));
  }

  public boolean presqueEgal(CPointAncetre A) {
   return (Kernel.zero(x - A.x) && (Kernel.zero(y - A.y)));
  }

  // A revoir . Inutile
  public final void tourne(double angle, VecteurDirecteur v)
  {
    double kc,ks;

    kc = Math.cos(angle);
    ks = Math.sin(angle);
    v.x = kc*x + ks*y;
    v.y = kc*y - ks*x;
  }
    
  @Override
  public boolean coincideAvec(CElementBase p) {
    if ((p.getNature() & NatureObjet.NToutPoint) == 0) return false;
    CPointAncetre pt = (CPointAncetre) p;
    return (Kernel.zero(pt.x-x) && Kernel.zero(pt.y-y));
  }

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    motif = inps.readByte();
    marquePourTrace = false;
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    motif = inps.readByte();
    if (listeProprietaire.numeroVersion < 8) marquePourTrace = false;
    else marquePourTrace = inps.readBoolean();
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeByte(motif);
    oups.writeBoolean(marquePourTrace);
  }
  
  /** Fonction ajoute pour la version 2.1 en vue d'amliorer l'outil de cration d'intersection
   * Renvoie true si le point est sur la droite par dfinition
   * A redfinir pour les descendants concerns
   * */
  public boolean appartientParDefinition(CDroiteAncetre droite)
  {
    return droite.contientParDefinition(this);
  }
  /** Fonction ajoute pour la version 2.1 en vue d'amliorer l'outil de cration d'intersection
   * Renvoie true si le point est sur la droite par dfinition
   * A redfinir pour les descendants concerns
   * */
  public boolean appartientParDefinition(CCercle cercle)
  {
    return cercle.contientParDefinition(this);
  }
      
  @Override
  public String chaineDesignation() {
    return getString("designationPoint");
  }
  
  /** A partir de la version 1.2.0, il est souvent ncessaire de connatre si un point est un point image par une transformation
   * Ne pouvant pas sans problme rajouter un nouveau type s'objet graphique, on cre cette fonction redfinie seulement
   * pour la classe CPointImage
   * @return
   */
  public boolean estPointImage() {
  	return false;
  }
  
  /**
   * Ajout version 4.8.1
   */
  @Override
  public String tikz(DimensionFenetre dimf, boolean nomaff, double coefmult, boolean bu) {
  	double x1,x2,x3,y1,y2,y3,ray,k2,k3,k5;
  	String ch = "";
  	if (masque || !dansFenetre) return "";
    // Modification version 5.0 pour augmenter la taille des points pour les fentres de haite dfinition
    double k = listeProprietaire.documentProprietaire.coefForPointsAndMarks;
    boolean hd = k > 1.2;
    // Version 5.0.1 : Si on rduit la figure, on rduit la taiile des poinst mais si on l'agrandit on ne la change pas
    double coef = (coefmult >= 1) ? 1 : coefmult;
    //
    switch (motif)
    {
      case MotifPoint.Carre:
      	k2 = (int)(coef*2);
    		x1 = listeProprietaire.tikzXcoord(x-k2, dimf, bu);
    		y1 = listeProprietaire.tikzYcoord(y-k2, dimf, bu);
    		x2 = listeProprietaire.tikzXcoord(x+k2, dimf, bu);
    		y2 = listeProprietaire.tikzYcoord(y+k2, dimf, bu);
        ch = "\\fill[color=" + tikzCouleur() + "] (" + 
    		Kernel.chaineNombre(x1,3) + "," + Kernel.chaineNombre(y1,3) + ") rectangle ("
        + Kernel.chaineNombre(x2,3) + "," + Kernel.chaineNombre(y2,3) + ");";
        break;
      case MotifPoint.Rond:
    		ray = listeProprietaire.tikzLongueur(3, dimf, bu);
    		x1 = listeProprietaire.tikzXcoord(x, dimf, bu);
    		y1 = listeProprietaire.tikzYcoord(y, dimf, bu);
        ch =  "\\fill[color=" + tikzCouleur() + "] (" + 
        		Kernel.chaineNombre(x1,3) + "," + Kernel.chaineNombre(y1,3) + ") circle (" +
        		Kernel.chaineNombre(ray,3) + ");";
        break;
      case MotifPoint.PetitRond:
    		ray = listeProprietaire.tikzLongueur(2, dimf, bu);
    		x1 = listeProprietaire.tikzXcoord(x, dimf, bu);
    		y1 = listeProprietaire.tikzYcoord(y, dimf, bu);
        ch = "\\fill[color=" + tikzCouleur() + "] (" + 
        		Kernel.chaineNombre(x1,3) + "," + Kernel.chaineNombre(y1,3) + ") circle (" +
        		Kernel.chaineNombre(ray,3) + ");";
        break;
      case MotifPoint.Croix:
      		k3 = hd ? (int)(coef*5) : (int)(coef*3);
      		ray = listeProprietaire.tikzLongueur(1, dimf, bu)*coefmult;
      		ch = "";
      		x1 = listeProprietaire.tikzXcoord(x-k3, dimf, bu);
      		y1 = listeProprietaire.tikzYcoord(y, dimf, bu);
      		x2 = listeProprietaire.tikzXcoord(x+k3, dimf, bu);
      		x3 = listeProprietaire.tikzXcoord(x, dimf, bu); 		
      		y2 = listeProprietaire.tikzYcoord(y-k3, dimf, bu);
      		y3 = listeProprietaire.tikzYcoord(y+k3, dimf, bu);

      		ch = "\\draw [line width = " + Kernel.chaineNombre(ray, 3) +"cm, color = " + tikzCouleur() + "] (" + Kernel.chaineNombre(x1, 3) + "," 
      		+ Kernel.chaineNombre(y1, 3) +")--(" + Kernel.chaineNombre(x2, 3) + "," + Kernel.chaineNombre(y1, 3) + ");" + System.lineSeparator();
      		ch += "\\draw [line width = "  + Kernel.chaineNombre(ray, 3) +"cm, color = " + tikzCouleur() + "] (" + Kernel.chaineNombre(x3, 3) + ","
      		+ Kernel.chaineNombre(y2, 3) +")--(" + Kernel.chaineNombre(x3, 3) + "," + Kernel.chaineNombre(y3, 3) +");";
      		break;
      case MotifPoint.Mult:
      	k3 = hd ? (int)(coef*5) : (int)(coef*3);
    		ray = listeProprietaire.tikzLongueur(1, dimf, bu)*coefmult;
    		ch = "";
    		x1 = listeProprietaire.tikzXcoord(x-k3, dimf, bu);
    		y1 = listeProprietaire.tikzYcoord(y-k3, dimf, bu);
    		x2 = listeProprietaire.tikzXcoord(x+k3, dimf, bu);
    		y2 = listeProprietaire.tikzYcoord(y+k3, dimf, bu);

    		ch = "\\draw [line width = " + Kernel.chaineNombre(ray, 3) +"cm, color = " + tikzCouleur() + "] (" + Kernel.chaineNombre(x1, 3) + "," 
    		+ Kernel.chaineNombre(y1, 3) +")--(" + Kernel.chaineNombre(x2, 3) + "," + Kernel.chaineNombre(y2, 3) + ");" + System.lineSeparator();
    		ch += "\\draw [line width = "  + Kernel.chaineNombre(ray, 3) +"cm, color = " + tikzCouleur() + "] (" + Kernel.chaineNombre(x1, 3) + ","
    		+ Kernel.chaineNombre(y2, 3) +")--(" + Kernel.chaineNombre(x2, 3) + "," + Kernel.chaineNombre(y1, 3) +");";
          break;
      case MotifPoint.Losange:
    		ch = "";
    		k3 = hd ? (int)(coef*5) : (int)(coef*3);
    		x1 = listeProprietaire.tikzXcoord(x-k3, dimf, bu);
    		y1 = listeProprietaire.tikzYcoord(y-k3, dimf, bu);
    		x2 = listeProprietaire.tikzXcoord(x+k3, dimf, bu);
    		y2 = listeProprietaire.tikzYcoord(y+k3, dimf, bu);
    		x3 = listeProprietaire.tikzXcoord(x, dimf, bu);
    		y3 = listeProprietaire.tikzYcoord(y, dimf, bu);
    		ch = "\\fill [color = " + tikzCouleur() + "] (" + Kernel.chaineNombre(x1, 3) + "," 
        		+ Kernel.chaineNombre(y3, 3) +")--(" + Kernel.chaineNombre(x3, 3) + "," + Kernel.chaineNombre(y1, 3) +
        		")--(" + Kernel.chaineNombre(x2, 3) + "," + Kernel.chaineNombre(y3, 3) +
        		")--(" + Kernel.chaineNombre(x3, 3) + "," + Kernel.chaineNombre(y2, 3) + ")--cycle;";
        break;
     case MotifPoint.Pixel:
   			ray = listeProprietaire.tikzLongueur(hd ? 2 : 1, dimf, bu);
   			x1 = listeProprietaire.tikzXcoord(x, dimf, bu);
   			y1 = listeProprietaire.tikzYcoord(y, dimf, bu);
   			ch = "\\fill[color=" + tikzCouleur() + "] (" + 
       		Kernel.chaineNombre(x1,3) + "," + Kernel.chaineNombre(y1,3) + ") circle (" +
       		Kernel.chaineNombre(ray,3) + ");";
   		break;
    	 
   	 // Ajout version 5.0. Deux motifs suppmentaires
     case MotifPoint.GrandRond:
   		ray = listeProprietaire.tikzLongueur(hd ? (int)(6*coef) : (int) (4*coef), dimf, bu);
   		x1 = listeProprietaire.tikzXcoord(x, dimf, bu);
   		y1 = listeProprietaire.tikzYcoord(y, dimf, bu);
       ch =  "\\fill[color=" + tikzCouleur() + "] (" + 
       		Kernel.chaineNombre(x1,3) + "," + Kernel.chaineNombre(y1,3) + ") circle (" +
       		Kernel.chaineNombre(ray,3) + ");";
       break;
     case MotifPoint.GrandMult:
   		ray = listeProprietaire.tikzLongueur(1, dimf, bu);
   		ch = "";
   		k5 = hd ? (int)(coef*7) : (int)(coef*5);
   		x1 = listeProprietaire.tikzXcoord(x-k5, dimf, bu);
   		y1 = listeProprietaire.tikzYcoord(y-k5, dimf, bu);
   		x2 = listeProprietaire.tikzXcoord(x+k5, dimf, bu);
   		y2 = listeProprietaire.tikzYcoord(y+k5, dimf, bu);

   		ch = "\\draw [line width = " + Kernel.chaineNombre(ray, 3) +"cm, color = " + tikzCouleur() + "] (" + Kernel.chaineNombre(x1, 3) + "," 
   		+ Kernel.chaineNombre(y1, 3) +")--(" + Kernel.chaineNombre(x2, 3) + "," + Kernel.chaineNombre(y2, 3) + ");" + System.lineSeparator();
   		ch += "\\draw [line width = "  + Kernel.chaineNombre(ray, 3) +"cm, color = " + tikzCouleur() + "] (" + Kernel.chaineNombre(x1, 3) + ","
   		+ Kernel.chaineNombre(y2, 3) +")--(" + Kernel.chaineNombre(x2, 3) + "," + Kernel.chaineNombre(y1, 3) +");";
      break;

    } // switch
    // On affiche le nom s'il n'est pas masqu et visible et si nomaff est true
    if (nomaff) {
	    if (nomMasque || nom.isEmpty()) return ch;
	    ch = ch + System.lineSeparator();
	    // int k = FonteNom.taille(indiceTailleNom, 1); Modifi version 5.0
	    int kn = indiceTailleNom;
	    double taille;
	    if (kn <= 14) taille = 0.7*kn; 
	    else if (kn <= 30) taille = 0.2*kn;
	    else taille = 0.1*kn;
	    x1 = listeProprietaire.tikzXcoord(x_nom+dec_x-taille/coefmult, dimf, bu);
	    y1 = listeProprietaire.tikzYcoord(y_nom+dec_y+kn, dimf, bu);
	    ch = ch + "\\draw (" + Kernel.chaineNombre(x1, 3) +", " + Kernel.chaineNombre(y1, 3) +
	    		") node [inner sep = 0pt, outer sep = 0pt, above right" + "," + tikzCouleur() + "] {" +
	    		tikzFont(dimf, indiceTailleNom, coefmult, bu) + "{" + tikzNom() + "}};";
    }
    return ch;
  }
  
  // Ajout version 4.9.9
  @Override
  public String genereNom() {
  	return listeProprietaire.genereNomPourPoint(true);
  }
  public String genereNom(String ch) {
  	indice++;
  	return ch + indice;
  }
}