/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;
import java.io.*;

import mathgraph32.MtgFrame;
import dialogs.MtgDlg;
import dialogs.SuiteRecurrenteComplexeDlg;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;


/**
 *
 * @author yves.biton
 */
public class CSuiteRecComplexe extends CCalculAncetre {
  // public CFoncComplexe fonctionAssociee; // Modification version 4.7.5
	public CCalcul fonctionAssociee;
  public CValeurAssocieeAVariable nombreTermes;
  int nbt;
  public CValeurAssocieeComp premierTerme;
  int indiceDernierTermeExistant;
  Complexe[] valeurs; // Pointeur sur un tableau contenant les valeurs
  // des termes successifs
  // Attributs crs pour optimisation des calculs
  Complexe u = new Complexe();
  Complexe v = new Complexe();
  // static int NombreMaxiTermesSuiteRec = 2000; // Modifi version 5.1
  public static int NombreMaxiTermesSuiteRec = CSuiteRec.NombreMaxiTermesSuiteRec;
 
  public CSuiteRecComplexe() {
  }
  
  /**
   * Constructeur pour cration d'objets
   * @param liste
   * @param nombreTermesInit
   * @param premierTermeInit
   */
  public CSuiteRecComplexe(CListeObjets liste, CValeurAssocieeAVariable nombreTermesInit, CValeurAssocieeComp premierTermeInit) {
  	super(liste, null, false, "");
  	fonctionAssociee = null;
    nombreTermes = nombreTermesInit;
    premierTerme = premierTermeInit;
    metAJourNombreTermes(); 	
  }
  
  // Constructeur pour clonage
  public CSuiteRecComplexe(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    String nomCalculInit, CCalcul fonctionAssocieeInit,
    CValeurAssocieeAVariable nombreTermesInit, CValeurAssocieeComp premierTermeInit) {
    super(liste, impProtoInit, estElementFinalInit, nomCalculInit);
    fonctionAssociee = fonctionAssocieeInit;
    nombreTermes = nombreTermesInit;
    premierTerme = premierTermeInit;
    metAJourNombreTermes();
  }
  
  final void metAJourNombreTermes() {
    double val = nombreTermes.rendValeur();
  	/* Modifi version 5.1
    if ((val == Math.floor(val)) && (val >= 1) && (val < NombreMaxiTermesSuiteRec)) nbt = (int) val;
    */
    if ((val == Math.floor(val)) && (val >= 1) && (val <= NombreMaxiTermesSuiteRec)) nbt = (int) val;  	
    else nbt = 2; // Pas d'importance car dans ce cas la suite n'existe pas
    valeurs = new Complexe[nbt];
    for (int i =0; i < nbt; i++) valeurs[i] = new Complexe();
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(fonctionAssociee);
    int ind2 = listeSource.indexOf(impProto);
    CValeurAssocieeComp premierTermeClone =
      (CValeurAssocieeComp) premierTerme.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable nombreTermesClone =
      (CValeurAssocieeAVariable) nombreTermes.getClone(listeSource, listeCible);
    CElementBase ptelb = new CSuiteRecComplexe(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, nomCalcul, (CFoncComplexe)(listeCible.get(ind1, CFoncComplexe.class)), nombreTermesClone, premierTermeClone);
    return ptelb;
  }
  
  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    indiceDernierTermeExistant = ((CSuiteRecComplexe) ptel).indiceDernierTermeExistant;
    // Les cinq lignes suivantes rajoutes version 3.3.0.4 pour corriger un bug de l'applet
    // qui faisait que les suites rcurrentes avec nombre d'objets dynamiques provoquaient
    // un indexOutOfBoundException
    int nbt = ((CSuiteRecComplexe) ptel).valeurs.length;
    if (valeurs.length != nbt) {
    	valeurs = new Complexe[nbt];
      for (int i =0; i < nbt; i++) valeurs[i] = new Complexe();
    }
    for (int i = 0; i <= indiceDernierTermeExistant; i++) {
      valeurs[i].set(((CSuiteRecComplexe) ptel).valeurs[i]);
    }
  }
  
  // Ajout version 3.1.1 pour corriger un bug
  public int nombreVariables() {
  	return 1;
  }  

  // Ajout version 4.8
  @Override
  public void initialisePourDependance() {
  	super.initialisePourDependance();
  	premierTerme.initialisePourDependance();
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || premierTerme.dependDe(p) || fonctionAssociee.dependDe(p) || nombreTermes.dependDe(p));
  }
  
  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || premierTerme.dependDePourReclassement(p)
    	|| fonctionAssociee.dependDePourReclassement(p) || nombreTermes.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || premierTerme.dependDePourBoucle(p) || fonctionAssociee.dependDePourBoucle(p)
    || nombreTermes.dependDePourBoucle(p);
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    premierTerme.positionne(infoRandom, dimfen);
    nombreTermes.positionne(infoRandom, dimfen);
    existe = fonctionAssociee.existe && premierTerme.existe() && nombreTermes.existe();
    if (!existe) return;
    double dnbt = Math.floor(nombreTermes.rendValeur() + 0.5);
    existe = (dnbt >= 1) && (dnbt <= NombreMaxiTermesSuiteRec);
    if (!existe) return;
    if ((int) dnbt != nbt) {
      nbt = (int) dnbt;
      metAJourNombreTermes();
    }    
    // Modifi version 3.1.2 pour autoriser la remise  jour des valeurs renvoyes par rand
    // premierTerme.rendValeurComplexe(infoRandom, u);
    premierTerme.rendValeurComplexe(u);
    valeurs[0].set(u);
    indiceDernierTermeExistant = 0;
    int i = 1;
    while (i < nbt) {
      try {
        listeProprietaire.initialiseNombreIterations();
        // fonctionAssociee.calcul.resultatFonctionComplexe(infoRandom, u, v);
        fonctionAssociee.calcul.resultatFonctionComplexe(true, u, v);
        if (v.isNanOrInfinite())  break;
        u.set(v);
        valeurs[i].set(u);
        i++;
      } catch (ErreurCalculException e) {
        break;
      }
    }
    i--;
    indiceDernierTermeExistant = i;
  }
  
  @Override
	public boolean confonduAvec(CElementBase p) {
    CSuiteRecComplexe pt;
    if (p.getClass() == this.getClass()) {
      pt = (CSuiteRecComplexe) p;
      return (fonctionAssociee == pt.fonctionAssociee) && (premierTerme.confonduAvec(pt.premierTerme)
      		&& (nombreTermes.confonduAvec(pt.nombreTermes)));
    }
    else return false;
	}
 
  // Modifi version 4.9.9
  @Override
  public String infoPourHistorique() {
  	String ch = nomCalcul + " : " + getString("chinfo87") + "\n" + " " + getString("chinfo84") + " " + fonctionAssociee.nom() + "\n" +
  	getString("chinfo85") + " " + premierTerme.chaineInfo() + "\n" +
  	getString("chinfo86") + " " + nombreTermes.chaineInfo();
    if (!existe) ch = ch + "\n" + getString("ch18");
    return ch;
  }
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	return nomCalcul + " : " + getString("chinfo87") + " " + getString("chinfo84") + " " + fonctionAssociee.nom();
	}
  
  
  void rendValeurFonctionComplexe(boolean infoRandom, Complexe parametre, Complexe zRes)
  throws ErreurCalculException {
    if (!existe || (parametre.y != 0) || (parametre.x != Math.floor(parametre.x))
    || (parametre.x < 0) || (parametre.x > indiceDernierTermeExistant)) {
      throw new ErreurCalculException();
    } else zRes.set(valeurs[(int) parametre.x]);
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(fonctionAssociee) || fonctionAssociee.estDefiniParObjetDans(listeOb))
		&&
		(premierTerme.estDefiniParObjetDans(listeOb))
		&&
		(nombreTermes.estDefiniParObjetDans(listeOb));
	}  
    
  /**
   * Pour savoir si le calcul est une fonction relle, une suite relle ou non
   * Redfini seulement pour CFonc, CFoncComplexe, CSuiteRec et CSuiteRecComplexe;
   */ 
  public boolean estFonctionOuSuite() {
    return true;
  }
  
  @Override
  public boolean modifiableParMenu() {
  	// Modification version 3.5
  	// return true;
  	return !estElementFinal;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new SuiteRecurrenteComplexeDlg(frame, this, true);
  }  
  
   @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps,list);
    nombreTermes = new CValeurAssocieeAVariable();
    if (nVersion == 1) {
      int nbtermes = inps.readCInt();
      nombreTermes.donneValeur(nbtermes);
    }
    else nombreTermes.chargeDepuisFlux(inps, list);
    premierTerme = new CValeurAssocieeComp();
    premierTerme.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    fonctionAssociee = (CFoncComplexe) (list.get(ind1,CFoncComplexe.class));
    // metAJourNombreTermes();
    // Le nombre de valeurs tant dynamique, on ne peut pas le connatre au chargement
    // On met 1 terme arbitrairement et cela sera rectifi dans positionne();
    nbt = 1;
    valeurs = new Complexe[nbt];
    nVersion = numeroVersion(); // On remet tout  zro pour la version java
  }

   @Override
   public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
  	 super.read(inps, list);
  	 nombreTermes = new CValeurAssocieeAVariable();
  	 nombreTermes.read(inps, list);
  	 premierTerme = new CValeurAssocieeComp();
  	 premierTerme.read(inps, list);  	 
  	 int ind1 = inps.readInt();
  	 // fonctionAssociee = (CFoncComplexe) (list.get(ind1,CFoncComplexe.class)); // Modifi version 4.7.5
  	 fonctionAssociee = (CCalcul) (list.get(ind1,CFoncComplexe.class));
  	 // metAJourNombreTermes();
  	 // Le nombre de valeurs tant dynamique, on ne peut pas le connatre au chargement
  	 // On met 1 terme arbitrairement et cela sera rectifi dans positionne();
  	 nbt = 1;
  	 valeurs = new Complexe[nbt];
   }
  
   @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    nombreTermes.write(oups, list);
    premierTerme.write(oups, list);  	 
    int ind1 = list.indexOf(fonctionAssociee);
    oups.writeInt(ind1);
  }
     
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NSuiteRecurrenteComplexe;
  }
  
}