/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.*;
import java.io.*;

import types.*;

import entreesSorties.MathGraphDataInputStream;

import mathgraph32.MtgFrame;

public class CSurfaceLieu extends CSurface {
  // Il faut deux tableaux pour y transfrer les coordonnes des points
  int[] abscisses;
  int[] ordonnees;
  // Pour le trac
  private int nbPointsLigne;  
  /** Creates new CSurfaceLieu */
  public CSurfaceLieu() {
  }

  // Constructeur pour clonage
   public CSurfaceLieu(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
     Color couleurInit, boolean masqueInit, int styleRemplissageInit, CLieuDeBase bordInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleRemplissageInit, bordInit);
    metAJourTableaux();
   }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(bord);
    int ind2 = listeSource.indexOf(impProto);        
    return new CSurfaceLieu(listeCible, (CImplementationProto)listeCible.get(ind2, CImplementationProto.class),
      estElementFinal, couleur, masque, styleRemplissage, (CLieuDeBase)listeCible.get(ind1, CLieuDeBase.class));
  }
  
  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    if (ptel.existe) {
	    nbPointsLigne = ((CSurfaceLieu)ptel).nbPointsLigne;
	    System.arraycopy(((CSurfaceLieu)ptel).abscisses, 0, abscisses, 0, nbPointsLigne);
	    System.arraycopy(((CSurfaceLieu)ptel).ordonnees, 0, ordonnees, 0, nbPointsLigne);
    }
  }
  
  /**
   * Fonction appele quand on change le nombre de points du lieu qui est le bord de la surface
   */
  @Override
  public void metAJourTableaux() {
    abscisses = new int[((CLieuDeBase)bord).infoLieu.nombreDePoints + 1];
    ordonnees = new int[((CLieuDeBase)bord).infoLieu.nombreDePoints + 1];  	
  }
      
  @Override
  public boolean aPourBord(CLieuDeBase lieu) {
  	return (lieu == bord);
  }  
  
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	return nom() + " : " + getString("chinfo60") + " " + bord.nom();
  }

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(bord);
  }   
  
  /**
   * A t profondment chang pour la version 3.4.0 pour que quand le lieu de points est form de plusieurs
   * lignes on remplisse la surface forme de tous les points du plan existant mis les uns  la suite des autres.
   */
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    CLieuDeBase lieu;
    lieu = (CLieuDeBase) bord;

    super.positionne(infoRandom, dimfen);
    existe = existe && lieu.contientAuMoinsUneLigne();
    if (existe) {
    	nbPointsLigne = 0;
      for (int j = 0; j < lieu.nombreLignes; j++) {
        int nombrePoints = lieu.infoLignes[j].nombrePoints;
        // Optimis version 3.9.
        int indPremPoint = lieu.infoLignes[j].indicePremierPoint;
        // Modifi version 3.9. On prend aussi les ventuels points isols
        //if (nombrePoints >= 2) {
          for (int k = 0; k < nombrePoints; k++) {
          	int k1 = k + nbPointsLigne;
            abscisses[k1] = lieu.abscissesCoordonneesPoints[indPremPoint + k];
            ordonnees[k1] = lieu.ordonneesCoordonneesPoints[indPremPoint + k];
          }
          nbPointsLigne += nombrePoints;
      //  }
      }
    }
  }
  
  /* Version vant 3.4.0
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    CLieuDeBase lieu;
    lieu = (CLieuDeBase) bord;

    super.positionne(infoRandom, dimfen);
    existe = existe && lieu.contientUneSeuleLigne();
    if (existe) {
      int indicePremiereLigneLieu = lieu.indicePremiereLigne;
      int indicePremierPointLigne = lieu.infoLignes[indicePremiereLigneLieu].indicePremierPoint;
      nbPointsLigne = lieu.infoLignes[indicePremiereLigneLieu].nombrePoints;
      System.arraycopy(lieu.abscissesCoordonneesPoints, indicePremierPointLigne,
        abscisses, 0, nbPointsLigne);
      System.arraycopy(lieu.ordonneesCoordonneesPoints, indicePremierPointLigne,
        ordonnees, 0, nbPointsLigne);
    }
  }
  */  

  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    int styleRemplissageEffectif;
    if (masque) styleRemplissageEffectif = StyleRemplissage.QuadrillageCroise;
    else styleRemplissageEffectif = styleRemplissage;
    if (styleRemplissageEffectif == StyleRemplissage.QuadrillagePleinTransp) {
    	if (transparence != 1) {
    		pDC.setColor(couleur);
  	    Composite originalComposite = pDC.getComposite();
  	    pDC.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, transparence));
  	    pDC.fillPolygon(abscisses, ordonnees, nbPointsLigne);
  	    pDC.setComposite(originalComposite);    		
    	}
    	else {
    		pDC.setColor(couleur);
    		pDC.fillPolygon(abscisses, ordonnees, nbPointsLigne);
    	}
    }
    else {
      if (styleRemplissageEffectif == StyleRemplissage.QuadrillagePlein) {
    		pDC.setColor(couleur);
    		pDC.fillPolygon(abscisses, ordonnees, nbPointsLigne);
      }
      else {
	    	TexturePaint tp = StyleRemplissage.getTexturePaint(couleur, styleRemplissageEffectif, listeProprietaire, couleurFond);
	    	pDC.setPaint(tp);
	    	pDC.fillPolygon(abscisses, ordonnees, nbPointsLigne);
      }
    } 
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (list.getClass() != CPrototype.class) {
      abscisses = new int[((CLieuDeBase)bord).infoLieu.nombreDePoints + 1];
      ordonnees = new int[((CLieuDeBase)bord).infoLieu.nombreDePoints + 1];
    }
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
  	super.read(inps, list);
    if (list.getClass() != CPrototype.class) {
      abscisses = new int[((CLieuDeBase)bord).infoLieu.nombreDePoints + 1];
      ordonnees = new int[((CLieuDeBase)bord).infoLieu.nombreDePoints + 1];
    }
  }
  
  /**
   * Ajout version 4.8.1
   */
  @Override
  public String tikz(DimensionFenetre dimf, boolean nomaff, double coefmult, boolean bu) {
  	CLieuDeBase lieu = (CLieuDeBase) bord;
  	String ch = lieu.tikzOnePath(dimf, nomaff, coefmult, bu);
  	return "\\fill" + tikzFillStyle() + ch + ";";
  }

}
