/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.geom.*;
import java.io.*;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CSymetrieCentrale extends CTransformation {
	CPointAncetre centre;

  /** Creates new CHomothetie */
  public CSymetrieCentrale() {
  }
  
  @Override
	public int natureTransformation() {
		return symetrieCentrale;
	}
  
  // Constructeur pour clonage
  public CSymetrieCentrale(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    CPointAncetre centreInit) {
    super(liste, impProtoInit, estElementFinalInit);
    centre = centreInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(centre);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CSymetrieCentrale(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)));
    return ptelb;
  }

  // Ajout version 4.9.9.
	/**
	 * Fonction renvoyant complment d'info pour liste totale d'objets
	 * @return
	 */
  @Override
	String complementInfo() {
		return getString("chinfo21") + " " + getString("chinfo202") + " " + centre.nom();
	}


  @Override
  void AjouteElementsVisuelsPourHist(CListeObjets liste) {
    liste.add(centre);
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || centre.dependDe(p));
  }

  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || centre.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || centre.dependDePourBoucle(p));
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = centre.existe;
  }
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CSymetrieCentrale pt;
    if (p.getClass() == this.getClass()) {
      pt = (CSymetrieCentrale) p;
      return centre.confonduAvec(pt.centre);
    } else return false;
  }

  @Override
  public double transformeLongueur(double L) {
    return L;
  }

  @Override
  public void transformePoint(double x, double y, Point2D.Double pointr) {
    pointr.x = 2*centre.x - x;
    pointr.y = 2*centre.y - y;
  }

  @Override
  public void transformeVecteur(VecteurDirecteur u, VecteurDirecteur ur) {
    ur.x = - u.x;
    ur.y = - u.y;
  }
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (centre == ancienPoint) centre = nouveauPoint;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return listeOb.contains(centre) || centre.estDefiniParObjetDans(listeOb);
	}  	  
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    centre = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    centre = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(centre);
    oups.writeInt(ind1);
  }      
}