/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;


import java.awt.Color;


import types.*;

import dialogs.AffichageValeurDlg;
import dialogs.MtgDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

import mathgraph32.MtgFrame;


public class CValeurAffichee extends CAffichageLieAPoint {
	// Ajout version 4.9.9 pour gnrer un nom
	public static int indice = 0;
	//
	public CValeurDynamique valeurAssociee;
	public String enTete;
	public String postChaine;
	public int nombreDecimales;

  /** Creates new CValeurAffichee */
  public CValeurAffichee() {
  }
  
  // Constructeur pour clonage
  public CValeurAffichee(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit,double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit, boolean masqueInit,
      CPointAncetre pointLieInit, int indiceTaillePoliceInit, int encadrementInit,
      boolean effacementFondInit, Color couleurFondInit, CValeurDynamique valeurAssocieeInit, String enTeteInit,
      String postChaineInit, int nombreDecimalesInit, int alignementHorizontalInit, int alignementVerticalInit) {
        super (liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
          dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit,
          encadrementInit, effacementFondInit, couleurFondInit, alignementHorizontalInit, alignementVerticalInit);
        valeurAssociee = valeurAssocieeInit;
        enTete = enTeteInit;
        postChaine = postChaineInit;
        nombreDecimales = nombreDecimalesInit;
  }
        
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(valeurAssociee);
    int ind2 = listeSource.indexOf(pointLie);
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CValeurAffichee(listeCible, (CImplementationProto)listeCible.get(ind3, CImplementationProto.class),
      estElementFinal, couleur,x_nom, y_nom, dec_x, dec_y, masque, (CPointAncetre)listeCible.get(ind2, CPointAncetre.class), 
      indiceTaillePolice, encadrement, effacementFond, couleurFondAffichage,
      (CValeurDynamique)listeCible.get(ind1, CValeurDynamique.class), enTete, postChaine, nombreDecimales,
      alignementHorizontal, alignementVertical);
    return ptelb;
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || valeurAssociee.dependDe(p));
  }

  // Modifie version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || valeurAssociee.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return super.dependDePourBoucle(p) || valeurAssociee.dependDePourBoucle(p);
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    super.positionne(infoRandom, dimfen);
    existe = existe && valeurAssociee.existe;
  }

  @Override
  public String rendChaineAffichage() {
    // Modification pour la version 2.0
    // On regarde s'il s'agit d'un affichage de complexe ou de rel.
    if ((valeurAssociee.getNatureCalcul() & NatureCalcul.NTouteValeurComplexe) != 0) {
      return enTete + valeurAssociee.rendChaineValeurComplexe(nombreDecimales) + postChaine;      
    }
    else {
      return enTete + valeurAssociee.rendChaineValeur(nombreDecimales) + postChaine;
    }
  }
  
  @Override
  public long getNature() {
    return NatureObjet.NValeurAffichee;
  }

  @Override
  public String infoPourHistorique() {
    String ch = nom() + " " + getString("chinfo13") + " " + valeurAssociee.nom();
    if (pointLie != null) ch = ch + "\n" + getString("chinfo49");
    return ch;
  }
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
    return nom() + " : " + getString("chinfo13") + " " + valeurAssociee.nom();
   }
  // Ajout version 4.9.9
  public String genereNom() {
  	indice++;
  	return getString("raffval") + indice;
  }
  
  @Override
  public String chaineDesignation() {
    return getString("designationAffichageValeur");
  }
  
  @Override
  public boolean modifiableParMenu() {
  	// Modification version 3.5 : les affichages de valeurs restent modifiables
  	// mme quand ce sont des objets finaux de construction
  	return true;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new AffichageValeurDlg(frame, this, true);
  }
  
  @Override
  public String informationModification() {
  	return getString("chaineOutil42");
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		super.estDefiniParObjetDans(listeOb)
		&&
		(listeOb.contains(valeurAssociee) || valeurAssociee.estDefiniParObjetDans(listeOb));
	}        

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    // Modification pour la version 1.9.5 : filtarge des caractres grecs    
    enTete = inps.readCStringAvecFiltrage();
    postChaine = inps.readCStringAvecFiltrage();
    nombreDecimales = inps.readByte();
    int ind1 = inps.readCInt();
    valeurAssociee = (CValeurDynamique) (list.get(ind1,CValeurDynamique.class));
    // Pour compatibilit d'affichage avec C++ (lengthdcalage ventuel est ngatif)
    int dec1 = Kernel.decalagePourCompatibilite(enTete);
    int dec2 = Kernel.decalagePourCompatibilite(postChaine);
    if (dec2 < dec1) dec1 = dec2;
    y_nom += dec1;
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    enTete = inps.readUTF();
    postChaine = inps.readUTF();
    nombreDecimales = inps.readByte();
    int ind1 = inps.readInt();
    valeurAssociee = (CValeurDynamique) (list.get(ind1,CValeurDynamique.class));
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeUTF(enTete);
    oups.writeUTF(postChaine);
    oups.writeByte(nombreDecimales);
    int ind1 = list.indexOf(valeurAssociee);
    oups.writeInt(ind1);
  }
  
}