/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.*;
import java.io.*;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.Kernel;

public class CVecteur extends CSegment {
  public int motifFleche;
  /** Creates new CVecteur */
  public CVecteur() {
  }
  
  // Constructeur pour clonage
  public CVecteur(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean masqueInit, StyleTrait styleInit,
      CPointAncetre point1Init, CPointAncetre point2Init, int motifFlecheInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit, point1Init, point2Init);
    motifFleche = motifFlecheInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(impProto);
    int ind2 = listeSource.indexOf(point1);
    int ind3 = listeSource.indexOf(point2);
    CElementBase ptelb = new CVecteur(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, (CPointAncetre) listeCible.get(ind2, CPointAncetre.class),
      (CPointAncetre) listeCible.get(ind3, CPointAncetre.class), motifFleche);
    return ptelb;
  }
  
  @Override
  public String complementInfo() {
    return getString("chinfo155");
  }
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo154") + " " + point1.nom() + point2.nom();
  }
  // Ajout version 4.9.9
 	/**
 	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
 	 * A redfinir pour les descendants.
 	 * @return
 	 */
   @Override
 	 public String info() {
     return getString("chinfo218") + "(" + point1.nom() + point2.nom() + ")";
   }
 
  @Override
  /**
   * A t modifi par rapport au C++
   */
	public boolean confonduAvec(CElementBase p) {
    CVecteur pt;
    if (p.getClass() == this.getClass()) {
      pt = (CVecteur) p;
  		return 
			(point1 == pt.point1) && (point2 == pt.point2);
   }
    else return false;
	}
  
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
  	if (horsFenetre) return;
    // Modification version 3.2.0
    // if (!(point1.horsEcran) && !(point2.horsEcran) && vecteur.nonNul())
    if (vecteur.nonNul())
    {
      super.trace(pDC, couleurFond, transparence);
      // StyleTrait styleTrait;
      // Modification version 5.0
      // if (masque) styleTrait = StyleTrait.traitPointille;
      // On trace la flche avec un style de trait continu
      // else styleTrait = style;
      StyleTrait styleTrait = new StyleTrait(StyleTrait.styleTraitContinu, style.epaisseur);
      pDC.setStroke(Pinceau.CreePinceau(styleTrait, listeProprietaire));
      pDC.setColor(couleur);
      // On ne trace la flche que si le vecteur n'est pas presque nul
      if (!vecteur.presqueNul()) {
      	StyleFleche.traceFleche(pDC,x1,y1,x2,y2,motifFleche, listeProprietaire);
      }
    }
  }
  
  @Override
  public long getNature() {
    return NatureObjet.NVecteur;
  }
  
  @Override
  public String chaineDesignation() {
    return getString("designationVecteur");
  }
  
  public void donneStyleFleche(int styleFleche) {
  	motifFleche = styleFleche;
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    motifFleche = inps.readByte();
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    motifFleche = inps.readByte();
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeByte(motifFleche);
  }
  
  /**
   * Ajout version 4.8.1
   * @param dimf
   * @return
   */
  public final String tikzFleche(DimensionFenetre dimf, double coefmult, boolean bu) {
    double[] abscisses = new double[3];
    double[] ordonnees = new double[3];
    VecteurDirecteur u0 = new VecteurDirecteur();
    VecteurDirecteur u1 = new VecteurDirecteur();
    VecteurDirecteur u2 = new VecteurDirecteur();
    double longueurFleche = StyleFleche.longeurFlechePourVecteur(motifFleche, listeProprietaire);

    u0.setVecteur(x2, y2, x1, y1);
    u0.vecteurColineaire(longueurFleche,u1);
    // On fait tourner le vecteur u1 de -pi/6 pour avoir u2
    u2.x = u1.x*Kernel.cos30 - u1.y*Kernel.sin30;
    u2.y = u1.x*Kernel.sin30 + u1.y*Kernel.cos30;
    abscisses[1] = x2;
    ordonnees[1] = y2;
    abscisses[0] = x2 + u2.x;
    ordonnees[0] = y2 + u2.y;
    // On fait tourner le vecteur u1 de pi/6 pour avoir u2
    u2.x = u1.x*Kernel.cos30 + u1.y*Kernel.sin30;
    u2.y = -u1.x*Kernel.sin30 + u1.y*Kernel.cos30;
    abscisses[2] = x2 + u2.x;
    ordonnees[2] = y2 + u2.y;
    switch (motifFleche)
    {
      case StyleFleche.FlecheLonguePleine:
      case StyleFleche.FlecheCourtePleine:
      	return "\\filldraw " + tikzdrawFillStyle(dimf, coefmult) + "(" +
      			Kernel.chaineNombre(listeProprietaire.tikzXcoord(abscisses[0], dimf, bu), 2) +
      			"," + Kernel.chaineNombre(listeProprietaire.tikzYcoord(ordonnees[0], dimf, bu), 3) + ")--(" +
      			Kernel.chaineNombre(listeProprietaire.tikzXcoord(abscisses[1], dimf, bu), 3) + "," +
      			Kernel.chaineNombre(listeProprietaire.tikzYcoord(ordonnees[1], dimf, bu), 3) + ")--(" +
      			Kernel.chaineNombre(listeProprietaire.tikzXcoord(abscisses[2], dimf, bu), 3) + "," +
      			Kernel.chaineNombre(listeProprietaire.tikzYcoord(ordonnees[2], dimf, bu), 3) + ")--cycle;";
      case StyleFleche.FlecheLongueSimple:
      case StyleFleche.FlecheCourteSimple:
      	return "\\draw " + tikzLineStyleContinu(dimf, coefmult) + "(" +
      			Kernel.chaineNombre(listeProprietaire.tikzXcoord(abscisses[0], dimf, bu), 2) +
      			"," + Kernel.chaineNombre(listeProprietaire.tikzYcoord(ordonnees[0], dimf, bu), 3) + ")--(" +
      			Kernel.chaineNombre(listeProprietaire.tikzXcoord(abscisses[1], dimf, bu), 3) + "," +
      			Kernel.chaineNombre(listeProprietaire.tikzYcoord(ordonnees[1], dimf, bu), 3) + ")--(" +
      			Kernel.chaineNombre(listeProprietaire.tikzXcoord(abscisses[2], dimf, bu), 3) + "," +
      			Kernel.chaineNombre(listeProprietaire.tikzYcoord(ordonnees[2], dimf, bu), 3) + ");";
      default : return "";
    }
  }

  /**
   * Ajout version 4.8.1
   */
  @Override
  public String tikz(DimensionFenetre dimf, boolean nomaff, double coefmult, boolean bu) {
  	if (horsFenetre || !vecteur.nonNul()) return "";
  	String ch = super.tikz(dimf, nomaff, coefmult, bu); // Code du segment
  	// Puis on rajoute le code de la flche
  	if (!vecteur.presqueNul()) ch += System.lineSeparator() + tikzFleche(dimf, coefmult, bu);
  	return ch;
  }

}
