/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;
import java.util.ListIterator;
import dialogs.ChoixMacroAvecListeDlg;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CElementBase;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CMacroAvecListe;
import objets.CSousListeObjets;
import types.NatureObjet;

public class OutilAjoutObjetMacro extends Outil {
  private CSousListeObjets sousListe;
  private CMacroAvecListe macro; // La macro laquelle on ajoute des objest

  public OutilAjoutObjetMacro(MtgFrame frameInit) {
    super(frameInit, true);
  }
    
  @Override
	public void select() {
    super.select();
    frame.indication(getString("ajoutObjetMacro1"));    
    ChoixMacroAvecListeDlg dlg = new ChoixMacroAvecListeDlg(frame);
		dlg.setLocationRelativeTo(frame);
    dlg.setVisible(true);
    if (!dlg.retourOK()) {
    	frame.activeOutilCapture();
    	return;
    }
    macro = dlg.macroChoisie;
  	sousListe = new CSousListeObjets();
    // on exclut de dsigner les objets dj traits par la macro
    ListIterator<CElementBase> li = macro.listeAssociee.listIterator();
    while (li.hasNext()) {
    	CElementGraphique elg = (CElementGraphique) li.next();
    	frame.listeExclusionDesignation.add(elg);
    	ajouteClignotementDe(elg);
    }
    frame.outilPointageActif = frame.outilPointageCre;
    frame.outilPointageActif.designationDe(NatureObjet.NToutObjet);
    frame.outilPointageActif.reset();
    frame.indication(getString("ajoutObjetMacro2"));
  }
	
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
  	CListeObjets liste = frame.cadreActif().listePr;
  	// Modifi version 4.7.4
  	// if (liste.indexOf(elg) > liste.indexOf(macro)) frame.dialogueAvertissement(getString("ch163"));
  	if (liste.indexOf(elg) >= liste.indexOf(macro)) frame.dialogueAvertissement(getString("ch163"));
  	else {
	  	frame.listeClignotante.add(elg);
	  	frame.listeExclusionDesignation.add(elg);
	  	sousListe.add(elg);
  	}
  }
	
  @Override
  public void creeObjetApresClicDroit() {
  	if (sousListe.size() != 0) {
  		MtgInternalFrame cadre = frame.cadreActif();
  		ListIterator<CElementBase> li = sousListe.listIterator();
  		while (li.hasNext()) {
  			CElementBase elb = li.next();
  			macro.listeAssociee.add(elb);
  		}
  		annuleClignotement();
		  cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil117"));  		
  	}
  	frame.activeOutilCapture();
  }
  
  @Override
  public boolean activationValide() {
    return frame.cadreActif().listePr.nombreMacrosAvecListe()!= 0;
  }
  
}
