/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 *
 * @author Yves
 * Outil anctre des outils de cration d'objets pas la cration successive de deux points
 */
package outils;

import java.awt.*;

import objets.*;
import types.*;
import mathgraph32.MtgFrame;

/**
 * Outil anctre de tous les outils devant attendre le clic sur deux points
 * Si les points doivent tre nomms, la fonction nomsPointsIndisp doit tre redfinie
 * @author Yves
 * Modifi version 5.0
 *
 */
public class OutilPar2pts extends Outil {
  CPointAncetre point1, point2;
  // objetCre servira dans le cas ou les points dsigns doivent tre renomms
  // Si l'objet n'a pas t cr, il faudra raffecter  point1 et point2 leurs
  // anciens noms au cas o ceux-ci auraient t changs.
  protected boolean objetCree;
  private boolean point1Nomme;
  private boolean point2Nomme;
  // Ajout version 5.0 : Si on cre des points par dfaut et qu'on abandonne la cration, on dtrit le premier point cr
  private boolean point1Cree;
  /** Creates a new instance of OutilPar2pts */
  public OutilPar2pts(MtgFrame frameInit) {
    super(frameInit, true);
  }
  
  /**
   * Les outils descendant de OutilPar2Pts et necessitant que les points dsigns
   * soient nomms s'ils n'ont pas de noms devront redfinir cette fonction en renvoyant true
   * Cela servira pour l'outil de mesure de longueur
   * @return true si les points dsigns doivent tre nomms
   */
  boolean nomsPointsIndisp() {
    return false;
  }
  
  /** Appel lorsque l'outil est slectionn */
  @Override
  public void select() {
    super.select();
    point1 = null;
    point2 = null;
    point1Nomme = false;
    point2Nomme = false;
    point1Cree = false; // Ajout version 5.0
    frame.outilPointageCre.designationDe(NatureObjet.NToutPoint);
    frame.outilPointageActif = frame.outilPointageCre;
    frame.indication(chaineIndication(1));
    objetCree = false;
  }
  
  @Override
  public void deselect() {
    if (nomsPointsIndisp() && !objetCree) {
      if ((point1 != null) && point1Nomme) point1.donneNom("");
      if ((point2 != null) && point2Nomme) point2.donneNom("");
    }
    // Ajout version 5.0
    if (!objetCree && (point1Cree)) frame.detruitDernierElement();
    //
    point1 = null;
    point2 = null;
    point1Nomme = false;
    point2Nomme = false;
    point1Cree = false;
    super.deselect();
  }
  
  /**
   * Action aprs dsignation du deuxime point
   * Ne fera rien et ne sera redfini qu pour l'outil de mesure d'ascisse
   */
  public void actionApresPoint2() {
    
  }
  
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (point1 == null) {
    	// Ajout version 5.0
    	if (elg == null) {
    		point1 = frame.ajoutePoint(pointClique.getX(), pointClique.getY(), false);
    		point1Cree = true;
    	}
    	//
    	else point1 = (CPointAncetre) elg;
      if (nomsPointsIndisp()) {
        if (point1.rendNom().length() == 0) {
          point1.donneNom(frame.cadreActif().listePr.genereNomPourPoint(false));
          point1Nomme =true;
        }
      }
      ajouteObjetsVisuels();
      /* Modifi version 5.0
      excluDeDesignation(elg);
      ajouteClignotementDe(elg);
      */
      excluDeDesignation(point1);
      ajouteClignotementDe(point1);
      resetClignotement();
      frame.indication(chaineIndication(2));
    }
    else {
      if (point2 == null) {
      	// Ajout version 5.0
      	if (elg == null) {
      		point2 = frame.ajoutePoint(pointClique.getX(), pointClique.getY(), false);
      	}
      	//
      	else point2 = (CPointAncetre) elg;
        if (nomsPointsIndisp()) { 
          if (point2.rendNom().length() == 0) {
            point2.donneNom(frame.cadreActif().listePr.genereNomPourPoint(false));
            point2Nomme = true;
          }
        }
        actionApresPoint2();
        point1Cree = false; // Ajout version 5.0
        creeObjet();
        annuleClignotement();
      }
    }
  }
}
