/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;

import dialogs.CoefficientDlg;
import mathgraph32.MtgFigurePanel;
import mathgraph32.MtgFrame;
import objets.CElementGraphique;
import objets.CHomothetie;
import objets.CListeObjets;
import objets.CPointAncetre;
import objets.CPointImage;
import objets.CPointParMultVect;
import objets.CTransParVect;
import objets.CValeurAssocieeAVariable;
import objets.CVecteur;
import types.Couleur;
import types.MotifPoint;
import types.NatureObjet;
import types.StyleFleche;
import types.StyleTrait;

public class OutilPointParMultVect extends Outil {
	private CVecteur vecteur1;
	private CPointAncetre point1;
  private CValeurAssocieeAVariable coef;
  private CPointParMultVect pointParMultVect;
	

  /** Creates a new instance of OutilPointParMultVec */
  public OutilPointParMultVect(MtgFrame frameInit) {
    super(frameInit, true);
  }
  
  @Override
  public void select() {
    super.select();
    vecteur1 = null;
    point1 = null;
    coef = null;
    frame.indication(getString("pointParMultVect1"));
    CListeObjets liste = frame.cadreActif().listePr;
    coef = new CValeurAssocieeAVariable(liste, 0);
    pointParMultVect = new CPointParMultVect(liste, null, false, frame.couleurActive(),
    		false, 0, 3, false, frame.genereNomPourPoint(), frame.pref_indiceTaillePoliceNom, frame.stylePointActif(), null, null, coef, false);
    CoefficientDlg dlg = new CoefficientDlg(frame, pointParMultVect, false);
		dlg.setLocationRelativeTo(frame);
    dlg.setVisible(true);
    if (dlg.retourOK()) {
    	pointParMultVect.coef = coef;
      frame.outilPointageCre.designationDe(NatureObjet.NVecteur);
      frame.outilPointageActif = frame.outilPointageCre;
      frame.outilPointageCre.reset();
      frame.indication(getString("pointParMultVect2"));
    }
    else frame.activeOutilCapture();
  }
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (vecteur1 == null) {
      vecteur1 = (CVecteur) elg;
      pointParMultVect.vecteur = vecteur1;
      ajouteClignotementDe(elg);
      frame.outilPointageCre.designationDe(NatureObjet.NToutPoint);
      ajouteObjetsVisuels();
      frame.indication(getString("pointParMultVect3"));
      resetClignotement();
    }
    else {
    	if (point1 == null) {
    		// annuleClignotement();
        point1 = (CPointAncetre) elg;
        pointParMultVect.antecedent = point1;
        creeObjet();
    	}
    }
  }   
	
  @Override
  public void creeObjet() {
    if (frame.ajouteElementAvecVerification(pointParMultVect)) {
      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil135"));
      frame.cadreActif().repaintFigureAvecCalcul();
    }
    coef = new CValeurAssocieeAVariable(coef); // POur que chaque point construit ait son propre multiplicateur de vecteur
    CListeObjets liste = frame.cadreActif().listePr;
    pointParMultVect = new CPointParMultVect(liste, null, false, frame.couleurActive(),
    		false, 0, 3, false, "", frame.pref_indiceTaillePoliceNom, frame.stylePointActif(), null, null, coef, false);
    pointParMultVect.vecteur = vecteur1;
    // annuleClignotement();
    point1 = null;
  }
  
  @Override
  public void ajouteObjetsVisuels() {
    CListeObjets liste = frame.listeObjetsVisuels;
    MtgFigurePanel pane = frame.cadreActif().paneFigure;
    
    CTransParVect trans1 = new CTransParVect(liste, null, false, vecteur1);
    frame.ajouteObjetVisuel(trans1);
    CPointImage ptim1 = new CPointImage(liste, null, false, Couleur.TraduitCouleur(1),
       MotifPoint.PetitRond, 2, pane.pointSouris, trans1);
    ptim1.cache();
    frame.ajouteObjetVisuel(ptim1);
    CHomothetie hom = new CHomothetie(liste, null, false, pane.pointSouris, pointParMultVect.coef);
    frame.ajouteObjetVisuel(hom);
    CPointImage ptim2 = new CPointImage(liste, null, false, Couleur.TraduitCouleur(1),
       MotifPoint.PetitRond, 2, ptim1, hom);
    frame.ajouteObjetVisuel(ptim2);
    CVecteur vect2 = new CVecteur(liste, null, false, vecteur1.couleur, false, StyleTrait.traitFin, pane.pointSouris, ptim2,
				StyleFleche.FlecheCourtePleine);
    frame.ajouteObjetVisuel(vect2);
  }
  
  @Override
	public boolean activationValide() {
		return frame.cadreActif().listePr.nombreObjetsParNatureVisibles(NatureObjet.NVecteur)>0;
	}
	
	@Override
	public String messageActivationNonValide() {
		return getString("messagePasDeVecteur");
	}	
  
}
