/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.*;

import objets.*;
import types.*;
import mathgraph32.MtgFigurePanel;
import mathgraph32.MtgFrame;

/**
 * Nouveau pour la version 1.6 : Possibilit de crer une image par translation
 * en cliquant sur un vecteur
 * @author Yves
 */
public class OutilTranslation extends Outil {
  private CPointAncetre point1;
  private CPointAncetre point2;
  private CVecteur vecteur;
  private CElementGraphique objet;
  private int nombreImagesCrees;
  CTransformation transfor;
  
  /** Creates a new instance of OutilTranslation */
  public OutilTranslation(MtgFrame frameInit) {
    super(frameInit, true);
  }


  /** Appel lorsque l'outil est slectionn */
  @Override
  public void select() {
    super.select();
    point1 = null;
    point2 = null;
    vecteur = null;
    objet = null;
    nombreImagesCrees = 0;
    frame.outilPointageCre.designationDe(NatureObjet.NToutPoint | NatureObjet.NVecteur);
    frame.outilPointageActif = frame.outilPointageCre;
    frame.indication(getString("translation1"));
  }
  
  /** On regarde si on n'a pas cr d'image. Dans ce cas il faut dtruire la translation
   * dans le cas o on a cliqu sur deux points
   */
  @Override
  public void deselect() {
    CListeObjets liste = frame.cadreActif().listePr;
    if (transfor == liste.get(liste.size() - 1)){
    	frame.detruitDernierElement();
    }
  	super.deselect();
  }
  
  @Override
  public void creeObjet() {
    CListeObjets liste = frame.cadreActif().listePr;
    if (nombreImagesCrees == 0) {
    	if (vecteur == null) {
    		CTranslation translation;
	    	translation = new CTranslation(liste, null, false, point1, point2);
	    	CTransformation trans = liste.transformationConfondueAvec(translation);
	    	if (trans != null) {
	    		translation = (CTranslation) trans;
	     	}
	    	else frame.ajouteElement(translation);
	    	transfor = translation;
    	}
    	else {
    		CTransParVect transVect;
	    	transVect = new CTransParVect(liste, null, false, vecteur);
	    	CTransformation trans = liste.transformationConfondueAvec(transVect);
	    	if (trans != null) {
	    		transVect = (CTransParVect) trans;
	     	}
	    	else frame.ajouteElement(transVect); 
	    	transfor = transVect;
    	}
    }
    Int nbImagesCrees = new Int();
    CElementGraphique elg = imageParTransformation(liste,  objet, transfor, nbImagesCrees);
    if (frame.ajouteElementsImageAvecVerification(elg, nbImagesCrees.getValue(), transfor)) {
    	nombreImagesCrees++;
    	if (nombreImagesCrees != 0) {
    		frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil19"));
    	}
  		frame.cadreActif().repaintFigureAvecCalcul();    	
    }
  }
  
  /** Mthode ajoutant les objets visuels  la liste listeObjetsVisuels de l'applet
   * au fur et  mesure que l'on dsigne les obejts  la souris
   * Doit tre redfini pour chaque outil utilisant une aide visuelle  la cration
   */
  @Override
  public void ajouteObjetsVisuels() {
    CListeObjets liste = frame.listeObjetsVisuels;
    MtgFigurePanel pane = frame.cadreActif().paneFigure;
    if (vecteur == null) {
	    CTranslation trans = new CTranslation(liste, null, false, point1, point2);
	    frame.ajouteObjetVisuel(trans);
	    CPointImage ptim = new CPointImage(liste, null, false, frame.couleurActive(),
	       MotifPoint.PetitRond, 13, pane.pointSouris, trans);
	    frame.ajouteObjetVisuel(ptim);
	    CVecteur vec = new CVecteur(frame.listeObjetsVisuels, null, false,
	      Couleur.TraduitCouleur(1), false, StyleTrait.traitPointille,
	      pane.pointSouris, ptim, StyleFleche.FlecheCourtePleine);
	    frame.ajouteObjetVisuel(vec);
    }
    else {
    	CTransParVect trans = new CTransParVect(liste, null, false, vecteur);
	    frame.ajouteObjetVisuel(trans);
	    CPointImage ptim = new CPointImage(liste, null, false, frame.couleurActive(),
	      MotifPoint.PetitRond, 13, pane.pointSouris, trans);
	    frame.ajouteObjetVisuel(ptim);
	    CVecteur vec = new CVecteur(frame.listeObjetsVisuels, null, false,
	      Couleur.TraduitCouleur(1), false, StyleTrait.traitPointille,
	      pane.pointSouris, ptim, StyleFleche.FlecheCourtePleine);
	    frame.ajouteObjetVisuel(vec);
    }
  }

  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
  	if ((vecteur != null) || ((point1 != null) && (point2 != null))) {
  		// Possibilit d'image multiples
    	// Cration d'images multiples possibles
      objet = elg;
      creeObjet();  		
  	}
  	else {
	  	if (elg.getNature() == NatureObjet.NVecteur) {
	  		if (vecteur == null) {
		      frame.indication(getString("translation3"));
		      excluDeDesignation(elg);
		  		vecteur = (CVecteur) elg;
	        ajouteObjetsVisuels();
	        ajouteClignotementDe(elg);
	        frame.outilPointageActif.designationDe(NatureObjet.NToutPoint | NatureObjet.NTouteDroiteSaufVecteur |
	                NatureObjet.NCercle | NatureObjet.NArc | NatureObjet.NPolygone
	                | NatureObjet.NLigneBrisee);
	      }
	  	}
	  	else {
		    if (point1 == null) {
		      frame.indication(getString("translation2"));
		      point1 = (CPointAncetre) elg;
		      ajouteClignotementDe(elg);
		      frame.outilPointageCre.designationDe(NatureObjet.NToutPoint);
		      resetClignotement();
		    }
		    else {
		      if (point2 == null) {
		        frame.indication(getString("translation3"));
		        point2 = (CPointAncetre) elg;
		        // Le vecteur servant aux clignotements
		        CVecteur vecteurClignotant = new CVecteur(frame.cadreActif().listePr, null, false,
		          Couleur.TraduitCouleur(2), false, StyleTrait.traitEpais, point1, point2,
		          StyleFleche.FlecheCourtePleine);
		        vecteurClignotant.positionne(false, frame.cadreActif().dimf);
		        ajouteObjetsVisuels();
		        ajouteClignotementDe(elg);
		        ajouteClignotementDe(vecteurClignotant);
		        frame.outilPointageActif.designationDe(NatureObjet.NToutPoint | NatureObjet.NTouteDroiteSaufVecteur |
		                NatureObjet.NCercle | NatureObjet.NArc | NatureObjet.NPolygone
		                | NatureObjet.NLigneBrisee);
		      }
		    }
		  }
	  }
  }
}