/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package types;

import glob.Kernel;

/**
 *
 * @author  Yves
 * @version 
 */
// Classe contenat des dimensions de fentre (quivalent des varaibels globales
// LongueurAbscisses et LongueurOrdonnees de la version C++
public class DimensionFenetre {
  public int longueurAbscisses, longueurOrdonnees;
  /** Creates new DimensionFenetre */
  public DimensionFenetre(int longueurAbscissesInit, int longueurOrdonneesInit) {
    longueurAbscisses = longueurAbscissesInit;
    longueurOrdonnees = longueurOrdonneesInit;
  }
  public final boolean dansFenetre(double xtest, double ytest) {
    return ((xtest >= 0) && (xtest <= longueurAbscisses) && (ytest >= 0) && (ytest <= longueurOrdonnees));
  }
  
  public void setCopy(DimensionFenetre dimfenSource) {
    longueurAbscisses = dimfenSource.longueurAbscisses;
    longueurOrdonnees = dimfenSource.longueurOrdonnees;
  }
  
  public void setSize(int width, int height) {
    longueurAbscisses = width;
    longueurOrdonnees = height;
  }
  
  public boolean testCercleDansFen(double cx, double cy, double rayon) {
    double d1 = Kernel.distancePointPoint(cx, cy, 0, 0);
    double d2 = Kernel.distancePointPoint(cx, cy, 0, longueurOrdonnees);
    double d3 = Kernel.distancePointPoint(cx, cy, longueurAbscisses, 0);
    double d4 = Kernel.distancePointPoint(cx, cy, longueurAbscisses, longueurOrdonnees);
    if ((rayon >=  d1) && (rayon >= d2) && (rayon >= d3) && (rayon >= d4)) return false;
    if (dansFenetre(cx, cy)) return true;
    if (cx>longueurAbscisses) {
      if (cy>longueurOrdonnees) return rayon >= d4;
      else if (cy<0) return rayon >= d3;
      else return rayon >= cx - longueurAbscisses;
    }
    else {
      if (cx<0) {
        if (cy>longueurOrdonnees) return rayon >= d2;
        else if (cy<0) return rayon >= d1;
        else return rayon >= -cx;
      }
      else {
        if (cy<0) return rayon>=-cy;
        else return rayon>=cy-longueurOrdonnees;
      }
    }

  }
      
}
