/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.util.*;
import java.awt.*;
import java.io.*;
import java.awt.geom.Point2D;
import outils.OutilCurseur;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import dialogs.EditeurNomCalcul;
import dialogs.PaneTaillePolice;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

@SuppressWarnings("serial")
public class CListeObjets extends ArrayList<CElementBase> {
	// Nouveaut pour la version 2.5 : Gestion d'un numro de version qui est
	// celui
	// du document qui a charg la liste lors d'un chargement et puis ensuite est
	// le numro
	// de version actuel du logiciel C++
	public int numeroVersion;
	// Dans certains cas on aura besoin de savoir quel est le document
	// propritaire de la liste,
	// en particulier lors du chargement des macros sonores et des images
	// Ce pointeur sera null par dfaut
	public CMathGrafDoc documentProprietaire;
	public int uniteAngle;
	public CLongueur pointeurLongueurUnite;
	// Version 5.0.2 : macroParDefaut plus utilis. A supprimer dans une version ultrieure
	public CMacro macroParDefaut = null; // Pointeur sur l'ventuelle macro d'animation pour JavaScript Modifi version 4.8
	public CMacro macroDemarrage = null; // Pointeur sur l'ventuelle macro de dmarrage (nouveau version 4.8)
	// Distance mini pour qu'un objet soit considr comme proche du pointeur
	// souris
	// Modifi version 5.0 pour tenir compe des crans haute rsolution
	// double distanceMiniPourEtreProche = 4;
	double distanceMiniPourEtreProche;
	// Modification  partir de la version java : quand on imprime
	// la figure il faut passer dans un systme de coordonnes logiques qui est
	// diffrent du systme de coordonnes cran
	// Chaque CListeObjets instanciera uun boolean basseDefinition qui contiendra
	// true si on est en rsolution
	// cran et false sinon
	public boolean basseDefinition;
	// Ajout version 4.6.4. Permet de connatre le coefficient multiplicateur en cas de haute dfinition
	public double coefMult = 1;
	// Ajout version 4.6.4. Permet de savoir si lors de l'affichage en haute rsolution le coeffcient multiplicateur
	// doit tre attnu (divis par 2) pour les paisseurs de traits.
	public boolean bAttenuationEpaisseurTrait = false;
	// Ajout version 4.7.0.3. Permet de savoir si lors du chargement de la liste via un flux,
	// les CEditeurFormule doivent recrer leurs composants MtgFormulaEditor
	public boolean bCreationComposants = true;
	// Ajout version 3.2.8 pour lutter contre les dpassements de capacit dans les sommes indices
	// et les calculs d'intgrales
	// Initialis  zro avant tout calcul par initialisenombreIterations()
	public long nombreIterations;

	// Macro  excuter avant copiage ou impression. null par dfaut
	/** Creates new CListeObjets */
	public CListeObjets() {
		super(20);
		numeroVersion = MathGraphDataInputStream.numeroVersionActif;
		documentProprietaire = null;
		basseDefinition = true;
		pointeurLongueurUnite = null;
		// Ajout version 5.0
		distanceMiniPourEtreProche = 4*MtgFrame.coefMultForDlgBoxIcons;
	}

	public CListeObjets(int uniteAngleInit, CLongueur pointeurLongueurUniteInit) {
		super(20);
		numeroVersion = MathGraphDataInputStream.numeroVersionActif;
		pointeurLongueurUnite = pointeurLongueurUniteInit;
		uniteAngle = uniteAngleInit;
		documentProprietaire = null;
		basseDefinition = true;
		// Ajout version 5.0
		distanceMiniPourEtreProche = 4*MtgFrame.coefMultForDlgBoxIcons;
	}

	public CListeObjets(CMathGrafDoc documentProprietaireInit) {
		super(20);
		documentProprietaire = documentProprietaireInit;
		numeroVersion = MathGraphDataInputStream.numeroVersionActif;
		basseDefinition = true;
		pointeurLongueurUnite = null;
		// Ajout version 5.0
		distanceMiniPourEtreProche = 4*MtgFrame.coefMultForDlgBoxIcons;
	}

	// Constructeur avec la mme unit de longueur et la mme longueur unit que
	// list
	public CListeObjets(CListeObjets list) {
		super(20);
		numeroVersion = MathGraphDataInputStream.numeroVersionActif; // Correction bug version 4.1.1.0
		documentProprietaire = list.documentProprietaire;
		pointeurLongueurUnite = list.pointeurLongueurUnite;
		uniteAngle = list.uniteAngle;
		basseDefinition = true;
		// Ajout version 5.0
		distanceMiniPourEtreProche = 4*MtgFrame.coefMultForDlgBoxIcons;
	}
	
	/**
	 * Ajout version 5.0 : Associe cette liste au document doc;
	 * @param doc
	 */
	public void associeA(CMathGrafDoc doc) {
		documentProprietaire = doc;
	}


	// Met  jour l'unit d'angle et la macro par dfaut aprs cration
	public final void initialise(int uniteAngleInit, CLongueur pointeurLongueurUniteInit) {
		pointeurLongueurUnite = pointeurLongueurUniteInit;
		uniteAngle = uniteAngleInit;
	}

	/** Mthode renvoyant true si au moins un des objets de la liste dpend de ptel */
	public final boolean dependDe(CElementBase ptel) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if(elb.dependDe(ptel)) return true;
		}
		return false;
	}

	/** Mthode renvoyant true si au moins un des objets de la liste dpend de ptel */
	public final boolean dependDePourReclassement(CElementBase ptel) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if(elb.dependDePourReclassement(ptel)) return true;
		}
		return false;
	}

	public final boolean dependDePourBoucle(CElementBase ptel) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if(elb.dependDePourBoucle(ptel)) return true;
		}
		return false;
	}

	/**
	 * Dtermination des dpendances d'appel dynamiques dans les commentaires de
	 * la liste
	 */
	public void determineDependancesCommentaires() {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.estDeNature(NatureObjet.NCommentaireOuLatex))
				((CCommentaire) elb).determineDependances();
			// if (elb.getNature() == NatureObjet.NLatex) ((CLatex)elb).creeFormule(); // Supprim version 4.9.9
		}
	}

	// Mthode renvoyant true si et seult si au moins un des lments de la liste
	// existe
	public final boolean existeAuMoinsUnElement() {
		boolean resultat = false;
		ListIterator<CElementBase> li = listIterator();
		// Optimis version 3.4.0.1
		// while (li.hasNext()) {
		while (li.hasNext() && ! resultat) {
			CElementBase elb = li.next();
			resultat = resultat || elb.existe;
		}
		return resultat;
	}

	/**
	 * Ajoute  la liste les lments de la liste list dpendant de l'objet ptel
	 */
	/*
	 * public void AjouteObjetsDependants(CElementBase ptel, CListeObjets list) {
	 * ListIterator<CElementBase> li = list.listIterator(); while (li.hasNext()) {
	 * CElementBase elb = li.next(); if (elb.dependDePourCapture(ptel)) {
	 * add(elb); } } }
	 */

	/**
	 * Fonction ajoutant les objets de la liste liste
	 */
	public final void ajouteElementsDe(CListeObjets liste) {
		ListIterator<CElementBase> li = liste.listIterator();
		while (li.hasNext())
			add(li.next());
	}

	public final void ajouteObjetsParNature(CListeObjets list, long nature) {
		ListIterator<CElementBase> li = list.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.estDeNature(nature))
				add(elb);
		}
	}
	
	// Ajout version 4.7.1
	public final void ajouteObjetsParNatureNonFinaux(CListeObjets list, long nature) {
		ListIterator<CElementBase> li = list.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (! elb.estElementFinal && elb.estDeNature(nature))
				add(elb);
		}
	}

	// Ajoute  la liste les lments de la liste list dpendant de l'objet ptel
	// sauf l'lment
	// exclu
	public final void ajouteObjetsDependantsSauf(CElementBase ptel, CListeObjets list, CElementBase exclu) {
		ListIterator<CElementBase> li = list.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if ((elb.dependDePourCapture(ptel)) && (elb != exclu)) {
				add(elb);
			}
		}
	}

	/**
	 * Fonction ajoutant les objets de la liste li qui sont des affichages lis 
	 * des points non masqus
	 * 
	 * @param list
	 */
	// Correction bug version 4.7.1 : Il ne faut pas donner la possibilit de librer un objet final
	// car alors si on le reclasse on aura une erreur
	public final void ajouteAffichagesLiesAPointVisibles(CListeObjets list) {
		ListIterator<CElementBase> li = list.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			// if (elb.estDeNature(NatureObjet.NAffichageLieAPoint)) {
			if (! elb.estElementFinal && elb.estDeNature(NatureObjet.NAffichageLieAPoint)) {
				CAffichageLieAPoint aff = (CAffichageLieAPoint) elb;
				if (!aff.masque && (aff.pointLie != null))
					add(aff);
			}
		}
	}

	/**
	 * Fonction renvoyant le nombre d'objets de la liste qui sont des affichages non masqus 
	 * lis  des points si blie est true
	 * non lis  des points si blie est false
	 */

	public final int nombreAffichagesLiesAPointVisibles(boolean blie) {
		int res = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.estDeNature(NatureObjet.NAffichageLieAPoint)) {
				CAffichageLieAPoint aff = (CAffichageLieAPoint) elb;
				// Correction version 5.4
				// if (!aff.masque && (blie ? aff.pointLie != null : aff.pointLie != null))
				if (!aff.masque && (blie ? aff.pointLie != null : aff.pointLie == null))
					res++;
			}
		}
		return res;
	}

	/**
	 * Fonction ajoutant les lments de la liste list qui sont libres dans le cas
	 * ou libre est true et sont punaiss sinon
	 * 
	 * @param list
	 * @param libres
	 */
	public final void ajoutePointsLibresVisibles(CListeObjets list, boolean libres) {
		ListIterator<CElementBase> li = list.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.estDeNature(NatureObjet.NPointMobile)) {
				CPointAncetrePointsMobiles pt = (CPointAncetrePointsMobiles) elb;
				if (!pt.masque) {
					if (libres) {
						if (pt.pointLibre)
							add(pt);
					} else {
						if (!pt.pointLibre)
							add(pt);
					}
				}
			}
		}
	}

	/**
	 * Fonction renvoyant : Si libre = true le nombre de points libres non
	 * punaiss non masqus Si libre = false : le nombre de points libres punaiss
	 * non masqus
	 * 
	 * @param libres
	 * @return
	 */
	public final int nombrePointsLibresVisibles(boolean libres) {
		int res = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.estDeNature(NatureObjet.NPointMobile)) {
				CPointAncetrePointsMobiles pt = (CPointAncetrePointsMobiles) li.next();
				if (!pt.masque) {
					if (libres) {
						res++;
					} else {
						if (!pt.pointLibre)
							res++;
					}
				}
			}
		}
		return res;
	}

	public final int nombrePointsLiesAVisibles(CElementBase el) {
		int compteur = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NPointLie) {
				CPointLie ptl = (CPointLie) elb;
				if (ptl.existe && !ptl.masque && (ptl.lieA() == el))
					compteur++;
			}
		}
		return compteur;
	}

	/**
	 * Fonction ajoutant les objets de la liste li qui sont des affichages lis 
	 * des points
	 * 
	 * @param list
	 */
	// Correction bug version 4.7.1 : Il ne faut pas donner la possibilit de librer un objet final
	// car alors si on le reclasse on aura une erreur
	public final void ajouteAffichagesNonLiesAPointVisibles(CListeObjets list) {
		ListIterator<CElementBase> li = list.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			// if (elb.estDeNature(NatureObjet.NAffichageLieAPoint)) {
			if (! elb.estElementFinal && elb.estDeNature(NatureObjet.NAffichageLieAPoint)) {
				CAffichageLieAPoint aff = (CAffichageLieAPoint) elb;
				if (aff.pointLie == null)
					add(aff);
			}
		}
	}
	
	/**
	 * Ajout version 3.0
	 * Ajoute les objets de listeSource pouvant gnrer un lieu d'objets
	 */
	public final void ajouteSurfacesNePouvantPasGenererLieuObjet(CListeObjets listeSource) {
		ListIterator<CElementBase> li = listeSource.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NSurface) {
				CSurfaceAncetre surf = (CSurfaceAncetre) elb;
				if (!surf.peutGenererLieuObjet()) add(surf);
			}
		}
	}
	
	/**
	 * Ajout version 3.0
	 * Ajoute  liste les objets pouvant gnrer un lieu d'objets
	 * et dpendant de el
	 */
	public final void ajouteObjetsPouvantGenererLieuObjetDepDe(CListeObjets listeSource, CElementBase el) {
		ListIterator<CElementBase> li = listeSource.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NSurface) {
				if (elb.dependDe(el)) {
					CSurfaceAncetre surf = (CSurfaceAncetre) elb;
					if (surf.peutGenererLieuObjet()) add(surf);
				}
			}
			else {
				if (elb.estDeNature(NatureObjet.NToutObjetPourLieu)) {
					if (elb.dependDe(el)) add(elb);
				}
			}
		}
	}
	
	/**
	 * Procedure utilise lors de la cration de la liaison entre un point libre
	 * et un objet dans le cas o le point libre a t cr avant l'objet. Il faut
	 * alors dcaler (du dernier au premier) tous les objets crs entre le point
	 * libre d'indice IndObj1 et l'objet d'indice indObj2 auquel il faut le lier
	 */
	// Revu version 4.9.3
	/*
	public final void decaleDependants(int indObj1, int indObj2) {
		int indiceObjet2;
		int indiceDebut;
		CElementBase el2 = get(indObj2);
		if (el2.estElementFinal) {
			indiceDebut = indexOf(el2.impProto) - 1;
			indiceObjet2 = indexOf(el2.impProto.dernierObjetFinal());
		} else {
			indiceDebut = indObj2 - 1;
			indiceObjet2 = indObj2;
		}
		CElementBase objet1 = get(indObj1);
		int i = indiceDebut;
		while (i >= indObj1) {
			CElementBase element = get(i);
			// 
			if (element.appartientABlocDependantPourReclassement(objet1)) {
				if (indiceObjet2 == size() - 1)
					add(element);
				else
					add(indiceObjet2 + 1, element);
				remove(i);
				indiceObjet2--;
				i--;
				// Si l'objet qu'on vient de reclasser est le dernier objet final d'une
				// implmentation de construction
				// il aut recoller lo corps de la construction avec lui
				if (element.estElementFinal) {
					CImplementationProto imp = element.impProto;
					int nb = imp.nbFinaux + imp.nbIntermediaires;
					for (int k = 0; k < nb; k++) {
						CElementBase el1 = get(i);
						add(indiceObjet2 + 1, el1);
						remove(i);
						indiceObjet2--;
						i--;
					}
				}
			} else {
				if (element.estElementFinal) {
					// Il ne faut pas dsoldariser le dernier lment final de son
					// implmentation de prototype
					CImplementationProto imp = element.impProto;
					int nb = imp.nbFinaux + imp.nbIntermediaires + 1;
					i -= nb;
				} else
					i--;
			}
		}
		// Ajout version 4.8.0
		updateIndexes();
		//

	}	
	*/
	public final void decaleDependants(int indObj1, int indObj2) {
		int indiceObjet2;
		int indiceDebut;
		CElementBase el2 = get(indObj2);
		if (el2.estElementFinal) {
			indiceDebut = indexOf(el2.impProto) - 1;
			indiceObjet2 = indexOf(el2.impProto.dernierObjetFinal());
		} else {
			indiceDebut = indObj2 - 1;
			indiceObjet2 = indObj2;
		}
		CElementBase objet1 = get(indObj1);
		int i = indiceDebut;
		while (i >= indObj1) {
			CElementBase element = get(i);
			// 
			if (element.appartientABlocDependantPourReclassement(objet1)) {
				if (indiceObjet2 == size() - 1)
					add(element);
				else
					add(indiceObjet2 + 1, element);
				remove(i);
				indiceObjet2--;
				i--;
				// Si l'objet qu'on vient de reclasser est le dernier objet final d'une
				// implmentation de construction
				// il aut recoller lo corps de la construction avec lui
				if (element.estElementFinal) {
					CImplementationProto imp = element.impProto;
					int nb = imp.nbFinaux + imp.nbIntermediaires;
					for (int k = 0; k < nb; k++) {
						CElementBase el1 = get(i);
						// Modifi version 4.9.3
						// add(indiceObjet2 + 1, el1);
						if (indiceObjet2 == size() - 1)
							add(el1);
						else
							add(indiceObjet2 + 1, el1);						
						// Fin modif
						remove(i);
						indiceObjet2--;
						i--;
					}
				}
			} else {
				if (element.estElementFinal) {
					// Il ne faut pas dsoldariser le dernier lment final de son
					// implmentation de prototype
					CImplementationProto imp = element.impProto;
					int nb = imp.nbFinaux + imp.nbIntermediaires + 1;
					i -= nb;
				} else
					i--;
			}
		}
		// Ajout version 4.8.0
		updateIndexes();
		//

	}	

	/**
	 * Fonction dcalant le plus possible vers la fin de la liste l'lment elb
	 * 
	 * @param elb
	 * @return true si le dcalage est possible et 0 sinon
	 */
	public final boolean reclasseVersFinAvecDependants(CElementBase elb) {
		// Si l'lment  reclasser est un lment final d'une implmentation de
		// prototype, c'est le dernier lment final
		// qu'il faut reclasser vers la fin
		int indexItem;
		int indexFin;
		int indDebut = 0; // Rajout version 4.9.9
		if (elb.estElementFinal) {
			indexItem = indexOf(elb.impProto);
			indexFin = indexOf(elb.impProto.dernierObjetFinal());
		} else {
			indexItem = indexOf(elb);
			indexFin = indexItem;
		}
		if (indexFin == size() - 1)
			return false;
		// On recherche  partir de la fin de la liste la position
		// du premier item qui ne dpende pas de elb
		int indMaxiDroite = size() - 1;
		boolean dependant = true;
		for (int i = size() - 1; (i != indexFin) && dependant; i--) {
			CElementBase element = get(i);
			if (element.estElementFinal) { 
				CImplementationProto imp = element.impProto;
				// Modifi version 4.2.2
				// if (!element.dependDeBlocPourReclassement(elb)) {
				if (!element.appartientABlocDependantPourReclassement(elb)) {
					dependant = false;
					indDebut = i - imp.nbFinaux - imp.nbIntermediaires -1;
					indMaxiDroite = i;
				}
				// Modifi version 4.2.2
				// i = i - imp.nbFinaux - imp.nbIntermediaires - 1;
				// Modifi version 4.9.9
				// i = i - imp.nbFinaux - imp.nbIntermediaires;
				// Remis version 4.9.9
				// Modifi version 5.4 : Il ne faut pas soustraire 1 car cela va tre fait en revenant au dbut de la boucle
				// i = i - imp.nbFinaux - imp.nbIntermediaires - 1;
				i = i - imp.nbFinaux - imp.nbIntermediaires;
			} else {
				if (!element.dependDeBlocPourReclassement(elb)) {
					dependant = false;
					indDebut = i - 1;
					indMaxiDroite = i;
				}
			}
		}
		if (dependant)
			return false;
		// indMaxiDroite est l'indice du premier lment qui ne dpend pas de elb 
		// partir de la fin de la liste
		// int indDebut = indMaxiDroite - 1; // Version 4.9.9
		/*
		 * for (int i = indDebut; i >= indexItem; i--) { CElementBase element =
		 * get(i); if (element.dependDePourReclassement(elb)) { if (indMaxiDroite ==
		 * size() - 1) add(element); else add(indMaxiDroite + 1, element);
		 * remove(i); indMaxiDroite --; } }
		 */
		int i = indDebut;
		while (i >= indexItem) {
			CElementBase element = get(i);
			// 
			if (element.appartientABlocDependantPourReclassement(elb)) {
				if (indMaxiDroite == size() - 1)
					add(element);
				else
					add(indMaxiDroite + 1, element);
				remove(i);
				indMaxiDroite--;
				i--;
				// Si l'objet qu'on vient de reclasser est le dernier objet final d'une
				// implmentation de construction
				// il faut recoller le corps de la construction avec lui
				if (element.estElementFinal) {
					CImplementationProto imp = element.impProto;
					int nb = imp.nbFinaux + imp.nbIntermediaires;
					for (int k = 0; k < nb; k++) {
						CElementBase el1 = get(i);
						// Modifi version 4.2.2
						// add(indMaxiDroite + 1, el1);
						if (indMaxiDroite == size() - 1)
							add(el1);
						else
							add(indMaxiDroite + 1, el1);						
						remove(i);
						indMaxiDroite--;
						i--;
					}
				}
			} else {
				if (element.estElementFinal) {
					// Il ne faut pas dsolidariser le dernier lment final de son
					// implmentation de prototype
					CImplementationProto imp = element.impProto;
					// Modifi version 4.2.2
					int nb = imp.nbFinaux + imp.nbIntermediaires + 1;
					i -= nb;
				} else
					i--;
			}
		}
		// Ajout version 4.8.0
		updateIndexes();
		//
		return true;
	}
	
	/**
	 * Fonction dcalant le plus possible vers le dbut de la liste l'lment elb
	 * 
	 * @param elb
	 * @return true si le dcalage est possible et 0 sinon
	 */

	public final boolean reclasseVersDebutAvecDependants(CElementBase elb) {
		// int indiceMiniDebut = indicePremierElementNonVariable(); // Modifi version 5.3
		int indiceMiniDebut = indicePremierElementNonConstante();
		int indiceItem = indexOf(elb);
		if (indiceItem == indiceMiniDebut)
			return false;
		// On recherche  partir du dbut de la liste la position
		// du premier item dont elb ne dpend pas
		// Modification version 3.6.1 : on ne commence qu'au deuxime objet car le premier est la constante pi
		// int indiceMiniGauche = 0;
		int indiceMiniGauche = 1;
		boolean dependant = true;
		int i = indiceMiniDebut;
		while ((i <= indiceItem) && dependant) {
			CElementBase el = (CElementBase) get(i);
			if (el.getNature() == NatureObjet.NImplementationProto) {
				CImplementationProto imp = (CImplementationProto) el;
				// dependant = imp.dependDeUnObjetFinal(el);
				dependant = elb.dependDePourReclassement(el);
				int nb = imp.nbFinaux + imp.nbIntermediaires;
				if (dependant)
					i += nb + 1;
				else
					// indiceMiniGauche = i + nb; // Modifi version 4.9.3
					indiceMiniGauche = i;
			} else {
				if (elb.dependDePourReclassement(el))
					i++;
				else {
					dependant = false;
					indiceMiniGauche = i;
				}
			}
		}
		if (dependant)
			return false; // elb qu'on veut reclasser dpend de tous les objets
										// prcdents
		i = indiceMiniGauche + 1;
		while (i <= indiceItem) {
			CElementBase el = (CElementBase) get(i);
			// Il ne faut pas sparer les lments d'une implmentation de prototype
			if (el.getNature() == NatureObjet.NImplementationProto) {
				CImplementationProto imp = (CImplementationProto) el;
				int nb = imp.nbFinaux + imp.nbIntermediaires + 1;
				// if (imp.dependDeUnObjetFinal(elb)) {
				if (elb.dependDePourReclassement(el)) {
					for (int k = 0; k < nb; k++) {
						CElementBase el1 = (CElementBase) get(i + k);
						remove(i + k);
						add(indiceMiniGauche + k, el1);
					}
					indiceMiniGauche += nb;
					i += nb;
				} else {
					i += nb;
				}
			} else {
				if (elb.dependDePourReclassement(el)) {
					remove(i);
					add(indiceMiniGauche, el);
					indiceMiniGauche++;
				}
				i++;
			}
		}

		// Ajout version 4.8.0
		updateIndexes();
		//
		return true;
	}
	/**
	 * Fonction remontant d'un rang vers le dbut de la liste l'lment elb
	 * en remontant aussi si ncessaire les lments dont il dpend
	 * @param elb
	 * @return true si le dcalage est possible et false sinon
	 */
	// Ajout version 4.9.9
	public final boolean remonteVersDebutAvecDependants(CElementBase elb) {
		CElementBase el = null;
		// int indicePremierNonVariable = indicePremierElementNonVariable();
		// int indicemini = (elb.estDeNatureCalcul(NatureCalcul.NVariable)) ? 1 : indicePremierNonVariable;
		// int indicemini = 0; // Modifi version 5.3 pour ne pas dcaler quelque chose au-dessus de la constante pi
		int indicemini = indicePremierElementNonConstante();
		int indDebut = 0;
		// Si l'lment  remonter est un lment final de construction ou une CImplementationProto il faut remonter tout le bloc.
		CImplementationProto imp = elb.estElementFinal ? elb.impProto : 
			(elb.estDeNature(NatureObjet.NImplementationProto) ? (CImplementationProto) elb : null);
		int indelb = (imp != null) ? indexOf(imp) : indexOf(elb);
		// On recherche vers le dbut  partir de elb l'indice du premier lment dont ne dpend pas elb;
		// int i = indelb;
		boolean trouve = false;
		int inddest = (imp == null) ? indelb : indelb + imp.nbFinaux + imp.nbIntermediaires;
		int i = indelb;
		while (!trouve && (i>indicemini)) {
		// for (int i = indelb - 1; (i > indicemini) && !trouve; i--) {
			i--;
			el = get(i);
			if (el.estElementFinal) { 
				CImplementationProto imp2 = el.impProto;
				if (!elb.appartientABlocDependantPourReclassement(el)) {
					trouve = true;
					indDebut = i - imp2.nbFinaux - imp2.nbIntermediaires;
					el = imp2;
					break;
				}
				i = i - imp2.nbFinaux - imp2 .nbIntermediaires;
			} else {
				if (!elb.dependDeBlocPourReclassement(el)) {
					trouve = true;
					indDebut = i;
					break;
				}
			}
		}
		if (!trouve) return false;
		if (el.estDeNature(NatureObjet.NImplementationProto)) {
			CImplementationProto imp2 = (CImplementationProto) el;
			for (int j = 0; j <= imp2.nbFinaux + imp2.nbIntermediaires; j++) {
				CElementBase e = get(indDebut);
				remove(e);
				if (inddest == size()) add(e);
				else add(inddest, e);
			}
		}
		else {
			remove(el);
			if (inddest == size()) add(el);
			else add(inddest, el);
		}
		updateIndexes();
		return true;
	}

	/**
	 * Fonction remontant d'un rang vers la fin de la liste l'lment elb
	 * en remontant aussi si ncessaire les lments qui en dpendent
	 * @param elb
	 * @return true si le dcalage est possible et false sinon
	 */
	// Ajout version 4.9.9
	public final boolean remonteVersFinAvecDependants(CElementBase elb) {
		int i;
		CElementBase el = null;
		// Si l'lment  remonter est un lment final de construction il faut remonter tout le bloc.
		int indelb = elb.estElementFinal ? indexOf(elb.impProto) : indexOf(elb);
		// if (indelb == 0) return false; // On ne peut pas reclasser la constante pi vers la fin
		if (elb.estElementFinal || elb.estDeNature(NatureObjet.NImplementationProto)) {
			CImplementationProto imp = elb.estElementFinal ? elb.impProto : (CImplementationProto) elb;
			i = indexOf(imp)+imp.nbFinaux+imp.nbIntermediaires;
			if (i+1 == size()) return false;
		}
		else if (indelb == size()-1) return false;
					else i = indelb;
		boolean trouve = false;
		while (!trouve && (i<size()-1)) {
			i++;
			el = get(i);
			if (el.estDeNature(NatureObjet.NImplementationProto)) {
				i++;
				el = get(i);
			}
 			if (!el.appartientABlocDependantPourReclassement(elb)) trouve = true;
			else {
				if (el.impProto != null) {
					CImplementationProto imp = el.impProto;
					i = indexOf(imp) + imp.nbFinaux + imp.nbIntermediaires;
				}
			}
		}
		if (!trouve) return false;
		if (el.impProto != null) {
			CImplementationProto imp = el.impProto;
			i = indexOf(imp);
			for (int j = 0; j <= imp.nbFinaux + imp.nbIntermediaires; j++) {
				CElementBase e = get(i+j);
				remove(e);
				add(indelb + j, e);
			}
		}
		else {
			remove(el);
			add(indelb, el);
		}
		updateIndexes();
		return true;
	}
 
	/**
	 * Fonction renvoyant l'indice du premier objet qui ne soit pas une variable
	 * ou un calcul constant. Est utilis pour le reclassement d'objets en dbut
	 * de liste
	 * 
	 * @return
	 */
	/* Modifi version 5.3
	public final int indicePremierElementNonVariable() {
		int i;
		for (i = 0; i < size(); i++) {
			CElementBase elb = (CElementBase) get(i);
			long nat = elb.getNatureCalcul();
			if ((nat != NatureCalcul.NVariable) && (elb.getClass() != CCalcConst.class))
				break;
		}
		return i;
	}
	*/
	public final int indicePremierElementNonConstante() {
		int i;
		for (i = 0; i < size(); i++) {
			CElementBase elb = (CElementBase) get(i);
			if (elb.getClass() != CCalcConst.class) break;
		}
		return i;
	}



	// Rend tous les objets de la liste visibles ou masques. Les objets doivent
	// tre de type graphique
	public final void montreTout(boolean visible) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.estDeNature(NatureObjet.NToutObjet)) {
				CElementGraphique elg = (CElementGraphique) elb;
				if (visible)
					elg.montre();
				else
					elg.cache();
			}
		}
	}

	public final void retireTout() {
		int s = size();
		for (int i = 0; i < s; i++) {
			remove(0);
		}
	}

	// Rajoute tous les lments clons de la liste  la liste listeCible
	// de la liste listeSource
	public final void setCopy(CListeObjets listeCible) {
		listeCible.documentProprietaire = documentProprietaire;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			 listeCible.add(elb.getClone(this, listeCible));
		}
		listeCible.uniteAngle = uniteAngle;
		if (pointeurLongueurUnite != null) {
			listeCible.pointeurLongueurUnite = (CLongueur) listeCible.get(indexOf(pointeurLongueurUnite));
		}
		// Ce n'est qu'une fois tous les lments clons qu'on peut tablir
		// certaines listes
		// maintenues de faon interne par certaines macros
		listeCible.etablitListesInternesMacros();
		// Ajout version 4.8 car sinon lors de annuler-refaire, la macro par dfaut et la macro de dmarrage ne sont pas conserves
		if (macroParDefaut != null) listeCible.macroParDefaut = (CMacro) listeCible.get(indexOf(macroParDefaut));
		if (macroDemarrage != null) listeCible.macroDemarrage = (CMacro) listeCible.get(indexOf(macroDemarrage));
		listeCible.basseDefinition = basseDefinition;
		listeCible.coefMult = coefMult;
		listeCible.bAttenuationEpaisseurTrait = bAttenuationEpaisseurTrait;
		listeCible.bCreationComposants = bCreationComposants;
	}

	// Cette mthode fait de cette liste dans listeProprietaireCible l'quivalent
	// de listeSource dans la
	// ListePropretaireSource. Bien sur ListeProprietaireCible doit tre une liste
	// clone
	// auparavant de la listeProprietaireSource.
	public final void setImage(CListeObjets listeSource, CListeObjets listeProprietaireSource,
			CListeObjets listeProprietaireCible) {
		ListIterator<CElementBase> li = listeSource.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			int indexSource = listeProprietaireSource.indexOf(elb);
			CElementBase elbCible = (CElementBase) listeProprietaireCible.get(indexSource, CElementBase.class);
			add(elbCible);
		}
		uniteAngle = listeSource.uniteAngle;
		if (listeSource.pointeurLongueurUnite != null) {
			pointeurLongueurUnite = (CLongueur) get(listeSource.indexOf(listeSource.pointeurLongueurUnite));
		}
	}

	/**
	 * Mthode faisant de la liste en cours la liste clone de listeSource
	 * Auparavant, la liste en cours doit avoir t cre comme copie de listeSource  l'aide de setCopy
	 * @param listeSource
	 */
	// Modifi version 4.6
	public final void setClone(CListeObjets listeSource) {
		ListIterator<CElementBase> li = listeSource.listIterator();
		int index = 0;
		while ((index < size()) && li.hasNext()) {
			CElementBase elbSource = li.next();
			CElementBase elb = (CElementBase) get(index);
			elb.setClone(elbSource);
			index++;
		}
	}

	/** Positionnement de tous les objets de la liste */
	// Synchronized  partir de version 3.2.6.1
	public synchronized final void positionne(boolean infoRandom, DimensionFenetre dimfen) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			elb.positionne(infoRandom, dimfen);
		}
	}
	
	// Ajout version 4.9.3. Utilis dans les CSomme et CProduit
	public synchronized final void positionneFull(boolean infoRandom, DimensionFenetre dimfen) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			elb.positionneFull(infoRandom, dimfen);
		}
	}

	// Ajout version 4.8.1
	public final void positionneTikz(boolean infoRandom, DimensionFenetre dimfen) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			elb.positionneTikz(infoRandom, dimfen);
		}
	}
	
	public final void positionneDependantsDe(boolean infoRandom, DimensionFenetre dimfen, CElementBase pt) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.dependDe(pt))
				elb.positionne(infoRandom, dimfen);
		}
	}

	/** Affichage de tous les lments affichables de la liste */
	public final void afficheTout(Graphics2D pDC, DimensionFenetre dimfen, boolean masquage, Color couleurFond,
			float transparence, boolean antialiasing) {
    pDC.setRenderingHint(RenderingHints.KEY_ANTIALIASING, antialiasing ? RenderingHints.VALUE_ANTIALIAS_ON : RenderingHints.VALUE_ANTIALIAS_OFF);
    pDC.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, antialiasing ? RenderingHints.VALUE_TEXT_ANTIALIAS_ON : RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.existe) {
				long nature = elb.getNature();
				if ((nature & NatureObjet.NToutObjet) != 0) {
					if (!elb.estElementIntermediaire()) {
						CElementGraphique elg = (CElementGraphique) elb;
						elg.affiche(pDC, masquage, couleurFond, transparence);
						if ((nature & NatureObjet.NObjetNommable) != 0) {
							// Modifi version 3.4.0 pour que quand le rideau est actif, les noms ou droites masqus 
							// soeint quand mme affichs.
							if (!masquage || (masquage && !elg.masque)) elg.afficheNom(pDC);
						}
					}
				}
			}
		}
	}
	
	/** Version 4.4 : Affichage de tous les points de la liste marqus pour laisser une trace */
	public final void affichePointsMarquesPourTrace(Graphics2D pDC, DimensionFenetre dimfen, boolean masquage, Color couleurFond,
			float transparence, boolean antialiasing) {
    pDC.setRenderingHint(RenderingHints.KEY_ANTIALIASING, antialiasing ? RenderingHints.VALUE_ANTIALIAS_ON : RenderingHints.VALUE_ANTIALIAS_OFF);
    pDC.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, antialiasing ? RenderingHints.VALUE_TEXT_ANTIALIAS_ON : RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.existe) {
				if (elb.estDeNature(NatureObjet.NToutPoint) && !elb.estElementIntermediaire()) {
					CPointAncetre pt = (CPointAncetre) elb;
					if (pt.marquePourTrace) pt.affiche(pDC, masquage, couleurFond, transparence);
				}
			}
		}
	}
	
	
	/** Affichage de tous les lments affichables de la liste sauf ceux de listeExclue */
	public final void afficheToutSaufElementsDe(Graphics2D pDC, DimensionFenetre dimfen, boolean masquage,
			Color couleurFond, float transparence, CListeObjets listeExclue, boolean antialiasing) {
    pDC.setRenderingHint(RenderingHints.KEY_ANTIALIASING, antialiasing ? RenderingHints.VALUE_ANTIALIAS_ON : RenderingHints.VALUE_ANTIALIAS_OFF);
    pDC.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, antialiasing ? RenderingHints.VALUE_TEXT_ANTIALIAS_ON : RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.existe) {
				long nature = elb.getNature();
				if ((nature & NatureObjet.NToutObjet) != 0) {
					if (!elb.estElementIntermediaire() && !listeExclue.contains(elb)) {
						CElementGraphique elg = (CElementGraphique) elb;
						elg.affiche(pDC, masquage, couleurFond, transparence);
						if ((nature & NatureObjet.NObjetNommable) != 0) {
							// Modifi version 3.4.0 pour que quand le rideau est actif, les noms ou droites masqus 
							// soeint quand mme affichs.
							if (!masquage || (masquage && !elg.masque)) elg.afficheNom(pDC);
						}
					}
				}
			}
		}
	}	
	
	
	/** Affichage de tous les lments affichables de la liste sans le nom des objets (utilis pour Mouvoir Trace d'objets) */
	public final void afficheToutSansNom(Graphics2D pDC, DimensionFenetre dimfen, boolean masquage, Color couleurFond,
			float transparence, boolean antialiasing) {
    pDC.setRenderingHint(RenderingHints.KEY_ANTIALIASING, antialiasing ? RenderingHints.VALUE_ANTIALIAS_ON : RenderingHints.VALUE_ANTIALIAS_OFF);
    pDC.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, antialiasing ? RenderingHints.VALUE_TEXT_ANTIALIAS_ON : RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.existe) {
				long nature = elb.getNature();
				if ((nature & NatureObjet.NToutObjet) != 0) {
					if (!elb.estElementIntermediaire()) ((CElementGraphique) elb).affiche(pDC, masquage, couleurFond, transparence);
				}
			}
		}
	}	

	// 
	// les noms)
	/**
	 * Affichage de tous les lments affichables de la liste mme masqus (sans
	 * les noms) Utilis pour les macres avec traces
	 * A n'utiliser que pour les liste d'lments tous graphiques
	 * 
	 */
	public final void afficheToutMemeMasquesPourTrace(Graphics2D pDC, DimensionFenetre dimfen, Color couleurFond,
			float transparence, boolean antialiasing) {
    pDC.setRenderingHint(RenderingHints.KEY_ANTIALIASING, antialiasing ? RenderingHints.VALUE_ANTIALIAS_ON : RenderingHints.VALUE_ANTIALIAS_OFF);
    pDC.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, antialiasing ? RenderingHints.VALUE_TEXT_ANTIALIAS_ON : RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementGraphique elg = (CElementGraphique) (li.next());
			if (elg.existe) {
				long nature = elg.getNature();
				if ((nature & NatureObjet.NToutObjet) != 0) {
					if (!elg.estElementIntermediaire()) {
						boolean estMasque = elg.masque;
						elg.masque = false;
						elg.affiche(pDC, true, couleurFond, transparence);
						elg.masque = estMasque;
					}
				}
			}
		}
	}

	
	/**
	 * Affichage de tous les lments affichables de la liste mme masqus (avec les noms)
	 * 
	 */
	public final void afficheToutMemeMasques(Graphics2D pDC, DimensionFenetre dimfen, Color couleurFond,
			float transparence, boolean antialiasing) {
    pDC.setRenderingHint(RenderingHints.KEY_ANTIALIASING, antialiasing ? RenderingHints.VALUE_ANTIALIAS_ON : RenderingHints.VALUE_ANTIALIAS_OFF);
    pDC.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, antialiasing ? RenderingHints.VALUE_TEXT_ANTIALIAS_ON : RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = (CElementBase) (li.next());
			if (elb.estDeNature(NatureObjet.NToutObjet)) {
				CElementGraphique elg = (CElementGraphique) elb;
				if (elg.existe) {
					long nature = elg.getNature();
					if (!elg.estElementIntermediaire()) {
						boolean estMasque = elg.masque;
						elg.masque = false;
						elg.affiche(pDC, true, couleurFond, transparence);
						elg.masque = estMasque;
						if ((nature & NatureObjet.NObjetNommable) != 0) {
							boolean nomMasque = elg.nomMasque;
							elg.nomMasque = false;
							elg.afficheNom(pDC);
							elg.nomMasque = nomMasque;
						}
					}
				}
			}
		}
	}
	
	/**
	 * Affichage de tous les lments affichables de la liste mme masqus (avec les noms)
	 * y compris les lments intermdiaires
	 */
	public final void afficheToutMemeMasquesPourHistorique(Graphics2D pDC, DimensionFenetre dimfen, Color couleurFond,
			float transparence, boolean antialiasing) {
    pDC.setRenderingHint(RenderingHints.KEY_ANTIALIASING, antialiasing ? RenderingHints.VALUE_ANTIALIAS_ON : RenderingHints.VALUE_ANTIALIAS_OFF);
    pDC.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, antialiasing ? RenderingHints.VALUE_TEXT_ANTIALIAS_ON : RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = (CElementBase) (li.next());
			if (elb.estDeNature(NatureObjet.NToutObjet)) {
				CElementGraphique elg = (CElementGraphique) elb;
				if (elg.existe) {
					long nature = elg.getNature();
					boolean estMasque = elg.masque;
					elg.masque = false;
					elg.affiche(pDC, true, couleurFond, transparence);
					elg.masque = estMasque;
					if (((nature & NatureObjet.NObjetNommable) != 0) && !elg.nom().equals(MtgFrame.getString("interm"))) {
						boolean nomMasque = elg.nomMasque;
						elg.nomMasque = false;
						elg.afficheNom(pDC);
						elg.nomMasque = nomMasque;
					}
				}
			}
		}
	}		

	/**
	 * Fonction affichant tous les lments de la liste jusqu' l'lment d'indice
	 * indDer. Utilis dans l'historique
	 * 
	 * @param pDC
	 * @param dimfen
	 * @param indDer
	 */
	public final void afficheJusque(Graphics2D pDC, DimensionFenetre dimfen, int indDer, boolean intermPermis, Color couleurFond, float transparence,
			boolean antialiasing) {
    pDC.setRenderingHint(RenderingHints.KEY_ANTIALIASING, antialiasing ? RenderingHints.VALUE_ANTIALIAS_ON : RenderingHints.VALUE_ANTIALIAS_OFF);
    pDC.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, antialiasing ? RenderingHints.VALUE_TEXT_ANTIALIAS_ON : RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		int ind = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext() && (ind <= indDer)) {
			CElementBase elb = li.next();
			ind++;
			if (elb.existe) {
				elb.montreComposant(); // Ajout version 4.5.2
				long nature = elb.getNature();
				if ((nature & NatureObjet.NToutObjet) != 0) {
					if (!elb.estElementIntermediaire() || intermPermis) {
						CElementGraphique elg = (CElementGraphique) elb;
						boolean masque = elg.masque;
						elg.masque = false;
						elg.affiche(pDC, true, couleurFond, transparence);
						if ((nature & NatureObjet.NObjetNommable) != 0) {
							boolean nomMasque = elg.nomMasque;
							elg.nomMasque = false;
							elg.afficheNom(pDC);
							elg.nomMasque = nomMasque;
						}
						elg.masque = masque;
					}
				}
			}
		}
		// Lignes suivantes ajoutes version 4.5.2
		while (li.hasNext()) {
			CElementBase elb = li.next();
			elb.cacheComposant();
		}

	}

	/**
	 * Fonction affichant tous les lments de la liste jusqu' l'lment d'indice
	 * indDer sauf ceux contenus dans listeEclue. Utilis dans l'historique
	 * 
	 * @param pDC
	 * @param dimfen
	 * @param indDer
	 */
	// Midifi version 4.9.9. Ajout d'un paramtre affMasques. Si ce paramtre est true, les ojets masqus sont quand
	// mme affichs.
	public final void afficheJusqueSaufElementsDe(Graphics2D pDC, DimensionFenetre dimfen, int indDer,
			CListeObjets listeExclue, boolean intermPermis, boolean affMasques, Color couleurFond, float transparence,
			boolean antialiasing) {
    pDC.setRenderingHint(RenderingHints.KEY_ANTIALIASING, antialiasing ? RenderingHints.VALUE_ANTIALIAS_ON : RenderingHints.VALUE_ANTIALIAS_OFF);
    pDC.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, antialiasing ? RenderingHints.VALUE_TEXT_ANTIALIAS_ON : RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		int ind = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext() && (ind <= indDer)) {
			CElementBase elb = li.next();
			ind++;
			if ((elb.existe) && !listeExclue.contains(elb)) {
				long nature = elb.getNature();
				if ((nature & NatureObjet.NToutObjet) != 0) {
					if (!elb.estElementIntermediaire() || intermPermis) {
						elb.montreComposant(); // Ajout version 4.5.2
						CElementGraphique elg = (CElementGraphique) elb;
						boolean masque = elg.masque;
						if (affMasques && masque) elg.masque = false;
						elg.affiche(pDC, true, couleurFond, transparence);
						if ((nature & NatureObjet.NObjetNommable) !=  0) {
							boolean nomMasque = elg.nomMasque;
							elg.nomMasque = !((affMasques && masque) || !masque);
							elg.afficheNom(pDC);
							elg.nomMasque = nomMasque;
						}
						elg.masque = masque;
					}
				}
			}
		}
		// Lignes suivantes ajoutes version 4.5.2
		while (li.hasNext()) {
			CElementBase elb = li.next();
			elb.cacheComposant();
		}
	}
	

	/**
	 * Fonction renvoyant le nombre d'objets de type typeCherche proche du point
	 * de coordonnes point  l'exclusion des objets de la liste
	 * listeExclusionDesignation
	 * 
	 * @param typeCherche
	 * @param point
	 * @param info
	 * @param listeExclusion
	 * @return
	 */
	public final int procheDe(long typeCherche, Point point, InfoProx info, CListeObjets listeExclusion,
			boolean prioritePoints) {
		int ind, indnat;
		long nat;
		CElementBase ptel;
		CElementGraphique ptelg;
		double dis;

		// lorsque la liste est vide, aucun objet n'est proche
		if (size() == 0) {
			return 0;
		}
		info.MetAZero();
		// On traite tous les lments de la liste

		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			ptel = li.next();
			// Modifi version 4.5 pour qu'on puisse capturer un lieu d'objet gnr par un CCommentaire ou un CLatex
			// if (listeExclusion.indexOf(ptel) == -1) {
			if (listeExclusion.indexOf(ptel) == -1) {
				nat = ptel.getNature();
				if ((nat & typeCherche) != 0) {
					// Modification par rapport au C++
					ptelg = (CElementGraphique) ptel;
					if (!ptelg.masque && !ptelg.estElementIntermediaire()) {
						dis = ptelg.distancePoint(point.x, point.y, true);
						if ((dis != -1) && (dis < distanceMiniPourEtreProche)) // distance
																																		// infrieure
																																		//  4
																																		// pixels
																																		// pour
																																		// tre considr comme proche
						{
							// Recherche de l'indice dans le tableau Tableau
							// correspondant  nat
							info.typesProches |= nat;
							indnat = NatureObjet.indiceDeNature(nat);

							info.infoParType[indnat].nombreVoisins++;
							if ((nat & NatureObjet.NToutPoint) != 0) // cas d'un point
							{
								info.nombrePointsVoisins++;
								if (info.premierPointVoisinRencontre) {
									info.premierPointVoisin = (CPointAncetre) ptelg;
									info.premierPointVoisinRencontre = false;
								} else {
									info.dernierPointVoisin = (CPointAncetre) ptelg;
								}
							}

							if (info.infoParType[indnat].premierVoisinRencontre) {
								info.infoParType[indnat].premierVoisin = ptelg;
								info.infoParType[indnat].premierVoisinRencontre = false;
							} else {
								info.infoParType[indnat].dernierVoisin = ptelg;
							}
						} // if Poindis
					} // if (!Ptel.Masque)
				} // if (nat and type_cherche <>0)
			} // if listeExclusion
		}// Fin de la boucle de traitement de chacun des lments de la liste

		// Si prioritePoint est true, priorits aux objets points sauf si on est proche d'un repre
		if (prioritePoints && (info.nombrePointsVoisins > 0)) {
			info.MetAZeroObjetsNonPoints(); // Ajout version 2.7
			info.typesProches &= NatureObjet.NToutPoint; // Car un repre se dsigne par son origine ds la version java
		}
		info.nombreObjetsProches = 0;
		for (ind = 0; ind < NatureObjet.nombreTypesDesignables; ind++) {
			info.nombreObjetsProches += info.infoParType[ind].nombreVoisins;
		}
		info.nombreTypesProches = 0;
		for (ind = 0; ind < NatureObjet.nombreTypesDesignables; ind++) {
			if ((info.typesProches & NatureObjet.conversionNumeroNature[ind]) != 0) {
				info.nombreTypesProches++;
			}
		}
		return info.nombreObjetsProches;
	}

	/**
	 * Fonction rajoute pour la version 4.5 et permettant de capturer un lieu d'objet gnr par un CCommentaire
	 * et un CLatex.
	 * Utilis dans OutilPointageCapt
	 * @param typeCherche
	 * @param point
	 * @param info
	 * @param listeExclusion
	 * @param prioritePoints
	 * @return
	 */
	public final int procheDePourCapture(long typeCherche, Point point, InfoProx info, CListeObjets listeExclusion,
			boolean prioritePoints) {
		int ind, indnat;
		long nat;
		CElementBase ptel;
		CElementGraphique ptelg;
		double dis;

		// lorsque la liste est vide, aucun objet n'est proche
		if (size() == 0) {
			return 0;
		}
		info.MetAZero();
		// On traite tous les lments de la liste

		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			ptel = li.next();
			if (ptel.estCapturableSouris() && listeExclusion.indexOf(ptel) == -1) {
				nat = ptel.getNature();
				if ((nat & typeCherche) != 0) {
					// Modification par rapport au C++
					ptelg = (CElementGraphique) ptel;
					if (!ptelg.masque && !ptelg.estElementIntermediaire()) {
						dis = ptelg.distancePoint(point.x, point.y, true);
						if ((dis != -1) && (dis < distanceMiniPourEtreProche)) // distance
																																		// infrieure
																																		//  4
																																		// pixels
																																		// pour
																																		// tre considr comme proche
						{
							// Recherche de l'indice dans le tableau Tableau
							// correspondant  nat
							info.typesProches |= nat;
							indnat = NatureObjet.indiceDeNature(nat);

							info.infoParType[indnat].nombreVoisins++;
							if ((nat & NatureObjet.NToutPoint) != 0) // cas d'un point
							{
								info.nombrePointsVoisins++;
								if (info.premierPointVoisinRencontre) {
									info.premierPointVoisin = (CPointAncetre) ptelg;
									info.premierPointVoisinRencontre = false;
								} else {
									info.dernierPointVoisin = (CPointAncetre) ptelg;
								}
							}

							if (info.infoParType[indnat].premierVoisinRencontre) {
								info.infoParType[indnat].premierVoisin = ptelg;
								info.infoParType[indnat].premierVoisinRencontre = false;
							} else {
								info.infoParType[indnat].dernierVoisin = ptelg;
							}
						} // if Poindis
					} // if (!Ptel.Masque)
				} // if (nat and type_cherche <>0)
			} // if listeExclusion
		}// Fin de la boucle de traitement de chacun des lments de la liste

		// Si prioritePoint est true, priorits aux objets points sauf si on est proche d'un repre
		if (prioritePoints && (info.nombrePointsVoisins > 0)) {
			info.MetAZeroObjetsNonPoints(); // Ajout version 2.7
			info.typesProches &= NatureObjet.NToutPoint; // Car un repre se dsigne par son origine ds la version java
		}
		info.nombreObjetsProches = 0;
		for (ind = 0; ind < NatureObjet.nombreTypesDesignables; ind++) {
			info.nombreObjetsProches += info.infoParType[ind].nombreVoisins;
		}
		info.nombreTypesProches = 0;
		for (ind = 0; ind < NatureObjet.nombreTypesDesignables; ind++) {
			if ((info.typesProches & NatureObjet.conversionNumeroNature[ind]) != 0) {
				info.nombreTypesProches++;
			}
		}
		return info.nombreObjetsProches;
	}

	/**
	 * Fonction renvoyant le nombre d'objets de type typeCherche proches du point
	 * de coordonnes point et possdant un dialogue permettant de les modifier 
	 * l'exclusion des objets de la liste listeExclusionDesignation
	 * 
	 * @param typeCherche
	 * @param point
	 * @param info
	 * @param listeExclusion
	 * @return
	 */
	public final int procheDePourDialogueModification(long typeCherche, Point point, InfoProx info,
			CListeObjets listeExclusion) {
		int ind, indnat;
		long nat;
		CElementBase ptel;
		CElementGraphique ptelg;
		double dis;

		// lorsque la liste est vide, aucun objet n'est proche
		if (size() == 0) {
			return 0;
		}
		info.MetAZero();
		// On traite tous les lments de la liste

		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			ptel = li.next();
			if (ptel.modifiableParMenu() && (listeExclusion.indexOf(ptel) == -1)) {
				nat = ptel.getNature();
				// if ((nat & typeCherche) != 0) {
				// Correction d'un bug version 4.8
				if (!ptel.estElementIntermediaire() && ((nat & typeCherche) != 0)) {
					// Modification par rapport au C++
					ptelg = (CElementGraphique) ptel;
					if (!ptelg.masque) {
						dis = ptelg.distancePoint(point.x, point.y, true);
						// LPtoDP(Dc,Poindis,1);
						if ((dis != -1) && (dis < distanceMiniPourEtreProche)) // distance
																																		// infrieur
																																		//  4
																																		// pixels
																																		// pour
						// tre considr comme proche
						{
							// Recherche de l'indice dans le tableau Tableau
							// correspondant  nat
							info.typesProches |= nat;
							indnat = NatureObjet.indiceDeNature(nat);

							info.infoParType[indnat].nombreVoisins++;
							if ((nat & NatureObjet.NToutPoint) != 0) // cas d'un point
							{
								info.nombrePointsVoisins++;
								if (info.premierPointVoisinRencontre) {
									info.premierPointVoisin = (CPointAncetre) ptelg;
									info.premierPointVoisinRencontre = false;
								} else {
									info.dernierPointVoisin = (CPointAncetre) ptelg;
								}
							}

							if (info.infoParType[indnat].premierVoisinRencontre) {
								info.infoParType[indnat].premierVoisin = ptelg;
								info.infoParType[indnat].premierVoisinRencontre = false;
							} else {
								info.infoParType[indnat].dernierVoisin = ptelg;
							}
						} // if Poindis
					} // if (!Ptel.Masque)
				} // if (nat and type_cherche <>0)
			} // if listeExclusion
		}// Fin de la boucle de traitement de chacun des lments de la liste

		// Priorits aux objets points sauf si on est proche d'un lieu
		// de points
		if (info.nombrePointsVoisins > 0) {
			info.MetAZeroObjetsNonPoints(); // Ajout version 2.7
			info.typesProches &= NatureObjet.NToutPoint; // Car un repre se dsigne par son origine ds la version java
		}
		info.nombreObjetsProches = 0;
		for (ind = 0; ind < NatureObjet.nombreTypesDesignables; ind++) {
			info.nombreObjetsProches += info.infoParType[ind].nombreVoisins;
		}
		info.nombreTypesProches = 0;
		for (ind = 0; ind < NatureObjet.nombreTypesDesignables; ind++) {
			if ((info.typesProches & NatureObjet.conversionNumeroNature[ind]) != 0) {
				info.nombreTypesProches++;
			}
		}
		return info.nombreObjetsProches;
	}

	public final int procheDePourOutilRideau(long typeCherche, Point point, InfoProx info) {
		int ind, indnat;
		long nat;
		CElementBase ptel;
		CElementGraphique ptelg;
		double dis;

		// lorsque la liste est vide, aucun objet n'est proche
		if (size() == 0) {
			return 0;
		}
		info.MetAZero();
		// On traite tous les lments de la liste

		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			ptel = li.next();
			nat = ptel.getNature();
			// On ne dmasque pas les macros avec cet outil
			if (!ptel.estElementIntermediaire() && (nat & typeCherche) != 0) {
				// Modification par rapport au C++
				ptelg = (CElementGraphique) ptel;
				if (ptelg.masque) {
					dis = ptelg.distancePoint(point.x, point.y, false);
					// LPtoDP(Dc,Poindis,1);
					if ((dis != -1) && (dis < distanceMiniPourEtreProche)) // distance
																																	// infrieur 
																																	// 4 pixels
																																	// pour
					// tre considr comme proche
					{
						// Recherche de l'indice dans le tableau Tableau
						// correspondant  nat
						info.typesProches |= nat;
						indnat = NatureObjet.indiceDeNature(nat);

						info.infoParType[indnat].nombreVoisins++;
						if ((nat & NatureObjet.NToutPoint) != 0) // cas d'un point
						{
							info.nombrePointsVoisins++;
							if (info.premierPointVoisinRencontre) {
								info.premierPointVoisin = (CPointAncetre) ptelg;
								info.premierPointVoisinRencontre = false;
							} else {
								info.dernierPointVoisin = (CPointAncetre) ptelg;
							}
						}

						if (info.infoParType[indnat].premierVoisinRencontre) {
							info.infoParType[indnat].premierVoisin = ptelg;
							info.infoParType[indnat].premierVoisinRencontre = false;
						} else {
							info.infoParType[indnat].dernierVoisin = ptelg;
						}
					} // if Poindis
				} // if (!Ptel.Masque)
			} // if (nat and type_cherche <>0)
		}// Fin de la boucle de traitement de chacun des lments de la liste

		// Priorits aux objets points sauf si on est proche d'un lieu
		// de points
		if (info.nombrePointsVoisins > 0) {
			info.MetAZeroObjetsNonPoints(); // Ajout version 2.7
			info.typesProches &= NatureObjet.NToutPoint; // Car un repre se dsigne par son origine ds la version java
		}
		info.nombreObjetsProches = 0;
		for (ind = 0; ind < NatureObjet.nombreTypesDesignables; ind++) {
			info.nombreObjetsProches += info.infoParType[ind].nombreVoisins;
		}
		info.nombreTypesProches = 0;
		for (ind = 0; ind < NatureObjet.nombreTypesDesignables; ind++) {
			if ((info.typesProches & NatureObjet.conversionNumeroNature[ind]) != 0) {
				info.nombreTypesProches++;
			}
		}
		return info.nombreObjetsProches;
	}

	// Mthode tablissant certaines listes maintenues de faon interne par
	// certaines macros
	public final void etablitListesInternesMacros() {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			long nature = elb.getNature();
			if (nature == NatureObjet.NMacro) {
				((CMacro) elb).etablitListesInternes();
			}
		}
	}

	// Mthode pour le CNED
	// Renvoie null si Intitule n'est pas l'intitul d'une macro ou sinon
	// renvoie un pointeur sur cette macro
	public final CMacro pointeurMacro(String intitule) {
		CMacro resultat = null;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext() && (resultat == null)) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NMacro) {
				CMacro mac = (CMacro) elb;
				if (mac.intitule.equals(intitule)) {
					resultat = mac;
				}
			}
		}
		return resultat;
	}

	// Ajoute  la liste les macros jouant un fichier sonore contenues dans list
	public final void ajouteMacrosJouantSon(CListeObjets list) {
		ListIterator<CElementBase> li = list.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NMacro) {
				if (elb.getClass() == CMacroJouantSon.class) {
					add(elb);
				}
			}
		}
	}

	/**
	 * Fonction renvoyant true si la liste contient une fonction drive de la
	 * fonction fonc qui ne soit pas un objet intermdiaire
	 */
	public final boolean contientDeriveeDe(CFonc fonc) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if ((elb.getNatureCalcul() == NatureCalcul.NDerivee) && !elb.estElementIntermediaire()) {
				if (((CDerivee) elb).fonctionAssociee == fonc)
					return true;
			}
		}
		return false;
	}

	/**
	 * Fonction renvoyant true si la liste contient au moins un objet qui dpend
	 * des positions gnres pas les dplacements du point li point li
	 * 
	 * @param pointlie
	 * @return
	 */

	public final boolean contientObjetGenereParPointLie(CPointLie pointLie) {
		boolean res = false;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			res = res || elb.estGenereParPointLie(pointLie);
		}
		return res;
	}

	/**
	 * Utilis dans CCommentaire pour savoir si la chaine utilise reprsente une
	 * valeur relle ou complexe dfini avant un certain indice Si indiceMaxi vaut
	 * -1, on recherche dans tous les objets Utilis aussi dans les diteurs de
	 * valeurs complexes
	 */
	// Bug corrig version 3.2.5 : il ne faut pas prendre en compte les objets intermdiaires
	public final CValeurDynamique pointeurValeurReelleOuComplexe(String nomValeur, int indiceMaxi,
			Bool definiApres) {
		if (indiceMaxi == -1) {
			indiceMaxi = size() - 1;
		}
		definiApres.setValue(false);
		int i = 0;
		CElementBase elb = null;
		boolean trouve = false;
		ListIterator<CElementBase> li = listIterator();
		while ((!(trouve)) && (i <= indiceMaxi)) {
			elb = li.next();
			if (!elb.estElementIntermediaire() && elb.estDeNatureCalcul(NatureCalcul.NTouteValeurReelleOuComplexe)) {
				if (((CValeurDynamique) elb).chaineEgaleANom(nomValeur)) {
					trouve = true;
				} else
					i++;
			} else
				i++;
		}
		if (trouve) {
			return (CValeurDynamique) elb;
		} else {
			// On regarde si nomValeur contient le nom d'une valeur d'indice suprieur
			//  indiceMaxi
			i = indiceMaxi + 1;
			while ((!(trouve)) && (i < size())) {
				elb = li.next();
				if (elb.estDeNatureCalcul(NatureCalcul.NTouteValeurReelleOuComplexe)) {
					if (((CValeurDynamique) elb).chaineEgaleANom(nomValeur)) {
						trouve = true;
					} else
						i++;
				} else
					i++;
			}
			if (trouve)
				definiApres.setValue(true);
			return null;
		}
	}
	
	// Utilis dans MtgConverter
	/* Supprim version 4.7.2
	public final CValeurDynamique pointeurValeurOuFonction(String nom, int indiceMaxi) {
		if (indiceMaxi == -1) {
			indiceMaxi = size() - 1;
		}
		int i = 0;
		CElementBase elb = null;
		boolean trouve = false;
		ListIterator<CElementBase> li = listIterator();
		while ((!(trouve)) && (i <= indiceMaxi)) {
			elb = li.next();
			if (elb.estDeNatureCalcul(NatureCalcul.NTouteValeurPourCommentaireDynamique)) {
				if (((CValeurDynamique) elb).nom().equals(nom)) {
					trouve = true;
				} else {
					i++;
				}
			} else {
				i++;
			}
		}
		if (trouve) {
			return (CValeurDynamique) elb;
		} else {
			return null;
		}
	}
	*/
	
	/**
	 * Ajout version 4.7.2 pour utilisation dans MtGConverter
	 * Renvoie un pointeur sur le calcul ou la fonction de nom dont le test nat & natureCalcul est non nul
	 * @param nat : le type de calcul cherch (peut contenir plusieurs types avec Or
	 * @param nom : le nom du calcul cherch
	 * @param indiceMaxi : L'indice maxi de recherche dans la liste
	 * @return
	 */
	public CValeurDynamique pointeurParNatureCalcul(long nat,String nom, int indiceMaxi) {
		if (indiceMaxi == -1) {
			indiceMaxi = size() - 1;
		}
		int i = 0;
		CElementBase elb = null;
		boolean trouve = false;
		ListIterator<CElementBase> li = listIterator();
		while ((!(trouve)) && (i <= indiceMaxi)) {
			elb = li.next();
			if (elb.estDeNatureCalcul(nat)) {
				if (((CValeurDynamique) elb).nom().equals(nom)) {
					trouve = true;
				} else {
					i++;
				}
			} else {
				i++;
			}
		}
		if (trouve) {
			return (CValeurDynamique) elb;
		} else return null;
	}
	

	/**
	 * Fonction renvoyant 0 si la chaine chaine pointe par PChaine ne commence
	 * pas par un nom de valeur relle et sinon renvoie la longueur du nom de
	 * cette valeur et dans PointeurValeur un pointeur sur celle-ci. 
	 * On n'aacepte pas les lments intermdiaires de construction
	 */
	public final int commenceParNomValeur(String pChaine, PointeurValDyn pointeurValeur) {
		CValeurDynamique ptv;
		Point longueurNom = new Point();

		int longueurTrouvee = 0;

		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (!elb.estElementIntermediaire() && elb.estDeNatureCalcul(NatureCalcul.NTouteValeurReelle)) {
				ptv = (CValeurDynamique) elb;
				if (ptv.chaineCommenceParNom(pChaine, longueurNom) && (longueurNom.x > longueurTrouvee)) {
					longueurTrouvee = longueurNom.x;
					pointeurValeur.setPointeur(ptv);
				}
			}
		} // while
		return longueurTrouvee;
	}

	/**
	 * Fonction renvoyant 0 si la chaine chaine pointe par PChaine ne commence
	 * pas par un nom de valeur relle et sinon renvoie la longueur du nom de
	 * cette valeur et dans PointeurValeur un pointeur sur celle-ci.
	 * On n'accepte pas les lments intermdiaires de construction
	 */
	public final int commenceParNomValeurReelleOuComplexe(String pChaine, PointeurValDyn pointeurValeur) {
		CValeurDynamique ptv;
		Point longueurNom = new Point();

		int longueurTrouvee = 0;

		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (! elb.estElementIntermediaire() && elb.estDeNatureCalcul(NatureCalcul.NTouteValeurReelleOuComplexe)) {
				ptv = (CValeurDynamique) elb;
				if (ptv.chaineCommenceParNom(pChaine, longueurNom) && (longueurNom.x > longueurTrouvee)) {
					longueurTrouvee = longueurNom.x;
					pointeurValeur.setPointeur(ptv);
				}
			}
		} // while
		return longueurTrouvee;
	}

	/**
	 * Fonction renvoyant 0 si la chaine pointe par PChaine ne commence pas par
	 * un nom de fonction ou de suite rcurrente et sinon renvoie la longueur du
	 * nom de cette fonction ou suite et dans PointeurFonction un pointeur sur
	 * celle-ci
	 */
	public final int commenceParNomFonctionUtilisateur(String pChaine, PointeurValDyn pointeurFonction) {
		CCalculAncetre ptf;
		Point longueurNom = new Point();

		int longueurTrouvee = 0;
		pointeurFonction.setPointeur(null);
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			// Modifi version 4.7.5
			// if (!elb.estElementIntermediaire() && elb.estDeNatureCalcul(NatureCalcul.NTouteFonctionReelle | NatureCalcul.NDerivee | NatureCalcul.NSuiteRecurrenteReelle)) {
			if (!elb.estElementIntermediaire() && elb.estDeNatureCalcul(NatureCalcul.NTouteFonctionReelle
					| NatureCalcul.NDerivee | NatureCalcul.NTouteSuiteRecurrenteReelle)) {
				ptf = (CCalculAncetre) elb;
				if (ptf.chaineCommenceParNom(pChaine, longueurNom) && (longueurNom.x > longueurTrouvee)) {
					longueurTrouvee = longueurNom.x;
					pointeurFonction.setPointeur(ptf);
				}
			}
		}
		return longueurTrouvee;
	}

	/**
	 * Fonction renvoyant 0 si la chaine pointe par PChaine ne commence pas par
	 * un nom de fonction ou de suite rcurrente et sinon renvoie la longueur du
	 * nom de cette fonction ou suite et dans PointeurFonction un pointeur sur
	 * celle-ci
	 */
	// Modifi version 4.9.7 : On accepte qu'une fonction complexe utilise dans sa formule une fonction relle.
	public final int commenceParNomFonctionComplexeUtilisateur(String pChaine, PointeurValDyn pointeurFonction) {
		CCalculAncetre ptf;
		Point longueurNom = new Point();

		int longueurTrouvee = 0;
		pointeurFonction.setPointeur(null);
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			// Modifi version 4.7.5
			// if (!elb.estElementIntermediaire() && elb.estDeNatureCalcul(NatureCalcul.NTouteFonctionComplexe | NatureCalcul.NSuiteRecurrenteComplexe)) {
			// Modifi version 4.9.7 : On accepte qu'une fonction complexe utilise dans sa formule une fonction relle.
			/*if (!elb.estElementIntermediaire() && elb.estDeNatureCalcul(NatureCalcul.NTouteFonctionComplexe
					| NatureCalcul.NTouteSuiteRecurrenteComplexe)) {*/
			if (!elb.estElementIntermediaire() && elb.estDeNatureCalcul(NatureCalcul.NTouteFonctionReelleOuComplexe
					| NatureCalcul.NTouteSuiteRecurrenteComplexe)) {
				ptf = (CCalculAncetre) elb;
				if (ptf.chaineCommenceParNom(pChaine, longueurNom) && (longueurNom.x > longueurTrouvee)) {
					longueurTrouvee = longueurNom.x;
					pointeurFonction.setPointeur(ptf);
				}
			}
		}
		// for
		return longueurTrouvee;
	}

	/**
	 * Fonction renvoyant 0 si la chaine pointe par pChaine ne pointe pas sur une
	 * chaine reprsentant une valeur ou une fonction mathmatique prdfinie ou
	 * une fonction dfinie par l'utilisateur ou le parametre. Sinon, elle renvoie
	 * la longueur du nom le plus long trouve. Si ParametreTrouve est non nul,
	 * c'est qu'on a trouv le paramtre formel et parametreTrouve contient son indice
	 * dans le tableau parametreFormel , sinon, si PointeurValeur est
	 * diffrent de NULL, c'est qu'un a trouv un nom de valeur ou de fonction
	 * dfinie par l'utilisateur (les suites tant considres comme des
	 * fonctions) sinon c'est qu'on a trouv un nom de fonction prdfinie dont
	 * la nature est renvoye dans NomFonction.
	 * Les calculs intermdiaires ne sont pas autoriss
	 */
	public final int commenceParNomFonctionOuValeurOuParametre(String pChaine, String[] parametreFormel,
			PointeurValDyn pointeurValeur, Int nomFonction, Int parametreTrouve, Int nombreVariables) {
		int longueurNomFonctionPredefinie;
		int longueurNomValeur;
		PointeurValDyn pointeurSurValeur = new PointeurValDyn();
		PointeurValDyn pointeurSurFonction = new PointeurValDyn();
		int longueurNomFonctionUtilisateur;
		int indiceParametre = -1;
		int longueurNomParametre = 0;

		parametreTrouve.setValue(-1);
		if (parametreFormel != null) {

			for (int i = 0; i < parametreFormel.length; i++) {
				if (pChaine.startsWith(parametreFormel[i]) && (parametreFormel[i].length() > longueurNomParametre)) {
					longueurNomParametre = parametreFormel[i].length();
					indiceParametre = i;
					parametreTrouve.setValue(i); // Ligne rajoute version 4.5.2
				}
			}
		}
		longueurNomFonctionPredefinie = CCalculBase.commenceParNomFonctionReellePredefinie(pChaine, nomFonction, nombreVariables);
		longueurNomValeur = commenceParNomValeur(pChaine, pointeurSurValeur);
		longueurNomFonctionUtilisateur = commenceParNomFonctionUtilisateur(pChaine, pointeurSurFonction);
		if ((longueurNomFonctionPredefinie == 0) && (longueurNomValeur == 0)
				&& (longueurNomFonctionUtilisateur == 0) && (longueurNomParametre == 0)) {
			return 0;
		} else // Si la chaine commence par un des trois types de noms, un des
		// trois noms est ncessairement strictement suprieur aux deux
		// autres, car il est interdit pour la syntaxe que deux types de
		// noms diffrents de mme longueur aient les mmes caractres
	  // Modification version 3.4.3 : un paramtre formel d'une fonction peut avoir le mme nom
		// qu'un objet calcul dj dfini. Il a alors priorit :  l'utilisateur de ne pas choisir
		// un paramtre formel ayant un nom d utilis
		{
			if ((longueurNomParametre >= longueurNomFonctionPredefinie)
					&& (longueurNomParametre >= longueurNomValeur)
					&& (longueurNomParametre >= longueurNomFonctionUtilisateur)) {
				parametreTrouve.setValue(indiceParametre);
				return longueurNomParametre;
			} else {
				parametreTrouve.setValue(-1); // Correction bug version 46
				if ((longueurNomFonctionPredefinie > longueurNomValeur)
						&& (longueurNomFonctionPredefinie > longueurNomFonctionUtilisateur)
						&& (longueurNomFonctionPredefinie > longueurNomParametre)) {
					pointeurValeur.setPointeur(null);
					return longueurNomFonctionPredefinie;
				} else {
					if ((longueurNomValeur > longueurNomFonctionPredefinie)
							&& (longueurNomValeur > longueurNomFonctionUtilisateur)
							&& (longueurNomValeur > longueurNomParametre)) {
						pointeurValeur.setPointeur(pointeurSurValeur.getPointeur());
						return longueurNomValeur;
					} else {
						if ((longueurNomFonctionUtilisateur > longueurNomFonctionPredefinie)
								&& (longueurNomFonctionUtilisateur > longueurNomValeur)
								&& (longueurNomFonctionUtilisateur > longueurNomParametre)) {
							pointeurValeur.setPointeur(pointeurSurFonction.getPointeur());
							nombreVariables.setValue(((CCalculAncetre)pointeurSurFonction.getPointeur()).nombreVariables());
							return longueurNomFonctionUtilisateur;
						} else {
							return 0;
						} // par scurit
					}
				}
			}
		}
	}

	/**
	 * Fonction renvoyant 0 si la chaine pointe par pChaine ne pointe pas sur une
	 * chaine reprsentant une valeur ou une fonction mathmatique prdfinie ou
	 * une fonction dfinie par l'utilisateur ou le complexe i ou le parametre.
	 * Sinon, elle renvoie la longueur du nom le plus long trouve. Si
	 * ParametreTrouve est  true, c'est qu'on a trouv le paramtre formel,
	 * sinon, si PointeurValeur est diffrent de NULL, c'est qu'un a trouv un nom
	 * de valeur ou de fonction dfinie par l'utilisateur (les suites tant
	 * considres comme des fonctions) sinon c'est qu'on a trouv un nom de
	 * fonction prdfinie dont la nature est renvoye dans NomFonction.
	 * Les calculs intermdiaires ne sont pas autoriss
	 */
	public final int commenceParNomFonctionOuValeurOuParametreComplexe(String pChaine, String parametreFormel[],
			PointeurValDyn pointeurValeur, Int nomFonction, Int parametreTrouve, Bool complexeiTrouve, Int nombreVariables) {
		int longueurNomFonctionPredefinie;
		int longueurNomValeur;
		PointeurValDyn pointeurSurValeur = new PointeurValDyn();
		PointeurValDyn pointeurSurFonction = new PointeurValDyn();
		int longueurNomFonctionUtilisateur;
		int indiceParametre = -1;
		int longueurNomParametre = 0;

		parametreTrouve.setValue(-1);
		complexeiTrouve.setValue(false);
		pointeurValeur.setPointeur(null);

		if (parametreFormel != null) {

			for (int i = 0; i < parametreFormel.length; i++) {
				if (pChaine.startsWith(parametreFormel[i]) && (parametreFormel[i].length() > longueurNomParametre)) {
					longueurNomParametre = parametreFormel[i].length();
					indiceParametre = i;
					parametreTrouve.setValue(i); // Ligne rajoute version 4.5.2
				}
			}
		}
		longueurNomFonctionPredefinie = CCalculBase.commenceParNomFonctionComplexePredefinie(pChaine, nomFonction, nombreVariables);
		longueurNomValeur = commenceParNomValeurReelleOuComplexe(pChaine, pointeurSurValeur);
		longueurNomFonctionUtilisateur = commenceParNomFonctionComplexeUtilisateur(pChaine, pointeurSurFonction);
		if ((longueurNomFonctionPredefinie == 0) && (longueurNomValeur == 0)
				&& (longueurNomFonctionUtilisateur == 0) && (longueurNomParametre == 0)) {
			if (pChaine.startsWith("i")) {
				complexeiTrouve.setValue(true);
				return 1;
			} else
				return 0;

		} else // Si la chaine commence par un des trois types de noms, un des
		// trois noms est ncessairement strictement suprieur aux deux
		// autres, car il est interdit pour la syntaxe que deux types de
		// noms diffrents de mme longueur aient les mmes caractres
		// Modification version 3.4.3 : un paramtre formel d'une fonction peut avoir le mme nom
		// qu'un objet calcul dj dfini. Il a alors priorit :  l'utilisateur de ne pas choisir
		// un paramtre formel ayant un nom d utilis
		{
			if ((longueurNomParametre >= longueurNomFonctionPredefinie)
					&& (longueurNomParametre >= longueurNomValeur)
					&& (longueurNomParametre >= longueurNomFonctionUtilisateur)) {
				parametreTrouve.setValue(indiceParametre);
				return longueurNomParametre;
			} else {
				parametreTrouve.setValue(-1); // Correction bug version 46
				if ((longueurNomFonctionPredefinie > longueurNomValeur)
						&& (longueurNomFonctionPredefinie > longueurNomFonctionUtilisateur)
						&& (longueurNomFonctionPredefinie > longueurNomParametre)) {
					return longueurNomFonctionPredefinie;
				} else {
					if ((longueurNomValeur > longueurNomFonctionPredefinie)
							&& (longueurNomValeur > longueurNomFonctionUtilisateur)
							&& (longueurNomValeur > longueurNomParametre)) {
						// On teste si on fait rfrence  un calcul ou une variable nomm i.
						// Si c'est le cas, on donne priorit au complexe i
						if ((longueurNomValeur == 1) && pChaine.startsWith("i")) {
							complexeiTrouve.setValue(true);
							return 1;
						} else {
							pointeurValeur.setPointeur(pointeurSurValeur.getPointeur());
							return longueurNomValeur;
						}
					} else {
						if ((longueurNomFonctionUtilisateur > longueurNomFonctionPredefinie)
								&& (longueurNomFonctionUtilisateur > longueurNomValeur)
								&& (longueurNomFonctionUtilisateur > longueurNomParametre)) {
							nombreVariables.setValue(((CCalculAncetre)pointeurSurFonction.getPointeur()).nombreVariables());
							pointeurValeur.setPointeur(pointeurSurFonction.getPointeur());
							return longueurNomFonctionUtilisateur;
						} else {
							return 0;
						} // par scurit
					}
				}
			}
		}
	}

	/**
	 * Fonction renvoyant true si un objet confondu avec l'objet point par ptel a
	 * dj t cr et false sinon. Dans la version Applet, confonduAvec() n'est
	 * pas redfini pour tous les objets mais seulement pour ceux qui peuvent tre
	 * crs  l'intrieur de l'applet
	 */

	public final boolean existeDeja(CElementBase ptel) {
		ListIterator<CElementBase> li = listIterator();
		boolean res = false;
		while (li.hasNext() && (!res)) {
			CElementBase elb = li.next();
			if (!elb.estElementIntermediaire()) res = res || elb.confonduAvec(ptel);
		}
		return res;
	}


	/**
	 * Fonction renvoyant true si un objet confondu avec l'objet point par ptel a
	 * dj t cr avant l'indice indiceDernier et false sinon. Dans la version Applet, confonduAvec() n'est
	 * pas redfini pour tous les objets mais seulement pour ceux qui peuvent tre
	 * crs  l'intrieur de l'applet
	 */

	public final boolean existeDeja(CElementBase ptel, int indiceDernier) {
		ListIterator<CElementBase> li = listIterator();
		int i = 0;
		boolean res = false;
		while (li.hasNext() && (!res) && (i <= indiceDernier)) {
			CElementBase elb = li.next();
			if (!elb.estElementIntermediaire()) res = res || elb.confonduAvec(ptel);
			i++;
		}
		return res;
	}
	
	/**
	 * Fonction renvoyant null si la liste ne contient pas de point image de pt
	 * par la transformation transf et sinon un pointeur sur ce point image
	 * 
	 * @param pt
	 * @param transf
	 * @return
	 */
	public final CPointImage pointImageDePar(CPointAncetre pt, CTransformation transf) {
		CPointImage im = null;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NPoint) {
				if (((CPointAncetre)elb).estPointImage()) {
					CPointImage ptim = (CPointImage) elb;
					if ((ptim.transformation == transf) && (ptim.antecedent == pt)) {
						im = ptim;
						break;
					}
				}
			}
		}
		return im;
	}

	// Ajout version 4.9.9.
	// Quand on supprime tous les objets  partir d'un objet donn il peut arriver u'on supprime ma longueur unit
	//  de la liste. Il faut alors modifier pointeurLongueurUnite
	/**
	 * Fonction retirant un objet de la liste aprs avoir vrifi s'il s'agit de la la longueur unit de la figure.
	 * Si c'est le cas on met pointeurLongueurUnit  null
	 */
	@Override
	public CElementBase remove(int index) {
		CElementBase elb = get(size()-1);
		if (elb.estDeNatureCalcul(NatureCalcul.NMesureLongueur)) if (elb == pointeurLongueurUnite) pointeurLongueurUnite = null;
		return super.remove(index);
	}
	/*
	 * Fonction dtruisant le dernier objet de la liste
	 */

	public final void detruitDernierObjet() {
		int ind = size() - 1;
		if (ind >= 0) {
			remove(ind);
		}
	}

	/**
	 * Fonction remplaant dans tous les objets de la liste l'objet ancienPoint
	 * par l'objet nouveauPoint
	 */
	public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			elb.remplacePoint(ancienPoint, nouveauPoint);
		}
		// Les listes gres de faon internes par les macros doivent tre mises  jour
		etablitListesInternesMacros();
		// Ajout version 3.4.2
		// initialiseDependances(); // Supprim version 4.9.2. Fait par CGestionnaireEdition.SauvegardeFigureEnCours()
	}

	/**
	 * Mthode renvoyant le nombre d'objets dpendant de l'objet pt
	 */
	public final int nombreDependants(CElementBase pt) {
		int res = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.dependDe(pt)) {
				res++;
			}
		}
		return res;
	}
	
	/**
	 * Ajout version 4.8
	 * Dtruit les objets dpendant du prototype proto.
	 * Ces objets ne peuvent tre que des CMacroConstructionIterative et CMacroConstructionRecursive
	 * @param proto
	 */
	public final void retireDependantsDeProto(CPrototype proto) {
		int index = size() - 1;
		while (index >= 0) {
			CElementBase elb = get(index);
			if (elb.dependDeProto(proto)) remove (elb);
			index--;
		}
		// Ajout version 4.8.0
		updateIndexes();
		//
	}

	/**
	 * Mthode retirant le la liste les objets dpendant de pt Pour cela la liste
	 * est parcourue de la fin vers le dbut.
	 * Modifi version 5.4
	 */
	public final void retireDependants(CElementBase pt, MtgInternalFrame cadre) {
		// Ajout version 3.4.2
		initialiseDependances();
		boolean existeImplementationsProtoATraiter = false;
		int index = size() - 1;
		while (index >= 0) {
			CElementBase elb = get(index);
			boolean dep = elb.dependDe(pt);
			// Ajout version 4.9.2 : Il faut dtruire les transformations orphelines, c'est- dire celes qui n'ont plus d'image.
			if (elb.getNature() == NatureObjet.NTransformation) {
				if (nombreImagesParTransformation((CTransformation) elb) == 0) dep = true;
			}
			// Rajout version 4.8 : Si on dtruit un objet source d'une implmentation de prototype, il faut aussi dtruire les objets intermdiaires
			// et finaux qui dpendent de l'implmentation de prototype
			if (elb.impProto != null) {
				CImplementationProto imp = (CImplementationProto) (elb.impProto);
				if (imp.dependDe(pt)) {
					existeImplementationsProtoATraiter = true;
					remove(elb);
					// Ajout version 5.0.1
					if ((cadre != null) && elb.estDeNatureCalcul(NatureCalcul.NVariable)) {
		  			if (((CVariableBornee)elb).dialogueAssocie) {
		  				cadre.creePaneAffichageVariables();
		  			}
					}
					//
					if (elb == pt) break;
					// index--; Modifi version 4.8
				}
				else {
					if (dep) {
						existeImplementationsProtoATraiter = true;
						if (elb.estElementFinal) {
							elb.impProto.nbFinaux--;
						}
						else elb.impProto.nbIntermediaires--;
						remove(elb);
						// Ajout version 5.0.1
						if ((cadre != null) && elb.estDeNatureCalcul(NatureCalcul.NVariable)) {
			  			if (((CVariableBornee)elb).dialogueAssocie) {
			  				cadre.creePaneAffichageVariables();
			  			}
						}
						//
						// Ajout version 3.6.1 pour corriger un bug. En effet le nombre d'objets intermdiaires
						// doitt aussi tre actualis car le reclassement d'objets l'utilise
						// elb.impProto.determineNombreObjetsIntermediaires();
						if (elb == pt) break;
						// index--; Modifi version 4.8
					}		
				}
			}
			else {
				// if (elb.dependDe(pt)) { // Modifi version 4.9.2
				if (dep) {
					if (elb == pointeurLongueurUnite) pointeurLongueurUnite = null;
					remove(elb); // Attention : ce remove(CObject obj) est redfini pour dtruire aussi le composant MtgFormulaEditor
					// Ajout version 5.0.1
					if ((cadre != null) && elb.estDeNatureCalcul(NatureCalcul.NVariable)) {
		  			if (((CVariableBornee)elb).dialogueAssocie) {
		  				cadre.creePaneAffichageVariables();
		  			}
					}
					// associ  un CEditeurFormule
				}
				// Modification version 3.3.3.1 ligne suivante pour optimiser
				if (elb == pt) break;
				// index--; Modifi version 4.8
			}
			index--; // Ajout version 4.8
		}
		//
		if (existeImplementationsProtoATraiter) traiteImplementationsProto();
		// Ajout version 4.8.0
		updateIndexes();
	}

	/*
	public final void retireDependants(CElementBase pt) {
		// Ajout version 3.4.2
		initialiseDependances();
		boolean existeImplementationsProtoOrphelines = false;
		int index = size() - 1;
		while (index >= 0) {
			CElementBase elb = get(index);
			boolean dep = elb.dependDe(pt);
			// Rajout version 4.8 : Si on dtruit un objet source d'une implmentation de prototype, il faut aussi dtruire les objets intermdiaires
			// et finaux qui dpendent de l'implmentation de prototype
			if (elb.impProto != null) {
				CImplementationProto imp = (CImplementationProto) (elb.impProto);
				if (imp.dependDe(pt)) {
					remove(elb);
					if (elb == pt) break;
					// index--; Modifi version 4.8
				}
				else {
					if (dep) {
						if (elb.estElementFinal) {
							elb.impProto.nbFinaux--;
							// Ajout version 3.6.1 pour corriger un bug. En effet le nombre d'objets intermdiaires
							// doitt aussi tre actualis car le reclassement d'objets l'utilise
							elb.impProto.determineNombreObjetsIntermediaires();
							existeImplementationsProtoOrphelines = existeImplementationsProtoOrphelines
									|| (elb.impProto.nbFinaux == 0);
							remove(elb);
							if (elb == pt) break;
							// index--; Modifi version 4.8
						}
					}		
				}
			}
			else {
				if (elb.dependDe(pt)) {
					if (elb == pointeurLongueurUnite) pointeurLongueurUnite = null;
					remove(elb); // Attention : ce remove(CObject obj) est redfini pour dtruire aussi le composant MtgFormulaEditor
					// associ  un CEditeurFormule
				}
				// Modification version 3.3.3.1 ligne suivante pour optimiser
				if (elb == pt) break;
				// index--; Modifi version 4.8
			}
			index--; // Ajout version 4.8
		}
		//
		if (existeImplementationsProtoOrphelines) retireImplementationsProtoOrphelines();
		// Ajout version 4.8.0
		updateIndexes();
	}
	*/
	/*
	// Version 4.9.2.
	public final void retireDependants(CElementBase pt, MtgInternalFrame cadre) {
		// Ajout version 3.4.2
		initialiseDependances();
		boolean existeImplementationsProtoOrphelines = false;
		int index = size() - 1;
		while (index >= 0) {
			CElementBase elb = get(index);
			boolean dep = elb.dependDe(pt);
			// Ajout version 4.9.2 : Il faut dtruire les transformations orphelines, c'est- dire celes qui n'ont plus d'image.
			if (elb.getNature() == NatureObjet.NTransformation) {
				if (nombreImagesParTransformation((CTransformation) elb) == 0) dep = true;
			}
			// Rajout version 4.8 : Si on dtruit un objet source d'une implmentation de prototype, il faut aussi dtruire les objets intermdiaires
			// et finaux qui dpendent de l'implmentation de prototype
			if (elb.impProto != null) {
				CImplementationProto imp = (CImplementationProto) (elb.impProto);
				if (imp.dependDe(pt)) {
					remove(elb);
					// Ajout version 5.0.1
					if ((cadre != null) && elb.estDeNatureCalcul(NatureCalcul.NVariable)) {
		  			if (((CVariableBornee)elb).dialogueAssocie) {
		  				cadre.creePaneAffichageVariables();
		  			}
					}
					//
					if (elb == pt) break;
					// index--; Modifi version 4.8
				}
				else {
					if (dep) {
						if (elb.estElementFinal) {
							elb.impProto.nbFinaux--;
							existeImplementationsProtoOrphelines = existeImplementationsProtoOrphelines
									|| (elb.impProto.nbFinaux == 0);
						}
						else elb.impProto.nbIntermediaires--;
						remove(elb);
						// Ajout version 5.0.1
						if ((cadre != null) && elb.estDeNatureCalcul(NatureCalcul.NVariable)) {
			  			if (((CVariableBornee)elb).dialogueAssocie) {
			  				cadre.creePaneAffichageVariables();
			  			}
						}
						//
						// Ajout version 3.6.1 pour corriger un bug. En effet le nombre d'objets intermdiaires
						// doitt aussi tre actualis car le reclassement d'objets l'utilise
						// elb.impProto.determineNombreObjetsIntermediaires();
						if (elb == pt) break;
						// index--; Modifi version 4.8
					}		
				}
			}
			else {
				// if (elb.dependDe(pt)) { // Modifi version 4.9.2
				if (dep) {
					if (elb == pointeurLongueurUnite) pointeurLongueurUnite = null;
					remove(elb); // Attention : ce remove(CObject obj) est redfini pour dtruire aussi le composant MtgFormulaEditor
					// Ajout version 5.0.1
					if ((cadre != null) && elb.estDeNatureCalcul(NatureCalcul.NVariable)) {
		  			if (((CVariableBornee)elb).dialogueAssocie) {
		  				cadre.creePaneAffichageVariables();
		  			}
					}
					// associ  un CEditeurFormule
				}
				// Modification version 3.3.3.1 ligne suivante pour optimiser
				if (elb == pt) break;
				// index--; Modifi version 4.8
			}
			index--; // Ajout version 4.8
		}
		//
		if (existeImplementationsProtoOrphelines) retireImplementationsProtoOrphelines();
		// Ajout version 4.8.0
		updateIndexes();
	}
*/
	
	// Ancienne version 4.7.9
	/*
	public final void retireDependants(CElementBase pt) {
		// Ajout version 3.4.2
		initialiseDependances();
		boolean existeImplementationsProtoOrphelines = false;
		int index = size() - 1;
		while (index >= 0) {
			CElementBase elb = get(index);
			if (elb.dependDe(pt)) {
				if (elb == pointeurLongueurUnite) pointeurLongueurUnite = null;
				if (elb.estElementFinal) {
					elb.impProto.nbFinaux--;
					// Ajout version 3.6.1 pour corriger un bug. En effet le nombre d'objets intermdiaires
					// doitt aussi tre actualis car le reclassement d'objets l'utilise
					elb.impProto.determineNombreObjetsIntermediaires();
					existeImplementationsProtoOrphelines = existeImplementationsProtoOrphelines
							|| (elb.impProto.nbFinaux == 0);
				}
				remove(elb); // Attention : ce remove(CObject obj) est redfini pour dtruire aussi le composant MtgFormulaEditor
				// associ  un CEditeurFormule
			}
			// Modification version 3.3.3.1 ligne suivante pour optimiser
			if (elb == pt) break;
			index--;
		}
		if (existeImplementationsProtoOrphelines)
			retireImplementationsProtoOrphelines();
	}
	*/

	public final void retireNDerniersElements(int N) {
		for (int i = 0; i < N; i++) {
			detruitDernierObjet();
		}
	}

	/**
	 * Quand on a dtruit le dernier lment final d'une implmentation de
	 * Prototype elle devient inutile et doit tre supprime mais il faut aussi retirer les derniers objets quand ce sont
	 * des objets intermdiaires
	 */
	// Modifi version 5.4
	private final void traiteImplementationsProto() {
		int i =0;
		while (i < size()) {
			CElementBase elb = get(i);
			if (elb.getNature() == NatureObjet.NImplementationProto) {
				CImplementationProto imp = (CImplementationProto) elb;
				if (imp.nbFinaux == 0) {
					// Modifi version 4.9.5.2
					// retireDependants(imp);
					int nbint = imp.nbIntermediaires;
					for (int k = 0; k <= nbint; k++) {
						CElementBase elb2 = get(i);
						remove(elb2);
					}
				}
				else {
	        int deb = i + imp.nbFinaux + imp.nbIntermediaires;
	        if (this.get(deb).estElementIntermediaire()) {
	          // Si le dernier lment d'une CImplmentationProto est un objet intermdiaire
	          // Il faut dtruire tous les objets intermdiaires finaux.
	          for (int j = deb; j > i; j--) {
	            if (this.get(j).estElementIntermediaire()) {
	              this.remove(j);
	              imp.nbIntermediaires--;
	            }
	            else break;
	          }
	        }
	        i += imp.nbFinaux + imp.nbIntermediaires + 1;				
				}
			}
			else i++;
		}
	}

/*
	private final void traiteImplementationsProto() {
		for (int i = 0; i < size(); i++) {
			CElementBase elb = get(i);
			if (elb.getNature() == NatureObjet.NImplementationProto) {
				CImplementationProto imp = (CImplementationProto) elb;
				if (imp.nbFinaux == 0) {
					// Modifi version 4.9.5.2
					// retireDependants(imp);
					int nbint = imp.nbIntermediaires;
					for (int k = 0; k <= nbint; k++) {
						CElementBase elb2 = get(i);
						remove(elb2);
					}
					if (i != 0) i--;
					// i = 0; // On recommence depuis le dbut Supprim version 4.9.5.2
				}
			}
		}
	}
*/
	/**
	 * Fonction rajoute pour la version 1.9.5 Cre une liste d'objets forme de
	 * NbCopies de l'objet ptelg Ces objets ne servent qu' mmoriser la position
	 * d'autres objets lors du positionnement des lieux d'objets ListeSource
	 */
	public final void ajouteClonesDe(CElementGraphique ptelg, int nombreCopies) {
		for (int i = 0; i < nombreCopies; i++) {
			add(ptelg.creeClone());
		}
	}

	/**
	 * // Si la formule d'une fonction a t modifie via DonneValeurA dans //
	 * AppletMathGraph32 // il faut vrifier si une drive de cette fonction a
	 * t cre et si oui // recalculer cette drive et si une drivee de cette
	 * drive // a t cre il faut recommencer et ainsi de suite. // La
	 * procdure doit donc tre rcursive / indiceDebut est le premier indice 
	 * partir duquel commence la recherche
	 * Modifi version 2.0 : fon pointera soit sur une CFonc soit sur une CFoncNVar
	 */
	/* Abandonn version 4.2
	public final void recalculeDeriveesDependantDe(CCalcul fon) {
		// Ajout version 3.4.2
		initialiseDependances();		
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNatureCalcul() == NatureCalcul.NDerivee) {
				CFonc ptfon = (CFonc) elb;
				if (ptfon.getClass() == CDerivee.class) {
					CDerivee ptder = (CDerivee) ptfon;
					if (ptder.dependDe(fon)) {
						// Au chargement d'une drive, depuis l'introduction de CProduitDansFormule et CSommeDansFormule
						// il faut vrifier que le calcul est possible. Si ce n'est pas le cas, on cre une division
						// par 0 pour que la drive n'existe pas.
						if (! ptder.fonctionAssociee.deriveePossible(0)) 
							ptder.calcul = new COperation(ptder.listeProprietaire, new CConstante(ptder.listeProprietaire, 1),
									new CConstante(ptder.listeProprietaire, 0), Operateur.Divi);
						else ptder.creeCalcul();

					}
				}
			}
		}
	}
	*/

	// Ajout pour la version 2.0
	/*
	 * redfinition de gte de faon que si l'argument est gal  -1 la valeur
	 * renvoye soir null
	 */
	public CElementBase get(int indice) {
		if (indice == -1) {
			return null;
		} else {
			return (CElementBase) super.get(indice);
		}
	}

	/*
	 * Ajout pour la version 2.5 pour le chargement des prototypes. cla contient
	 * la classe attendue pour l'objet renvoy& Si l'lment renvoy par get() est
	 * du type CElementGenerique, on va remplacer dans list cet lmnt par un
	 * nouvel lment de type cla sinon on renvoie un pointeur du type souhait
	 * vers l'lment d'indice indice de list
	 */

	@SuppressWarnings({ "rawtypes" })
	public CElementBase get(int indice, Class cla) {
		CElementBase nouv = null;
		if (indice == -1) {
			return null;
		}
		CElementBase elb = (CElementBase) super.get(indice);
		if (elb.getClass() == CElementGenerique.class) {
			// On remplace le CElementGenerique par un nouvel objet de classe cla
			try {
				nouv = (CElementBase) cla.newInstance();
				nouv.elementGeneriqueAssocie = (CElementGenerique) elb;
			} catch (InstantiationException e) {
			} catch (IllegalAccessException e) {
			}
			// On leurre le prototype en lui renvoyant un pointeur sur un lment pas
			// rellement implant;
			return nouv;
		} else {
			return (CElementBase) super.get(indice);
		}
	}

	// Chargement de la liste depuis un flux
	public void chargeDepuisFlux(MathGraphDataInputStream inps) throws IOException {
		try {
			// On lit d'abord l'unit d'angle
			uniteAngle = inps.readCShort();
			// On lit d'abord le nombre d'objets de la liste
			int nombreObjets = inps.readCCount();
			for (int i = 0; i < nombreObjets; i++) {
				CElementBase elb = (CElementBase) inps.readCObject(this);
				elb.index = i; // Ajout version 4.8.0
				add(elb);
			}
			// Il est maintenant inutile se sauvegarder dans le flux l'indice de la
			// longueur unit car c'est lors
			// du chargement de la premire longueur que celle-ci informa la liste
			// qu'elle est la longueur unit
			@SuppressWarnings("unused")
			int indiceLongueurUnite = inps.readCInt();
			/*
			 * if (indiceLongueurUnite == -1) { pointeurLongueurUnite = null; } else {
			 * pointeurLongueurUnite = (CLongueur) get(indiceLongueurUnite); }
			 */
			int indiceMacroParDefaut = inps.readCInt();
			if (indiceMacroParDefaut != -1) {
				macroParDefaut = (CMacro) get(indiceMacroParDefaut);
			}
			//  revoir en java une fois les macros dfinies
			// Certaines macros qui maintiennent une liste interne d'objets doivent
			// tablir cette liste
			//  ce niveau
			etablitListesInternesMacros();
		}
		catch  (IndexOutOfBoundsException e) {
			throw(new IOException());
		}			
		catch  (ClassCastException e) {
			throw(new IOException());
		}
	}

	// Ajout pour la version 2.5 pour le chargement des prototypes
	public void chargePourProtoDepuisFlux(MathGraphDataInputStream inps) throws IOException {
		int nombreObjets = inps.readCCount();
		for (int i = 0; i < nombreObjets; i++) {
			CElementBase elb = (CElementBase) inps.readCObject(this);
			elb.index = i; // Ajout version 4.8.0
			add(elb);
		}
	}

	/* Chargement de la liste depuis un flux java */
	public void read(MathGraphDataInputStream inps) throws IOException {
		try {
			// On lit d'abord l'unit d'angle
			uniteAngle = inps.readInt();
			// On lit d'abord le nombre d'objets de la liste
			int nombreObjets = inps.readInt();
			for (int i = 0; (i < nombreObjets); i++) {
				CElementBase elb = (CElementBase) inps.readObject(this);
				elb.index = i; // Ajout version 4.8.0
				add(elb);
			}
			int indiceLongueurUnite = inps.readInt();
			if (indiceLongueurUnite == -1) {
				pointeurLongueurUnite = null;
			} else {
				pointeurLongueurUnite = (CLongueur) get(indiceLongueurUnite);
			}
			// Inutile dans l'applet
			int indiceMacroParDefaut = inps.readInt();
			if (indiceMacroParDefaut != -1) {
				macroParDefaut = (CMacro) get(indiceMacroParDefaut);
			}
			// Modification version 4.8 (version de document 12) : On peut ventuellement choisir une macro de dmarrage
			// qui est une macro de construction itrative ou rcursive			
			if (numeroVersion < 12) macroDemarrage = null;
			else {
				int indiceMacroDemarrage = inps.readInt();
				macroDemarrage = (CMacro) get(indiceMacroDemarrage);
			}
			//  revoir en java une fois les macros dfinies
			// Certaines macros qui maintiennent une liste interne d'objets doivent
			// tablir cette liste
			//  ce niveau
			etablitListesInternesMacros();
		}
		catch  (IndexOutOfBoundsException e) {
			throw(new IOException());
		}
		catch  (ClassCastException e) {
			throw(new IOException());
		}
	}

	/* Enregistrement de la liste dans un flux java */
	public void write(MathGraphDataOutputStream oups) throws IOException {
		oups.writeInt(uniteAngle);
		oups.writeInt(size());
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			oups.writeObject(elb);
		}
		int indiceLongueurUnite;
		if (pointeurLongueurUnite == null)
			indiceLongueurUnite = -1;
		else
			indiceLongueurUnite = indexOf(pointeurLongueurUnite);
		oups.writeInt(indiceLongueurUnite);
		// Inutile pour l'applet
		int indiceMacroParDefaut;
		if (macroParDefaut == null)
			indiceMacroParDefaut = -1;
		else
			indiceMacroParDefaut = indexOf(macroParDefaut);
		oups.writeInt(indiceMacroParDefaut);
		// Ajout version 4.8
		int indiceMacroDemarrage;
		if (macroDemarrage == null)
			indiceMacroDemarrage = -1;
		else
			indiceMacroDemarrage = indexOf(macroDemarrage);
		oups.writeInt(indiceMacroDemarrage);
		//
	}

	// Pour le chargement des prototypes
	public void readPourProtoDepuisFlux(MathGraphDataInputStream inps) throws IOException {
		int nombreObjets = inps.readInt();
		for (int i = 0; i < nombreObjets; i++) {
			CElementBase elb = (CElementBase) inps.readObject(this);
			elb.index = i; // Ajout version 4.8
			add(elb);
		}
	}

	// Pour l'enregistrement des prototypes
	public void writePourProtoDepuisFlux(MathGraphDataOutputStream oups) throws IOException {
		oups.writeInt(size());
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			oups.writeObject(elb);
		}
	}

	public final boolean egalNomValeurOuFonctionOuMesure(String chaine, int indiceMaxiDansListe) {
		CValeurDynamique ptv;

		boolean trouve = false;
		int i = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext() && !trouve && (i <= indiceMaxiDansListe)) {
			i++;
			CElementBase elb = li.next();
			long nat = elb.getNatureCalcul();
			// Modifi version 4.7.5
			if (((nat & (NatureCalcul.NTouteValeurReelleOuComplexe | NatureCalcul.NTouteFonctionReelle
					| NatureCalcul.NTouteFonctionComplexe | NatureCalcul.NDerivee | NatureCalcul.NTouteSuiteRecurrenteReelle
					| NatureCalcul.NTouteSuiteRecurrenteComplexe)) != 0)
					&& !elb.estElementIntermediaire()) {
				ptv = (CValeurDynamique) elb;
				if (ptv.chaineEgaleANom(chaine))
					trouve = true;
			}
		}
		return trouve;
	}
	
	
	// Modifi version 5.0.2
	// Cette fonction n'est appele par par CImplementationProto.implemente
	// Il faut inclure dans la recherche des calculs ayant dj le mme nom les objets intermdiaires de constructions.
	public final boolean egalNomValeurOuFonctionOuMesure(String chaine, int indiceMaxiDansListe, CElementBase exclu) {
		CValeurDynamique ptv;

		boolean trouve = false;
		int i = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext() && !trouve && (i <= indiceMaxiDansListe)) {
			i++;
			CElementBase elb = li.next();
			if (elb != exclu) {
				long nat = elb.getNatureCalcul();
				// Modifi version 4.7.5
				if (((nat & (NatureCalcul.NTouteValeurReelleOuComplexe | NatureCalcul.NTouteFonctionReelle
						| NatureCalcul.NTouteFonctionComplexe | NatureCalcul.NDerivee | NatureCalcul.NTouteSuiteRecurrenteReelle
						| NatureCalcul.NTouteSuiteRecurrenteComplexe)) != 0)) {
						// && !elb.estElementIntermediaire()) { // Supprim version 5.0.2
					ptv = (CValeurDynamique) elb;
					if (ptv.chaineEgaleANom(chaine))
						trouve = true;
				}
			}
		}
		return trouve;
	}
	
	
	/**
	 * Fonction renvoyant true si chaine est le nom donn  un paramtre dans une fonction relle ou complexe
	 * @param chaine
	 * @return
	 */
	public final boolean egalNomParametreFonction(String chaine) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNatureCalcul() == NatureCalcul.NFonction) {
				CFonc fonc = (CFonc) elb;
				if (chaine.toLowerCase().equals(fonc.chaineVariable)) return true;
			}
			else {
				if (elb.getNatureCalcul() == NatureCalcul.NFonctionComplexe) {
					CFoncComplexe foncComp = (CFoncComplexe) elb;
					if (chaine.toLowerCase().equals(foncComp.chaineVariable)) return true;
				}
			}
		}
		return false;
	}
	
	/**
	 * Fonction renvoyant true seulement si nom est valide pour un nouveau nom de calcul ou de fonction
	 * Si le nom n'est pas valide un message d'erreur est affich
	 * @param nom
	 * @return
	 */
	public final boolean validationNomCalcul(MtgFrame frame, String chNom) {
    if ((chNom.length() == 0) || !EditeurNomCalcul.validationNomVariableOuCalcul(chNom)) {
      frame.dialogueAvertissement(MtgFrame.getString("ch1"));
      return false;
    }
    if (chNom.equals("i")) {
      frame.dialogueAvertissement(MtgFrame.getString("ch2"));
      return false;
    }
    if (CCalculBase.egalNomFonctionReelleOuComplexePredefinie(chNom)) {
      frame.dialogueAvertissement(MtgFrame.getString("ch91"));
      return false;	    	
    }		
    if (egalNomValeurOuFonctionOuMesure(chNom, size()-1)) {
      frame.dialogueAvertissement(MtgFrame.getString("ch3"));
      return false;
    }
    // Modification version 3.4.3 : un calcul peut avoir le mme nom qu'une variable formelle
    // d'une fonction
    /*
    if (egalNomParametreFonction(chNom)) {
    	frame.dialogueAvertissement(getString("ch156"));
    	return false;
    }
    */
    return true;
	}
	
	/**
	 * Fonction renvoyant true seulement si nom est valide pour un nouveau nom de calcul ou de fonction
	 * N'affiche pas de message en cas d'erreur
	 * les paramtres formels de fonctions sont autoriss
	 * @param chNom
	 * @return
	 */
	public final boolean validationNomCalculSansMessage(String chNom) {
    if ((chNom.length() == 0) || !EditeurNomCalcul.validationNomVariableOuCalcul(chNom)) {
      return false;
    }
    if (chNom.equals("i")) {
      return false;
    }
    if (CCalculBase.egalNomFonctionReelleOuComplexePredefinie(chNom)) {
      return false;	    	
    }		
    if (egalNomValeurOuFonctionOuMesure(chNom, size()-1)) {
      return false;
    }
    return true;
	}	
	
	/**
	 * Fonction renvoyant true seulement si nom est valide pour un variable formelle de fonction
	 * Si le nom n'est pas valide un message d'erreur est affich
	 * @param nom
	 * @return
	 */
	public final boolean validationNomVariableFormelle(MtgFrame frame, String chNom) {
    if ((chNom.length() == 0) || !EditeurNomCalcul.validationNomVariableOuCalcul(chNom)) {
      frame.dialogueAvertissement(MtgFrame.getString("ch1"));
      return false;
    }
    if (chNom.equals("i")) {
      frame.dialogueAvertissement(MtgFrame.getString("ch2"));
      return false;
    }
    /*
     * Modification version 3.4.3 : on autorise une  variable formelle  avoir le mme nom
     * qu'un objet de type calcul dj dfini
    if (CCalculBase.egalNomFonctionReelleOuComplexePredefinie(chNom)) {
      frame.dialogueAvertissement(getString("ch91"));
      return false;	    	
    }		
    if (egalNomValeurOuFonctionOuMesure(chNom, size()-1)) {
      frame.dialogueAvertissement(getString("ch3"));
      return false;
    }
    */
    return true;
	}	

	/**
	 * Permet de savoir si la liste contient un point qui soit sur la droite Droite1 et la droite Droite2
	 * Si oui renvoie true, sinon renvoie false
	 * */
	private final boolean existePointConfonduPourIntersection(double x, double y, CDroiteAncetre droite1,
			CDroiteAncetre droite2) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.estDeNature(NatureObjet.NToutPoint)) {
				CPointAncetre ptp = (CPointAncetre) elb;
				// Modification version 3.9 : Si l'interection a dj t cre pour un objet intermdiaire de construction
				// on cre de nouveau l'objet
				// if (ptp.existe) {
				if (! ptp.estElementIntermediaire() && ptp.existe) {
					if (Kernel.zero(ptp.x - x) && Kernel.zero(ptp.y - y) && ptp.appartientParDefinition(droite1)
							&& ptp.appartientParDefinition(droite2)) {
						return true;
					}
				}
			}
		}
		return false;
	}

	/**
	 * Permet de savoir si la liste contient un point dont les coordonnes sont
	 * "presque" gales  x et y et qui soit sur la droite Droite1 et le cercle
	 * Cercle1 Si oui renvoirt un pointeur sur ce point, sinon renvoit null
	 * */
	private final CPointAncetre existePointConfonduPourIntersection(double x, double y, CDroiteAncetre droite1,
			CCercle cercle1) {
		ListIterator<CElementBase> li = listIterator();
		CPointAncetre trouve = null;
		while (li.hasNext() && (trouve == null)) {
			CElementBase elb = li.next();
			if (elb.estDeNature(NatureObjet.NToutPoint)) {
				CPointAncetre ptp = (CPointAncetre) elb;
				// Modification version 3.9 : Si l'interection a dj t cre pour un objet intermdiaire de construction
				// on cre de nouveau l'objet
				// if (ptp.existe) {
				if (! ptp.estElementIntermediaire() && ptp.existe) {
					if (Kernel.zero(ptp.x - x) && Kernel.zero(ptp.y - y)
							&& ptp.appartientParDefinition(droite1) && ptp.appartientParDefinition(cercle1)) {
						trouve = ptp;
					}
				}
			}
		}
		return trouve;
	}

	/**
	 * Permet de savoir si la liste contient un point dont les coordonnes sont
	 * "presque" gales  x et y et qui soit sur la droite Droite1 et le cerlce
	 * Cercle1 Si oui renvoirt un pointeur sur ce point, sinon renvoit null
	 * */
	public final CPointAncetre existePointConfonduPourIntersection(double x, double y, CCercle cercle1,
			CCercle cercle2) {
		ListIterator<CElementBase> li = listIterator();
		CPointAncetre trouve = null;
		while (li.hasNext() && (trouve == null)) {
			CElementBase elb = li.next();
			if (elb.estDeNature(NatureObjet.NToutPoint)) {
				CPointAncetre ptp = (CPointAncetre) elb;
				// Modification version 3.9 : Si l'interection a dj t cre pour un objet intermdiaire de construction
				// on cre de nouveau l'objet
				// if (ptp.existe) {
				if (! ptp.estElementIntermediaire() && ptp.existe) {
					if (Kernel.zero(ptp.x - x) && Kernel.zero(ptp.y - y) && ptp.appartientParDefinition(cercle1)
							&& ptp.appartientParDefinition(cercle2)) {
						trouve = ptp;
					}
				}
			}
		}
		return trouve;
	}

	/**
	 * Etudie si la droite droite1 et la droite droite2 sont scants. S'ils le sont
	 * et si un des deux points d'intersection a dj t cr, renvoit un
	 * pointeur sur ce point. Sinon renvoit null. La droite et le cercle existent
	 * avant appel Renvoie true dans DejaCree quand les deux points d'intersection
	 * ont dj t crs et false sinon
	 * */
	public final boolean pointIntersectionDejaCree(CDroiteAncetre droite1, CDroiteAncetre droite2) {
		byte positionRelative;
		// Coordonnes des points d'intersection
		Point2D.Double pointInt1 = new Point2D.Double();
		positionRelative = Kernel.intersection(droite1.point_x, droite1.point_y, droite1.vecteur, droite2.point_x,
				droite2.point_y, droite2.vecteur, pointInt1);
		if (positionRelative != PositionDroites.Paralleles) {
			return existePointConfonduPourIntersection(pointInt1.x, pointInt1.y, droite1, droite2);
		} else
			return false;
	}

	/**
	 * Etudie si la droite Droite et le cercle Cercle sont scants. S'ils le sont
	 * et si un des deux points d'intersection a dj t cr, renvoit un
	 * pointeur sur ce point. Sinon renvoit null. La droite et le cercle existent
	 * avant appel Renvoie true dans DejaCree quand les deux points d'intersection
	 * ont dj t crs et false sinon
	 * */
	public final CPointAncetre pointIntersectionDejaCree(CDroiteAncetre droite1, CCercle cercle1, Bool dejaCree) {
		byte positionRelative;
		// Coordonnes des points d'intersection
		Point2D.Double pointInt1 = new Point2D.Double();
		Point2D.Double pointInt2 = new Point2D.Double();
		dejaCree.setValue(false);
		positionRelative = Kernel.intersectionDroiteCercle(droite1.point_x, droite1.point_y, droite1.vecteur,
				cercle1.centre_x, cercle1.centre_y, cercle1.rayon, pointInt1, pointInt2);
		CPointAncetre ptPoint1 = existePointConfonduPourIntersection(pointInt1.x, pointInt1.y, droite1, cercle1);
		CPointAncetre ptPoint2 = existePointConfonduPourIntersection(pointInt2.x, pointInt2.y, droite1, cercle1);
		if (positionRelative == PositionDroiteCercle.Secants) {
			dejaCree.setValue((ptPoint1 != null) && (ptPoint2 != null));
			if (dejaCree.getValue())
				return null;
			else {
				if (ptPoint1 != null)
					return ptPoint1;
				else {
					if (ptPoint2 != null)
						return ptPoint2;
					else
						return null;
				}
			}
		}
		else {
			if (positionRelative == PositionDroiteCercle.Tangents) {
				dejaCree.setValue(ptPoint1 != null);
				if (dejaCree.getValue()) return ptPoint1;
				else return null;
			}
			else return null;
		}			
	}

	/**
	 * Etudie si la droite Droite et le cercle Cercle sont scants. S'ils le sont
	 * et si un des deux points d'intersection a dj t cr, renvoie un
	 * pointeur sur ce point. Sinon renvoit null. La droite et le cercle existent
	 * avant appel Renvoit true dans DejaCree quand les deux points d'intersection
	 * ont dj t crs et false sinon
	 * */
	public final CPointAncetre pointIntersectionDejaCree(CCercle cercle1, CCercle cercle2, Bool dejaCree) {
		byte positionRelative;
		// Coordonnes des points d'intersection
		Point2D.Double pointInt1 = new Point2D.Double();
		Point2D.Double pointInt2 = new Point2D.Double();
		dejaCree.setValue(false);
		positionRelative = Kernel.intersectionCercleCercle(cercle1.centre_x, cercle1.centre_y, cercle1.rayon,
				cercle2.centre_x, cercle2.centre_y, cercle2.rayon, pointInt1, pointInt2);
		CPointAncetre ptPoint1 = existePointConfonduPourIntersection(pointInt1.x, pointInt1.y, cercle1, cercle2);
		CPointAncetre ptPoint2 = existePointConfonduPourIntersection(pointInt2.x, pointInt2.y, cercle1, cercle2);
		if (positionRelative == PositionCercleCercle.Secants) {
			dejaCree.setValue((ptPoint1 != null) && (ptPoint2 != null));
			if (dejaCree.getValue())
				return null;
			else {
				if (ptPoint1 != null)
					return ptPoint1;
				else {
					if (ptPoint2 != null)
						return ptPoint2;
					else
						return null;
				}
			}
		}
		else {
			if (positionRelative == PositionCercleCercle.Tangents) {
				dejaCree.setValue(ptPoint1 != null);
				if (dejaCree.getValue()) return ptPoint1;
				else return null;
			}
			else return null;
		}
	}

	/**
	 * Fonction renvoyant true si la liste ne contient pas de point ou de droite
	 * autre que element ayant pour nom st
	 * Modifi version 4.8.0
	 * @param element peut tre null
	 * @param st
	 * @return
	 */
	private final boolean existePointOuDroiteMemeNom(CElementGraphique element, String st) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (! elb.estElementIntermediaire() && (elb.estDeNature(NatureObjet.NToutPoint + NatureObjet.NDroite))) {
				CElementGraphique ptp = (CElementGraphique) elb;
				if (ptp != element) {
					if (ptp.nom.length() != 0) {
						// Modification de la version 2.5
						// if (!NomEl.Compare(St))
						if (ptp.nom.equals(st)) return true;
					}
				}
			}
		}
		return false;
	}

	/**
	 * Fonction renvoyant true si la liste ne contient pas de point ou de droite
	 * ayant pour nom st
	 * Modifi version 4.8.0
	 */
	// Modifi version 4.9.9. Ajout d'un paramtre intermPermis
	private final boolean existePointOuDroiteMemeNom(StringBuffer st, boolean intermPermis) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			// if (!elb.estElementIntermediaire() && (elb.estDeNature(NatureObjet.NToutPoint + NatureObjet.NDroite))) {
			if ((!elb.estElementIntermediaire() || intermPermis) && (elb.estDeNature(NatureObjet.NToutPoint + NatureObjet.NDroite))) {
				CElementGraphique ptp = (CElementGraphique) elb;
				if (ptp.nom.length() != 0) {
					// Modification de la version 2.5
					// if (!NomEl.Compare(St))
					if (ptp.nom.contentEquals(st)) return true;
				}
			}
		}
		return false;
	}		
	/**
	 * Fonction renvoyant true s'il existe un calcul, une fonction ou un paramre de fonction
	 * autre que valeur, qui ne soit pas un objet intermdiaire et ayant pour nom st
	 * @param valeur
	 * @param st
	 * @return
	 */
	public final boolean existeCalculOuVariableOuFonctionOuParametreMemeNom
		(CValeurDynamique valeur, String st, boolean intermPermis) {
		for (int i = 0; i < size(); i++) {
			CElementBase elb = get(i);
			if (elb.estDeNatureCalcul(NatureCalcul.NToutCalculNomme) && (!elb.estElementIntermediaire() || intermPermis)) {
				CValeurDynamique ptv = (CValeurDynamique) elb;
				if (ptv != valeur) {
					if (ptv.nom().equals(st))
						return true;
				}
			}
		}
		return false;
	}

	/**
	 * Fonction gnrant un nom libre pour un point nom nomm
	 * 
	 * @return
	 */
	public final String genereNomPourPoint(boolean intermPermis) {
		String ch = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
		StringBuffer nomGenere = new StringBuffer(64);
		// Modifi version 4.8
		for (int i = 0;; i++) {
			for (int j = 0; j < ch.length(); j++) {
				nomGenere.delete(0, nomGenere.length());
				nomGenere.append(ch.charAt(j));
				if (i != 0)
					nomGenere.append(Integer.toString(i));
				if (!existePointOuDroiteMemeNom(nomGenere, intermPermis))
					return nomGenere.toString();
			}
		}
		// return ""; // Pour satisfaire le compilateur
	}
	
	/**
	 * Fonction gnrant un nom libre pour une droite
	 * commenant par la lettre contenue dans ch
	 * @return
	 */
	// Modifi version 4.9.9
	// public final String genereNomPourDroite() {
	public final String genereNomPourDroite(String ch) {

		StringBuffer nomGenere = new StringBuffer(64);
		nomGenere.append(ch);
		// Modifi version 4.8
		for (int i = 0;; i++) {
			nomGenere.delete(ch.length(), nomGenere.length());
			if (i != 0)
				nomGenere.append(Integer.toString(i));
			if (!existePointOuDroiteMemeNom(null, nomGenere.toString()))
				return nomGenere.toString();
		}
	}	

	/**
	 * Fonction gnrant un nom libre pour un point ou un edroite nom nomm commenant par le
	 * caractre car
	 * Si car est accept, on ne rajoute pas de chiffre
	 * @return
	 */
	public final String genereNomPourPointOuDroite(char car) {
		StringBuffer nomGenere = new StringBuffer(64);
		// Modifi version 4.8
		for (int i = 0;; i++) {
			nomGenere.delete(0, nomGenere.length());
			nomGenere.append(car);
			if (i != 0) nomGenere.append(Integer.toString(i));
			if (!existePointOuDroiteMemeNom(null, nomGenere.toString()))
				return nomGenere.toString();
		}
		// return "";
	}
	
	/**
	 * Fonction gnrant un nom libre pour un point ou une droite dj nomm commenant par le
	 * caractre car
	 * Si car est accept, on ne rajoute pas de chiffre
	 * @param el : le nom ou la droite dont on doit changer le nom
	 * @return
	 * Modifi version 4.8 pour rajouter un paramtre ideb qui est le premier entier  rajouter au nom pour tester la validit du nouveau nom
	 */
	public final String genereNomPourPointOuDroite(char car, CElementGraphique el, int ideb) {
		StringBuffer nomGenere = new StringBuffer(64);
		// Modifi version 4.8
		for (int i = ideb;; i++) {
			nomGenere.delete(0, nomGenere.length());
			nomGenere.append(car);
			if (i != 0) nomGenere.append(Integer.toString(i));
			if (!existePointOuDroiteMemeNom(el, nomGenere.toString()))
				return nomGenere.toString();
		}
		// return "";
	}	

	/**
	 * Fonction gnrant un nom libre pour un point nom nomm commenant par le
	 * caractre car suivi d'un chiffre  partir de 1
	 * Rajout pour la version 1.5.1 pour les courbes de fonctions
	 * @return
	 */
	public final String genereNomPourPointOuDroiteAvecChiffre(char car) {
		StringBuffer nomGenere = new StringBuffer(64);
		// Modifi version 4.8
		for (int i = 1;; i++) {
			nomGenere.delete(0, nomGenere.length());
			nomGenere.append(car);
			nomGenere.append(Integer.toString(i));
			if (!existePointOuDroiteMemeNom(null, nomGenere.toString()))
				return nomGenere.toString();
		}
		// return "";
	}	
	
	/**
	 * Fonction gnrant un calcul dont le nom commence par ch et qui se soit pas dj un nom
	 * utilis pour un calcul ou une mesure nomme comme la mesure de coefficieint directeur
	 * Si chiffreImpose est true, on commence  ch suivi du chiffre 1 sinon on commence  ch
	 * @param ch
	 * @return
	 */
	public final String genereNomPourCalcul(String ch, boolean chiffreImpose) {
		StringBuffer nomGenere = new StringBuffer(64);
		int ideb;
		if (chiffreImpose) ideb = 1; else ideb = 0;
		// Modifi version 4.8
		for (int i = ideb;; i++) {
			nomGenere.delete(0, nomGenere.length());
			nomGenere.append(ch);
			if (i != 0)
				nomGenere.append(Integer.toString(i));
			if (!egalNomValeurOuFonctionOuMesure(nomGenere.toString(), size() - 1)
					&& !egalNomParametreFonction(nomGenere.toString())
					&& !CCalculBase.egalNomFonctionReelleOuComplexePredefinie(nomGenere.toString())) // Ligne rajoute version 3.5.1
				return nomGenere.toString();
		}
	}
	
	/**
	 * Fonction gnrant un calcul dont le nom commence par ch et qui se soit pas dj un nom
	 * utilis pour un calcul ou une mesure nomme comme la mesure de coeffcieint directeur
	 * Utilise dans CImplemnetationProto.implemente()
	 * @param ch
	 * @parma exclu l'lment dont le nom n'est pas pris en compte
	 * @return
	 * Modifi version 4.8 : On ajoute un troisime paramtre qui est l'indice du premier entier  rajouter au nom ch pour gnrer un nouveau nom
	 * Entirement revu. Plus de StringBuffer
	 */
	public final String genereNomPourCalcul(String ch, CElementBase exclu, int ideb) {
		String nomGenere;
		// Modifi version 4.8
		for (int i = ideb; ; i++) {
			nomGenere = ch;
			// if (i != 0) Supprim version 5.0.2
			nomGenere += Integer.toString(i);
			if (!egalNomValeurOuFonctionOuMesure(nomGenere, size() - 1, exclu)
					&& !egalNomParametreFonction(nomGenere)
					&& !CCalculBase.egalNomFonctionReelleOuComplexePredefinie(nomGenere)) // Ligne rajoute version 3.5.1
					
				return nomGenere.toString();
		}
		// return "";
	} 

	/**
	 * Fonction remplaant dans tous les commentaires faisant un affichage
	 * dynamique de la valeur nomm ancienNom par son nouveau nom nouveauNom
	 * 
	 * @param ancienNom
	 * @param nouveauNom
	 */
	public void remplaceNomValeurDynamiqueDansCommentairesOuLatex(String ancienNom, String nouveauNom) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.estDeNature(NatureObjet.NCommentaireOuLatex)) {
				((CCommentaire) elb).remplaceNomValeurDynamique(ancienNom, nouveauNom);
			}
		}
	}
	/**
	 * Fonction remplaant dans tous les commentaires dpendant de el faisant un affichage
	 * dynamique de la valeur nomm ancienNom par son nouveau nom nouveauNom
	 * 
	 * @param ancienNom
	 * @param nouveauNom
	 */

	public void remplaceNomValeurDynamiqueDansCommentairesOuLatexDependantDe(String ancienNom, String nouveauNom, CElementBase el) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			// Modification version 3.5
			// if ((elb.estDeNature(NatureObjet.NCommentaireOuLatex)) && (elb.dependDe(el))) {
			if (elb.estDeNature(NatureObjet.NCommentaireOuLatex)) {
				// Modification version 3.5
				/*
				((CCommentaire) elb).remplaceNomValeurDynamique(ancienNom, nouveauNom);
				if (elb.getNature() == NatureObjet.NLatex) ((CLatex)elb).creeFormule();
				*/
				CCommentaire com = (CCommentaire) elb;
				if (com.listeValeursDynamiquesUtilisees.contains(el)) {
					com.remplaceNomValeurDynamique(ancienNom, nouveauNom);
					if (elb.getNature() == NatureObjet.NLatex) ((CLatex)elb).creeFormule();
				}
			}
		}
	}
	
	/**
	 * Fonction renvoyant le nombre d'objets de la liste pour l'utilisateur. Les
	 * trabsformations et bipoints ne sont pas comptes Si traceObjetsInterm est
	 * true les objets intermdiaires sont pris en compte
	 * 
	 * @param traceObjetsinTerm
	 * @return
	 */
	public final int nombreElementsPourHistorique(boolean traceObjetsInTerm) {
		ListIterator<CElementBase> li = listIterator();
		int i = 0;
		while (li.hasNext()) {
			CElementBase elb = li.next();
			boolean estInterm = elb.estElementIntermediaire();
			if (!estInterm || (estInterm && traceObjetsInTerm)) {
				if (!elb.estDeNature(NatureObjet.NTransformation | NatureObjet.NBipoint)) {
					i++;
				}
			}
		}
		return i;
	}

	public final int nombreObjetsParNature(long nat) {
		ListIterator<CElementBase> li = listIterator();
		int compt = 0;
		boolean trouve = false;
		while (li.hasNext() && !(trouve)) {
			CElementBase elb = li.next();
			if (elb.estDeNature(nat))
				compt++;
		}
		return compt;
	}

	public final int nombreObjetsParNatureVisibles(long nat) {
		ListIterator<CElementBase> li = listIterator();
		int compt = 0;
		boolean trouve = false;
		while (li.hasNext() && !(trouve)) {
			CElementBase elb = li.next();
			if (!elb.estElementIntermediaire() && elb.estDeNature(nat)) {
				if (!((CElementGraphique) elb).masque)
					compt++;
			}
		}
		return compt;
	}

	public final int nombreObjetsParNatureCalcul(long nat) {
		ListIterator<CElementBase> li = listIterator();
		int compt = 0;
		boolean trouve = false;
		while (li.hasNext() && !(trouve)) {
			CElementBase elb = li.next();
			if (elb.estDeNatureCalcul(nat))
				compt++;
		}
		return compt;
	}
	
	public final int nombreObjetsParNatureCalculNonIntermediaires(long nat) {
		ListIterator<CElementBase> li = listIterator();
		int compt = 0;
		// boolean trouve = false; Inutile version 5.1.4
		// while (li.hasNext() && !(trouve)) {
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (!elb.estElementIntermediaire() && elb.estDeNatureCalcul(nat))
				compt++;
		}
		return compt;
	}
	
	public final int nombreObjetsParNatureGraphiqueNonIntermediaires(long nat) {
		ListIterator<CElementBase> li = listIterator();
		int compt = 0;
		boolean trouve = false;
		while (li.hasNext() && !(trouve)) {
			CElementBase elb = li.next();
			if (!elb.estElementIntermediaire() && elb.estDeNature(nat))
				compt++;
		}
		return compt;
	}

	/**
	 * Fonction renvoyant le nombre d'images par la transformation transf ne
	 * dpendant pas de exclu
	 * 
	 * @param transf
	 */
	// Modifi version 4.9.2 pour optimisation
	/*
	public final int nombreImagesParTransformationNonDepDe(CTransformation transf, CElementBase exclu) {
		ListIterator<CElementBase> li = listIterator();
		int res = 0;
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (!elb.dependDe(exclu)) {
				CElementBase elb2 = elb.antecedentDirect();
				if ((elb2 != elb) && (elb2.getNature() == NatureObjet.NTransformation) && (elb2 == transf))
					res++;
			}
		}
		return res;
	}
	*/
	public final int nombreImagesParTransformationNonDepDe(CTransformation transf, CElementBase exclu) {
		int ind = indexOf(transf);
		int res = 0;
		for (int i = ind+1; i < size(); i++) {
			CElementBase elb = get(i);
			if (!elb.dependDe(exclu)) {
				CElementBase elb2 = elb.antecedentDirect();
				if ((elb2 != elb) && (elb2.getNature() == NatureObjet.NTransformation) && (elb2 == transf))
					res++;
			}
		}
		return res;
	}


	/*
	public final int nombreImagesParTransformation(CTransformation transf) {
		ListIterator<CElementBase> li = listIterator();
		int res = 0;
		while (li.hasNext()) {
			CElementBase elb = li.next();
			CElementBase elb2 = elb.antecedentDirect();
			if ((elb2 != elb) && (elb2.getNature() == NatureObjet.NTransformation) && (elb2 == transf))
				res++;
		}
		return res;
	}
	*/
	// Rcrit version 4.9.2
	public final int nombreImagesParTransformation(CTransformation transf) {
		int ind = indexOf(transf);
		int res = 0;
		for (int i = ind+1; i < size(); i++) {
			CElementBase elb = get(i);
			CElementBase elb2 = elb.antecedentDirect();
			if ((elb2 != elb) && (elb2.getNature() == NatureObjet.NTransformation) && (elb2 == transf))
				res++;
		}
		return res;
	}

	
	public CTransformation transformationConfondueAvec(CTransformation trans) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (!elb.estElementIntermediaire() && (elb.getNature() == NatureObjet.NTransformation)) {
				CTransformation t = (CTransformation) elb;
				if (t.getClass() == trans.getClass()) {
					if (t.confonduAvec(trans)) return t;
				}
			}
		}
		return null;
	}


	/**
	 * Fonction renvoyant true seulement si la liste contient un objet non
	 * intermdiaire autre que element ayant pour nom st
	 * 
	 * @param element
	 * @param st
	 * @return
	 */
	public final boolean existeMemeNomNonIntermediaire(CElementGraphique element, String st) {
		ListIterator<CElementBase> li = listIterator();
		boolean trouve = false;
		while ((li.hasNext()) && !trouve) {
			CElementBase elb = li.next();
			if (elb.estDeNature(NatureObjet.NToutObjetSaufDuplique) && !elb.estElementIntermediaire()) {
				CElementGraphique elg = (CElementGraphique) elb;
				if (elg != element) {
					if (elg.nom.length() != 0) {
						if (elg.nom.equals(st))
							trouve = true;
					}
				}
			}
		}
		return trouve;
	}

	public final boolean validationNomPoint(String chaine) {
		char premierCar, carPrecedent, carEnCours;
		boolean valable;
		// CString ChaineVerification;

		if (chaine.length() == 0)
			return true;
		premierCar = chaine.charAt(0);
		if (!FonteNom.lettre(premierCar))
			return false;
		// Le premier caractre est bien une lettre
		carPrecedent = premierCar;
		valable = true;
		for (int i = 1; i < chaine.length(); i++) {
			carEnCours = chaine.charAt(i);
			// Cas d'une lettre
			if (FonteNom.lettre(carEnCours)) {
				if (FonteNom.chiffre(carPrecedent) || FonteNom.prime(carPrecedent) || FonteNom.lettre(carPrecedent))
					valable = false;
			} else {
				if (FonteNom.prime(carEnCours)) {
					if (FonteNom.chiffre(carPrecedent) || FonteNom.prime(carPrecedent))
						valable = false;
				} else {
					if (!(FonteNom.chiffre(carEnCours)))
						valable = false;
				}
			}
			carPrecedent = carEnCours;
		}
		return valable;
	}
	
	public final boolean validationNomDroite(String chaine) {
		if (chaine.startsWith("(") && chaine.endsWith(")")) return validationNomPoint(chaine.substring(1, chaine.length() - 1));
		else return validationNomPoint(chaine);
	}

	/**
	 * Fonction rajoute pour pourvoir dplacer le nom d'un objet
	 * 
	 * @param typeCherche
	 * @param point
	 * @param info
	 * @param listeExclusion
	 * @return
	 */
	public final CElementGraphique premierObjetNomProcheDe(Graphics2D dc, long typeCherche, Point point) {
		long nat;
		CElementBase ptel;
		CElementGraphique ptelg;
		double dis;

		// lorsque la liste est vide, aucun objet n'est proche
		if (size() == 0)
			return null;
		// On traite tous les lments de la liste

		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			ptel = li.next();
			nat = ptel.getNature();
			if ((nat & typeCherche) != 0) {
				// Modification par rapport au C++
				ptelg = (CElementGraphique) ptel;
				if (!(ptelg.masque || ptelg.nomMasque)) {
					dis = ptelg.distanceNom(dc, point.x, point.y, true);
					// LPtoDP(Dc,Poindis,1);
					if (dis == 0)
						return ptelg;// On est  l'intrieur de l'affichage du nom de
					// l'objet
				} // if (!Ptel.Masque)
			} // if (nat and type_cherche <>0)
		}// Fin de la boucle de traitement de chacun des lments de la liste
		return null; // Aucun nom d'objet proche
	}

	public final boolean nomIndispensable(CElementGraphique el) {
		CElementBase ptel;
		boolean res = false;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			ptel = li.next();
			res = res || ptel.nomIndispensable(el);
		}
		return res;
	}

	/**
	 * Fonction ajoutant  la figure un point libre nomm U, une droite
	 * horizontale un point li  cette droite nomm V et une longueur unit UV de
	 * faon que  l'cran UV mesure vraiment 1 cm
	 */
	public final void ajouteSegmentLongueurUnite(int width, int height) {
		double res = (double) Toolkit.getDefaultToolkit().getScreenResolution() / 2.54;
		// res contient la rsolution de l'cran en points par cm
		// D'abord un point libre nomm U cach par dfaut
		// Changement pour vrsion 3.3 : Le point U est punais par dfaut et ainsi en zoomant sur la figure il restera visible
		double k = MtgFrame.coefMultForFonts(width,  height);
		CPointBase ptb = new CPointBase(this, null, false, Couleur.TraduitCouleur(1), false, -10d*k, 4d*k, false, "U",
				(int)(k*11), MotifPoint.PetitRond, false, res / 2, res / 2, false);
		add(ptb);
		// Ensuite une droite horizontale passant par ce point
		CDroiteDirectionFixe ptd = new CDroiteDirectionFixe(this, null, false, Couleur.TraduitCouleur(1), false,
				0d, 0d, true, "", (int)(k*13), StyleTrait.traitFin, CDroite.abscisseNomParDefaut, ptb, true, 1);
		add(ptd);
		// On ajoute un point li  la droite horizontale
		CPointLieDroite ptl = new CPointLieDroite(this, null, false, Couleur.TraduitCouleur(1), false, 3d*k, 4d*k,
				false, "V", (int)(k*11), MotifPoint.PetitRond, true, res, ptd, false);
		add(ptl);
		// On cre un segment joignant ces deux points
		CSegment ptseg = new CSegment(this, null, false, Couleur.TraduitCouleur(1), false, StyleTrait.traitFin,
				ptb, ptl);
		add(ptseg);
		// On cre maintenant le milieu de ce segment
		CMilieu ptm = new CMilieu(this, null, false, Couleur.TraduitCouleur(1), false, 0d, 0d, true, "", (int)(k*13),
				MotifPoint.PetitRond, ptb, ptl, false);
		add(ptm);
		// On ajoute un commentaire li  ce milieu contenant la lettre 1
		// Ligne suivante modifie version 2.0
		/* Modifi version 5.0
		CCommentaire ptcom = new CCommentaire(this, null, false, Couleur.TraduitCouleur(1), 0d, 0d, 0d, 6d,
				false, ptm, 1, StyleEncadrement.Sans, false, documentProprietaire.couleurFond, "1", CAffichageLieAPoint.alignementHorizontalCentre,
				CAffichageLieAPoint.alignementVerticalHaut);
		*/
		CCommentaire ptcom = new CCommentaire(this, null, false, Couleur.TraduitCouleur(1), 0d, 0d, 0d, 6d,
				false, ptm, (int)(k*11), StyleEncadrement.Sans, false, documentProprietaire.couleurFond, "1", CAffichageLieAPoint.alignementHorizontalCentre,
				CAffichageLieAPoint.alignementVerticalHaut);
		add(ptcom);
		ptcom.determineDependances();
		// On cre maintenant la longueur unit
		CLongueur lon = new CLongueur(this, null, false, ptb, ptl);
		add(lon);
		pointeurLongueurUnite = lon;
	}

	/**
	 * Ajout de la constante pi
	 */
	public final void ajouteConstantePi() {
		CCalcConst ptCalc = new CCalcConst(this, null, false);
		ptCalc.nomCalcul = "pi";
		ptCalc.chaineCalcul = "3.14159265358979323846";
		ptCalc.creeCalcul();
		ptCalc.calculeValeurConstante();
		add(ptCalc);
	}

	/**
	 * Fonction ajoutant un repre orthonormal au centre de la fentre et crant
	 * une longueur unit
	 * Modification pour la version 1.3.6 : On n'utilise plus de rotation pour que le repre reste
	 * orthonormal quand on change l'unit d'angle de la figure.
	 * 
	 * @param dimf
	 *          : dimensions de la fentre contenant la figure
	 * @param quadrillageHor
	 *          :true pour un quadrillage horizontal
	 * @param quadrillageVer
	 *          : true pour un quadrillage vertical
	 * @param pointille
	 *          : true pour un repre avec pointills
	 * @param nomO
	 *          : Nom de l'origine
	 * @param nomI
	 *          : Nom du point de cooronnes (1;0)
	 * @param nomJ
	 *          : Nom du point de cooronnes (0;1)
	 * @param nomVecteuri
	 *          : si chaine non vide, on rajoute une vecteur OI et un commentaire
	 *          vecteur
	 * @param nomVecteurj
	 *          : si chaine non vide, on rajoute une vecteur OJ et un commentaire
	 *          vecteur
	 */
	public final void ajouteRepereOrthonormal(DimensionFenetre dimf, boolean quadrillageHor,
			boolean quadrillageVer, boolean pointille, String nomO, String nomI, String nomJ, String nomVecteuri,
			String nomVecteurj, CValeurAssocieeAVariable originex, CValeurAssocieeAVariable originey,
			CValeurAssocieeAVariable unitex, CValeurAssocieeAVariable unitey, boolean nomsIJmasques) {
		double k = MtgFrame.coefMultForFonts(dimf.longueurAbscisses,  dimf.longueurOrdonnees);
		double res = (double) Toolkit.getDefaultToolkit().getScreenResolution() / 2.54;
		// res contient la rsolution de l'cran en points par cm
		// D'abord un point libre nomm U cach par dfaut
		CPointBase O = new CPointBase(this, null, false, Couleur.TraduitCouleur(1), false, -15d*k, 4d*k, false, nomO,
				(int)(k*16), MotifPoint.PetitRond, true, dimf.longueurAbscisses / 2, dimf.longueurOrdonnees / 2, false);
		add(O);
		// Ensuite une droite horizontale passant par ce point cache
		CDroiteDirectionFixe ptdh = new CDroiteDirectionFixe(this, null, false, Couleur.TraduitCouleur(1), false,
				0d, 0d, true, "", (int)(k*16), StyleTrait.traitFin, CDroite.abscisseNomParDefaut, O, true, 1);
		add(ptdh);
		// On ajoute un point nomm li  la droite horizontale
		CPointLieDroite I = new CPointLieDroite(this, null, false, Couleur.TraduitCouleur(1), nomsIJmasques,
			-4d*k, 4d*k, false, nomI, (int)(k*16), MotifPoint.PetitRond, true, res, ptdh, false);
		add(I);
		// Une droite reprsentant l'axe des abscisses.
		CDroiteAB ptd = new CDroiteAB(this, null, false, Color.black, StyleTrait.traitFin, O, I, (int)(k*16));
		add(ptd);
		// On cre une perpendiculaire
		CDroitePerpendiculaire perp = new CDroitePerpendiculaire(this, null, false, Couleur.TraduitCouleur(1), StyleTrait.traitFin,
				O, ptd, (int)(k*16));
		add(perp);
		// On cre un cercle de centre O et passant par I
		CCercleOA cercle = new CCercleOA(this, null, false, Couleur.TraduitCouleur(1), true, StyleTrait.traitFin, O, I);
		add(cercle);
		// On cre l'intersection de ce cercle avec la perpendiculaire
		CIntDroiteCercle inter = new CIntDroiteCercle(this, null, false, perp, cercle);
		add(inter);
		// Puis deux points reprsentant les deux points d'intersection
		// Le premier point est cach sans nom
		CPointLieBipoint K = new CPointLieBipoint(this, null, false, Couleur.TraduitCouleur(1), false,
				-12d*k, -4d*k, true, "", (int)(k*16), MotifPoint.PetitRond, inter, 1, false);
		add(K);
		CPointLieBipoint J = new CPointLieBipoint(this, null, false, Couleur.TraduitCouleur(1), nomsIJmasques,
				-12d*k, -4d*k, false, nomJ, (int)(k*16), MotifPoint.PetitRond, inter, 2, false);
		add(J);
		// puis le repre
		CRepere rep = new CRepere(this, null, false, Color.gray, false, StyleTrait.traitPointille, O, I, J,
				originex, originey, quadrillageHor,
				quadrillageVer, pointille, unitex, unitey);
		add(rep);
		// On cre la longueur OI qu'on tablit comme longueur nit de la liste
		CLongueur lon = new CLongueur(this, null, false, O, I);
		add(lon);
		pointeurLongueurUnite = lon;
		if (nomVecteuri.length() != 0) {
			CVecteur vecti = new CVecteur(this, null, false, Color.black, false, StyleTrait.traitFin, O, I,
					StyleFleche.FlecheCourtePleine);
			add(vecti);
			CLatex commi = new CLatex(this, null, false, Color.black, 0d, 0d, -10d*k, 2d*k, false, I, (int)(k*16),
					StyleEncadrement.Sans, false, Color.white, "\\vec{" + nomVecteuri + "}",
					CAffichageLieAPoint.alignementHorizontalDroit, CAffichageLieAPoint.alignementVerticalHaut);
			commi.creeFormule();
			add(commi);
		}
		if (nomVecteurj.length() != 0) {
			CVecteur vecti = new CVecteur(this, null, false, Color.black, false, StyleTrait.traitFin, O, J,
					StyleFleche.FlecheCourtePleine);
			add(vecti);
			CLatex commj = new CLatex(this, null, false, Color.black, 0d, 0d, -8d*k, 5d*k, false, J, (int)(k*16),
					StyleEncadrement.Sans, false, Color.white, "\\vec{" + nomVecteurj + "}",
					CAffichageLieAPoint.alignementHorizontalDroit, CAffichageLieAPoint.alignementVerticalHaut);
			commj.creeFormule();
			add(commj);
		}
	}

	/**
	 * Fonction ajoutant un repre orthogonal au centre de la fentre et crant
	 * une longueur unit
	 * 
	 * @param dimf
	 *          : dimensions de la fentre contenant la figure
	 * @param quadrillageHor
	 *          :true pour un quadrillage horizontal
	 * @param quadrillageVer
	 *          : true pour un quadrillage vertical
	 * @param pointille
	 *          : true pour un repre avec pointills
	 * @param nomO
	 *          : Nom de l'origine
	 * @param nomI
	 *          : Nom du point de cooronnes (1;0)
	 * @param nomJ
	 *          : Nom du point de cooronnes (0;1)
	 * @param nomVecteuri
	 *          : si chaine non vide, on rajoute une vecteur OI et un commentaire
	 *          vecteur
	 * @param nomVecteurj
	 *          : si chaine non vide, on rajoute une vecteur OJ et un commentaire
	 *          vecteur
	 */

	public final void ajouteRepereOrthogonal(DimensionFenetre dimf, boolean quadrillageHor,
			boolean quadrillageVer, boolean pointille, String nomO, String nomI, String nomJ, String nomVecteuri,
			String nomVecteurj, CValeurAssocieeAVariable originex, CValeurAssocieeAVariable originey,
			CValeurAssocieeAVariable unitex, CValeurAssocieeAVariable unitey, boolean nomsIJmasques) {
		double k = MtgFrame.coefMultForFonts(dimf.longueurAbscisses,  dimf.longueurOrdonnees);
		double res = (double) Toolkit.getDefaultToolkit().getScreenResolution() / 2.54;
		// res contient la rsolution de l'cran en points par cm
		// D'abord un point libre nomm U cach par dfaut
		CPointBase O = new CPointBase(this, null, false, Couleur.TraduitCouleur(1), false, -15d*k, 4d*k, false, nomO,
				(int)(k*16), MotifPoint.PetitRond, true, dimf.longueurAbscisses / 2, dimf.longueurOrdonnees / 2, false);
		add(O);
		// Ensuite une droite horizontale passant par ce point cache
		CDroiteDirectionFixe ptdh = new CDroiteDirectionFixe(this, null, false, Couleur.TraduitCouleur(1), false,
				0d, 0d, true, "", (int)(k*16), StyleTrait.traitFin, CDroite.abscisseNomParDefaut, O, true, 1);
		add(ptdh);
		// On ajoute un point nomm li  la droite horizontale
		CPointLieDroite I = new CPointLieDroite(this, null, false, Couleur.TraduitCouleur(1), nomsIJmasques,
				-4d*k, 4d*k, false, nomI, (int)(k*16), MotifPoint.PetitRond, true, res, ptdh, false);
		add(I);
		// Une droite reprsentant l'axe des abscisses.
		CDroiteAB ptd = new CDroiteAB(this, null, false, Color.black, StyleTrait.traitFin, O, I, (int)(k*16));
		add(ptd);		
		// On cre une perpendiculaire  la droite prcdente cache
		CDroitePerpendiculaire perp = new CDroitePerpendiculaire(this, null, false, Color.black, true, 0d, 0d,
				true, "", (int)(k*16), StyleTrait.traitFin, CDroite.abscisseNomParDefaut, O, ptdh);
		add(perp);
		// puis un point li  cette droite
		CPointLieDroite J = new CPointLieDroite(this, null, false, Couleur.TraduitCouleur(1), nomsIJmasques,
				-11d*k, 0d, false, nomJ, (int)(k*16), MotifPoint.PetitRond, true, -res * 1.5, perp, false);
		add(J);
		// Une droite reprsentant l'axe des ordonnes.
		CDroiteAB axeord = new CDroiteAB(this, null, false, Color.black, StyleTrait.traitFin, O, J, (int)(k*16));
		add(axeord);		
		
		// puis le repre
		CRepere rep = new CRepere(this, null, false, Color.gray, false, StyleTrait.traitPointille, O, I, J,
				originex, originey, quadrillageHor,	quadrillageVer, pointille, unitex, unitey);
		add(rep);
		// On cre la longueur OI qu'on tablit comme longueur nit de la liste
		CLongueur lon = new CLongueur(this, null, false, O, I);
		add(lon);
		pointeurLongueurUnite = lon;
		if (nomVecteuri.length() != 0) {
			CVecteur vecti = new CVecteur(this, null, false, Color.black, false, StyleTrait.traitFin, O, I,
					StyleFleche.FlecheCourtePleine);
			add(vecti);
			CLatex commi = new CLatex(this, null, false, Color.black, 0d, 0d, -10d*k, 2d*k, false, I, (int)(k*16),
					StyleEncadrement.Sans, false, Color.white, "\\vec{" + nomVecteuri + "}",
					CAffichageLieAPoint.alignementHorizontalDroit, CAffichageLieAPoint.alignementVerticalHaut);
			commi.creeFormule();
			add(commi);
		}
		if (nomVecteurj.length() != 0) {
			CVecteur vecti = new CVecteur(this, null, false, Color.black, false, StyleTrait.traitFin, O, J,
					StyleFleche.FlecheCourtePleine);
			add(vecti);
			CLatex commj = new CLatex(this, null, false, Color.black, 0d, 0d, -8d*k, 5d*k, false, J, (int)(k*16),
					StyleEncadrement.Sans, false, Color.white, "\\vec{" + nomVecteurj + "}",
					CAffichageLieAPoint.alignementHorizontalDroit, CAffichageLieAPoint.alignementVerticalHaut);
			commj.creeFormule();
			add(commj);
		}
	}

	/**
	 * Fonction ajoutant un repre oblique au centre de la fentre et crant une
	 * longueur unit
	 * 
	 * @param dimf
	 *          : dimensions de la fentre contenant la figure
	 * @param quadrillageHor
	 *          :true pour un quadrillage horizontal
	 * @param quadrillageVer
	 *          : true pour un quadrillage vertical
	 * @param pointille
	 *          : true pour un repre avec pointills
	 * @param nomO
	 *          : Nom de l'origine
	 * @param nomI
	 *          : Nom du point de cooronnes (1;0)
	 * @param nomJ
	 *          : Nom du point de cooronnes (0;1)
	 * @param nomVecteuri
	 *          : si chaine non vide, on rajoute une vecteur OI et un commentaire
	 *          vecteur
	 * @param nomVecteurj
	 *          : si chaine non vide, on rajoute une vecteur OJ et un commentaire
	 *          vecteur
	 */
	public final void ajouteRepereObliqueQuelconque(DimensionFenetre dimf, boolean quadrillageHor,
			boolean quadrillageVer, boolean pointille, String nomO, String nomI, String nomJ, String nomVecteuri,
			String nomVecteurj, CValeurAssocieeAVariable originex, CValeurAssocieeAVariable originey,
			CValeurAssocieeAVariable unitex, CValeurAssocieeAVariable unitey, boolean nomsIJmasques) {
		double k = MtgFrame.coefMultForFonts(dimf.longueurAbscisses,  dimf.longueurOrdonnees);
		double res = (double) Toolkit.getDefaultToolkit().getScreenResolution() / 2.54;
		// res contient la rsolution de l'cran en points par cm
		// D'abord un point libre nomm U cach par dfaut
		CPointBase O = new CPointBase(this, null, false, Couleur.TraduitCouleur(1), false, -15d*k, 4d*k, false, nomO,
				(int)(k*16), MotifPoint.PetitRond, true, dimf.longueurAbscisses / 2, dimf.longueurOrdonnees / 2, false);
		add(O);
		// Ensuite une droite horizontale passant par ce point cache
		CDroiteDirectionFixe ptdh = new CDroiteDirectionFixe(this, null, false, Couleur.TraduitCouleur(1), false,
				0d, 0d, true, "", (int)(k*16), StyleTrait.traitFin, CDroite.abscisseNomParDefaut, O, true, 1);
		add(ptdh);
		// On ajoute un point nomm li  la droite horizontale
		CPointLieDroite I = new CPointLieDroite(this, null, false, Couleur.TraduitCouleur(1), nomsIJmasques,
			-9d*k, 4d*k, false, nomI, (int)(k*16), MotifPoint.PetitRond, true, res, ptdh, false);
		add(I);
		// Une droite reprsentant l'axe des abscisses.
		CDroiteAB ptd = new CDroiteAB(this, null, false, Color.black, StyleTrait.traitFin, O, I, (int)(k*16));
		add(ptd);		
		// Un point libre J
		CPointBase J = new CPointBase(this, null, false, Couleur.TraduitCouleur(1), nomsIJmasques,
				-11d*k, 0d, false, nomJ, (int)(k*16), MotifPoint.PetitRond, true, dimf.longueurAbscisses / 2 + res / 4,
				dimf.longueurOrdonnees / 2 - res, false);
		add(J);
		// puis l'axe des ordonne
		CDroiteAB axeOrd = new CDroiteAB(this, null, false, Color.black, StyleTrait.traitFin, O, J, (int)(k*16));
		add(axeOrd);
		// puis le repre
		CRepere rep = new CRepere(this, null, false, Color.gray, false, StyleTrait.traitPointille, O, I, J,
				originex, originey, quadrillageHor, quadrillageVer, pointille, unitex, unitey);
		add(rep);
		// On cre la longueur OI qu'on tablit comme longueur nit de la liste
		CLongueur lon = new CLongueur(this, null, false, O, I);
		add(lon);
		pointeurLongueurUnite = lon;
		if (nomVecteuri.length() != 0) {
			CVecteur vecti = new CVecteur(this, null, false, Color.black, false, StyleTrait.traitFin, O, I,
					StyleFleche.FlecheCourtePleine);
			add(vecti);
			CLatex commi = new CLatex(this, null, false, Color.black, 0d, 0d, -10d*k, 2d*k, false, I, (int)(k*16),
					StyleEncadrement.Sans, false, Color.white, "\\vec{" + nomVecteuri + "}",
					CAffichageLieAPoint.alignementHorizontalDroit, CAffichageLieAPoint.alignementVerticalHaut);
			commi.creeFormule();
			add(commi);
		}
		if (nomVecteurj.length() != 0) {
			CVecteur vecti = new CVecteur(this, null, false, Color.black, false, StyleTrait.traitFin, O, J,
					StyleFleche.FlecheCourtePleine);
			add(vecti);
			CLatex commj = new CLatex(this, null, false, Color.black, 0d, 0d, -8d*k, 5d*k, false, J, (int)(k*16),
					StyleEncadrement.Sans, false, Color.white, "\\vec{" + nomVecteurj + "}",
					CAffichageLieAPoint.alignementHorizontalDroit, CAffichageLieAPoint.alignementVerticalHaut);
			commj.creeFormule();
			add(commj);
		}
	}

	/**
	 * Fonction ajoutant un repre oblique au centre de la fentre et crant une
	 * longueur unit Les units sur chacun des axes sont les mmes
	 * 
	 * @param dimf
	 *          : dimensions de la fentre contenant la figure
	 * @param quadrillageHor
	 *          :true pour un quadrillage horizontal
	 * @param quadrillageVer
	 *          : true pour un quadrillage vertical
	 * @param pointille
	 *          : true pour un repre avec pointills
	 * @param nomO
	 *          : Nom de l'origine
	 * @param nomI
	 *          : Nom du point de cooronnes (1;0)
	 * @param nomJ
	 *          : Nom du point de cooronnes (0;1)
	 * @param nomVecteuri
	 *          : si chaine non vide, on rajoute une vecteur OI et un commentaire
	 *          vecteur
	 * @param nomVecteurj
	 *          : si chaine non vide, on rajoute une vecteur OJ et un commentaire
	 *          vecteur
	 */
	public final void ajouteRepereObliqueNorme(DimensionFenetre dimf, boolean quadrillageHor,
			boolean quadrillageVer, boolean pointille, String nomO, String nomI, String nomJ, String nomVecteuri,
			String nomVecteurj, CValeurAssocieeAVariable originex, CValeurAssocieeAVariable originey,
			CValeurAssocieeAVariable unitex, CValeurAssocieeAVariable unitey, boolean nomsIJmasques) {
		double k = MtgFrame.coefMultForFonts(dimf.longueurAbscisses,  dimf.longueurOrdonnees);
		double res = (double) Toolkit.getDefaultToolkit().getScreenResolution() / 2.54;
		// res contient la rsolution de l'cran en points par cm
		// D'abord un point libre nomm U cach par dfaut
		CPointBase O = new CPointBase(this, null, false, Couleur.TraduitCouleur(1), false, -17d*k, 4d*k, false, nomO,
				(int)(k*16), MotifPoint.PetitRond, true, dimf.longueurAbscisses / 2, dimf.longueurOrdonnees / 2, false);
		add(O);
		// Ensuite une droite horizontale passant par ce point cache
		CDroiteDirectionFixe ptdh = new CDroiteDirectionFixe(this, null, false, Couleur.TraduitCouleur(1), false,
				0d, 0d, true, "", (int)(k*16), StyleTrait.traitFin, CDroite.abscisseNomParDefaut, O, true, 1);
		add(ptdh);
		// On ajoute un point nomm li  la droite horizontale
		CPointLieDroite I = new CPointLieDroite(this, null, false, Couleur.TraduitCouleur(1), nomsIJmasques,
				2d*k, 4d*k, false, nomI, (int)(k*16), MotifPoint.PetitRond, true, res, ptdh, false);
		add(I);
		// Une droite reprsentant l'axe des abscisses.
		CDroiteAB ptd = new CDroiteAB(this, null, false, Color.black, StyleTrait.traitFin, O, I, (int)(k*16));
		add(ptd);		
		// On cre un cercle de centre O et passant par I
		CCercleOA cerc = new CCercleOA(this, null, false, Color.black, true, StyleTrait.traitPointille, O, I);
		add(cerc);
		// Puis un point li  ce cercle
		CPointLieCercle J = new CPointLieCercle(this, null, false, Color.black, nomsIJmasques, -12d*k,
				0d, false, nomJ, (int)(k*16), MotifPoint.PetitRond, true, Math.PI / 3, cerc, false);
		add(J);
		// puis l'axe des ordonne
		CDroiteAB axeOrd = new CDroiteAB(this, null, false, Color.black, StyleTrait.traitFin, O, J, (int)(k*16));
		add(axeOrd);
		// puis le repre
		CRepere rep = new CRepere(this, null, false, Color.gray, false, StyleTrait.traitPointille, O, I, J,
				originex, originey, quadrillageHor, quadrillageVer, pointille, unitex, unitey);
		add(rep);
		// On cre la longueur OI qu'on tablit comme longueur nit de la liste
		CLongueur lon = new CLongueur(this, null, false, O, I);
		add(lon);
		pointeurLongueurUnite = lon;
		if (nomVecteuri.length() != 0) {
			CVecteur vecti = new CVecteur(this, null, false, Color.black, false, StyleTrait.traitFin, O, I,
					StyleFleche.FlecheCourtePleine);
			add(vecti);
			CLatex commi = new CLatex(this, null, false, Color.black, 0d, 0d, -10d*k, 2d*k, false, I, (int)(k*16),
					StyleEncadrement.Sans, false, Color.white, "\\vec{" + nomVecteuri + "}",
					CAffichageLieAPoint.alignementHorizontalDroit, CAffichageLieAPoint.alignementVerticalHaut);
			commi.creeFormule();
			add(commi);
		}
		if (nomVecteurj.length() != 0) {
			CVecteur vecti = new CVecteur(this, null, false, Color.black, false, StyleTrait.traitFin, O, J,
					StyleFleche.FlecheCourtePleine);
			add(vecti);
			CLatex commj = new CLatex(this, null, false, Color.black, 0d, 0d, -8d*k, 0d, false, J, (int)(k*16),
					StyleEncadrement.Sans, false, Color.white, "\\vec{" + nomVecteurj + "}",
					CAffichageLieAPoint.alignementHorizontalDroit, CAffichageLieAPoint.alignementVerticalCentre);
			commj.creeFormule();
			add(commj);
		}
	}

	/**
	 * Fonction renvoyant un pointeur sur une droite s'il existe dans la liste une
	 * droite dfinie par les deux points passs en paramtre, sinon renvoie null.
	 * Sert pour la cration des repres
	 */
	public final CDroiteAB existeDroiteABDefiniePar(CPointAncetre point1, CPointAncetre point2) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NDroite) {
				if (elb.getClass() == CDroiteAB.class) {
					CDroiteAB dte = (CDroiteAB) elb;
					if (((dte.A == point1) && (dte.B == point2)) || ((dte.A == point2) && (dte.B == point1)))
						return dte;
				}
			}
		}
		return null;
	}

	/**
	 * Fonction renvoyant un pointeur sur une droite s'il existe dans la liste un
	 * segement dfini par les deux points passs en paramtre, sinon renvoie
	 * null. Sert pour la cration des repres
	 */
	public final CSegment existeSegmentDefiniPar(CPointAncetre point1, CPointAncetre point2) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NSegment) {
				CSegment seg = (CSegment) elb;
				if (((seg.point1 == point1) && (seg.point2 == point2))
						|| ((seg.point1 == point2) && (seg.point2 == point1)))
					return seg;
			}
		}
		return null;
	}

	/**
	 * Fonction renvoyant un pointeur sur une droite s'il existe dans la liste une
	 * demi-droite dfinie par les deux points passs en paramtre, sinon renvoie
	 * null. Sert pour la cration des repres
	 */
	public final CDemiDroiteOA existeDemidroiteDefiniePar(CPointAncetre point1, CPointAncetre point2) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NDemiDroite) {
				if (elb.getClass() == CDemiDroiteOA.class) {
					CDemiDroiteOA dem = (CDemiDroiteOA) elb;
					if ((dem.O == point1) && (dem.A == point2))
						return dem;
				}
			}
		}
		return null;
	}

	/**
	 * Fonction renvoyant un pointeur sur un translation s'il existe dans la liste
	 * une translation dfinie par les deux points passs en paramtre, sinon
	 * renvoie null. Sert pour la cration des repres
	 */
	public final CTranslation existeTranslationDefiniePar(CPointAncetre point1, CPointAncetre point2) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NTransformation) {
				if (elb.getClass() == CTranslation.class) {
					CTranslation trans = (CTranslation) elb;
					if ((trans.origineVecteur == point1) && (trans.extremiteVecteur == point2))
						return trans;
				}
			}
		}
		return null;
	}

	/** Optimis version 4.7.5.2
	public final boolean estDefiniParObjetDans(CListeObjets listeOb) {
		boolean res = true;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase el = li.next();
			res = res && el.estDefiniParObjetDans(listeOb);
		}
		return res;
	}
	*/
	public final boolean estDefiniParObjetDans(CListeObjets listeOb) {
		boolean res = true;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext() && res) {
			CElementBase el = li.next();
			res = res && el.estDefiniParObjetDans(listeOb);
		}
		return res;
	}


	public final void setListeDependantePourCapture(CListeObjets listeSource, CElementBase elb) {
		retireTout();
		ListIterator<CElementBase> li = listeSource.listIterator();
		while (li.hasNext()) {
			CElementBase el = li.next();
			if (el.dependDePourCapture(elb)) {
				add(el);
			}
		}
		pointeurLongueurUnite = listeSource.pointeurLongueurUnite;
		uniteAngle = listeSource.uniteAngle;
	}

	/**
	 * Fonction renvoyant le nombre de macros hritant de CMacroAvecListe
	 * 
	 * @return
	 */
	public int nombreMacrosAvecListe() {
		int res = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NMacro) {
				if (((CMacro) elb).ajoutOuRetraitObjetsPossible())
					res++;
			}
		}
		return res;
	}

	// Modifi version 3.2.4.2 pour ne pas prendre en compte les variables objets intermdiaires
	/**
	 * Fonction renvoyant le nombre de variables devant figurer en bas et  droite
	 * de la figure sauf les objets intermdiaires de construction
	 */
	public final int nombreVariablesAvecDialogueAssocie() {
		int res = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (!elb.estElementIntermediaire() && (elb.getNatureCalcul() == NatureCalcul.NVariable)) {
				if (((CVariableBornee) elb).dialogueAssocie)
					res++;
			}
		}
		return res;
	}
	
	public final int nombreCalculsModifiablesParMenu() {
		int res = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
      if (!elb.estElementIntermediaire()
      		&& elb.estDeNatureCalcul(NatureCalcul.NCalculOuVariableOuFonctionReelOuComplexe) && elb.modifiableParMenu())
      	res++;
		}
		return res;
	}

/**
 * Ajout version 4.8
 * Fonction renvoyant le nombre total d'implmentations de la construction de nom nomc que contient la figure
 * @param nom
 * @return
 */
	public final int nombreImplementationsConstruction(String nomc) {
		int res = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
      if (elb.estDeNature(NatureObjet.NImplementationProto)) {
      	CImplementationProto imp = (CImplementationProto) elb;
      	// if (imp.nomProto == nomc) res++; Modifi version 4.8
      	if (imp.nomProto.equals(nomc)) res++;
      }
		}
		return res;
	}
	
	/**
	 * Fonction renvoyant true si la liste contient un point li non masqu dont
	 * dpent elb
	 * 
	 * @param elb
	 * @return
	 */
	public final boolean existePointLieVisibleDontDepend(CElementBase elb) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase el = li.next();
			if (el.getNature() == NatureObjet.NPointLie) {
				CElementGraphique elg = (CElementGraphique) el;
				if (!elg.masque && elg.dependDe(elb))
					return true;
			}
		}
		return false;
	}

	/**
	 * Fonction rajoutant  la liste les lments de listeSource qui sont des
	 * points lis visibles et qui ne sont pas dj dans la liste et dont dpend
	 * elb
	 * 
	 * @param listeSource
	 * @param elb
	 */
	public final void ajoutePointsLiesVisiblesDontDepend(CListeObjets listeSource, CElementBase elb) {
		int indelb = listeSource.indexOf(elb);
		for (int i = 0; i <= indelb; i++) {
			CElementBase el = listeSource.get(i);
			if (el.getNature() == NatureObjet.NPointLie) {
				CElementGraphique elg = (CElementGraphique) el;
				if (!elg.masque && elb.dependDe(elg) && !contains(elg))
					add(elg);
			}
		}
	}

	public final void metAJourSurfacesAyantPourBord(CLieuDeBase lieu) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase el = li.next();
			if (el.getNature() == NatureObjet.NSurface) {
				CSurfaceAncetre surf = (CSurfaceAncetre) el;
				if (surf.aPourBord(lieu))
					surf.metAJourTableaux();
			}
		}
	}

	/**
	 * Fonction ajoutant les points de listeSource dpendant de elb
	 * 
	 * @param listeSource
	 * @param elb
	 */
	public final void ajoutePointsDependantDe(CListeObjets listeSource, CElementBase elb) {
		ListIterator<CElementBase> li = listeSource.listIterator();
		while (li.hasNext()) {
			CElementBase el = li.next();
			if (el.estDeNature(NatureObjet.NToutPoint) && el.dependDe(elb))
				add(el);
		}
	}

	/**
	 * Fonction ajoutant les objets graphiques de listeSource dpendant de elb
	 * 
	 * @param listeSource
	 * @param elb
	 */
	public final void ajouteObjetsDependantDe(CListeObjets listeSource, CElementBase elb) {
		ListIterator<CElementBase> li = listeSource.listIterator();
		while (li.hasNext()) {
			CElementBase el = li.next();
			if (el.estDeNature(NatureObjet.NToutObjet) && el.dependDe(elb))
				add(el);
		}
	}

	/**
	 * Mthode d'appeler pour chaque objet dpendant de elb la mthode metAJour()
	 * qui est charge de mettre  jour tous les lments crs dynamiquement par
	 * certians objets comme les lieux de points et lieux d'objets
	 * 
	 * @param elb
	 */
	public final void metAJourObjetsDependantDe(CElementBase elb) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase el = li.next();
			if (el.dependDe(elb))
				el.metAJour();
		}
	}

	/**
	 * Ajout version 4.7.2
	 * Mthode d'appeler pour chaque objet dpendant de elb la mthode metAJour() sauf pour l'lment sauf
	 * qui est charge de mettre  jour tous les lments crs dynamiquement par
	 * certains objets comme les lieux de points et lieux d'objets, calcus de drives
	 * Les diteurs de formules eux ne sont pas mis  jour 
	 * 
	 * @param elb
	 */
	public final void metAJourObjetsDependantDeSaufEditeursFormule(CElementBase elb) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase el = li.next();
			if ((!el.estDeNature(NatureObjet.NEditeurFormule)) && el.dependDe(elb))
				el.metAJour();
		}
	}
	
	
	// Ajout version 4.6
	/**
	 * Appelle la mthode metAJour de chaque objet qui est de nature objet nature
	 * @param nature
	 */
	public void metAJourParNatureObjet(long nature) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase el = li.next();
			if (el.estDeNature(nature)) el.metAJour();
		}		
	}

	/**
	 * Mthode d'appeler pour chaque objet la mthode metAJour() qui est charge
	 * de mettre  jour tous les lments crs dynamiquement par certains objets
	 * comme les lieux de points et lieux d'objets
	 */

	public final void metAJour() {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase el = li.next();
			el.metAJour();
		}
	}
	
	// Ajout version 4.9.8.2
	/**
	 * Fonction mettant  jour certains objets lorsqu'un objet a t rajout  la figure.
	 * Les objets  mettre  jour sont les macros d'affectation de valeur  une variable et de modification de variable
	 */
	public void metAJourPourCreation() {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase el = li.next();
			el.metAJourPourCreation();
		}
	}

	
	// Ajout version 4.5.7.
	/** MetAJour tous les lments de la liste tout en les positionannt un fosi MetAJour appel
	 * 
	 */
	public final void metAJourEtPositionne(boolean infoRandom, DimensionFenetre dimfen) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase el = li.next();
			el.metAJour();
			el.positionne(infoRandom, dimfen);
		}
	}
	

	/**
	 * Fonction rordonnant les objets de la liste suivant l'ordre de
	 * ListePrincipale qui est la liste contenant tous les objets
	 * 
	 * @param listePrincipale
	 */
	public void reordonne(CListeObjets listePrincipale) {
		CListeObjets lis = new CListeObjets();
		ListIterator<CElementBase> li = listePrincipale.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (contains(elb))
				lis.add(elb);
		}
		// lis contient maintenant les lments dans le bon ordre
		removeRange(0, size());
		ListIterator<CElementBase> li2 = lis.listIterator();
		while (li2.hasNext())
			add(li2.next());
	}

	/**
	 * Fonction renvoyant true si la liste ne contient pas de macro ayant pour
	 * intitul intitule autre que la macro macro
	 * 
	 * @param nom
	 * @param macro
	 * @return
	 */
	public final boolean validationIntituleMacro(String intitule, CMacro macro) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNature() == NatureObjet.NMacro) {
				CMacro mac = (CMacro) elb;
				// Modifi version 4.8
				// if ((macro != mac) && (mac.intitule == intitule))
				if ((macro != mac) && (mac.intitule.equals(intitule)))
					return false;
			}
		}
		return true;
	}

	/**
	 * Fonction utilise quand on zoome ou dzoome sur la figure
	 * Modifi version 3.1.6 : Si un point libre est punais on ne zoome pas ses coordonnes
	 * @param xcentre
	 * @param ycentre
	 * @param rapport
	 */
	public final void zoom(double xcentre, double ycentre, double rapport) {
		CPointAncetre origineUnite = null;
		if (pointeurLongueurUnite != null) {
			CPointAncetre pt1 = pointeurLongueurUnite.point1;
			if (pt1.getNature() == NatureObjet.NPointBase)
				origineUnite = pt1;
		}
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if ((elb.getNature() == NatureObjet.NPointBase) && (elb != origineUnite)) {
				CPointBase ptb = (CPointBase) elb;
				if (ptb.estLibre()) ptb.zoom(xcentre, ycentre, rapport);
			}
			else {
				if (elb.getNature() == NatureObjet.NDroite) {
					if (elb.getClass() == CDroiteDirectionFixe.class) {
						// Modification version 4.5
						// ((CDroiteDirectionFixe) elb).zoom(rapport);
						if (elb.impProto != null) {
							if (!elb.impProto.nomProto.equals(OutilCurseur.nomCurseur)) ((CDroiteDirectionFixe) elb).zoom(rapport);
						}
						else ((CDroiteDirectionFixe) elb).zoom(rapport);
					}
				}
			}
		}
	}

	/**
	 * Adaptation de certains objets pour passer en haute rsolution avant
	 * impression notamment
	 */
	public final void adaptePourHauteResolution(double coef) {
		coefMult = coef;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			elb.adaptePourHauteResolution(coef);
		}
	}
	

	/**
	 * Fonction reconstruisant toutes les chanes des calculs et des fonctions.
	 * Utilis quand on charge un document dont la langue n'est pas la langue
	 * active.
	 */
	public void reconstruitChainesCalcul() {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.estDeNatureCalcul(NatureCalcul.NCalculOuFonctionNonConstante))
				((CCalcul) elb).reconstruitChaineCalcul();
		}
	}
	
	/**
	 * Fonction reconstruisant toutes les chanes des calculs et des fonctions.
	 * Utilis quand on charge un document dont la langue n'est pas la langue
	 * active.
	 */
	public void reconstruitChainesCalcul(int inddeb, int indfin) {
		for (int i = inddeb; i <= indfin; i++) {
			CElementBase elb = get(i);
			if (elb.estDeNatureCalcul(NatureCalcul.NCalculOuFonctionNonConstante))
				((CCalcul) elb).reconstruitChaineCalcul();
		}
	}

	/**
	 * Fonction reconstruisant toutes les chanes des calculs et des fonctions dpendant de elb
	 *  partir de l'indice inddeb
	*/
	public void reconstruitChainesCalculDependantsDe(int inddeb, CElementBase el) {
		for (int i = inddeb; i < size(); i++) {
			CElementBase elb = get(i);
			if (elb.estDeNatureCalcul(NatureCalcul.NCalculOuFonctionNonConstante) && elb.dependDe(el)) {
				((CCalcul) elb).reconstruitChaineCalcul();				
			}
		}
	}
	
	
	/**
	 * Fonction fusionnant les implmentations de constructions de la figure et les remplaant
	 * par des objest normaux en les renommant si ncessaire
	 */
	public void fusionneImplementationsConstructions() {
		boolean calculIntermediaireRenomme = false;
		// On retire d'abord toutes les implmentations de constructions
		// Pour cela on ne peut pas utiliser un listIterator sous peine de concurrentModificationException
		int i = 0;
		while (i < size()) {
			CElementBase elb = get(i);
			if (elb.getNature() == NatureObjet.NImplementationProto) remove (elb);
			else i++;
		}
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.estElementIntermediaire()) {
				// On renomme les points ou droites intermdiaires si leur nom existe dj
				if (elb.estDeNature(NatureObjet.NToutPoint | NatureObjet.NDroite)) {
					String ancienNom = ((CElementGraphique) elb).rendNom();
					if (ancienNom.length() != 0) {
						if (existePointOuDroiteMemeNom((CElementGraphique) elb, ancienNom)) {
							String nouveauNom = genereNomPourPointOuDroite(ancienNom.charAt(0));
							((CElementGraphique) elb).donneNom(nouveauNom);
						}
					}
				}
				else {
					// Il faut renommer aussi les calculs intermdiaires ayant mme
					// nom que d'autres calculs crs aprs implmentation de la construction		
					if (elb.estDeNatureCalcul(NatureCalcul.NToutCalculNomme)) {
						CValeurDynamique val = (CValeurDynamique) elb;
						String ancienNom = val.nom();
						if (existeCalculOuVariableOuFonctionOuParametreMemeNom(val, ancienNom, true)) {
							String nouveauNom = genereNomPourCalcul(ancienNom, false);
							// Ligne suivante amlioration version 3.5
							remplaceNomValeurDynamiqueDansCommentairesOuLatexDependantDe(ancienNom, nouveauNom, val);
							val.donneNom(nouveauNom);
							calculIntermediaireRenomme = true;
						}
					}
				}
				elb.impProto = null;
				elb.estElementFinal = false;			
			}
			else {
				if (elb.estElementFinal) {
					elb.impProto = null;
					elb.estElementFinal = false;
				}
			}
		}
		// Ajout version 4.8.0
		updateIndexes();
		//
		// On reconstruit toutes les formules de la figure
		if (calculIntermediaireRenomme) reconstruitChainesCalcul();
	}
	
	/**
	 * Fonction renvoyant true si la liste contient des objets utilisant la prsence d'une longueur unit
	 * @return
	 */
	public boolean utiliseLongueurUnite() {
		boolean res = false;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			res = li.next().utiliseLongueurUnite() || res; // Ordre indispensable sinon boucle infine
		}
		return res;
	}
	
	/**
	 * Fonction renvoyant un pointeur sur la premire longueur utilise
	 * Utilis dans les implmentations de prototypes
	 * @return
	 */
	public CLongueur premiereLongueur() {
		CLongueur res = null;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.getNatureCalcul() == NatureCalcul.NMesureLongueur) {
				res = (CLongueur) elb;
				break;
			}
		}
		return res;
		
	}
	
	
	public boolean sontParDefinitionSurParallele(CPointAncetre pt1, CPointAncetre pt2, CDroiteAncetre droite) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.estDeNature(NatureObjet.NTouteDroiteSaufVecteur)) {
				CDroiteAncetre da = (CDroiteAncetre) elb;
				if (pt1.appartientParDefinition(da) && pt2.appartientParDefinition(da)
						&& (da.estParalleleParDefinition(droite) || droite.estParalleleParDefinition(da))) return true;
			}
		}
		return false;
	}
	
	/**
	 * Renvoie un pointeur sur le premier repre cr
	 * @param list
	 * @return
	 */
	public final CRepere premierRepere() {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (! elb.estElementIntermediaire() && (elb.getNatureCalcul() == NatureCalcul.NRepere)) {
				CRepere rep = (CRepere) elb;
				return rep;
			}
		}
		return null;
	}
	
	/**
	 * Renvoie un pointeur sur le premier repre cr visible
	 * @param list
	 * @return
	 */
	// Revu version 5.1.4 car un repre est toujours visible !
	public final CRepere premierRepereVisible() {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			// Bug corrig version 4.5.0.1
			// if (elb.getNatureCalcul() == NatureCalcul.NRepere) {
			if (! elb.estElementIntermediaire() && (elb.getNatureCalcul() == NatureCalcul.NRepere)) {
				CRepere rep = (CRepere) elb;
				// if (!rep.masque) return rep;
				return rep;
			}
		}
		return null;
	}
	
	
	public final void recreeListesCopiesLieuxObjetsDependantDe(CLieuObjetAncetre lieu) {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if ((elb.getNature() == NatureObjet.NLieuObjet) && (elb != lieu) && elb.dependDe(lieu)) {
				((CLieuObjetAncetre)elb).recreeListeCopieObjets();
			}
		}
	}
	
	public final void initialiseNombreIterations() {
		nombreIterations = 0;
	}
	
	public final int nombreFonctionsDerivables() {
		int res = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (!elb.estElementIntermediaire() && elb.getNatureCalcul() == NatureCalcul.NFonction) {
				CFonc fon = (CFonc) elb;
				if (fon.deriveePossible(0)) res++;
			}	
		}
		return res;
	}
	
	// Ajout version 4.6
	public final int nombreFonctionsNVar() {
		int res = 0;
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (!elb.estElementIntermediaire() && elb.estDeNatureCalcul(NatureCalcul.NFonctionReelleNvar | NatureCalcul.NDeriveePartielle)) {
				res++;
			}	
		}
		return res;
	}
	
	/**
	 * Ajout version 3.4.2 pour optimisation
	 * Rinitialize le membre elementTestePourDependDe de chaque objet  null
	 * Doit tre appel lorsque les liens entre objets peuvent avoir chang
	 * Cette opimisation a t rajoute pour optimiser le temps de chargement et de cration
	 * des lieux de points et lieux d'objets (on pouvait opour certaines figures
	 * avoir une latence de plusieurs minutes)
	 */
	public final void initialiseDependances() {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			elb.initialisePourDependance();
		}
	}
	
	/**
	 * Fonction ajoutant  la liste tous les points de listeSource qui sont marqus ou non pour la trace suivant la valeur de bmarque
	 * @param listeSource
	 * @param bmarque
	 */
	public final void ajoutePointsMarquesPourTrace(CListeObjets listeSource, boolean bmarque) {
		ListIterator<CElementBase> li = listeSource.listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.estDeNature(NatureObjet.NToutPoint) && !elb.estElementIntermediaire()){
				CPointAncetre pt = (CPointAncetre) elb;
				if (pt.marquePourTrace == bmarque) add(pt);
			}
		}
	}
	
	// Ajout version 4.5.2
	/**
	 * On retire ventuellement les CComponent instancis par les objets sur le MtgInternalFrame
	 * C'est le cas pour CEditeurFormule
	 */
	@Override
	public boolean remove(Object arg0) {
		// TODO Auto-generated method stub
		CElementBase elb = (CElementBase) arg0;
		CListeObjets liste = elb.listeProprietaire;
		if (liste != null) {
			CMathGrafDoc doc = liste.documentProprietaire;
			if (doc != null) {
				MtgInternalFrame cadre = doc.cadreProprietaire;
				if (cadre != null) elb.retireComposant(cadre);
			}		
		}
		return super.remove(arg0);
	}

	/**
	 * Fonction retirant l'objet de la liste mais ne retirant pas le comosant associ (utilis quand on annule le clignotement d'objets)
	 * @param arg0
	 * @return
	 */
	public boolean removeAndKeepComposant(Object arg0) {
		return super.remove(arg0);
	}
	
	// Ajout version 4.5.2
	public void retablitComposants() {
		ListIterator<CElementBase> li = listIterator();
		while (li.hasNext()) {
			CElementBase elb = li.next();
			if (elb.estDeNature(NatureObjet.NToutObjet)) {
				CElementGraphique elg = (CElementGraphique) elb;
				if (elg.masque) elg.cacheComposant();
				else elg.montreComposant();
			}
		}
	}

	/**
	 * Ajout version 4.8
	 * Fonction renvoyant un pointeur sur l'lment final d'indice ind de la liste en sautant ls lments graphiques
	 * intermdiaires comme les transformations ou les bipoints
	 *  partir de l'indice inddeb
	 * @param inddeb;
	 * @param nb
	 * @return
	 */
	public final CElementBase getFinal(int inddeb, int ind) {
		CElementBase el = null;
		int i = inddeb;
		int j = -1;
		while (j != ind) {
			el = get(i);
			if (el.estElementFinal) {
				if (el.estDeNature(NatureObjet.NToutObjet) || !el.estDeNatureCalcul(NatureCalcul.NAucunCalcul)) j++;
			}
			i++;
		}
		return el;
	}

	/**
	 * Ajout version 4.8.0 : On redfinit la fonction indexOf pour acclrer le traitement.
	 * Chaque Objet CElementBase possde un membre index qui donne son indice dans la liste qui le possde.
	 * @param elb
	 * @return
	 */
	public int indexOf(CElementBase elb) {
		if (elb == null) return -1;
		if (elb.listeProprietaire == this) return elb.index;
		else return super.indexOf(elb);
	}

	/**
	 * Ajout version 4.8.0
	 * Modifi version 5.0.0.2 pour corriger un bug dans le cas o on reclasse la longueur unit de la figure
	 * Si on ajoute une longueur et que la liste ne comporte pas de longueur unit on tablikt la longuur unit de la liste
	 */
	public boolean add(CElementBase elb) {
		if (elb.listeProprietaire == this) elb.index = size();
		if (elb.estDeNatureCalcul(NatureCalcul.NMesureLongueur) && (pointeurLongueurUnite == null)) pointeurLongueurUnite = (CLongueur) elb;
		return super.add(elb);
	}
	
	/**
	 * Ajout version 5.0.0.2 pour corriger un bug dans le cas o on reclasse la longueur unit de la figure
	 */
	@Override
	public void add(int index, CElementBase elb) {
		// TODO Auto-generated method stub
		if (elb.estDeNatureCalcul(NatureCalcul.NMesureLongueur) && (pointeurLongueurUnite == null)) pointeurLongueurUnite = (CLongueur) elb;
		super.add(index, elb);
	}
	
	/**
	 * Ajout version 4.8.0
	 * Met  jour les index de chaque descendant de CElementBase
	 */
	public void updateIndexes() {
		ListIterator<CElementBase> li = listIterator();
		int i = 0;
		while (li.hasNext()) {
			CElementBase elb = li.next();
			elb.index = i;
			i++;
		}
	}
	
	/**
	 * Ajout version 4.8.0
	 * Fonction renvoyant true si la liste ne contient pas de point ou de droite non intermdiaire
	 * ayant un nom commenant par st en arrtant la recherche
	 *  l'indice indFin
	 * @param element peut tre null
	 * @param st
	 * @return
	 */
	private final boolean existePointOuDroiteNomCommencantPar(String st, int indFin) {
		for (int i = 0; i < indFin; i++) {
			CElementBase elb = get(i);
			if (! elb.estElementIntermediaire() && (elb.estDeNature(NatureObjet.NToutPoint + NatureObjet.NDroite))) {
				CElementGraphique ptp = (CElementGraphique) elb;
				if (ptp.nom.length() != 0) {
					if (ptp.nom.startsWith(st)) return true;
				}
			}
		}
		return false;
	}
	/**
	 * Ajout version 4.8.0
	 * Fonction renvoyant true s'il existe un calcul, une fonction ou un paramre de fonction non intermdiaire
	 * qui ne soit pas un objet non intermdiaire et ayant un nom commenant par st
	 * en arrtant la recherche  l'indice indFin
	 * @param valeur
	 * @param st
	 * @return
	 */
	public final boolean existeCalculOuVariableOuFonctionOuParametreNomCommencantPar
		(String st, int indFin) {
		for (int i = 0; i <= indFin; i++) {
			CElementBase elb = get(i);
			if (elb.estDeNatureCalcul(NatureCalcul.NTouteValeurReelleOuComplexe) && !elb.estElementIntermediaire()) {
				CValeurDynamique ptv = (CValeurDynamique) elb;
				if (ptv.nomCommencePar(st)) return true;
			}
		}
		return false;
	}

	/**
	 * Fonction gnrant un dbut de nom libre pour un point nom nomm
	 * en arrtant la recherche des noms  indFin.
	 * Utilis pour les implmentations itratives et rcursives de constructions
	 * et pour le nouvel historique (version 4.9.9)
	 * @return
	 */
	public final String genereDebutNomPourPoint(int indFin) {
		String ch = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
		StringBuffer nomGenere = new StringBuffer(64);
		// Modifi version 4.8
		for (int i = 0;; i++) {
			for (int j = 0; j < ch.length(); j++) {
				nomGenere.delete(0, nomGenere.length());
				nomGenere.append(ch.charAt(j));
				if (i != 0)
					nomGenere.append(Integer.toString(i));
				if (!existePointOuDroiteNomCommencantPar(nomGenere.toString(), indFin))
					return nomGenere.toString();
			}
		}
		// return ""; // Pour satisfaire le compilateur
	}
	/**
	 * Fonction gnrant un dbut de nom libre pour un point nom nomm
	 * en arrtant la recherche des noms  indFin.
	 * Utilis pour les implmentations itratives et rcursives de constructions
	 * et pour le nouvel historique (version 4.9.9)
	 * @return
	 */
	public final String genereDebutNomPourDroite(int indFin) {
		StringBuffer nomGenere = new StringBuffer(64);
		nomGenere.append("d");
		// Modifi version 4.8
		for (int i = 0;; i++) {
			nomGenere.delete(1, nomGenere.length());
			if (i != 0)
				nomGenere.append(Integer.toString(i));
			if (!existePointOuDroiteNomCommencantPar(nomGenere.toString(), indFin))
				return nomGenere.toString();
		}
	}

	public final String genereDebutNomPourCalcul(String debNom, int indFin) {
		if (!existeCalculOuVariableOuFonctionOuParametreNomCommencantPar(debNom, indFin)) return debNom;
		String ch = "abcdefghijklmnopqrstuvwxyz";
		StringBuffer nomGenere = new StringBuffer(64);
		nomGenere.append(debNom);
		// Modifi version 4.8
		for (int i = 0;; i++) {
			for (int j = 0; j < ch.length(); j++) {
				nomGenere.delete(debNom.length(), nomGenere.length());
				nomGenere.append(ch.charAt(j));
				if (i != 0)
					nomGenere.append(Integer.toString(i));
				// Bug corrig version 4.9.9
				// if (!existePointOuDroiteNomCommencantPar(nomGenere.toString(), indFin))
				if (!existeCalculOuVariableOuFonctionOuParametreNomCommencantPar(nomGenere.toString(), indFin))
					return nomGenere.toString();
			}
		}
	}

	/**
	 * Ajout version 4.8.
	 * Fonction appele aprs avoir implment de faon itrative une construction pour renommer les points ou droites
	 * qui ont besoin d'tre nomms, renommer les calculs qui ont besoin d'tre nomms et reconstituer les formules des calculs
	 * qui sont des objets finaux
	 * @param inddeb : L'indice de l'implmentation de prototype
	 * @param proto
	 */
	// Modifi version 4.9.9
	public final void traiteImplementationsProtoSuccessivesIter(int inddeb, CPrototype proto, MtgInternalFrame cadre, int nbObjImp) {
		int lon = this.size();
		// On commence par renommer les points ou droites qui sont utiliss comme sources qui sont nomms dans le prototype
		for (int i = 0; i < proto.nbObjSources; i++) {
			CElementGenerique ptgen = (CElementGenerique) proto.get(i);
			CElementBase elb = ptgen.elementAssocie;
			if (elb.estDeNature(NatureObjet.NToutPoint|NatureObjet.NDroite)) {
				CElementGraphique ptp = (CElementGraphique) elb;
				String nomGen = ptgen.nom;
				// On regarde si l'lment correspondant tait nomm.
				// Si oui on rajout cet lment  la liste des points devant tre
				// nomms
				if ((nomGen.length() != 0) && (ptp.rendNom().length() == 0)) {
					ptp.donneNom(genereDebutNomPourPoint(inddeb-1));
				}
			}
		}
		// Ensuite on gnre des noms pour tous les objets implments noms ou droites qui ont un nom
		// On compte le nombre d'objets finaux nomms
		int nbfn = 0;
		for (int k = proto.nbObjSources; k < proto.size(); k++) {
			CElementBase elb = proto.get(k);
			if (elb.estDeNature(NatureObjet.NToutPoint|NatureObjet.NDroite|NatureObjet.NDroiteClone)) {
				CElementGraphique elg = (CElementGraphique) elb;
				if (!elg.rendNom().isEmpty()) nbfn++;
			}
		}
		String ch = genereDebutNomPourPoint(inddeb-1); 	// Le dbut du nom gnr qui sera le mme pour tous
		// les objets de mme type
		int k = 0;
		for (int i = 0; i < nbObjImp; i++) {
			CElementBase elb = get(inddeb + 1 + i);
			if (elb.estDeNature(NatureObjet.NToutPoint|NatureObjet.NDroite|NatureObjet.NDroiteClone)) {
				CElementGraphique ptp = (CElementGraphique) elb;
				CElementGraphique elg = (CElementGraphique) proto.get(proto.nbObjSources + i);
				if (!elg.rendNom().isEmpty()) {
					k++;
					ptp.donneNom(ch+k);
					int j = 1;
					int deb = inddeb + 1 + i+j*(nbObjImp+1);
					while (deb < lon) {
						CElementGraphique elb2 = (CElementGraphique)get(deb);
						elb2.donneNom(ch+Integer.toString(k+j*nbfn));
						j++;
						deb = inddeb + 1 + i + j*(nbObjImp+1);
					}
				}
			}
			else {
				if (elb.estDeNatureCalcul(NatureCalcul.NToutCalculNomme)) {
					CValeurDynamique vald = (CValeurDynamique) elb;
					ch = genereDebutNomPourCalcul(FonteNom.premiersCaracteresNonChiffresNonPrime(vald.nom()), inddeb - 1);
					String oldName = vald.nom();
					int j = 0;
					int deb = inddeb + 1 + i + j*(nbObjImp+1);
					while (deb < lon) {
						CValeurDynamique vald2 = (CValeurDynamique)get(deb);
						oldName = vald2.nom();
						String newName = ch+Integer.toString(j+1);
						vald2.donneNom(newName);
						for (int m = deb + 1; m<lon; m++) {
							CElementBase e = get(m);
							if (e.estDeNature(NatureObjet.NImplementationProto)) break;
							if (e.estDeNature(NatureObjet.NCommentaire | NatureObjet.NLatex)) {
								if (((CCommentaire)e).listeValeursDynamiquesUtilisees.size()!=0) 
										if (((CCommentaire)e).remplaceNomValeurDynamique(oldName, newName)) ((CCommentaire)e).determineDependances();
							}
						}
						j++;
						deb = inddeb + 1 + i + j*(nbObjImp+1);
					}
				}
			}
		}
		// On reconstitue les formules des calculs ou fonctions
		for (int i = inddeb + 1; i < lon; i++) {
			CElementBase elb = get(i);
			if (elb.estDeNatureCalcul(NatureCalcul.NCalculOuFonctionNonConstante)) {
				CCalcul calc = (CCalcul) elb;
				calc.reconstruitChaineCalcul();
			}
		}
		// On met  jour les lments ajouts et on les met  jour
		for (int i = inddeb + 1; i < lon; i++) {
			CElementBase elb = get(i); 
			elb.metAJour();
		}
	}

	/**
	 * Pour les implmentations rcursives, la structure est trop complexe pour respecter les noms initiaux des
	 * points, droites ou calculs dans le prototype
	 * @param inddeb
	 * @param proto
	 * @param cadre
	 */
	public final void traiteImplementationsProtoSuccessivesRecur(int inddeb, CPrototype proto, MtgInternalFrame cadre) {
		int lon = this.size();
		// On commence par renommer les points ou droites qui sont utiliss comme sources qui sont nomms dans le prototype
		for (int i = 0; i < proto.nbObjSources; i++) {
			CElementGenerique ptgen = (CElementGenerique) proto.get(i);
			CElementBase elb = ptgen.elementAssocie;
			if (elb.estDeNature(NatureObjet.NToutPoint|NatureObjet.NDroite|NatureObjet.NDroiteClone)) {
				CElementGraphique ptp = (CElementGraphique) elb;
				String nomGen = ptgen.nom;
				// On regarde si l'lment correspondant tait nomm. 
				if ((nomGen.length() != 0) && (ptp.rendNom().length() == 0)) {
					ptp.donneNom(genereDebutNomPourPoint(inddeb-1));
				}
			}
		}
		String nomPoint = genereDebutNomPourPoint(inddeb - 1);
		String nomCalcul = genereDebutNomPourCalcul("", inddeb - 1);
		int nomp = 1;
		// int nomc = 1; // Modifi version 4.9.9
		int nomc = 0;
		int k = 0;
		for (int i = inddeb+1; i < lon; i++) {
			CElementBase elb = get(i);
			if (elb.estDeNature(NatureObjet.NImplementationProto)) 
				k = -1;
			else {
				if (elb.estDeNature(NatureObjet.NToutPoint|NatureObjet.NDroite|NatureObjet.NDroiteClone)) {
					CElementGraphique elg = (CElementGraphique)elb;
					CElementGraphique elg2 = (CElementGraphique) proto.get(proto.nbObjSources + k);
					if (!elg2.rendNom().isEmpty()) elg.donneNom(nomPoint+nomp++);
				}
				else { 
					if (elb.estDeNatureCalcul(NatureCalcul.NToutCalculNomme)) {
						CValeurDynamique vald = (CValeurDynamique) elb;
						String oldName = vald.nom();
						String newName = nomCalcul+nomc++;
						vald.donneNom(newName);
						// Ajout version 4.9.9. On remet  jour les CCommentaire ou CLateX qui uitiliseraient un affichage dynamique de la valeur
						for (int j=i+1; j<lon; j++) {
							CElementBase e = get(j);
							if (e.estDeNature(NatureObjet.NImplementationProto)) break;
							if (e.estDeNature(NatureObjet.NCommentaire | NatureObjet.NLatex)) {
								if (((CCommentaire)e).listeValeursDynamiquesUtilisees.size()!=0) 
 									if (((CCommentaire)e).remplaceNomValeurDynamique(oldName, newName)) ((CCommentaire)e).determineDependances();
							}
						}
					}
					// Correction version 4.9.9
					// else if (elb.estDeNatureCalcul(NatureCalcul.NCalculOuFonctionNonConstante)) {
					if (elb.estDeNatureCalcul(NatureCalcul.NCalculOuFonctionNonConstante)) {
						CCalcul calc = (CCalcul) elb;
						calc.reconstruitChaineCalcul();
					}
				}
			}
			k++;
		}
		// Ajout version 4.9.9 : On met  jour les lments ajouts et on les met  jour
		for (int i = inddeb + 1; i < lon; i++) {
			CElementBase elb = get(i);
			elb.metAJour();
		}
	}
	
/**
 * Ajout version 4.8.1 pour exportation tikz.
 * Convertit l'abscisse en coordonnes dans un repre classique avec la longueur unit reprsentant 1 cm
 * @param x
 * @param dimf
 * @return
 */
	public final double tikzXcoord(double x, DimensionFenetre dimf, boolean bu) {
		if ((pointeurLongueurUnite != null) && bu) {
			double lo = pointeurLongueurUnite.rendLongueur();
			return x/lo;
		}
		else {
			double res = (double) Toolkit.getDefaultToolkit().getScreenResolution() / 2.54;
			return x/res;
		}
	}

	/**
	 * Ajout version 4.8.1 pour exportation tikz.
	 * Convertit l'ordonne en coordonnes dans un repre classique avec la longueur unit reprsentant 1 cm
	 * @param x
	 * @param dimf
	 * @return
	 */
	public final double tikzYcoord(double y, DimensionFenetre dimf, boolean bu) {
		if ((pointeurLongueurUnite != null) && bu) {
			double lo = pointeurLongueurUnite.rendLongueur();
			return (dimf.longueurOrdonnees - y)/lo;
		}
		else {
			double res = (double) Toolkit.getDefaultToolkit().getScreenResolution() / 2.54;
			return (dimf.longueurOrdonnees - y)/res;		
		}
	}

	// Ajout version 4.8.1
	public final double tikzLongueur(double lon, DimensionFenetre dimf, boolean bu) {
		if ((pointeurLongueurUnite != null) && bu) {
			double lo = pointeurLongueurUnite.rendLongueur();
			return lon/lo;		
		}
		else {
			double res = (double) Toolkit.getDefaultToolkit().getScreenResolution() / 2.54;
			return lon/res;
		}
	}
	
	/**
	 * Ajout version 4.9.1
	 * Exporte toutes les images de la figure dans le dossier contenant la figure.
	 * Les fichiers ont pour nom le nom de la figure suivi du numro de l'image dans la figure.
	 */
	public final void exporteImages() {
		int j = 1;
		for (int i = 0; i < size(); i++) {
			CElementBase el = get(i);
			if (el.getNature() == NatureObjet.NImage) {
				CImage im = (CImage) el;
				im.exporteFichierPNG(j);
				j++;
			}
		}
	}
	
	// Ajout version 4.9.1
	/**
	 * Renvoie l'inidce de l'image dans la figure (en commenant  1)
	 * @param image
	 * @return
	 */
	public final int indexOfImage(CImage image) {
		int j = 1;
		for (int i = 0; i < size(); i++) {
			CElementBase el = get(i);
			if (el.getNature() == NatureObjet.NImage) {
				CImage im = (CImage) el;
				if (im == image) return j;
				j++;
			}
		}	
		return -1;
	}
	
	/**
	 * Fonction gnrant des noms pour les objets non nomms de la figure.
	 * Si interm est true, des noms sont aussi gnrs pour les objets intermmdiaires de constructions.
	 * @param liste
	 * @param interm
	 */
	public final void genereNoms(CListeObjets liste, boolean interm) {
		CAbscisseOrigineRep.indice = 0;
		COrdonneeOrigineRep.indice = 0;
		CUnitexRep.indice = 0;
		CUniteyRep.indice = 0; // Corrig version 5.3
		CArcDeCercleAncetre.indice = 0; // Modifi version 5.3
		CCercle.indice = 0;
		CCommentaire.indice = 0;
		CDemiDroite.indice = 0;
		CDemiPlan.indice = 0;
		CEditeurFormule.indice = 0;
		CGrapheSuiteRec.indice = 0;
		CGrapheSuiteRecComplexe.indice = 0;
		CImage.indice = 0;
		CLatex.indice = 0;
		CLieuDeBase.indice = 0;
		CLieuDiscretDeBase.indice = 0;
		CLieuObjetAncetre.indice = 0;
		CMacro.indice = 0;
		CMarqueAngleAncetre.indice = 0;
		CMarqueSegment.indice = 0;
		CPointAncetre.indice = 0;
		CDroiteAncetre.indice = 0;
		CSurface.indice = 0;
		CValeurAffichee.indice = 0;
		// On regarde combien a figure contient de points ou de droites qu'il faut nommer. Si ce nombre est nfrieur  50
		// On gnre des noms construits de faon gnrique pour les points et droites, ce ui oblige pour chacun  faire une gnration
		// alatoire et peut tre chronophage sur les figures trs lourdes obtenues par rcursion par exemple.
		// Sinon les points auront un nom commenant par Pt et le sdroites par dt
		int compteur = 0;
		for (int i = 0; i < size(); i++) {
			CElementBase el = get(i);
			if (el.nom().isEmpty()) compteur ++;
		}
		if ((compteur <= 100) && (this.size() <= 500)) {
			for (int i = 0; i < size(); i++) {
				CElementBase el = get(i);
				if (el.nom().isEmpty()) {
					if ((!el.estElementIntermediaire() || interm)) {
						liste.add(el);
						el.nom = el.genereNom();
						if (el.estDeNature(NatureObjet.NToutObjet)) {
							CElementGraphique elg = (CElementGraphique) el;
							elg.nomMasque = false;
							elg.indiceTailleNom = (int)(MtgFrame.coefMultForDlgBoxIcons*13);
						}
					}
					else {
						el.nom = MtgFrame.getString("interm");
						liste.add(el);
					}
				}
			}
		}
		else {
			for (int i = 0; i < size(); i++) {
				CElementBase el = get(i);
				if (el.nom().isEmpty()) {
					if (!el.estElementIntermediaire() || interm) {
						if (el.estDeNature(NatureObjet.NDroite)) el.nom = ((CDroiteAncetre)el).genereNom(MtgFrame.getString("rdt"));
						else if (el.estDeNature(NatureObjet.NToutPoint)) el.nom = ((CPointAncetre)el).genereNom(MtgFrame.getString("rPt"));
						else el.nom = el.genereNom();				
						liste.add(el);
					}
					else {
						el.nom = MtgFrame.getString("interm");
						liste.add(el);
					}
				}
			}			
		}
	}
	
	/**
	 * Fonction supprimant les noms affects aux objets pour l'historique
	 * @param liste
	 */
	// Ajout version 4.9.9
	/* Modifi version 5.1. Pas assez optimis
	public final void effaceNoms(CListeObjets liste) {
		for (int i = 0; i < size(); i++) {
			CElementBase el = get(i);
			if (liste.contains(el)) el.nom = "";
		}	
	}
	*/
	public final void effaceNoms() {
		for (int i = 0; i < size(); i++) {
			CElementBase el = get(i);
			el.nom = "";
		}	
	}

	

	/**
	 * Fonction renvoyant une chane de caractres forme de tous les noms des lmnts de la liste spars par des virgules
	 * @return
	 */
	// Ajout version 4.9.9
  public final String listeNoms() {
  	int s = size();
  	String ch = "";
  	for (int i = 0; i < s; i++) {
  		ch += get(i).nom();
  		if (i != s-1) ch+= ",";
  	}
  	return ch;
  }

  public final boolean augmenteTaillePolices(MtgInternalFrame cadre) {
  	boolean done = false;
		for (int i = 0; i < size(); i++) {
			CElementBase el = get(i);
			if (el.estDeNature(NatureObjet.NAffichageLieAPoint)) {
				if (el.estDeNature(NatureObjet.NEditeurFormule)) {
					CEditeurFormule ed = (CEditeurFormule) el;
					if (ed.indiceTaillePolice < PaneTaillePolice.taillePolicemax) {
						ed.indiceTaillePolice++;
						done = true;
						ed.retireComposant(cadre);
						ed.creeComposant();
						ed.metAJour();
					}
				}
				else {
					CAffichageLieAPoint aff = (CAffichageLieAPoint)el;
					if (aff.indiceTaillePolice < PaneTaillePolice.taillePolicemax) {
						aff.indiceTaillePolice++;
						done = true;
					}
					if (el.estDeNature(NatureObjet.NLatex)) ((CLatex)el).dejaPositionne = false;
				}
			}
			else {
				if (el.estDeNature(NatureObjet.NToutPoint | NatureObjet.NTouteDroite)) {
					CElementGraphique elg = (CElementGraphique) el;
					if (elg.indiceTailleNom < PaneTaillePolice.taillePolicemax) {
						elg.indiceTailleNom++;
						done = true;
					}
				}
			}
		}
		return done;
  }
  public final boolean changeStyleTousPoints(MtgFrame frame) {
  	boolean done = false;
		for (int i = 0; i < size(); i++) {
			CElementBase el = get(i);
			if (el.estDeNature(NatureObjet.NToutPoint)) {
				CPointAncetre pt = (CPointAncetre) el;
				if (pt.motif != frame.stylePointActif) {
					pt.motif = frame.stylePointActif;
					done = true;
				}
			}
		}
		return done;
  }

  public final boolean diminueTaillePolices(MtgInternalFrame cadre) {
  	boolean done = false;
		for (int i = 0; i < size(); i++) {
			CElementBase el = get(i);
			if (el.estDeNature(NatureObjet.NAffichageLieAPoint)) {
				if (el.estDeNature(NatureObjet.NEditeurFormule)) {
					CEditeurFormule ed = (CEditeurFormule) el;
					if (ed.indiceTaillePolice > PaneTaillePolice.taillePolicemin) {
						ed.indiceTaillePolice--;
						done = true;
						ed.retireComposant(cadre);
						ed.creeComposant();
						ed.metAJour();
					}
				}
				else {
					CAffichageLieAPoint aff = (CAffichageLieAPoint)el;
					if (aff.indiceTaillePolice > PaneTaillePolice.taillePolicemin) {
						aff.indiceTaillePolice--;
						done = true;
					}
					if (el.estDeNature(NatureObjet.NLatex)) ((CLatex)el).dejaPositionne = false;
				}
			}
			else {
				if (el.estDeNature(NatureObjet.NToutPoint | NatureObjet.NTouteDroite)) {
					CElementGraphique elg = (CElementGraphique) el;
					if (elg.indiceTailleNom > PaneTaillePolice.taillePolicemin) {
						elg.indiceTailleNom--;
						done = true;
					}
				}
			}
		}
		return done;
  }

}